@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/heaptuple.c,v 1.27 1997/09/24 17:44:24 thomas Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/heaptuple.c,v 1.37 1998/02/26 04:29:15 momjian Exp $
  *
  * NOTES
  *	  The old interface functions have been converted to macros
@@ -23,6 +23,7 @@
 #include <access/htup.h>
 #include <access/transam.h>
 #include <access/tupmacs.h>
+#include <catalog/pg_type.h>
 #include <storage/bufpage.h>
 #include <utils/memutils.h>
 
@@ -32,12 +33,16 @@
 #include <string.h>
 #endif
 
-
-/* this is so the sparcstation debugger works */
-
-#if !defined(NO_ASSERT_CHECKING) && defined(sparc) && defined(sunos4)
-#define register
-#endif							/* !NO_ASSERT_CHECKING && sparc && sunos4 */
+/* Used by heap_getattr() macro, for speed */
+long		heap_sysoffset[] = {
+/* Only the first one is pass-by-ref, and is handled specially in the macro */
+	offsetof(HeapTupleData, t_ctid),
+	offsetof(HeapTupleData, t_oid),
+	offsetof(HeapTupleData, t_xmin),
+	offsetof(HeapTupleData, t_cmin),
+	offsetof(HeapTupleData, t_xmax),
+	offsetof(HeapTupleData, t_cmax)
+};
 
 /* ----------------------------------------------------------------
  *						misc support routines
@@ -93,7 +98,7 @@ ComputeDataSize(TupleDesc tupleDesc,
 				break;
 			default:
 				if (att[i]->attlen < sizeof(int32))
-					elog(WARN, "ComputeDataSize: attribute %d has len %d",
+					elog(ERROR, "ComputeDataSize: attribute %d has len %d",
 						 i, att[i]->attlen);
 				if (att[i]->attalign == 'd')
 					data_length = DOUBLEALIGN(data_length) + att[i]->attlen;
@@ -115,7 +120,7 @@ DataFill(char *data,
 		 TupleDesc tupleDesc,
 		 Datum value[],
 		 char nulls[],
-		 char *infomask,
+		 uint16 *infomask,
 		 bits8 *bit)
 {
 	bits8	   *bitP = 0;
@@ -194,7 +199,7 @@ DataFill(char *data,
 				break;
 			default:
 				if (att[i]->attlen < sizeof(int32))
-					elog(WARN, "DataFill: attribute %d has len %d",
+					elog(ERROR, "DataFill: attribute %d has len %d",
 						 i, att[i]->attlen);
 				if (att[i]->attalign == 'd')
 				{
@@ -246,18 +251,13 @@ heap_attisnull(HeapTuple tup, int attnum)
 			case MinCommandIdAttributeNumber:
 			case MaxTransactionIdAttributeNumber:
 			case MaxCommandIdAttributeNumber:
-			case ChainItemPointerAttributeNumber:
-			case AnchorItemPointerAttributeNumber:
-			case MinAbsoluteTimeAttributeNumber:
-			case MaxAbsoluteTimeAttributeNumber:
-			case VersionTypeAttributeNumber:
 				break;
 
 			case 0:
-				elog(WARN, "heap_attisnull: zero attnum disallowed");
+				elog(ERROR, "heap_attisnull: zero attnum disallowed");
 
 			default:
-				elog(WARN, "heap_attisnull: undefined negative attnum");
+				elog(ERROR, "heap_attisnull: undefined negative attnum");
 		}
 
 	return (0);
@@ -293,21 +293,9 @@ heap_sysattrlen(AttrNumber attno)
 			return sizeof f->t_xmax;
 		case MaxCommandIdAttributeNumber:
 			return sizeof f->t_cmax;
-		case ChainItemPointerAttributeNumber:
-			return sizeof f->t_chain;
-		case MinAbsoluteTimeAttributeNumber:
-			return sizeof f->t_tmin;
-		case MaxAbsoluteTimeAttributeNumber:
-			return sizeof f->t_tmax;
-		case VersionTypeAttributeNumber:
-			return sizeof f->t_vtype;
-
-		case AnchorItemPointerAttributeNumber:
-			elog(WARN, "heap_sysattrlen: field t_anchor does not exist!");
-			return 0;
 
 		default:
-			elog(WARN, "sysattrlen: System attribute number %d unknown.", attno);
+			elog(ERROR, "sysattrlen: System attribute number %d unknown.", attno);
 			return 0;
 	}
 }
@@ -343,24 +331,9 @@ heap_sysattrbyval(AttrNumber attno)
 		case MaxCommandIdAttributeNumber:
 			byval = true;
 			break;
-		case ChainItemPointerAttributeNumber:
-			byval = false;
-			break;
-		case AnchorItemPointerAttributeNumber:
-			byval = false;
-			break;
-		case MinAbsoluteTimeAttributeNumber:
-			byval = true;
-			break;
-		case MaxAbsoluteTimeAttributeNumber:
-			byval = true;
-			break;
-		case VersionTypeAttributeNumber:
-			byval = true;
-			break;
 		default:
 			byval = true;
-			elog(WARN, "sysattrbyval: System attribute number %d unknown.",
+			elog(ERROR, "sysattrbyval: System attribute number %d unknown.",
 				 attno);
 			break;
 	}
@@ -389,49 +362,19 @@ heap_getsysattr(HeapTuple tup, Buffer b, int attnum)
 			return ((Datum) (long) tup->t_xmax);
 		case MaxCommandIdAttributeNumber:
 			return ((Datum) (long) tup->t_cmax);
-		case ChainItemPointerAttributeNumber:
-			return ((Datum) &tup->t_chain);
-		case AnchorItemPointerAttributeNumber:
-			elog(WARN, "heap_getsysattr: t_anchor does not exist!");
-			break;
-
-			/*
-			 * For tmin and tmax, we need to do some extra work.  These
-			 * don't get filled in until the vacuum cleaner runs (or we
-			 * manage to flush a page after setting the value correctly
-			 * below).	If the vacuum cleaner hasn't run yet, then the
-			 * times stored in the tuple are wrong, and we need to look up
-			 * the commit time of the transaction. We cache this value in
-			 * the tuple to avoid doing the work more than once.
-			 */
-
-		case MinAbsoluteTimeAttributeNumber:
-			if (!AbsoluteTimeIsBackwardCompatiblyValid(tup->t_tmin) &&
-				TransactionIdDidCommit(tup->t_xmin))
-				tup->t_tmin = TransactionIdGetCommitTime(tup->t_xmin);
-			return ((Datum) (long) tup->t_tmin);
-		case MaxAbsoluteTimeAttributeNumber:
-			if (!AbsoluteTimeIsBackwardCompatiblyReal(tup->t_tmax))
-			{
-				if (TransactionIdDidCommit(tup->t_xmax))
-					tup->t_tmax = TransactionIdGetCommitTime(tup->t_xmax);
-				else
-					tup->t_tmax = CURRENT_ABSTIME;
-			}
-			return ((Datum) (long) tup->t_tmax);
-		case VersionTypeAttributeNumber:
-			return ((Datum) (long) tup->t_vtype);
 		default:
-			elog(WARN, "heap_getsysattr: undefined attnum %d", attnum);
+			elog(ERROR, "heap_getsysattr: undefined attnum %d", attnum);
 	}
 	return ((Datum) NULL);
 }
 
 /* ----------------
- *		fastgetattr
+ *		nocachegetattr
  *
- *		This is a newer version of fastgetattr which attempts to be
- *		faster by caching attribute offsets in the attribute descriptor.
+ *		This only gets called from fastgetattr() macro, in cases where
+ *		we can't use a cacheoffset and the value is not null.
+ *
+ *		This caches attribute offsets in the attribute descriptor.
  *
  *		an alternate way to speed things up would be to cache offsets
  *		with the tuple, but that seems more difficult unless you take
@@ -445,23 +388,25 @@ heap_getsysattr(HeapTuple tup, Buffer b, int attnum)
  * ----------------
  */
 Datum
-fastgetattr(HeapTuple tup,
-			int attnum,
-			TupleDesc tupleDesc,
-			bool *isnull)
+nocachegetattr(HeapTuple tup,
+			   int attnum,
+			   TupleDesc tupleDesc,
+			   bool *isnull)
 {
 	char	   *tp;				/* ptr to att in tuple */
-	bits8	   *bp = NULL;		/* ptr to att in tuple */
+	bits8	   *bp = tup->t_bits;		/* ptr to att in tuple */
 	int			slow;			/* do we have to walk nulls? */
 	AttributeTupleForm *att = tupleDesc->attrs;
 
-	/* ----------------
-	 *	sanity checks
-	 * ----------------
-	 */
 
+#if IN_MACRO
+/* This is handled in the macro */
 	Assert(attnum > 0);
 
+	if (isnull)
+		*isnull = false;
+#endif
+
 	/* ----------------
 	 *	 Three cases:
 	 *
@@ -471,13 +416,13 @@ fastgetattr(HeapTuple tup,
 	 * ----------------
 	 */
 
-	if (isnull)
-		*isnull = false;
-
 	if (HeapTupleNoNulls(tup))
 	{
 		attnum--;
-		if (att[attnum]->attcacheoff > 0)
+
+#if IN_MACRO
+/* This is handled in the macro */
+		if (att[attnum]->attcacheoff != -1)
 		{
 			return (Datum)
 				fetchatt(&(att[attnum]),
@@ -491,8 +436,7 @@ fastgetattr(HeapTuple tup,
 			 */
 			return ((Datum) fetchatt(&(att[0]), (char *) tup + tup->t_hoff));
 		}
-
-		tp = (char *) tup + tup->t_hoff;
+#endif
 
 		slow = 0;
 	}
@@ -503,7 +447,6 @@ fastgetattr(HeapTuple tup,
 		 * there's a null somewhere in the tuple
 		 */
 
-		bp = tup->t_bits;
 		tp = (char *) tup + tup->t_hoff;
 		slow = 0;
 		attnum--;
@@ -513,51 +456,76 @@ fastgetattr(HeapTuple tup,
 		 * ----------------
 		 */
 
+#if IN_MACRO
+/* This is handled in the macro */
 		if (att_isnull(attnum, bp))
 		{
 			if (isnull)
 				*isnull = true;
 			return (Datum) NULL;
 		}
+#endif
 
 		/* ----------------
 		 *		Now check to see if any preceeding bits are null...
 		 * ----------------
 		 */
-
 		{
-			register int i = 0; /* current offset in bp */
+			int			i = 0;	/* current offset in bp */
+			int			mask;	/* bit in byte we're looking at */
+			char		n;		/* current byte in bp */
+			int			byte,
+						finalbit;
 
-			for (i = 0; i < attnum && !slow; i++)
+			byte = attnum >> 3;
+			finalbit = attnum & 0x07;
+
+			for (; i <= byte && !slow; i++)
 			{
-				if (att_isnull(i, bp))
-					slow = 1;
+				n = bp[i];
+				if (i < byte)
+				{
+					/* check for nulls in any "earlier" bytes */
+					if ((~n) != 0)
+						slow = 1;
+				}
+				else
+				{
+					/* check for nulls "before" final bit of last byte */
+					mask = (1 << finalbit) - 1;
+					if ((~n) & mask)
+						slow = 1;
+				}
 			}
 		}
 	}
 
+	tp = (char *) tup + tup->t_hoff;
+
 	/*
 	 * now check for any non-fixed length attrs before our attribute
 	 */
 	if (!slow)
 	{
-		if (att[attnum]->attcacheoff > 0)
+		if (att[attnum]->attcacheoff != -1)
 		{
-			return (Datum)
-				fetchatt(&(att[attnum]),
-						 tp + att[attnum]->attcacheoff);
+			return (Datum) fetchatt(&(att[attnum]),
+									tp + att[attnum]->attcacheoff);
 		}
 		else if (attnum == 0)
 		{
-			return (Datum)
-				fetchatt(&(att[0]), (char *) tup + tup->t_hoff);
+			return ((Datum) fetchatt(&(att[0]), (char *) tp));
 		}
 		else if (!HeapTupleAllFixed(tup))
 		{
-			register int j = 0;
+			int			j = 0;
 
-			for (j = 0; j < attnum && !slow; j++)
-				if (att[j]->attlen < 1)
+			/*
+			 * In for(), we make this <= and not < because we want to test
+			 * if we can go past it in initializing offsets.
+			 */
+			for (j = 0; j <= attnum && !slow; j++)
+				if (att[j]->attlen < 1 && !VARLENA_FIXED_SIZE(att[j]))
 					slow = 1;
 		}
 	}
@@ -569,8 +537,8 @@ fastgetattr(HeapTuple tup,
 	 */
 	if (!slow)
 	{
-		register int j = 1;
-		register long off;
+		int			j = 1;
+		long		off;
 
 		/*
 		 * need to set cache for some atts
@@ -581,10 +549,25 @@ fastgetattr(HeapTuple tup,
 		while (att[j]->attcacheoff > 0)
 			j++;
 
-		off = att[j - 1]->attcacheoff + att[j - 1]->attlen;
+		if (!VARLENA_FIXED_SIZE(att[j - 1]))
+			off = att[j - 1]->attcacheoff + att[j - 1]->attlen;
+		else
+			off = att[j - 1]->attcacheoff + att[j - 1]->atttypmod;
 
-		for (; j < attnum + 1; j++)
+		for (; j <= attnum ||
+		/* Can we compute more?  We will probably need them */
+			 (j < tup->t_natts &&
+			  att[j]->attcacheoff == -1 &&
+			  (HeapTupleNoNulls(tup) || !att_isnull(j, bp)) &&
+			  (HeapTupleAllFixed(tup) ||
+			   att[j]->attlen > 0 || VARLENA_FIXED_SIZE(att[j]))); j++)
 		{
+
+			/*
+			 * Fix me when going to a machine with more than a four-byte
+			 * word!
+			 */
+
 			switch (att[j]->attlen)
 			{
 				case -1:
@@ -600,31 +583,46 @@ fastgetattr(HeapTuple tup,
 					off = INTALIGN(off);
 					break;
 				default:
-					if (att[j]->attlen < sizeof(int32))
-					{
-						elog(WARN,
-							 "fastgetattr: attribute %d has len %d",
-							 j, att[j]->attlen);
-					}
-					if (att[j]->attalign == 'd')
-						off = DOUBLEALIGN(off);
+					if (att[j]->attlen > sizeof(int32))
+						off = (att[j]->attalign == 'd') ?
+							DOUBLEALIGN(off) : LONGALIGN(off);
 					else
-						off = LONGALIGN(off);
+						elog(ERROR, "nocache_index_getattr: attribute %d has len %d",
+							 j, att[j]->attlen);
 					break;
 			}
 
 			att[j]->attcacheoff = off;
-			off += att[j]->attlen;
+
+			switch (att[j]->attlen)
+			{
+				case sizeof(char):
+					off++;
+					break;
+				case sizeof(short):
+					off += sizeof(short);
+					break;
+				case sizeof(int32):
+					off += sizeof(int32);
+					break;
+				case -1:
+					Assert(!VARLENA_FIXED_SIZE(att[j]) ||
+						   att[j]->atttypmod == VARSIZE(tp + off));
+					off += VARSIZE(tp + off);
+					break;
+				default:
+					off += att[j]->attlen;
+					break;
+			}
 		}
 
-		return
-			(Datum) fetchatt(&(att[attnum]), tp + att[attnum]->attcacheoff);
+		return (Datum) fetchatt(&(att[attnum]), tp + att[attnum]->attcacheoff);
 	}
 	else
 	{
-		register bool usecache = true;
-		register int off = 0;
-		register int i;
+		bool		usecache = true;
+		int			off = 0;
+		int			i;
 
 		/*
 		 * Now we know that we have to walk the tuple CAREFULLY.
@@ -646,41 +644,37 @@ fastgetattr(HeapTuple tup,
 					continue;
 				}
 			}
-			switch (att[i]->attlen)
-			{
-				case -1:
-					off = (att[i]->attalign == 'd') ?
-						DOUBLEALIGN(off) : INTALIGN(off);
-					break;
-				case sizeof(char):
-					break;
-				case sizeof(short):
-					off = SHORTALIGN(off);
-					break;
-				case sizeof(int32):
-					off = INTALIGN(off);
-					break;
-				default:
-					if (att[i]->attlen < sizeof(int32))
-						elog(WARN,
-							 "fastgetattr2: attribute %d has len %d",
-							 i, att[i]->attlen);
-					if (att[i]->attalign == 'd')
-						off = DOUBLEALIGN(off);
-					else
-						off = LONGALIGN(off);
-					break;
-			}
-			if (usecache && att[i]->attcacheoff > 0)
-			{
+
+			/* If we know the next offset, we can skip the rest */
+			if (usecache && att[i]->attcacheoff != -1)
 				off = att[i]->attcacheoff;
-				if (att[i]->attlen == -1)
-				{
-					usecache = false;
-				}
-			}
 			else
 			{
+				switch (att[i]->attlen)
+				{
+					case -1:
+						off = (att[i]->attalign == 'd') ?
+							DOUBLEALIGN(off) : INTALIGN(off);
+						break;
+					case sizeof(char):
+						break;
+					case sizeof(short):
+						off = SHORTALIGN(off);
+						break;
+					case sizeof(int32):
+						off = INTALIGN(off);
+						break;
+					default:
+						if (att[i]->attlen < sizeof(int32))
+							elog(ERROR,
+							  "nocachegetattr2: attribute %d has len %d",
+								 i, att[i]->attlen);
+						if (att[i]->attalign == 'd')
+							off = DOUBLEALIGN(off);
+						else
+							off = LONGALIGN(off);
+						break;
+				}
 				if (usecache)
 					att[i]->attcacheoff = off;
 			}
@@ -690,21 +684,25 @@ fastgetattr(HeapTuple tup,
 				case sizeof(char):
 					off++;
 					break;
-				case sizeof(int16):
-					off += sizeof(int16);
+				case sizeof(short):
+					off += sizeof(short);
 					break;
 				case sizeof(int32):
 					off += sizeof(int32);
 					break;
 				case -1:
-					usecache = false;
+					Assert(!VARLENA_FIXED_SIZE(att[i]) ||
+						   att[i]->atttypmod == VARSIZE(tp + off));
 					off += VARSIZE(tp + off);
+					if (!VARLENA_FIXED_SIZE(att[i]))
+						usecache = false;
 					break;
 				default:
 					off += att[i]->attlen;
 					break;
 			}
 		}
+
 		switch (att[attnum]->attlen)
 		{
 			case -1:
@@ -721,7 +719,7 @@ fastgetattr(HeapTuple tup,
 				break;
 			default:
 				if (att[attnum]->attlen < sizeof(int32))
-					elog(WARN, "fastgetattr3: attribute %d has len %d",
+					elog(ERROR, "nocachegetattr3: attribute %d has len %d",
 						 attnum, att[attnum]->attlen);
 				if (att[attnum]->attalign == 'd')
 					off = DOUBLEALIGN(off);
@@ -729,7 +727,8 @@ fastgetattr(HeapTuple tup,
 					off = LONGALIGN(off);
 				break;
 		}
-		return ((Datum) fetchatt(&(att[attnum]), tp + off));
+
+		return (Datum) fetchatt(&(att[attnum]), tp + off);
 	}
 }
 
@@ -750,7 +749,7 @@ heap_copytuple(HeapTuple tuple)
 	/* XXX For now, just prevent an undetectable executor related error */
 	if (tuple->t_len > MAXTUPLEN)
 	{
-		elog(WARN, "palloctup: cannot handle length %d tuples",
+		elog(ERROR, "palloctup: cannot handle length %d tuples",
 			 tuple->t_len);
 	}
 
@@ -783,7 +782,6 @@ heap_deformtuple(HeapTuple tuple,
 		bool		isnull;
 
 		values[i] = heap_getattr(tuple,
-								 InvalidBuffer,
 								 i + 1,
 								 tdesc,
 								 &isnull);
@@ -837,7 +835,7 @@ heap_formtuple(TupleDesc tupleDescriptor,
 	}
 
 	if (numberOfAttributes > MaxHeapAttributeNumber)
-		elog(WARN, "heap_formtuple: numberOfAttributes of %d > %d",
+		elog(ERROR, "heap_formtuple: numberOfAttributes of %d > %d",
 			 numberOfAttributes, MaxHeapAttributeNumber);
 
 	if (hasnull)
@@ -858,8 +856,6 @@ heap_formtuple(TupleDesc tupleDescriptor,
 	tuple->t_len = len;
 	tuple->t_natts = numberOfAttributes;
 	tuple->t_hoff = hoff;
-	tuple->t_tmin = INVALID_ABSTIME;
-	tuple->t_tmax = CURRENT_ABSTIME;
 
 	DataFill((char *) tuple + tuple->t_hoff,
 			 tupleDescriptor,
@@ -868,6 +864,8 @@ heap_formtuple(TupleDesc tupleDescriptor,
 			 &tuple->t_infomask,
 			 (hasnull ? tuple->t_bits : NULL));
 
+	tuple->t_infomask |= HEAP_XMAX_INVALID;
+
 	return (tuple);
 }
 
@@ -938,7 +936,6 @@ heap_modifytuple(HeapTuple tuple,
 		{
 			value[attoff] =
 				heap_getattr(tuple,
-							 InvalidBuffer,
 							 AttrOffsetGetAttrNumber(attoff),
 							 RelationGetTupleDescriptor(relation),
 							 &isNull);
@@ -947,7 +944,7 @@ heap_modifytuple(HeapTuple tuple,
 		}
 		else if (repl[attoff] != 'r')
 		{
-			elog(WARN, "heap_modifytuple: repl is \\%3d", repl[attoff]);
+			elog(ERROR, "heap_modifytuple: repl is \\%3d", repl[attoff]);
 
 		}
 		else
@@ -970,9 +967,9 @@ heap_modifytuple(HeapTuple tuple,
 	 * ----------------
 	 */
 	infomask = newTuple->t_infomask;
-	memmove((char *) &newTuple->t_ctid, /* XXX */
-			(char *) &tuple->t_ctid,
-			((char *) &tuple->t_hoff - (char *) &tuple->t_ctid));		/* XXX */
+	memmove((char *) &newTuple->t_oid,	/* XXX */
+			(char *) &tuple->t_oid,
+			((char *) &tuple->t_hoff - (char *) &tuple->t_oid));		/* XXX */
 	newTuple->t_infomask = infomask;
 	newTuple->t_natts = numberOfAttributes;		/* fix t_natts just in
 												 * case */
@@ -998,7 +995,7 @@ heap_addheader(uint32 natts,	/* max domain index */
 			   int structlen,	/* its length */
 			   char *structure) /* pointer to the struct */
 {
-	register char *tp;			/* tuple data pointer */
+	char	   *tp;				/* tuple data pointer */
 	HeapTuple	tup;
 	long		len;
 	int			hoff;
@@ -1013,10 +1010,11 @@ heap_addheader(uint32 natts,	/* max domain index */
 	tup = (HeapTuple) tp;
 	MemSet((char *) tup, 0, len);
 
-	tup->t_len = (short) len;	/* XXX */
+	tup->t_len = len;
 	tp += tup->t_hoff = hoff;
 	tup->t_natts = natts;
 	tup->t_infomask = 0;
+	tup->t_infomask |= HEAP_XMAX_INVALID;
 
 	memmove(tp, structure, structlen);
 
