@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/indextuple.c,v 1.19 1997/09/18 20:19:37 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/indextuple.c,v 1.28 1998/02/26 04:29:18 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,6 +19,7 @@
 #include <access/ibit.h>
 #include <access/itup.h>
 #include <access/tupmacs.h>
+#include <catalog/pg_type.h>
 
 #ifndef HAVE_MEMMOVE
 #include <regex/utils.h>
@@ -26,11 +27,6 @@
 #include <string.h>
 #endif
 
-static Size IndexInfoFindDataOffset(unsigned short t_info);
-static char *
-fastgetiattr(IndexTuple tup, int attnum,
-			 TupleDesc att, bool *isnull);
-
 /* ----------------------------------------------------------------
  *				  index_ tuple interface routines
  * ----------------------------------------------------------------
@@ -45,18 +41,18 @@ index_formtuple(TupleDesc tupleDescriptor,
 				Datum value[],
 				char null[])
 {
-	register char *tp;			/* tuple pointer */
+	char	   *tp;				/* tuple pointer */
 	IndexTuple	tuple;			/* return tuple */
 	Size		size,
 				hoff;
 	int			i;
 	unsigned short infomask = 0;
 	bool		hasnull = false;
-	char		tupmask = 0;
+	uint16		tupmask = 0;
 	int			numberOfAttributes = tupleDescriptor->natts;
 
 	if (numberOfAttributes > MaxIndexAttributeNumber)
-		elog(WARN, "index_formtuple: numberOfAttributes of %d > %d",
+		elog(ERROR, "index_formtuple: numberOfAttributes of %d > %d",
 			 numberOfAttributes, MaxIndexAttributeNumber);
 
 
@@ -103,7 +99,7 @@ index_formtuple(TupleDesc tupleDescriptor,
 	 */
 
 	if (size & 0xE000)
-		elog(WARN, "index_formtuple: data takes %d bytes: too big", size);
+		elog(ERROR, "index_formtuple: data takes %d bytes: too big", size);
 
 
 	infomask |= size;
@@ -117,10 +113,12 @@ index_formtuple(TupleDesc tupleDescriptor,
 }
 
 /* ----------------
- *		fastgetiattr
+ *		nocache_index_getattr
  *
- *		This is a newer version of fastgetiattr which attempts to be
- *		faster by caching attribute offsets in the attribute descriptor.
+ *		This gets called from index_getattr() macro, and only in cases
+ *		where we can't use cacheoffset and the value is not null.
+ *
+ *		This caches attribute offsets in the attribute descriptor.
  *
  *		an alternate way to speed things up would be to cache offsets
  *		with the tuple, but that seems more difficult unless you take
@@ -133,16 +131,16 @@ index_formtuple(TupleDesc tupleDescriptor,
  *		the same attribute descriptor will go much quicker. -cim 5/4/91
  * ----------------
  */
-static char *
-fastgetiattr(IndexTuple tup,
-			 int attnum,
-			 TupleDesc tupleDesc,
-			 bool *isnull)
+Datum
+nocache_index_getattr(IndexTuple tup,
+					  int attnum,
+					  TupleDesc tupleDesc,
+					  bool *isnull)
 {
-	register char *tp;			/* ptr to att in tuple */
-	register char *bp = NULL;	/* ptr to att in tuple */
+	char	   *tp;				/* ptr to att in tuple */
+	char	   *bp = NULL;		/* ptr to att in tuple */
 	int			slow;			/* do we have to walk nulls? */
-	register int data_off;		/* tuple data offset */
+	int			data_off;		/* tuple data offset */
 	AttributeTupleForm *att = tupleDesc->attrs;
 
 	/* ----------------
@@ -150,9 +148,6 @@ fastgetiattr(IndexTuple tup,
 	 * ----------------
 	 */
 
-	Assert(PointerIsValid(isnull));
-	Assert(attnum > 0);
-
 	/* ----------------
 	 *	 Three cases:
 	 *
@@ -162,37 +157,43 @@ fastgetiattr(IndexTuple tup,
 	 * ----------------
 	 */
 
+#ifdef IN_MACRO
+/* This is handled in the macro */
+	Assert(PointerIsValid(isnull));
+	Assert(attnum > 0);
+
 	*isnull = false;
+#endif
+
 	data_off = IndexTupleHasMinHeader(tup) ? sizeof *tup :
 		IndexInfoFindDataOffset(tup->t_info);
 
 	if (IndexTupleNoNulls(tup))
 	{
+		attnum--;
+
+#ifdef IN_MACRO
+/* This is handled in the macro */
 
 		/* first attribute is always at position zero */
 
 		if (attnum == 1)
 		{
-			return (fetchatt(&(att[0]), (char *) tup + data_off));
+			return (Datum) fetchatt(&(att[0]), (char *) tup + data_off);
 		}
-		attnum--;
-
-		if (att[attnum]->attcacheoff > 0)
+		if (att[attnum]->attcacheoff != -1)
 		{
-			return (fetchatt(&(att[attnum]),
-							 (char *) tup + data_off +
-							 att[attnum]->attcacheoff));
+			return (Datum) fetchatt(&(att[attnum]),
+									(char *) tup + data_off +
+									att[attnum]->attcacheoff);
 		}
-
-		tp = (char *) tup + data_off;
+#endif
 
 		slow = 0;
 	}
 	else
 	{							/* there's a null somewhere in the tuple */
 
-		bp = (char *) tup + sizeof(*tup);		/* "knows" t_bits are
-												 * here! */
 		slow = 0;
 		/* ----------------
 		 *		check to see if desired att is null
@@ -200,66 +201,74 @@ fastgetiattr(IndexTuple tup,
 		 */
 
 		attnum--;
+
+		bp = (char *) tup + sizeof(*tup);		/* "knows" t_bits are
+												 * here! */
+#ifdef IN_MACRO
+/* This is handled in the macro */
+
+		if (att_isnull(attnum, bp))
 		{
-			if (att_isnull(attnum, bp))
-			{
-				*isnull = true;
-				return NULL;
-			}
+			*isnull = true;
+			return (Datum) NULL;
 		}
+#endif
+
 		/* ----------------
 		 *		Now check to see if any preceeding bits are null...
 		 * ----------------
 		 */
 		{
-			register int i = 0; /* current offset in bp */
-			register int mask;	/* bit in byte we're looking at */
-			register char n;	/* current byte in bp */
-			register int byte,
+			int			i = 0;	/* current offset in bp */
+			int			mask;	/* bit in byte we're looking at */
+			char		n;		/* current byte in bp */
+			int			byte,
 						finalbit;
 
 			byte = attnum >> 3;
 			finalbit = attnum & 0x07;
 
-			for (; i <= byte; i++)
+			for (; i <= byte && !slow; i++)
 			{
 				n = bp[i];
 				if (i < byte)
 				{
 					/* check for nulls in any "earlier" bytes */
 					if ((~n) != 0)
-					{
-						slow++;
-						break;
-					}
+						slow = 1;
 				}
 				else
 				{
 					/* check for nulls "before" final bit of last byte */
-					mask = (finalbit << 1) - 1;
+					mask = (1 << finalbit) - 1;
 					if ((~n) & mask)
-						slow++;
+						slow = 1;
 				}
 			}
 		}
-		tp = (char *) tup + data_off;
 	}
 
+	tp = (char *) tup + data_off;
+
 	/* now check for any non-fixed length attrs before our attribute */
 
 	if (!slow)
 	{
-		if (att[attnum]->attcacheoff > 0)
+		if (att[attnum]->attcacheoff != -1)
 		{
-			return (fetchatt(&(att[attnum]),
-							 tp + att[attnum]->attcacheoff));
+			return (Datum) fetchatt(&(att[attnum]),
+									tp + att[attnum]->attcacheoff);
+		}
+		else if (attnum == 0)
+		{
+			return ((Datum) fetchatt(&(att[0]), (char *) tp));
 		}
 		else if (!IndexTupleAllFixed(tup))
 		{
-			register int j = 0;
+			int			j = 0;
 
 			for (j = 0; j < attnum && !slow; j++)
-				if (att[j]->attlen < 1)
+				if (att[j]->attlen < 1 && !VARLENA_FIXED_SIZE(att[j]))
 					slow = 1;
 		}
 	}
@@ -272,8 +281,8 @@ fastgetiattr(IndexTuple tup,
 
 	if (!slow)
 	{
-		register int j = 1;
-		register long off;
+		int			j = 1;
+		long		off;
 
 		/*
 		 * need to set cache for some atts
@@ -281,11 +290,13 @@ fastgetiattr(IndexTuple tup,
 
 		att[0]->attcacheoff = 0;
 
-		while (att[j]->attcacheoff > 0)
+		while (att[j]->attcacheoff != -1)
 			j++;
 
-		off = att[j - 1]->attcacheoff +
-			att[j - 1]->attlen;
+		if (!VARLENA_FIXED_SIZE(att[j - 1]))
+			off = att[j - 1]->attcacheoff + att[j - 1]->attlen;
+		else
+			off = att[j - 1]->attcacheoff + att[j - 1]->atttypmod;
 
 		for (; j < attnum + 1; j++)
 		{
@@ -314,24 +325,31 @@ fastgetiattr(IndexTuple tup,
 						off = (att[j]->attalign == 'd') ?
 							DOUBLEALIGN(off) : LONGALIGN(off);
 					else
-						elog(WARN, "fastgetiattr: attribute %d has len %d",
+						elog(ERROR, "nocache_index_getattr: attribute %d has len %d",
 							 j, att[j]->attlen);
 					break;
 
 			}
 
 			att[j]->attcacheoff = off;
-			off += att[j]->attlen;
+
+			/* The only varlena/-1 length value to get here is this */
+			if (!VARLENA_FIXED_SIZE(att[j]))
+				off += att[j]->attlen;
+			else
+			{
+				Assert(att[j]->atttypmod == VARSIZE(tp + off));
+				off += att[j]->atttypmod;
+			}
 		}
 
-		return (fetchatt(&(att[attnum]),
-						 tp + att[attnum]->attcacheoff));
+		return (Datum) fetchatt(&(att[attnum]), tp + att[attnum]->attcacheoff);
 	}
 	else
 	{
-		register bool usecache = true;
-		register int off = 0;
-		register int i;
+		bool		usecache = true;
+		int			off = 0;
+		int			i;
 
 		/*
 		 * Now we know that we have to walk the tuple CAREFULLY.
@@ -348,51 +366,64 @@ fastgetiattr(IndexTuple tup,
 				}
 			}
 
-			if (usecache && att[i]->attcacheoff > 0)
-			{
+			/* If we know the next offset, we can skip the rest */
+			if (usecache && att[i]->attcacheoff != -1)
 				off = att[i]->attcacheoff;
-				if (att[i]->attlen == -1)
-					usecache = false;
-				else
-					continue;
+			else
+			{
+				switch (att[i]->attlen)
+				{
+					case -1:
+						off = (att[i]->attalign == 'd') ?
+							DOUBLEALIGN(off) : INTALIGN(off);
+						break;
+					case sizeof(char):
+						break;
+					case sizeof(short):
+						off = SHORTALIGN(off);
+						break;
+					case sizeof(int32):
+						off = INTALIGN(off);
+						break;
+					default:
+						if (att[i]->attlen < sizeof(int32))
+							elog(ERROR,
+							 "nocachegetiattr2: attribute %d has len %d",
+								 i, att[i]->attlen);
+						if (att[i]->attalign == 'd')
+							off = DOUBLEALIGN(off);
+						else
+							off = LONGALIGN(off);
+						break;
+				}
+				if (usecache)
+					att[i]->attcacheoff = off;
 			}
 
-			if (usecache)
-				att[i]->attcacheoff = off;
 			switch (att[i]->attlen)
 			{
 				case sizeof(char):
 					off++;
 					break;
 				case sizeof(short):
-					off = SHORTALIGN(off) +sizeof(short);
+					off += sizeof(short);
 					break;
 				case sizeof(int32):
-					off = INTALIGN(off) +sizeof(int32);
+					off += sizeof(int32);
 					break;
 				case -1:
-					usecache = false;
-					off = (att[i]->attalign == 'd') ?
-						DOUBLEALIGN(off) : INTALIGN(off);
+					Assert(!VARLENA_FIXED_SIZE(att[i]) ||
+						   att[i]->atttypmod == VARSIZE(tp + off));
 					off += VARSIZE(tp + off);
+					if (!VARLENA_FIXED_SIZE(att[i]))
+						usecache = false;
 					break;
 				default:
-					if (att[i]->attlen > sizeof(int32))
-						off = (att[i]->attalign == 'd') ?
-							DOUBLEALIGN(off) + att[i]->attlen :
-							LONGALIGN(off) + att[i]->attlen;
-					else
-						elog(WARN, "fastgetiattr2: attribute %d has len %d",
-							 i, att[i]->attlen);
-
+					off += att[i]->attlen;
 					break;
 			}
 		}
 
-		/*
-		 * I don't know why this code was missed here! I've got it from
-		 * heaptuple.c:fastgetattr(). - vadim 06/12/97
-		 */
 		switch (att[attnum]->attlen)
 		{
 			case -1:
@@ -409,7 +440,7 @@ fastgetiattr(IndexTuple tup,
 				break;
 			default:
 				if (att[attnum]->attlen < sizeof(int32))
-					elog(WARN, "fastgetattr3: attribute %d has len %d",
+					elog(ERROR, "nocache_index_getattr: attribute %d has len %d",
 						 attnum, att[attnum]->attlen);
 				if (att[attnum]->attalign == 'd')
 					off = DOUBLEALIGN(off);
@@ -418,26 +449,10 @@ fastgetiattr(IndexTuple tup,
 				break;
 		}
 
-		return (fetchatt(&att[attnum], tp + off));
+		return (Datum) fetchatt(&att[attnum], tp + off);
 	}
 }
 
-/* ----------------
- *		index_getattr
- * ----------------
- */
-Datum
-index_getattr(IndexTuple tuple,
-			  AttrNumber attNum,
-			  TupleDesc tupDesc,
-			  bool *isNullOutP)
-{
-	Assert(attNum > 0);
-
-	return (Datum)
-		fastgetiattr(tuple, attNum, tupDesc, isNullOutP);
-}
-
 RetrieveIndexResult
 FormRetrieveIndexResult(ItemPointer indexItemPointer,
 						ItemPointer heapItemPointer)
@@ -455,29 +470,6 @@ FormRetrieveIndexResult(ItemPointer indexItemPointer,
 	return (result);
 }
 
-/*
- * Takes an infomask as argument (primarily because this needs to be usable
- * at index_formtuple time so enough space is allocated).
- *
- * Change me if adding an attribute to IndexTuples!!!!!!!!!!!
- */
-static Size
-IndexInfoFindDataOffset(unsigned short t_info)
-{
-	if (!(t_info & INDEX_NULL_MASK))
-		return ((Size) sizeof(IndexTupleData));
-	else
-	{
-		Size		size = sizeof(IndexTupleData);
-
-		if (t_info & INDEX_NULL_MASK)
-		{
-			size += sizeof(IndexAttributeBitMapData);
-		}
-		return DOUBLEALIGN(size);		/* be conservative */
-	}
-}
-
 /*
  * Copies source into target.  If *target == NULL, we palloc space; otherwise
  * we assume we have space that is already palloc'ed.
