@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/printtup.c,v 1.18 1997/09/12 04:07:12 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/printtup.c,v 1.27 1998/02/26 04:29:20 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,7 +45,7 @@ typtoout(Oid type)
 		return ((Oid)
 				((TypeTupleForm) GETSTRUCT(typeTuple))->typoutput);
 
-	elog(WARN, "typtoout: Cache lookup of type %d failed", type);
+	elog(ERROR, "typtoout: Cache lookup of type %d failed", type);
 	return (InvalidOid);
 }
 
@@ -62,7 +62,7 @@ gettypelem(Oid type)
 		return ((Oid)
 				((TypeTupleForm) GETSTRUCT(typeTuple))->typelem);
 
-	elog(WARN, "typtoout: Cache lookup of type %d failed", type);
+	elog(ERROR, "typtoout: Cache lookup of type %d failed", type);
 	return (InvalidOid);
 }
 
@@ -97,7 +97,7 @@ printtup(HeapTuple tuple, TupleDesc typeinfo)
 	{
 		i++;					/* heap_getattr is a macro, so no
 								 * increment */
-		attr = heap_getattr(tuple, InvalidBuffer, i, typeinfo, &isnull);
+		attr = heap_getattr(tuple, i, typeinfo, &isnull);
 		if (!isnull)
 			j |= k;
 		k >>= 1;
@@ -117,14 +117,15 @@ printtup(HeapTuple tuple, TupleDesc typeinfo)
 	 */
 	for (i = 0; i < tuple->t_natts; ++i)
 	{
-		attr = heap_getattr(tuple, InvalidBuffer, i + 1, typeinfo, &isnull);
+		attr = heap_getattr(tuple, i + 1, typeinfo, &isnull);
 		typoutput = typtoout((Oid) typeinfo->attrs[i]->atttypid);
 
 		if (!isnull && OidIsValid(typoutput))
 		{
 			outputstr = fmgr(typoutput, attr,
-							 gettypelem(typeinfo->attrs[i]->atttypid));
-			pq_putint(strlen(outputstr) + 4, 4);
+							 gettypelem(typeinfo->attrs[i]->atttypid),
+							 typeinfo->attrs[i]->atttypmod);
+			pq_putint(strlen(outputstr) + VARHDRSZ, VARHDRSZ);
 			pq_putnchar(outputstr, strlen(outputstr));
 			pfree(outputstr);
 		}
@@ -175,21 +176,22 @@ showatts(char *name, TupleDesc tupleDesc)
 void
 debugtup(HeapTuple tuple, TupleDesc typeinfo)
 {
-	register int i;
-	Datum	    attr;
+	int			i;
+	Datum		attr;
 	char	   *value;
 	bool		isnull;
 	Oid			typoutput;
 
 	for (i = 0; i < tuple->t_natts; ++i)
 	{
-		attr = heap_getattr(tuple, InvalidBuffer, i + 1, typeinfo, &isnull);
+		attr = heap_getattr(tuple, i + 1, typeinfo, &isnull);
 		typoutput = typtoout((Oid) typeinfo->attrs[i]->atttypid);
 
 		if (!isnull && OidIsValid(typoutput))
 		{
 			value = fmgr(typoutput, attr,
-						 gettypelem(typeinfo->attrs[i]->atttypid));
+						 gettypelem(typeinfo->attrs[i]->atttypid),
+						 typeinfo->attrs[i]->atttypmod);
 			printatt((unsigned) i + 1, typeinfo->attrs[i], value);
 			pfree(value);
 		}
@@ -231,7 +233,7 @@ printtup_internal(HeapTuple tuple, TupleDesc typeinfo)
 	{
 		i++;					/* heap_getattr is a macro, so no
 								 * increment */
-		attr = heap_getattr(tuple, InvalidBuffer, i, typeinfo, &isnull);
+		attr = heap_getattr(tuple, i, typeinfo, &isnull);
 		if (!isnull)
 			j |= k;
 		k >>= 1;
@@ -256,7 +258,7 @@ printtup_internal(HeapTuple tuple, TupleDesc typeinfo)
 	{
 		int32		len = typeinfo->attrs[i]->attlen;
 
-		attr = heap_getattr(tuple, InvalidBuffer, i + 1, typeinfo, &isnull);
+		attr = heap_getattr(tuple, i + 1, typeinfo, &isnull);
 		if (!isnull)
 		{
 			/* # of bytes, and opaque data */
@@ -265,7 +267,7 @@ printtup_internal(HeapTuple tuple, TupleDesc typeinfo)
 				/* variable length, assume a varlena structure */
 				len = VARSIZE(attr) - VARHDRSZ;
 
-				pq_putint(len, sizeof(int32));
+				pq_putint(len, VARHDRSZ);
 				pq_putnchar(VARDATA(attr), len);
 #ifdef IPORTAL_DEBUG
 				{
@@ -311,7 +313,7 @@ printtup_internal(HeapTuple tuple, TupleDesc typeinfo)
 					pq_putnchar(DatumGetPointer(attr), len);
 #ifdef IPORTAL_DEBUG
 					fprintf(stderr, "byref length %d data %x\n", len,
-												DatumGetPointer(attr));
+							DatumGetPointer(attr));
 #endif
 				}
 			}
