@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtscan.c,v 1.10 1997/09/08 20:54:24 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtscan.c,v 1.13 1998/02/28 13:53:18 vadim Exp $
  *
  *
  * NOTES
@@ -77,7 +77,7 @@ _bt_dropscan(IndexScanDesc scan)
 	}
 
 	if (chk == (BTScanList) NULL)
-		elog(WARN, "btree scan list trashed; can't find 0x%lx", scan);
+		elog(ERROR, "btree scan list trashed; can't find 0x%lx", scan);
 
 	if (last == (BTScanList) NULL)
 		BTScans = chk->btsl_next;
@@ -154,7 +154,7 @@ _bt_scandel(IndexScanDesc scan, int op, BlockNumber blkno, OffsetNumber offno)
 				_bt_step(scan, &buf, BackwardScanDirection);
 				break;
 			default:
-				elog(WARN, "_bt_scandel: bad operation '%d'", op);
+				elog(ERROR, "_bt_scandel: bad operation '%d'", op);
 				/* NOTREACHED */
 		}
 		so->btso_curbuf = buf;
@@ -170,6 +170,9 @@ _bt_scandel(IndexScanDesc scan, int op, BlockNumber blkno, OffsetNumber offno)
 		tmp = *current;
 		*current = scan->currentItemData;
 		scan->currentItemData = tmp;
+		so->btso_curbuf = so->btso_mrkbuf;
+		so->btso_mrkbuf = buf;
+		buf = so->btso_curbuf;
 		switch (op)
 		{
 			case BT_INSERT:
@@ -179,9 +182,10 @@ _bt_scandel(IndexScanDesc scan, int op, BlockNumber blkno, OffsetNumber offno)
 				_bt_step(scan, &buf, BackwardScanDirection);
 				break;
 			default:
-				elog(WARN, "_bt_scandel: bad operation '%d'", op);
+				elog(ERROR, "_bt_scandel: bad operation '%d'", op);
 				/* NOTREACHED */
 		}
+		so->btso_curbuf = so->btso_mrkbuf;
 		so->btso_mrkbuf = buf;
 		tmp = *current;
 		*current = scan->currentItemData;
