@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtsearch.c,v 1.26 1997/09/08 20:54:27 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtsearch.c,v 1.34 1998/04/10 22:07:41 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -199,6 +199,9 @@ _bt_moveright(Relation rel,
 			 * scankey is x = 2. Scankey == (2,1) because of we compare
 			 * first attrs only, but we shouldn't to move right of here. -
 			 * vadim 04/15/97
+			 *
+			 * Also, if this page is not LEAF one (and # of attrs > keysize)
+			 * then we can't move too. - vadim 10/22/97
 			 */
 
 			if (_bt_skeycmp(rel, keysz, scankey, page, hikey,
@@ -216,7 +219,8 @@ _bt_moveright(Relation rel,
 						if (_bt_skeycmp(rel, keysz, scankey, page,
 										PageGetItemId(page, P_FIRSTKEY),
 										BTEqualStrategyNumber))
-							elog(FATAL, "btree: BTP_CHAIN flag was expected");
+							elog(FATAL, "btree: BTP_CHAIN flag was expected in %s (access = %s)", 
+										rel->rd_rel->relname, access ? "bt_write" : "bt_read");
 						if (_bt_skeycmp(rel, keysz, scankey, page,
 										PageGetItemId(page, offmax),
 										BTEqualStrategyNumber))
@@ -227,6 +231,8 @@ _bt_moveright(Relation rel,
 							elog(FATAL, "btree: unexpected greater last item");
 						/* move right */
 					}
+					else if (!(opaque->btpo_flags & BTP_LEAF))
+						break;
 					else if (_bt_skeycmp(rel, keysz, scankey, page,
 										 PageGetItemId(page, offmax),
 										 BTLessEqualStrategyNumber))
@@ -611,7 +617,7 @@ _bt_compare(Relation rel,
 		 */
 		if (!P_RIGHTMOST(opaque))
 		{
-			elog(WARN, "_bt_compare: invalid comparison to high key");
+			elog(ERROR, "_bt_compare: invalid comparison to high key");
 		}
 
 #if 0
@@ -671,8 +677,7 @@ _bt_compare(Relation rel,
 		}
 		else
 		{
-			tmpres = (long) FMGR_PTR2(entry->sk_func, entry->sk_procedure,
-									  entry->sk_argument, datum);
+			tmpres = (long) FMGR_PTR2(&entry->sk_func, entry->sk_argument, datum);
 		}
 		result = tmpres;
 
@@ -833,7 +838,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	/* _bt_orderkeys disallows it, but it's place to add some code latter */
 	if (so->keyData[0].sk_flags & SK_ISNULL)
 	{
-		elog(WARN, "_bt_first: btree doesn't support is(not)null, yet");
+		elog(ERROR, "_bt_first: btree doesn't support is(not)null, yet");
 		return ((RetrieveIndexResult) NULL);
 	}
 	proc = index_getprocid(rel, 1, BTORDER_PROC);
@@ -1325,7 +1330,7 @@ _bt_twostep(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 	 * us up less often since they're only done by the vacuum daemon.
 	 */
 
-	elog(WARN, "btree synchronization error:  concurrent update botched scan");
+	elog(ERROR, "btree synchronization error:  concurrent update botched scan");
 
 	return (false);
 }
@@ -1410,7 +1415,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 	if (ScanDirectionIsForward(dir))
 	{
 		if (!P_LEFTMOST(opaque))/* non-leftmost page ? */
-			elog(WARN, "_bt_endpoint: leftmost page (%u) has not leftmost flag", blkno);
+			elog(ERROR, "_bt_endpoint: leftmost page (%u) has not leftmost flag", blkno);
 		start = P_RIGHTMOST(opaque) ? P_HIKEY : P_FIRSTKEY;
 
 		/*
@@ -1434,7 +1439,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		if (PageIsEmpty(page))
 		{
 			if (start != P_HIKEY)		/* non-rightmost page */
-				elog(WARN, "_bt_endpoint: non-rightmost page (%u) is empty", blkno);
+				elog(ERROR, "_bt_endpoint: non-rightmost page (%u) is empty", blkno);
 
 			/*
 			 * It's left- & right- most page - root page, - and it's
@@ -1506,7 +1511,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 	}
 	else
 	{
-		elog(WARN, "Illegal scan direction %d", dir);
+		elog(ERROR, "Illegal scan direction %d", dir);
 	}
 
 	btitem = (BTItem) PageGetItem(page, PageGetItemId(page, start));
