@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtutils.c,v 1.15 1997/09/18 20:19:55 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtutils.c,v 1.18 1998/01/15 19:42:15 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -119,7 +119,7 @@ _bt_orderkeys(Relation relation, BTScanOpaque so)
 
 	cur = &key[0];
 	if (cur->sk_attno != 1)
-		elog(WARN, "_bt_orderkeys: key(s) for attribute 1 missed");
+		elog(ERROR, "_bt_orderkeys: key(s) for attribute 1 missed");
 
 	if (numberOfKeys == 1)
 	{
@@ -159,7 +159,7 @@ _bt_orderkeys(Relation relation, BTScanOpaque so)
 		{
 			if (cur->sk_attno != attno + 1 && i < numberOfKeys)
 			{
-				elog(WARN, "_bt_orderkeys: key(s) for attribute %d missed", attno + 1);
+				elog(ERROR, "_bt_orderkeys: key(s) for attribute %d missed", attno + 1);
 			}
 
 			/*
@@ -263,8 +263,7 @@ _bt_orderkeys(Relation relation, BTScanOpaque so)
 		{
 			/* yup, use the appropriate value */
 			test =
-				(long) FMGR_PTR2(cur->sk_func, cur->sk_procedure,
-								 cur->sk_argument, xform[j].sk_argument);
+				(long) FMGR_PTR2(&cur->sk_func, cur->sk_argument, xform[j].sk_argument);
 			if (test)
 				xform[j].sk_argument = cur->sk_argument;
 			else if (j == (BTEqualStrategyNumber - 1))
@@ -296,7 +295,7 @@ _bt_formitem(IndexTuple itup)
 	/*
 	 * see comments in btbuild
 	 *
-	 * if (itup->t_info & INDEX_NULL_MASK) elog(WARN, "btree indices cannot
+	 * if (itup->t_info & INDEX_NULL_MASK) elog(ERROR, "btree indices cannot
 	 * include null keys");
 	 */
 
@@ -381,13 +380,13 @@ _bt_checkkeys(IndexScanDesc scan, IndexTuple tuple, Size *keysok)
 
 		if (key[0].sk_flags & SK_COMMUTE)
 		{
-			test = (int) (*(key[0].sk_func))
+			test = (int) (*fmgr_faddr(&key[0].sk_func))
 				(DatumGetPointer(key[0].sk_argument),
 				 datum);
 		}
 		else
 		{
-			test = (int) (*(key[0].sk_func))
+			test = (int) (*fmgr_faddr(&key[0].sk_func))
 				(datum,
 				 DatumGetPointer(key[0].sk_argument));
 		}
