@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtproc.c,v 1.12 1997/09/18 20:19:56 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtproc.c,v 1.16 1998/02/26 04:30:06 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -29,7 +29,7 @@ rt_box_union(BOX *a, BOX *b)
 	BOX		   *n;
 
 	if ((n = (BOX *) palloc(sizeof(*n))) == (BOX *) NULL)
-		elog(WARN, "Cannot allocate box for union");
+		elog(ERROR, "Cannot allocate box for union");
 
 	n->high.x = Max(a->high.x, b->high.x);
 	n->high.y = Max(a->high.y, b->high.y);
@@ -39,13 +39,13 @@ rt_box_union(BOX *a, BOX *b)
 	return (n);
 }
 
-BOX		   *
+BOX *
 rt_box_inter(BOX *a, BOX *b)
 {
 	BOX		   *n;
 
 	if ((n = (BOX *) palloc(sizeof(*n))) == (BOX *) NULL)
-		elog(WARN, "Cannot allocate box for union");
+		elog(ERROR, "Cannot allocate box for union");
 
 	n->high.x = Min(a->high.x, b->high.x);
 	n->high.y = Min(a->high.y, b->high.y);
@@ -91,10 +91,10 @@ rt_poly_union(POLYGON *a, POLYGON *b)
 {
 	POLYGON    *p;
 
-	p = (POLYGON *) PALLOCTYPE(POLYGON);
+	p = (POLYGON *) palloc(sizeof(POLYGON));
 
 	if (!PointerIsValid(p))
-		elog(WARN, "Cannot allocate polygon for union");
+		elog(ERROR, "Cannot allocate polygon for union");
 
 	MemSet((char *) p, 0, sizeof(POLYGON));		/* zero any holes */
 	p->size = sizeof(POLYGON);
@@ -133,10 +133,10 @@ rt_poly_inter(POLYGON *a, POLYGON *b)
 {
 	POLYGON    *p;
 
-	p = (POLYGON *) PALLOCTYPE(POLYGON);
+	p = (POLYGON *) palloc(sizeof(POLYGON));
 
 	if (!PointerIsValid(p))
-		elog(WARN, "Cannot allocate polygon for intersection");
+		elog(ERROR, "Cannot allocate polygon for intersection");
 
 	MemSet((char *) p, 0, sizeof(POLYGON));		/* zero any holes */
 	p->size = sizeof(POLYGON);
