@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/varsup.c,v 1.12 1997/09/08 21:41:49 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/varsup.c,v 1.15 1998/01/07 21:02:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -23,9 +23,7 @@
 static void GetNewObjectIdBlock(Oid *oid_return, int oid_block_size);
 static void VariableRelationGetNextOid(Oid *oid_return);
 static void VariableRelationGetNextXid(TransactionId *xidP);
-static void VariableRelationPutLastXid(TransactionId xid);
 static void VariableRelationPutNextOid(Oid *oidP);
-static void VariableRelationGetLastXid(TransactionId *xidP);
 
 /* ---------------------
  *		spin lock for oid generation
@@ -71,7 +69,7 @@ VariableRelationGetNextXid(TransactionId *xidP)
 	if (!BufferIsValid(buf))
 	{
 		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationGetNextXid: ReadBuffer failed");
+		elog(ERROR, "VariableRelationGetNextXid: ReadBuffer failed");
 	}
 
 	var = (VariableRelationContents) BufferGetBlock(buf);
@@ -80,49 +78,6 @@ VariableRelationGetNextXid(TransactionId *xidP)
 	ReleaseBuffer(buf);
 }
 
-/* --------------------------------
- *		VariableRelationGetLastXid
- * --------------------------------
- */
-static void
-VariableRelationGetLastXid(TransactionId *xidP)
-{
-	Buffer		buf;
-	VariableRelationContents var;
-
-	/* ----------------
-	 * We assume that a spinlock has been acquire to guarantee
-	 * exclusive access to the variable relation.
-	 * ----------------
-	 */
-
-	/* ----------------
-	 *	do nothing before things are initialized
-	 * ----------------
-	 */
-	if (!RelationIsValid(VariableRelation))
-		return;
-
-	/* ----------------
-	 *	read the variable page, get the the lastXid field and
-	 *	release the buffer
-	 * ----------------
-	 */
-	buf = ReadBuffer(VariableRelation, 0);
-
-	if (!BufferIsValid(buf))
-	{
-		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationGetNextXid: ReadBuffer failed");
-	}
-
-	var = (VariableRelationContents) BufferGetBlock(buf);
-
-	TransactionIdStore(var->lastXidData, xidP);
-
-	ReleaseBuffer(buf);
-}
-
 /* --------------------------------
  *		VariableRelationPutNextXid
  * --------------------------------
@@ -157,7 +112,7 @@ VariableRelationPutNextXid(TransactionId xid)
 	if (!BufferIsValid(buf))
 	{
 		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationPutNextXid: ReadBuffer failed");
+		elog(ERROR, "VariableRelationPutNextXid: ReadBuffer failed");
 	}
 
 	var = (VariableRelationContents) BufferGetBlock(buf);
@@ -169,49 +124,6 @@ VariableRelationPutNextXid(TransactionId xid)
 	SetBufferWriteMode(flushmode);
 }
 
-/* --------------------------------
- *		VariableRelationPutLastXid
- * --------------------------------
- */
-static void
-VariableRelationPutLastXid(TransactionId xid)
-{
-	Buffer		buf;
-	VariableRelationContents var;
-
-	/* ----------------
-	 * We assume that a spinlock has been acquire to guarantee
-	 * exclusive access to the variable relation.
-	 * ----------------
-	 */
-
-	/* ----------------
-	 *	do nothing before things are initialized
-	 * ----------------
-	 */
-	if (!RelationIsValid(VariableRelation))
-		return;
-
-	/* ----------------
-	 *	read the variable page, update the lastXid field and
-	 *	force the page back out to disk.
-	 * ----------------
-	 */
-	buf = ReadBuffer(VariableRelation, 0);
-
-	if (!BufferIsValid(buf))
-	{
-		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationPutLastXid: ReadBuffer failed");
-	}
-
-	var = (VariableRelationContents) BufferGetBlock(buf);
-
-	TransactionIdStore(xid, &(var->lastXidData));
-
-	WriteBuffer(buf);
-}
-
 /* --------------------------------
  *		VariableRelationGetNextOid
  * --------------------------------
@@ -252,7 +164,7 @@ VariableRelationGetNextOid(Oid *oid_return)
 	if (!BufferIsValid(buf))
 	{
 		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationGetNextXid: ReadBuffer failed");
+		elog(ERROR, "VariableRelationGetNextXid: ReadBuffer failed");
 	}
 
 	var = (VariableRelationContents) BufferGetBlock(buf);
@@ -312,7 +224,7 @@ VariableRelationPutNextOid(Oid *oidP)
 	if (!PointerIsValid(oidP))
 	{
 		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationPutNextOid: invalid oid pointer");
+		elog(ERROR, "VariableRelationPutNextOid: invalid oid pointer");
 	}
 
 	/* ----------------
@@ -325,7 +237,7 @@ VariableRelationPutNextOid(Oid *oidP)
 	if (!BufferIsValid(buf))
 	{
 		SpinRelease(OidGenLockId);
-		elog(WARN, "VariableRelationPutNextOid: ReadBuffer failed");
+		elog(ERROR, "VariableRelationPutNextOid: ReadBuffer failed");
 	}
 
 	var = (VariableRelationContents) BufferGetBlock(buf);
@@ -449,40 +361,6 @@ GetNewTransactionId(TransactionId *xid)
 	prefetched_xid_count--;
 }
 
-/* ----------------
- *		UpdateLastCommittedXid
- * ----------------
- */
-
-void
-UpdateLastCommittedXid(TransactionId xid)
-{
-	TransactionId lastid;
-
-
-	/*
-	 * we assume that spinlock OidGenLockId has been acquired prior to
-	 * entering this function
-	 */
-
-	/* ----------------
-	 *	get the "last committed" transaction id from
-	 *	the variable relation page.
-	 * ----------------
-	 */
-	VariableRelationGetLastXid(&lastid);
-
-	/* ----------------
-	 *	if the transaction id is greater than the last committed
-	 *	transaction then we update the last committed transaction
-	 *	in the variable relation.
-	 * ----------------
-	 */
-	if (TransactionIdIsLessThan(lastid, xid))
-		VariableRelationPutLastXid(xid);
-
-}
-
 /* ----------------------------------------------------------------
  *					object id generation support
  * ----------------------------------------------------------------
