@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/xid.c,v 1.10 1997/09/08 21:41:56 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/xid.c,v 1.14 1998/04/07 18:10:01 momjian Exp $
  *
  * OLD COMMENTS
  * XXX WARNING
@@ -30,18 +30,6 @@ extern TransactionId DisabledTransactionId;
 extern TransactionId AmiTransactionId;
 extern TransactionId FirstTransactionId;
 
-/* ----------------------------------------------------------------
- *		TransactionIdIsValid
- *
- *		Macro-ize me.
- * ----------------------------------------------------------------
- */
-bool
-TransactionIdIsValid(TransactionId transactionId)
-{
-	return ((bool) (transactionId != NullTransactionId));
-}
-
 /* XXX char16 name for catalogs */
 TransactionId
 xidin(char *representation)
@@ -50,7 +38,7 @@ xidin(char *representation)
 }
 
 /* XXX char16 name for catalogs */
-char	   *
+char *
 xidout(TransactionId transactionId)
 {
 /*	  return(TransactionIdFormString(transactionId)); */
@@ -65,32 +53,6 @@ xidout(TransactionId transactionId)
 
 }
 
-/* ----------------------------------------------------------------
- *		StoreInvalidTransactionId
- *
- *		Maybe do away with Pointer types in these routines.
- *		Macro-ize this one.
- * ----------------------------------------------------------------
- */
-void
-StoreInvalidTransactionId(TransactionId *destination)
-{
-	*destination = NullTransactionId;
-}
-
-/* ----------------------------------------------------------------
- *		TransactionIdStore
- *
- *		Macro-ize this one.
- * ----------------------------------------------------------------
- */
-void
-TransactionIdStore(TransactionId transactionId,
-				   TransactionId *destination)
-{
-	*destination = transactionId;
-}
-
 /* ----------------------------------------------------------------
  *		TransactionIdEquals
  * ----------------------------------------------------------------
