@@ -7,7 +7,7 @@
  * Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/bootstrap/bootstrap.c,v 1.26 1997/09/18 20:20:04 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/bootstrap/bootstrap.c,v 1.39 1998/04/07 18:10:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -40,7 +40,6 @@
 #include "storage/itemptr.h"
 #include "utils/nabstime.h"
 #include "access/htup.h"
-#include "utils/tqual.h"
 #include "storage/buf.h"
 #include "access/relscan.h"
 #include "access/heapam.h"
@@ -200,11 +199,19 @@ static char *values[MAXATTR];	/* cooresponding attribute values */
 int			numattr;			/* number of attributes for cur. rel */
 extern int	fsyncOff;			/* do not fsync the database */
 
-#ifndef HAVE_SIGSETJMP
+/* The test for HAVE_SIGSETJMP fails on Linux 2.0.x because the test
+ *	explicitly disallows sigsetjmp being a #define, which is how it
+ *	is declared in Linux. So, to avoid compiler warnings about
+ *	sigsetjmp() being redefined, let's not redefine unless necessary.
+ * - thomas 1997-12-27
+ */
+
+#if !defined(HAVE_SIGSETJMP) && !defined(sigsetjmp)
 static jmp_buf Warn_restart;
 
 #define sigsetjmp(x,y)	setjmp(x)
 #define siglongjmp longjmp
+
 #else
 static sigjmp_buf Warn_restart;
 
@@ -307,17 +314,15 @@ BootstrapMain(int argc, char *argv[])
 	 * ----------------
 	 */
 	pqsignal(SIGINT, (sig_func) die);
-#ifndef win32
 	pqsignal(SIGHUP, (sig_func) die);
 	pqsignal(SIGTERM, (sig_func) die);
-#endif							/* win32 */
 
 	/* --------------------
 	 *	initialize globals
 	 * -------------------
 	 */
 
-	MasterPid = getpid();
+	MyProcPid = getpid();
 
 	/* ----------------
 	 *	process command arguments
@@ -401,12 +406,6 @@ BootstrapMain(int argc, char *argv[])
 		exitpg(1);
 	}
 
-#ifdef win32
-	_nt_init();
-	_nt_attach();
-#endif							/* win32 */
-
-
 	/* ----------------
 	 *	backend initialization
 	 * ----------------
@@ -426,18 +425,13 @@ BootstrapMain(int argc, char *argv[])
 		hashtable[i] = NULL;
 
 	/* ----------------
-	 *	abort processing resumes here  - What to do in WIN32?
+	 *	abort processing resumes here
 	 * ----------------
 	 */
-#ifndef win32
 	pqsignal(SIGHUP, handle_warn);
 
 	if (sigsetjmp(Warn_restart, 1) != 0)
 	{
-#else
-	if (setjmp(Warn_restart) != 0)
-	{
-#endif							/* win32 */
 		Warnings++;
 		AbortCurrentTransaction();
 	}
@@ -480,21 +474,21 @@ boot_openrel(char *relname)
 	HeapScanDesc sdesc;
 	HeapTuple	tup;
 
-	if (strlen(relname) > 15)
-		relname[15] = '\000';
+	if (strlen(relname) >= NAMEDATALEN - 1)
+		relname[NAMEDATALEN - 1] = '\0';
 
 	if (Typ == (struct typmap **) NULL)
 	{
 		StartPortalAllocMode(DefaultAllocMode, 0);
 		rdesc = heap_openr(TypeRelationName);
-		sdesc = heap_beginscan(rdesc, 0, NowTimeQual, 0, (ScanKey) NULL);
+		sdesc = heap_beginscan(rdesc, 0, false, 0, (ScanKey) NULL);
 		for (i = 0; PointerIsValid(tup = heap_getnext(sdesc, 0, (Buffer *) NULL)); ++i);
 		heap_endscan(sdesc);
 		app = Typ = ALLOC(struct typmap *, i + 1);
 		while (i-- > 0)
 			*app++ = ALLOC(struct typmap, 1);
 		*app = (struct typmap *) NULL;
-		sdesc = heap_beginscan(rdesc, 0, NowTimeQual, 0, (ScanKey) NULL);
+		sdesc = heap_beginscan(rdesc, 0, false, 0, (ScanKey) NULL);
 		app = Typ;
 		while (PointerIsValid(tup = heap_getnext(sdesc, 0, (Buffer *) NULL)))
 		{
@@ -567,18 +561,18 @@ closerel(char *name)
 		if (reldesc)
 		{
 			if (namestrcmp(RelationGetRelationName(reldesc), name) != 0)
-				elog(WARN, "closerel: close of '%s' when '%s' was expected",
+				elog(ERROR, "closerel: close of '%s' when '%s' was expected",
 					 name, relname ? relname : "(null)");
 		}
 		else
-			elog(WARN, "closerel: close of '%s' before any relation was opened",
+			elog(ERROR, "closerel: close of '%s' before any relation was opened",
 				 name);
 
 	}
 
 	if (reldesc == NULL)
 	{
-		elog(WARN, "Warning: no opened relation to close.\n");
+		elog(ERROR, "Warning: no opened relation to close.\n");
 	}
 	else
 	{
@@ -634,6 +628,8 @@ DefineAttr(char *name, char *type, int attnum)
 		attlen = attrtypes[attnum]->attlen = Procid[t].len;
 		attrtypes[attnum]->attbyval = (attlen == 1) || (attlen == 2) || (attlen == 4);
 	}
+	attrtypes[attnum]->attcacheoff = -1;
+	attrtypes[attnum]->atttypmod = -1;
 }
 
 
@@ -852,7 +848,7 @@ gettype(char *type)
 		if (DebugMode)
 			printf("bootstrap.c: External Type: %s\n", type);
 		rdesc = heap_openr(TypeRelationName);
-		sdesc = heap_beginscan(rdesc, 0, NowTimeQual, 0, (ScanKey) NULL);
+		sdesc = heap_beginscan(rdesc, 0, false, 0, (ScanKey) NULL);
 		i = 0;
 		while (PointerIsValid(tup = heap_getnext(sdesc, 0, (Buffer *) NULL)))
 			++i;
@@ -861,7 +857,7 @@ gettype(char *type)
 		while (i-- > 0)
 			*app++ = ALLOC(struct typmap, 1);
 		*app = (struct typmap *) NULL;
-		sdesc = heap_beginscan(rdesc, 0, NowTimeQual, 0, (ScanKey) NULL);
+		sdesc = heap_beginscan(rdesc, 0, false, 0, (ScanKey) NULL);
 		app = Typ;
 		while (PointerIsValid(tup = heap_getnext(sdesc, 0, (Buffer *) NULL)))
 		{
@@ -874,7 +870,7 @@ gettype(char *type)
 		heap_close(rdesc);
 		return (gettype(type));
 	}
-	elog(WARN, "Error: unknown type '%s'.\n", type);
+	elog(ERROR, "Error: unknown type '%s'.\n", type);
 	err_out();
 	/* not reached, here to make compiler happy */
 	return 0;
@@ -913,7 +909,7 @@ AllocateAttribute()
  * be freed by the CALLER.
  * ----------------
  */
-char	   *
+char *
 MapArrayTypeName(char *s)
 {
 	int			i,
@@ -966,7 +962,7 @@ EnterString(char *str)
  *		associated with the idnum
  * ----------------
  */
-char	   *
+char *
 LexIDStr(int ident_num)
 {
 	return (strtable[ident_num]);
@@ -985,7 +981,7 @@ LexIDStr(int ident_num)
 static int
 CompHash(char *str, int len)
 {
-	register int result;
+	int			result;
 
 	result = (NUM * str[0] + NUMSQR * str[len - 1] + NUMCUBE * str[(len - 1) / 2]);
 
