@@ -10,7 +10,7 @@
 #
 #
 # IDENTIFICATION
-#    $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/genbki.sh,v 1.8 1997/09/06 18:27:11 momjian Exp $
+#    $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/genbki.sh,v 1.10 1998/04/06 00:22:16 momjian Exp $
 #
 # NOTES
 #    non-essential whitespace is removed from the generated file.
@@ -18,6 +18,10 @@
 #    end can be changed into another awk script or something smarter..
 #
 #-------------------------------------------------------------------------
+trap "rm -f /tmp/genbki.tmp" 0 1 2 3 15
+
+# make sure it is empty
+>/tmp/genbki.tmp
 
 PATH=$PATH:/lib:/usr/ccs/lib		# to find cpp
 BKIOPTS=''
@@ -48,6 +52,10 @@ while test $x -le $numargs ; do
     shift
 done
 
+# Get NAMEDATALEN and OIDNAMELEN from postgres_ext.h
+NAMEDATALEN=`grep '#define.*NAMEDATALEN' ../../include/postgres_ext.h | awk '{ print $3 }'`
+OIDNAMELEN=`grep '#define.*OIDNAMELEN' ../../include/postgres_ext.h | awk '{ print $3 }'`
+
 # ----------------
 # 	strip comments and trash from .h before we generate
 #	the .bki file...
@@ -64,14 +72,16 @@ sed -e 's;/\*.*\*/;;g' \
     -e 's;\*/;\
 */\
 ;g' | # we must run a new sed here to see the newlines we added
-sed -e 's/;[ 	]*$//g' \
-    -e 's/^[ 	]*//' \
-    -e 's/[ 	]Oid/\ oid/g' \
-    -e 's/[ 	]NameData/\ name/g' \
-    -e 's/^Oid/oid/g' \
-    -e 's/^NameData/\name/g' \
-    -e 's/(NameData/(name/g' \
-    -e 's/(Oid/(oid/g' | \
+sed -e "s/;[ 	]*$//g" \
+    -e "s/^[ 	]*//" \
+    -e "s/[ 	]Oid/\ oid/g" \
+    -e "s/[ 	]NameData/\ name/g" \
+    -e "s/^Oid/oid/g" \
+    -e "s/^NameData/\name/g" \
+    -e "s/(NameData/(name/g" \
+    -e "s/(Oid/(oid/g" \
+	-e "s/NAMEDATALEN/$NAMEDATALEN/g" \
+	-e "s/OIDNAMELEN/$OIDNAMELEN/g" | \
 awk '
 # ----------------
 #	now use awk to process remaining .h file..
@@ -118,6 +128,27 @@ raw == 1 	{ print; next; }
 /^DATA\(/ {
 	data = substr($0, 6, length($0) - 6);
 	print data;
+	nf = 1;
+	oid = 0;
+	while (nf <= NF-3)
+	{
+		if ($nf == "OID" && $(nf+1) == "=")
+		{
+			oid = $(nf+2);
+			break;
+		}
+		nf++;
+	}
+	next;
+}
+
+/^DESCR\(/ {
+	if (oid != 0)
+	{
+		data = substr($0, 8, length($0) - 9);
+		if (data != "")
+			printf "%d	%s\n", oid, data >> "/tmp/genbki.tmp";
+	}
 	next;
 }
 
@@ -235,6 +266,9 @@ cpp $BKIOPTS | \
 sed -e '/^[ 	]*$/d' \
     -e 's/[ 	][ 	]*/ /g'
 
+# send pg_description file contents to standard error
+cat /tmp/genbki.tmp 1>&2
+
 # ----------------
 #	all done
 # ----------------
