@@ -7,17 +7,18 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/heap.c,v 1.29 1997/09/18 20:20:11 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/heap.c,v 1.47 1998/02/26 04:30:35 momjian Exp $
  *
  * INTERFACE ROUTINES
- *		heap_creatr()			- Create an uncataloged heap relation
- *		heap_create()			- Create a cataloged relation
- *		heap_destroy()			- Removes named relation from catalogs
+ *		heap_create()			- Create an uncataloged heap relation
+ *		heap_create_with_catalog() - Create a cataloged relation
+ *		heap_destroy_with_catalog()			- Removes named relation from catalogs
  *
  * NOTES
  *	  this code taken from access/heap/create.c, which contains
- *	  the old heap_creater, amcreate, and amdestroy.  those routines
- *	  will soon call these routines using the function manager,
+ *	  the old heap_create_with_catalogr, amcreate, and amdestroy.
+ *	  those routines will soon call these routines using the function
+ *	  manager,
  *	  just like the poorly named "NewXXX" routines do.	The
  *	  "New" routines are all going to die soon, once and for all!
  *		-cim 1/13/91
@@ -34,24 +35,28 @@
 #include <catalog/heap.h>
 #include <catalog/index.h>
 #include <catalog/indexing.h>
-#include <catalog/pg_ipl.h>
-#include <catalog/pg_inherits.h>
-#include <catalog/pg_proc.h>
-#include <catalog/pg_index.h>
-#include <catalog/pg_type.h>
 #include <catalog/pg_attrdef.h>
+#include <catalog/pg_index.h>
+#include <catalog/pg_inherits.h>
+#include <catalog/pg_ipl.h>
+#include <catalog/pg_proc.h>
 #include <catalog/pg_relcheck.h>
+#include <catalog/pg_type.h>
 #include <commands/trigger.h>
+#include <nodes/plannodes.h>
+#include <optimizer/tlist.h>
+#include <parser/parse_expr.h>
+#include <parser/parse_node.h>
+#include <parser/parse_type.h>
+#include <rewrite/rewriteRemove.h>
 #include <storage/bufmgr.h>
 #include <storage/lmgr.h>
 #include <storage/smgr.h>
-#include <parser/catalog_utils.h>
-#include <parser/parse_query.h>
-#include <rewrite/rewriteRemove.h>
+#include <tcop/tcopprot.h>
 #include <utils/builtins.h>
 #include <utils/mcxt.h>
 #include <utils/relcache.h>
-#include <nodes/plannodes.h>
+#include <utils/tqual.h>
 #ifndef HAVE_MEMMOVE
 #include <regex/utils.h>
 #else
@@ -60,7 +65,7 @@
 
 static void
 AddPgRelationTuple(Relation pg_class_desc,
-	   Relation new_rel_desc, Oid new_rel_oid, int arch, unsigned natts);
+				 Relation new_rel_desc, Oid new_rel_oid, unsigned natts);
 static void AddToTempRelList(Relation r);
 static void DeletePgAttributeTuples(Relation rdesc);
 static void DeletePgRelationTuple(Relation rdesc);
@@ -93,61 +98,36 @@ static void RemoveConstraints(Relation rel);
 
 static FormData_pg_attribute a1 = {
 	0xffffffff, {"ctid"}, 27l, 0l, sizeof(ItemPointerData),
-	SelfItemPointerAttributeNumber, 0, -1, '\0', '\0', 'i', '\0', '\0'
+	SelfItemPointerAttributeNumber, 0, -1, -1, '\0', '\0', 'i', '\0', '\0'
 };
 
 static FormData_pg_attribute a2 = {
 	0xffffffff, {"oid"}, 26l, 0l, sizeof(Oid),
-	ObjectIdAttributeNumber, 0, -1, '\001', '\0', 'i', '\0', '\0'
+	ObjectIdAttributeNumber, 0, -1, -1, '\001', '\0', 'i', '\0', '\0'
 };
 
 static FormData_pg_attribute a3 = {
 	0xffffffff, {"xmin"}, 28l, 0l, sizeof(TransactionId),
-	MinTransactionIdAttributeNumber, 0, -1, '\0', '\0', 'i', '\0', '\0'
+	MinTransactionIdAttributeNumber, 0, -1, -1, '\0', '\0', 'i', '\0', '\0'
 };
 
 static FormData_pg_attribute a4 = {
 	0xffffffff, {"cmin"}, 29l, 0l, sizeof(CommandId),
-	MinCommandIdAttributeNumber, 0, -1, '\001', '\0', 's', '\0', '\0'
+	MinCommandIdAttributeNumber, 0, -1, -1, '\001', '\0', 'i', '\0', '\0'
 };
 
 static FormData_pg_attribute a5 = {
 	0xffffffff, {"xmax"}, 28l, 0l, sizeof(TransactionId),
-	MaxTransactionIdAttributeNumber, 0, -1, '\0', '\0', 'i', '\0', '\0'
+	MaxTransactionIdAttributeNumber, 0, -1, -1, '\0', '\0', 'i', '\0', '\0'
 };
 
 static FormData_pg_attribute a6 = {
 	0xffffffff, {"cmax"}, 29l, 0l, sizeof(CommandId),
-	MaxCommandIdAttributeNumber, 0, -1, '\001', '\0', 's', '\0', '\0'
-};
-
-static FormData_pg_attribute a7 = {
-	0xffffffff, {"chain"}, 27l, 0l, sizeof(ItemPointerData),
-	ChainItemPointerAttributeNumber, 0, -1, '\0', '\0', 'i', '\0', '\0'
-};
-
-static FormData_pg_attribute a8 = {
-	0xffffffff, {"anchor"}, 27l, 0l, sizeof(ItemPointerData),
-	AnchorItemPointerAttributeNumber, 0, -1, '\0', '\0', 'i', '\0', '\0'
-};
-
-static FormData_pg_attribute a9 = {
-	0xffffffff, {"tmin"}, 702l, 0l, sizeof(AbsoluteTime),
-	MinAbsoluteTimeAttributeNumber, 0, -1, '\001', '\0', 'i', '\0', '\0'
-};
-
-static FormData_pg_attribute a10 = {
-	0xffffffff, {"tmax"}, 702l, 0l, sizeof(AbsoluteTime),
-	MaxAbsoluteTimeAttributeNumber, 0, -1, '\001', '\0', 'i', '\0', '\0'
-};
-
-static FormData_pg_attribute a11 = {
-	0xffffffff, {"vtype"}, 18l, 0l, sizeof(char),
-	VersionTypeAttributeNumber, 0, -1, '\001', '\0', 'c', '\0', '\0'
+	MaxCommandIdAttributeNumber, 0, -1, -1, '\001', '\0', 'i', '\0', '\0'
 };
 
 static AttributeTupleForm HeapAtt[] =
-{&a1, &a2, &a3, &a4, &a5, &a6, &a7, &a8, &a9, &a10, &a11};
+{&a1, &a2, &a3, &a4, &a5, &a6};
 
 /* ----------------------------------------------------------------
  *				XXX END OF UGLY HARD CODED BADNESS XXX
@@ -172,7 +152,7 @@ static TempRelList *tempRels = NULL;
 
 
 /* ----------------------------------------------------------------
- *		heap_creatr		- Create an uncataloged heap relation
+ *		heap_create		- Create an uncataloged heap relation
  *
  *		Fields relpages, reltuples, reltuples, relkeys, relhistory,
  *		relisindexed, and relkind of rdesc->rd_rel are initialized
@@ -184,16 +164,15 @@ static TempRelList *tempRels = NULL;
  *		into the transaction context block.
  *
  *
- * if heap_creatr is called with "" as the name, then heap_creatr will create a
- * temporary name	"temp_$RELOID" for the relation
+ * if heap_create is called with "" as the name, then heap_create will create
+ * a temporary name "temp_$RELOID" for the relation
  * ----------------------------------------------------------------
  */
 Relation
-heap_creatr(char *name,
-			unsigned smgr,
+heap_create(char *name,
 			TupleDesc tupDesc)
 {
-	register unsigned i;
+	unsigned	i;
 	Oid			relid;
 	Relation	rdesc;
 	int			len;
@@ -216,7 +195,7 @@ heap_creatr(char *name,
 
 	if (IsSystemRelationName(relname) && IsNormalProcessingMode())
 	{
-		elog(WARN,
+		elog(ERROR,
 		 "Illegal class name: %s -- pg_ is reserved for system catalogs",
 			 relname);
 	}
@@ -308,7 +287,6 @@ heap_creatr(char *name,
 	namestrcpy(&(rdesc->rd_rel->relname), relname);
 	rdesc->rd_rel->relkind = RELKIND_UNCATALOGED;
 	rdesc->rd_rel->relnatts = natts;
-	rdesc->rd_rel->relsmgr = smgr;
 	if (tupDesc->constr)
 		rdesc->rd_rel->relchecks = tupDesc->constr->num_check;
 
@@ -338,7 +316,7 @@ heap_creatr(char *name,
 	 */
 
 	rdesc->rd_tmpunlinked = TRUE;		/* change once table is created */
-	rdesc->rd_fd = (File) smgrcreate(smgr, rdesc);
+	rdesc->rd_fd = (File) smgrcreate(DEFAULT_SMGR, rdesc);
 	rdesc->rd_tmpunlinked = FALSE;
 
 	RelationRegisterRelation(rdesc);
@@ -357,7 +335,7 @@ heap_creatr(char *name,
 
 
 /* ----------------------------------------------------------------
- *		heap_create		- Create a cataloged relation
+ *		heap_create_with_catalog		- Create a cataloged relation
  *
  *		this is done in 6 steps:
  *
@@ -368,8 +346,8 @@ heap_creatr(char *name,
  *		   preforms a scan to ensure that no relation with the
  *		   same name already exists.
  *
- *		3) heap_creater() is called to create the new relation on
- *		   disk.
+ *		3) heap_create_with_catalogr() is called to create the new relation
+ *		   on disk.
  *
  *		4) TypeDefine() is called to define a new type corresponding
  *		   to the new relation.
@@ -401,8 +379,8 @@ heap_creatr(char *name,
  *		create new relation
  *		insert new relation into attribute catalog
  *
- *		Should coordinate with heap_creater().	Either it should
- *		not be called or there should be a way to prevent
+ *		Should coordinate with heap_create_with_catalogr(). Either
+ *		it should not be called or there should be a way to prevent
  *		the relation from being removed at the end of the
  *		transaction if it is successful ('u'/'r' may be enough).
  *		Also, if the transaction does not commit, then the
@@ -419,7 +397,7 @@ heap_creatr(char *name,
  *
  *		this is used to make certain the tuple descriptor contains a
  *		valid set of attribute names.  a problem simply generates
- *		elog(WARN) which aborts the current transaction.
+ *		elog(ERROR) which aborts the current transaction.
  * --------------------------------
  */
 static void
@@ -444,7 +422,7 @@ CheckAttributeNames(TupleDesc tupdesc)
 			if (nameeq(&(HeapAtt[j]->attname),
 					   &(tupdesc->attrs[i]->attname)))
 			{
-				elog(WARN,
+				elog(ERROR,
 					 "create: system attribute named \"%s\"",
 					 HeapAtt[j]->attname.data);
 			}
@@ -468,7 +446,7 @@ CheckAttributeNames(TupleDesc tupdesc)
 			if (nameeq(&(tupdesc->attrs[j]->attname),
 					   &(tupdesc->attrs[i]->attname)))
 			{
-				elog(WARN,
+				elog(ERROR,
 					 "create: repeated attribute \"%s\"",
 					 tupdesc->attrs[j]->attname.data);
 			}
@@ -525,7 +503,7 @@ RelationAlreadyExists(Relation pg_class_desc, char relname[])
 	 */
 	pg_class_scan = heap_beginscan(pg_class_desc,
 								   0,
-								   NowTimeQual,
+								   false,
 								   1,
 								   &key);
 
@@ -656,7 +634,6 @@ static void
 AddPgRelationTuple(Relation pg_class_desc,
 				   Relation new_rel_desc,
 				   Oid new_rel_oid,
-				   int arch,
 				   unsigned natts)
 {
 	Form_pg_class new_rel_reltup;
@@ -681,7 +658,6 @@ AddPgRelationTuple(Relation pg_class_desc,
 		new_rel_reltup->relkind = RELKIND_SEQUENCE;
 	else
 		new_rel_reltup->relkind = RELKIND_RELATION;
-	new_rel_reltup->relarch = arch;
 	new_rel_reltup->relnatts = natts;
 
 	/* ----------------
@@ -751,14 +727,14 @@ addNewRelationType(char *typeName, Oid new_rel_oid)
 	 */
 	new_type_oid = TypeCreate(typeName, /* type name */
 							  new_rel_oid,		/* relation oid */
-							  tlen(type("oid")),		/* internal size */
-							  tlen(type("oid")),		/* external size */
+							  typeLen(typeidType(OIDOID)),		/* internal size */
+							  typeLen(typeidType(OIDOID)),		/* external size */
 							  'c',		/* type-type (catalog) */
 							  ',',		/* default array delimiter */
 							  "int4in", /* input procedure */
 							  "int4out",		/* output procedure */
-							  "int4in", /* send procedure */
-							  "int4out",		/* receive procedure */
+							  "int4in", /* receive procedure */
+							  "int4out",		/* send procedure */
 							  NULL,		/* array element type - irrelevent */
 							  "-",		/* default type value */
 							  (bool) 1, /* passed by value */
@@ -766,23 +742,19 @@ addNewRelationType(char *typeName, Oid new_rel_oid)
 }
 
 /* --------------------------------
- *		heap_create
+ *		heap_create_with_catalog
  *
  *		creates a new cataloged relation.  see comments above.
  * --------------------------------
  */
 Oid
-heap_create(char relname[],
-			char *typename,		/* not used currently */
-			int arch,
-			unsigned smgr,
-			TupleDesc tupdesc)
+heap_create_with_catalog(char relname[],
+						 TupleDesc tupdesc)
 {
 	Relation	pg_class_desc;
 	Relation	new_rel_desc;
 	Oid			new_rel_oid;
 
-/*	  NameData			  typeNameData; */
 	int			natts = tupdesc->natts;
 
 	/* ----------------
@@ -791,7 +763,7 @@ heap_create(char relname[],
 	 */
 	AssertState(IsNormalProcessingMode() || IsBootstrapProcessingMode());
 	if (natts == 0 || natts > MaxHeapAttributeNumber)
-		elog(WARN, "amcreate: from 1 to %d attributes must be specified",
+		elog(ERROR, "amcreate: from 1 to %d attributes must be specified",
 			 MaxHeapAttributeNumber);
 
 	CheckAttributeNames(tupdesc);
@@ -806,7 +778,7 @@ heap_create(char relname[],
 	if (RelationAlreadyExists(pg_class_desc, relname))
 	{
 		heap_close(pg_class_desc);
-		elog(WARN, "amcreate: %s relation already exists", relname);
+		elog(ERROR, "amcreate: %s relation already exists", relname);
 	}
 
 	/* ----------------
@@ -814,11 +786,11 @@ heap_create(char relname[],
 	 *	create an uncataloged relation and pull its relation oid
 	 *	from the newly formed relation descriptor.
 	 *
-	 *	Note: The call to heap_creatr() does all the "real" work
+	 *	Note: The call to heap_create() does all the "real" work
 	 *	of creating the disk file for the relation.
 	 * ----------------
 	 */
-	new_rel_desc = heap_creatr(relname, smgr, tupdesc);
+	new_rel_desc = heap_create(relname, tupdesc);
 	new_rel_oid = new_rel_desc->rd_att->attrs[0]->attrelid;
 
 	/* ----------------
@@ -826,8 +798,6 @@ heap_create(char relname[],
 	 *	we add a new system type corresponding to the new relation.
 	 * ----------------
 	 */
-/*	  namestrcpy(&typeNameData, relname);*/
-/*	  addNewRelationType(&typeNameData, new_rel_oid);*/
 	addNewRelationType(relname, new_rel_oid);
 
 	/* ----------------
@@ -844,7 +814,6 @@ heap_create(char relname[],
 	AddPgRelationTuple(pg_class_desc,
 					   new_rel_desc,
 					   new_rel_oid,
-					   arch,
 					   natts);
 
 	StoreConstraints(new_rel_desc);
@@ -864,7 +833,7 @@ heap_create(char relname[],
 
 
 /* ----------------------------------------------------------------
- *		heap_destroy	- removes all record of named relation from catalogs
+ *		heap_destroy_with_catalog	- removes all record of named relation from catalogs
  *
  *		1)	open relation, check for existence, etc.
  *		2)	remove inheritance information
@@ -928,7 +897,7 @@ RelationRemoveInheritance(Relation relation)
 
 	scan = heap_beginscan(catalogRelation,
 						  false,
-						  NowTimeQual,
+						  false,
 						  1,
 						  &entry);
 
@@ -942,7 +911,7 @@ RelationRemoveInheritance(Relation relation)
 		heap_endscan(scan);
 		heap_close(catalogRelation);
 
-		elog(WARN, "relation <%d> inherits \"%s\"",
+		elog(ERROR, "relation <%d> inherits \"%s\"",
 			 ((InheritsTupleForm) GETSTRUCT(tuple))->inhrel,
 			 RelationGetRelationName(relation));
 	}
@@ -956,7 +925,7 @@ RelationRemoveInheritance(Relation relation)
 
 	scan = heap_beginscan(catalogRelation,
 						  false,
-						  NowTimeQual,
+						  false,
 						  1,
 						  &entry);
 
@@ -984,7 +953,7 @@ RelationRemoveInheritance(Relation relation)
 
 	scan = heap_beginscan(catalogRelation,
 						  false,
-						  NowTimeQual,
+						  false,
 						  1,
 						  &entry);
 
@@ -1023,7 +992,7 @@ RelationRemoveIndexes(Relation relation)
 
 	scan = heap_beginscan(indexRelation,
 						  false,
-						  NowTimeQual,
+						  false,
 						  1,
 						  &entry);
 
@@ -1071,7 +1040,7 @@ DeletePgRelationTuple(Relation rdesc)
 
 	pg_class_scan = heap_beginscan(pg_class_desc,
 								   0,
-								   NowTimeQual,
+								   false,
 								   1,
 								   &key);
 
@@ -1086,7 +1055,7 @@ DeletePgRelationTuple(Relation rdesc)
 	{
 		heap_endscan(pg_class_scan);
 		heap_close(pg_class_desc);
-		elog(WARN, "DeletePgRelationTuple: %s relation nonexistent",
+		elog(ERROR, "DeletePgRelationTuple: %s relation nonexistent",
 			 &rdesc->rd_rel->relname);
 	}
 
@@ -1135,7 +1104,7 @@ DeletePgAttributeTuples(Relation rdesc)
 
 	pg_attribute_scan = heap_beginscan(pg_attribute_desc,
 									   0,
-									   NowTimeQual,
+									   false,
 									   1,
 									   &key);
 
@@ -1204,7 +1173,7 @@ DeletePgTypeTuple(Relation rdesc)
 
 	pg_type_scan = heap_beginscan(pg_type_desc,
 								  0,
-								  NowTimeQual,
+								  false,
 								  1,
 								  &key);
 
@@ -1219,7 +1188,7 @@ DeletePgTypeTuple(Relation rdesc)
 	{
 		heap_endscan(pg_type_scan);
 		heap_close(pg_type_desc);
-		elog(WARN, "DeletePgTypeTuple: %s type nonexistent",
+		elog(ERROR, "DeletePgTypeTuple: %s type nonexistent",
 			 &rdesc->rd_rel->relname);
 	}
 
@@ -1240,7 +1209,7 @@ DeletePgTypeTuple(Relation rdesc)
 
 	pg_attribute_scan = heap_beginscan(pg_attribute_desc,
 									   0,
-									   NowTimeQual,
+									   false,
 									   1,
 									   &attkey);
 
@@ -1261,7 +1230,7 @@ DeletePgTypeTuple(Relation rdesc)
 		heap_endscan(pg_attribute_scan);
 		heap_close(pg_attribute_desc);
 
-		elog(WARN, "DeletePgTypeTuple: att of type %s exists in relation %d",
+		elog(ERROR, "DeletePgTypeTuple: att of type %s exists in relation %d",
 			 &rdesc->rd_rel->relname, relid);
 	}
 	heap_endscan(pg_attribute_scan);
@@ -1280,12 +1249,12 @@ DeletePgTypeTuple(Relation rdesc)
 }
 
 /* --------------------------------
- *		heap_destroy
+ *		heap_destroy_with_catalog
  *
  * --------------------------------
  */
 void
-heap_destroy(char *relname)
+heap_destroy_with_catalog(char *relname)
 {
 	Relation	rdesc;
 	Oid			rid;
@@ -1297,7 +1266,7 @@ heap_destroy(char *relname)
 	 */
 	rdesc = heap_openr(relname);
 	if (rdesc == NULL)
-		elog(WARN, "Relation %s Does Not Exist!", relname);
+		elog(ERROR, "Relation %s Does Not Exist!", relname);
 
 	RelationSetLockForWrite(rdesc);
 	rid = rdesc->rd_id;
@@ -1307,7 +1276,7 @@ heap_destroy(char *relname)
 	 * ----------------
 	 */
 	if (IsSystemRelationName(RelationGetRelationName(rdesc)->data))
-		elog(WARN, "amdestroy: cannot destroy %s relation",
+		elog(ERROR, "amdestroy: cannot destroy %s relation",
 			 &rdesc->rd_rel->relname);
 
 	/* ----------------
@@ -1379,7 +1348,7 @@ heap_destroy(char *relname)
 	 */
 	if (!(rdesc->rd_istemp) || !(rdesc->rd_tmpunlinked))
 	{
-		smgrunlink(rdesc->rd_rel->relsmgr, rdesc);
+		smgrunlink(DEFAULT_SMGR, rdesc);
 	}
 	rdesc->rd_tmpunlinked = TRUE;
 
@@ -1392,19 +1361,17 @@ heap_destroy(char *relname)
 }
 
 /*
- * heap_destroyr
+ * heap_destroy
  *	  destroy and close temporary relations
  *
  */
 
 void
-heap_destroyr(Relation rdesc)
+heap_destroy(Relation rdesc)
 {
 	ReleaseRelationBuffers(rdesc);
 	if (!(rdesc->rd_istemp) || !(rdesc->rd_tmpunlinked))
-	{
-		smgrunlink(rdesc->rd_rel->relsmgr, rdesc);
-	}
+		smgrunlink(DEFAULT_SMGR, rdesc);
 	rdesc->rd_tmpunlinked = TRUE;
 	heap_close(rdesc);
 	RemoveFromTempRelList(rdesc);
@@ -1511,17 +1478,13 @@ DestroyTempRels(void)
 		rdesc = tempRels->rels[i];
 		/* rdesc may be NULL if it has been removed from the list already */
 		if (rdesc)
-			heap_destroyr(rdesc);
+			heap_destroy(rdesc);
 	}
 	free(tempRels->rels);
 	free(tempRels);
 	tempRels = NULL;
 }
 
-extern List *flatten_tlist(List *tlist);
-extern List *
-pg_plan(char *query_string, Oid *typev, int nargs,
-		QueryTreeList **queryListP, CommandDest dest);
 
 static void
 StoreAttrDefault(Relation rel, AttrDefault *attrdef)
@@ -1548,13 +1511,13 @@ start:;
 	sprintf(str, "select %s%s from %.*s", attrdef->adsrc, cast,
 			NAMEDATALEN, rel->rd_rel->relname.data);
 	setheapoverride(true);
-	planTree_list = (List *) pg_plan(str, NULL, 0, &queryTree_list, None);
+	planTree_list = (List *) pg_parse_and_plan(str, NULL, 0, &queryTree_list, None);
 	setheapoverride(false);
 	query = (Query *) (queryTree_list->qtrees[0]);
 
 	if (length(query->rtable) > 1 ||
 		flatten_tlist(query->targetList) != NIL)
-		elog(WARN, "DEFAULT: cannot use attribute(s)");
+		elog(ERROR, "DEFAULT: cannot use attribute(s)");
 	te = (TargetEntry *) lfirst(query->targetList);
 	resdom = te->resdom;
 	expr = te->expr;
@@ -1564,13 +1527,13 @@ start:;
 		if (((Const *) expr)->consttype != atp->atttypid)
 		{
 			if (*cast != 0)
-				elog(WARN, "DEFAULT: const type mismatched");
-			sprintf(cast, ":: %s", get_id_typname(atp->atttypid));
+				elog(ERROR, "DEFAULT: const type mismatched");
+			sprintf(cast, ":: %s", typeidTypeName(atp->atttypid));
 			goto start;
 		}
 	}
 	else if (exprType(expr) != atp->atttypid)
-		elog(WARN, "DEFAULT: type mismatched");
+		elog(ERROR, "DEFAULT: type mismatched");
 
 	adbin = nodeToString(expr);
 	oldcxt = MemoryContextSwitchTo((MemoryContext) CacheCxt);
@@ -1618,12 +1581,12 @@ StoreRelCheck(Relation rel, ConstrCheck *check)
 	sprintf(str, "select 1 from %.*s where %s",
 			NAMEDATALEN, rel->rd_rel->relname.data, check->ccsrc);
 	setheapoverride(true);
-	planTree_list = (List *) pg_plan(str, NULL, 0, &queryTree_list, None);
+	planTree_list = (List *) pg_parse_and_plan(str, NULL, 0, &queryTree_list, None);
 	setheapoverride(false);
 	query = (Query *) (queryTree_list->qtrees[0]);
 
 	if (length(query->rtable) > 1)
-		elog(WARN, "CHECK: only relation %.*s can be referenced",
+		elog(ERROR, "CHECK: only relation %.*s can be referenced",
 			 NAMEDATALEN, rel->rd_rel->relname.data);
 
 	plan = (Plan *) lfirst(planTree_list);
@@ -1695,7 +1658,7 @@ RemoveAttrDefault(Relation rel)
 
 	RelationSetLockForWrite(adrel);
 
-	adscan = heap_beginscan(adrel, 0, NowTimeQual, 1, &key);
+	adscan = heap_beginscan(adrel, 0, false, 1, &key);
 
 	while (tup = heap_getnext(adscan, 0, (Buffer *) NULL), PointerIsValid(tup))
 		heap_delete(adrel, &tup->t_ctid);
@@ -1722,7 +1685,7 @@ RemoveRelCheck(Relation rel)
 
 	RelationSetLockForWrite(rcrel);
 
-	rcscan = heap_beginscan(rcrel, 0, NowTimeQual, 1, &key);
+	rcscan = heap_beginscan(rcrel, 0, false, 1, &key);
 
 	while (tup = heap_getnext(rcscan, 0, (Buffer *) NULL), PointerIsValid(tup))
 		heap_delete(rcrel, &tup->t_ctid);
