@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/pg_operator.c,v 1.14 1997/09/18 20:20:18 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/pg_operator.c,v 1.23 1998/04/01 15:35:04 scrappy Exp $
  *
  * NOTES
  *	  these routines moved here from commands/define.c and somewhat cleaned up.
@@ -18,10 +18,13 @@
 
 #include <catalog/pg_proc.h>
 #include <utils/syscache.h>
+#include <utils/tqual.h>
+#include <utils/builtins.h>
 #include <access/heapam.h>
-#include <parser/catalog_utils.h>
 #include <catalog/catname.h>
 #include <catalog/pg_operator.h>
+#include <catalog/pg_type.h>
+#include <parser/parse_oper.h>
 #include <storage/bufmgr.h>
 #include <fmgr.h>
 #include <miscadmin.h>
@@ -95,12 +98,12 @@ OperatorGetWithOpenRelation(Relation pg_operator_desc,
 		{0, Anum_pg_operator_oprright, ObjectIdEqualRegProcedure},
 	};
 
-	fmgr_info(NameEqualRegProcedure,
-			  &opKey[0].sk_func, &opKey[0].sk_nargs);
-	fmgr_info(ObjectIdEqualRegProcedure,
-			  &opKey[1].sk_func, &opKey[1].sk_nargs);
-	fmgr_info(ObjectIdEqualRegProcedure,
-			  &opKey[2].sk_func, &opKey[2].sk_nargs);
+	fmgr_info(NameEqualRegProcedure, &opKey[0].sk_func);
+	fmgr_info(ObjectIdEqualRegProcedure, &opKey[1].sk_func);
+	fmgr_info(ObjectIdEqualRegProcedure, &opKey[2].sk_func);
+	opKey[0].sk_nargs = opKey[0].sk_func.fn_nargs;
+	opKey[1].sk_nargs = opKey[1].sk_func.fn_nargs;
+	opKey[2].sk_nargs = opKey[2].sk_func.fn_nargs;
 
 	/* ----------------
 	 *	form scan key
@@ -116,7 +119,7 @@ OperatorGetWithOpenRelation(Relation pg_operator_desc,
 	 */
 	pg_operator_scan = heap_beginscan(pg_operator_desc,
 									  0,
-									  SelfTimeQual,
+									  true,
 									  3,
 									  opKey);
 
@@ -169,7 +172,7 @@ OperatorGet(char *operatorName,
 		leftObjectId = TypeGet(leftTypeName, &leftDefined);
 
 		if (!OidIsValid(leftObjectId) || !leftDefined)
-			elog(WARN, "OperatorGet: left type '%s' nonexistent", leftTypeName);
+			elog(ERROR, "OperatorGet: left type '%s' nonexistent", leftTypeName);
 	}
 
 	if (rightTypeName)
@@ -177,13 +180,13 @@ OperatorGet(char *operatorName,
 		rightObjectId = TypeGet(rightTypeName, &rightDefined);
 
 		if (!OidIsValid(rightObjectId) || !rightDefined)
-			elog(WARN, "OperatorGet: right type '%s' nonexistent",
+			elog(ERROR, "OperatorGet: right type '%s' nonexistent",
 				 rightTypeName);
 	}
 
 	if (!((OidIsValid(leftObjectId) && leftDefined) ||
 		  (OidIsValid(rightObjectId) && rightDefined)))
-		elog(WARN, "OperatorGet: no argument types??");
+		elog(ERROR, "OperatorGet: no argument types??");
 
 	/* ----------------
 	 *	open the pg_operator relation
@@ -222,11 +225,12 @@ OperatorShellMakeWithOpenRelation(Relation pg_operator_desc,
 								  Oid leftObjectId,
 								  Oid rightObjectId)
 {
-	register int i;
+	int			i;
 	HeapTuple	tup;
 	Datum		values[Natts_pg_operator];
 	char		nulls[Natts_pg_operator];
 	Oid			operatorObjectId;
+	NameData	oname;
 	TupleDesc	tupDesc;
 
 	/* ----------------
@@ -244,7 +248,8 @@ OperatorShellMakeWithOpenRelation(Relation pg_operator_desc,
 	 * ----------------
 	 */
 	i = 0;
-	values[i++] = PointerGetDatum(operatorName);
+	namestrcpy(&oname, operatorName);
+	values[i++] = NameGetDatum(&oname);
 	values[i++] = Int32GetDatum(GetUserId());
 	values[i++] = (Datum) (uint16) 0;
 
@@ -325,7 +330,7 @@ OperatorShellMake(char *operatorName,
 
 	if (!((OidIsValid(leftObjectId) && leftDefined) ||
 		  (OidIsValid(rightObjectId) && rightDefined)))
-		elog(WARN, "OperatorShellMake: no valid argument types??");
+		elog(ERROR, "OperatorShellMake: no valid argument types??");
 
 	/* ----------------
 	 *	open pg_operator
@@ -450,7 +455,7 @@ OperatorDef(char *operatorName,
 			char *leftSortName,
 			char *rightSortName)
 {
-	register	i,
+	int			i,
 				j;
 	Relation	pg_operator_desc;
 
@@ -472,6 +477,7 @@ OperatorDef(char *operatorName,
 	char	   *name[4];
 	Oid			typeId[8];
 	int			nargs;
+	NameData		oname;
 	TupleDesc	tupDesc;
 
 	static ScanKeyData opKey[3] = {
@@ -480,19 +486,19 @@ OperatorDef(char *operatorName,
 		{0, Anum_pg_operator_oprright, ObjectIdEqualRegProcedure},
 	};
 
-	fmgr_info(NameEqualRegProcedure,
-			  &opKey[0].sk_func, &opKey[0].sk_nargs);
-	fmgr_info(ObjectIdEqualRegProcedure,
-			  &opKey[1].sk_func, &opKey[1].sk_nargs);
-	fmgr_info(ObjectIdEqualRegProcedure,
-			  &opKey[2].sk_func, &opKey[2].sk_nargs);
+	fmgr_info(NameEqualRegProcedure, &opKey[0].sk_func);
+	fmgr_info(ObjectIdEqualRegProcedure, &opKey[1].sk_func);
+	fmgr_info(ObjectIdEqualRegProcedure, &opKey[2].sk_func);
+	opKey[0].sk_nargs = opKey[0].sk_func.fn_nargs;
+	opKey[1].sk_nargs = opKey[1].sk_func.fn_nargs;
+	opKey[2].sk_nargs = opKey[2].sk_func.fn_nargs;
 
 	operatorObjectId = OperatorGet(operatorName,
 								   leftTypeName,
 								   rightTypeName);
 
 	if (OidIsValid(operatorObjectId) && !definedOK)
-		elog(WARN, "OperatorDef: operator \"%s\" already defined",
+		elog(ERROR, "OperatorDef: operator \"%s\" already defined",
 			 operatorName);
 
 	if (leftTypeName)
@@ -503,7 +509,7 @@ OperatorDef(char *operatorName,
 
 	if (!((OidIsValid(leftTypeId && leftDefined)) ||
 		  (OidIsValid(rightTypeId && rightDefined))))
-		elog(WARN, "OperatorGet: no argument types??");
+		elog(ERROR, "OperatorGet: no argument types??");
 
 	for (i = 0; i < Natts_pg_operator; ++i)
 	{
@@ -606,7 +612,8 @@ OperatorDef(char *operatorName,
 	 * ----------------
 	 */
 	i = 0;
-	values[i++] = PointerGetDatum(operatorName);
+	namestrcpy(&oname, operatorName);
+	values[i++] = NameGetDatum(&oname);
 	values[i++] = Int32GetDatum(GetUserId());
 	values[i++] = UInt16GetDatum(precedence);
 	values[i++] = leftTypeName ? (rightTypeName ? 'b' : 'r') : 'l';
@@ -666,7 +673,7 @@ OperatorDef(char *operatorName,
 				}
 
 				if (!OidIsValid(other_oid))
-					elog(WARN,
+					elog(ERROR,
 						 "OperatorDef: can't create operator '%s'",
 						 name[j]);
 				values[i++] = ObjectIdGetDatum(other_oid);
@@ -697,7 +704,7 @@ OperatorDef(char *operatorName,
 
 		pg_operator_scan = heap_beginscan(pg_operator_desc,
 										  0,
-										  SelfTimeQual,
+										  true,
 										  3,
 										  opKey);
 
@@ -717,7 +724,7 @@ OperatorDef(char *operatorName,
 			setheapoverride(false);
 		}
 		else
-			elog(WARN, "OperatorDef: no operator %d", other_oid);
+			elog(ERROR, "OperatorDef: no operator %d", other_oid);
 
 		heap_endscan(pg_operator_scan);
 
@@ -765,7 +772,7 @@ OperatorDef(char *operatorName,
 static void
 OperatorUpd(Oid baseId, Oid commId, Oid negId)
 {
-	register	i;
+	int			i;
 	Relation	pg_operator_desc;
 	HeapScanDesc pg_operator_scan;
 	HeapTuple	tup;
@@ -779,8 +786,8 @@ OperatorUpd(Oid baseId, Oid commId, Oid negId)
 		{0, ObjectIdAttributeNumber, ObjectIdEqualRegProcedure},
 	};
 
-	fmgr_info(ObjectIdEqualRegProcedure,
-			  &opKey[0].sk_func, &opKey[0].sk_nargs);
+	fmgr_info(ObjectIdEqualRegProcedure, &opKey[0].sk_func);
+	opKey[0].sk_nargs = opKey[0].sk_func.fn_nargs;
 
 	for (i = 0; i < Natts_pg_operator; ++i)
 	{
@@ -796,7 +803,7 @@ OperatorUpd(Oid baseId, Oid commId, Oid negId)
 
 	pg_operator_scan = heap_beginscan(pg_operator_desc,
 									  0,
-									  SelfTimeQual,
+									  true,
 									  1,
 									  opKey);
 
@@ -886,7 +893,7 @@ OperatorUpd(Oid baseId, Oid commId, Oid negId)
 
 	pg_operator_scan = heap_beginscan(pg_operator_desc,
 									  0,
-									  SelfTimeQual,
+									  true,
 									  1,
 									  opKey);
 
@@ -992,7 +999,7 @@ OperatorCreate(char *operatorName,
 	int			definedOK;
 
 	if (!leftTypeName && !rightTypeName)
-		elog(WARN, "OperatorCreate : at least one of leftarg or rightarg must be defined");
+		elog(ERROR, "OperatorCreate : at least one of leftarg or rightarg must be defined");
 
 	/* ----------------
 	 *	get the oid's of the operator's associated operators, if possible.
