@@ -14,7 +14,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/cluster.c,v 1.15 1997/09/08 02:21:55 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/cluster.c,v 1.23 1998/02/26 04:30:49 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -125,7 +125,7 @@ cluster(char oldrelname[], char oldindexname[])
 	OldHeap = heap_openr(oldrelname);
 	if (!RelationIsValid(OldHeap))
 	{
-		elog(WARN, "cluster: unknown relation: \"%s\"",
+		elog(ERROR, "cluster: unknown relation: \"%s\"",
 			 oldrelname);
 	}
 	OIDOldHeap = OldHeap->rd_id;/* Get OID for the index scan	*/
@@ -133,7 +133,7 @@ cluster(char oldrelname[], char oldindexname[])
 	OldIndex = index_openr(oldindexname);		/* Open old index relation	*/
 	if (!RelationIsValid(OldIndex))
 	{
-		elog(WARN, "cluster: unknown index: \"%s\"",
+		elog(ERROR, "cluster: unknown index: \"%s\"",
 			 oldindexname);
 	}
 	OIDOldIndex = OldIndex->rd_id;		/* OID for the index scan		  */
@@ -175,7 +175,10 @@ cluster(char oldrelname[], char oldindexname[])
 
 
 	/* Destroy old heap (along with its index) and rename new. */
-	heap_destroy(oldrelname);
+	heap_destroy_with_catalog(oldrelname);
+
+	CommitTransactionCommand();
+	StartTransactionCommand();
 
 	renamerel(NewHeapName, saveoldrelname);
 	TypeRename(NewHeapName, saveoldrelname);
@@ -209,20 +212,16 @@ copy_heap(Oid OIDOldHeap)
 	OldHeapDesc = RelationGetTupleDescriptor(OldHeap);
 
 	/*
-	 * Need to make a copy of the tuple descriptor, heap_create modifies
-	 * it.
+	 * Need to make a copy of the tuple descriptor,
+	 * heap_create_with_catalog modifies it.
 	 */
 
 	tupdesc = CreateTupleDescCopy(OldHeapDesc);
 
-	OIDNewHeap = heap_create(NewName,
-							 NULL,
-							 OldHeap->rd_rel->relarch,
-							 OldHeap->rd_rel->relsmgr,
-							 tupdesc);
+	OIDNewHeap = heap_create_with_catalog(NewName, tupdesc);
 
 	if (!OidIsValid(OIDNewHeap))
-		elog(WARN, "clusterheap: cannot create temporary heap relation\n");
+		elog(ERROR, "clusterheap: cannot create temporary heap relation\n");
 
 	NewHeap = heap_open(OIDNewHeap);
 
@@ -355,7 +354,7 @@ rebuildheap(Oid OIDNewHeap, Oid OIDOldHeap, Oid OIDOldIndex)
 	{
 
 		HeapTid = &ScanResult->heap_iptr;
-		LocalHeapTuple = heap_fetch(LocalOldHeap, 0, HeapTid, &LocalBuffer);
+		LocalHeapTuple = heap_fetch(LocalOldHeap, false, HeapTid, &LocalBuffer);
 		OIDNewHeapInsert =
 			heap_insert(LocalNewHeap, LocalHeapTuple);
 		pfree(ScanResult);
