@@ -6,7 +6,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/copy.c,v 1.34 1997/09/12 04:07:25 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/copy.c,v 1.45 1998/03/30 16:45:55 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -32,7 +32,7 @@
 #include <access/genam.h>
 #include <catalog/pg_type.h>
 #include <catalog/catname.h>
-#include <catalog/pg_user.h>
+#include <catalog/pg_shadow.h>
 #include <commands/copy.h>
 #include "commands/trigger.h"
 #include <storage/fd.h>
@@ -67,11 +67,8 @@ static int	CountTuples(Relation relation);
 extern FILE *Pfout,
 		   *Pfin;
 
-#ifdef COPY_DEBUG
 static int	lineno;
 
-#endif
-
 /*
  *	 DoCopy executes a the SQL COPY statement.
  */
@@ -115,15 +112,15 @@ DoCopy(char *relname, bool binary, bool oids, bool from, bool pipe,
 
 	rel = heap_openr(relname);
 	if (rel == NULL)
-		elog(WARN, "COPY command failed.  Class %s "
+		elog(ERROR, "COPY command failed.  Class %s "
 			 "does not exist.", relname);
 
 	result = pg_aclcheck(relname, UserName, required_access);
 	if (result != ACLCHECK_OK)
-		elog(WARN, "%s: %s", relname, aclcheck_error_strings[result]);
+		elog(ERROR, "%s: %s", relname, aclcheck_error_strings[result]);
 	/* Above should not return */
 	else if (!superuser() && !pipe)
-		elog(WARN, "You must have Postgres superuser privilege to do a COPY "
+		elog(ERROR, "You must have Postgres superuser privilege to do a COPY "
 			 "directly to or from a file.  Anyone can COPY to stdout or "
 			 "from stdin.  Psql's \\copy command also works for anyone.");
 	/* Above should not return. */
@@ -132,7 +129,7 @@ DoCopy(char *relname, bool binary, bool oids, bool from, bool pipe,
 		if (from)
 		{						/* copy from file to database */
 			if (rel->rd_rel->relkind == RELKIND_SEQUENCE)
-				elog(WARN, "You can't change sequence relation %s", relname);
+				elog(ERROR, "You can't change sequence relation %s", relname);
 			if (pipe)
 			{
 				if (IsUnderPostmaster)
@@ -147,7 +144,7 @@ DoCopy(char *relname, bool binary, bool oids, bool from, bool pipe,
 			{
 				fp = AllocateFile(filename, "r");
 				if (fp == NULL)
-					elog(WARN, "COPY command, running in backend with "
+					elog(ERROR, "COPY command, running in backend with "
 						 "effective uid %d, could not open file '%s' for "
 						 "reading.  Errno = %s (%d).",
 						 geteuid(), filename, strerror(errno), errno);
@@ -175,7 +172,7 @@ DoCopy(char *relname, bool binary, bool oids, bool from, bool pipe,
 				fp = AllocateFile(filename, "w");
 				umask(oumask);
 				if (fp == NULL)
-					elog(WARN, "COPY command, running in backend with "
+					elog(ERROR, "COPY command, running in backend with "
 						 "effective uid %d, could not open file '%s' for "
 						 "writing.  Errno = %s (%d).",
 						 geteuid(), filename, strerror(errno), errno);
@@ -205,10 +202,10 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 	int32		attr_count,
 				i;
 	AttributeTupleForm *attr;
-	func_ptr   *out_functions;
-	int			dummy;
+	FmgrInfo   *out_functions;
 	Oid			out_func_oid;
 	Oid		   *elements;
+	int16	   *typmod;
 	Datum		value;
 	bool		isnull;			/* The attribute we are copying is null */
 	char	   *nulls;
@@ -224,7 +221,7 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 	int32		ntuples;
 	TupleDesc	tupDesc;
 
-	scandesc = heap_beginscan(rel, 0, NULL, 0, NULL);
+	scandesc = heap_beginscan(rel, 0, false, 0, NULL);
 
 	attr_count = rel->rd_att->natts;
 	attr = rel->rd_att->attrs;
@@ -232,13 +229,15 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 
 	if (!binary)
 	{
-		out_functions = (func_ptr *) palloc(attr_count * sizeof(func_ptr));
+		out_functions = (FmgrInfo *) palloc(attr_count * sizeof(FmgrInfo));
 		elements = (Oid *) palloc(attr_count * sizeof(Oid));
+		typmod = (int16 *) palloc(attr_count * sizeof(int16));
 		for (i = 0; i < attr_count; i++)
 		{
 			out_func_oid = (Oid) GetOutputFunction(attr[i]->atttypid);
-			fmgr_info(out_func_oid, &out_functions[i], &dummy);
+			fmgr_info(out_func_oid, &out_functions[i]);
 			elements[i] = GetTypeElement(attr[i]->atttypid);
+			typmod[i] = attr[i]->atttypmod;
 		}
 		nulls = NULL;			/* meaningless, but compiler doesn't know
 								 * that */
@@ -246,6 +245,7 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 	else
 	{
 		elements = NULL;
+		typmod = NULL;
 		out_functions = NULL;
 		nulls = (char *) palloc(attr_count);
 		for (i = 0; i < attr_count; i++)
@@ -270,12 +270,13 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 
 		for (i = 0; i < attr_count; i++)
 		{
-			value = heap_getattr(tuple, InvalidBuffer, i + 1, tupDesc, &isnull);
+			value = heap_getattr(tuple, i + 1, tupDesc, &isnull);
 			if (!binary)
 			{
 				if (!isnull)
 				{
-					string = (char *) (out_functions[i]) (value, elements[i]);
+					string = (char *) (*fmgr_faddr(&out_functions[i]))
+						(value, elements[i], typmod[i]);
 					CopyAttributeOut(fp, string, delim);
 					pfree(string);
 				}
@@ -349,6 +350,7 @@ CopyTo(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 	{
 		pfree(out_functions);
 		pfree(elements);
+		pfree(typmod);
 	}
 
 	heap_close(rel);
@@ -360,9 +362,8 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 	HeapTuple	tuple;
 	AttrNumber	attr_count;
 	AttributeTupleForm *attr;
-	func_ptr   *in_functions;
-	int			i,
-				dummy;
+	FmgrInfo   *in_functions;
+	int			i;
 	Oid			in_func_oid;
 	Datum	   *values;
 	char	   *nulls,
@@ -381,6 +382,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 				tuples_read = 0;
 	bool		reading_to_eof = true;
 	Oid		   *elements;
+	int16	   *typmod;
 	FuncIndexInfo *finfo,
 			  **finfoP = NULL;
 	TupleDesc  *itupdescArr;
@@ -501,19 +503,22 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 
 	if (!binary)
 	{
-		in_functions = (func_ptr *) palloc(attr_count * sizeof(func_ptr));
+		in_functions = (FmgrInfo *) palloc(attr_count * sizeof(FmgrInfo));
 		elements = (Oid *) palloc(attr_count * sizeof(Oid));
+		typmod = (int16 *) palloc(attr_count * sizeof(int16));
 		for (i = 0; i < attr_count; i++)
 		{
 			in_func_oid = (Oid) GetInputFunction(attr[i]->atttypid);
-			fmgr_info(in_func_oid, &in_functions[i], &dummy);
+			fmgr_info(in_func_oid, &in_functions[i]);
 			elements[i] = GetTypeElement(attr[i]->atttypid);
+			typmod[i] = attr[i]->atttypmod;
 		}
 	}
 	else
 	{
 		in_functions = NULL;
 		elements = NULL;
+		typmod = NULL;
 		fread(&ntuples, sizeof(int32), 1, fp);
 		if (ntuples != 0)
 			reading_to_eof = false;
@@ -532,9 +537,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 		byval[i] = (bool) IsTypeByVal(attr[i]->atttypid);
 	}
 
-#ifdef COPY_DEBUG
 	lineno = 0;
-#endif
 	while (!done)
 	{
 		if (!binary)
@@ -543,10 +546,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 			int			newline = 0;
 
 #endif
-#ifdef COPY_DEBUG
 			lineno++;
-			elog(DEBUG, "line %d", lineno);
-#endif
 			if (oids)
 			{
 #ifdef COPY_PATCH
@@ -560,7 +560,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 				{
 					loaded_oid = oidin(string);
 					if (loaded_oid < BootstrapObjectIdData)
-						elog(WARN, "COPY TEXT: Invalid Oid");
+						elog(ERROR, "COPY TEXT: Invalid Oid");
 				}
 			}
 			for (i = 0; i < attr_count && !done; i++)
@@ -582,9 +582,9 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 				else
 				{
 					values[i] =
-						(Datum) (in_functions[i]) (string,
-												   elements[i],
-												   attr[i]->attlen);
+						(Datum) (*fmgr_faddr(&in_functions[i])) (string,
+															 elements[i],
+															  typmod[i]);
 
 					/*
 					 * Sanity check - by reference attributes cannot
@@ -593,12 +593,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 					if (!PointerIsValid(values[i]) &&
 						!(rel->rd_att->attrs[i]->attbyval))
 					{
-#ifdef COPY_DEBUG
-						elog(WARN,
-						 "copy from: line %d - Bad file format", lineno);
-#else
-						elog(WARN, "copy from: Bad file format");
-#endif
+						elog(ERROR, "copy from line %d: Bad file format", lineno);
 					}
 				}
 			}
@@ -622,7 +617,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 				{
 					fread(&loaded_oid, sizeof(int32), 1, fp);
 					if (loaded_oid < BootstrapObjectIdData)
-						elog(WARN, "COPY BINARY: Invalid Oid");
+						elog(ERROR, "COPY BINARY: Invalid Oid");
 				}
 				fread(&null_ct, sizeof(int32), 1, fp);
 				if (null_ct > 0)
@@ -661,7 +656,7 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 								ptr += sizeof(int32);
 								break;
 							default:
-								elog(WARN, "COPY BINARY: impossible size!");
+								elog(ERROR, "COPY BINARY: impossible size!");
 								break;
 						}
 					}
@@ -816,9 +811,13 @@ CopyFrom(Relation rel, bool binary, bool oids, FILE *fp, char *delim)
 			done = true;
 	}
 	pfree(values);
-	if (!binary)
-		pfree(in_functions);
 	pfree(nulls);
+	if (!binary)
+	{
+		pfree(in_functions);
+		pfree(elements);
+		pfree(typmod);
+	}
 	pfree(byval);
 	heap_close(rel);
 }
@@ -837,7 +836,7 @@ GetOutputFunction(Oid type)
 	if (HeapTupleIsValid(typeTuple))
 		return ((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typoutput);
 
-	elog(WARN, "GetOutputFunction: Cache lookup of type %d failed", type);
+	elog(ERROR, "GetOutputFunction: Cache lookup of type %d failed", type);
 	return (InvalidOid);
 }
 
@@ -854,7 +853,7 @@ GetTypeElement(Oid type)
 	if (HeapTupleIsValid(typeTuple))
 		return ((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typelem);
 
-	elog(WARN, "GetOutputFunction: Cache lookup of type %d failed", type);
+	elog(ERROR, "GetOutputFunction: Cache lookup of type %d failed", type);
 	return (InvalidOid);
 }
 
@@ -870,7 +869,7 @@ GetInputFunction(Oid type)
 	if (HeapTupleIsValid(typeTuple))
 		return ((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typinput);
 
-	elog(WARN, "GetInputFunction: Cache lookup of type %d failed", type);
+	elog(ERROR, "GetInputFunction: Cache lookup of type %d failed", type);
 	return (InvalidOid);
 }
 
@@ -886,7 +885,7 @@ IsTypeByVal(Oid type)
 	if (HeapTupleIsValid(typeTuple))
 		return ((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typbyval);
 
-	elog(WARN, "GetInputFunction: Cache lookup of type %d failed", type);
+	elog(ERROR, "GetInputFunction: Cache lookup of type %d failed", type);
 
 	return (InvalidOid);
 }
@@ -921,7 +920,7 @@ GetIndexRelations(Oid main_relation_oid,
 	bool		isnull;
 
 	pg_index_rel = heap_openr(IndexRelationName);
-	scandesc = heap_beginscan(pg_index_rel, 0, NULL, 0, NULL);
+	scandesc = heap_beginscan(pg_index_rel, 0, false, 0, NULL);
 	tupDesc = RelationGetTupleDescriptor(pg_index_rel);
 
 	*n_indices = 0;
@@ -936,12 +935,12 @@ GetIndexRelations(Oid main_relation_oid,
 	{
 
 		index_relation_oid =
-			(Oid) DatumGetInt32(heap_getattr(tuple, InvalidBuffer, 2,
+			(Oid) DatumGetInt32(heap_getattr(tuple, 2,
 											 tupDesc, &isnull));
 		if (index_relation_oid == main_relation_oid)
 		{
 			scan->index_rel_oid =
-				(Oid) DatumGetInt32(heap_getattr(tuple, InvalidBuffer,
+				(Oid) DatumGetInt32(heap_getattr(tuple,
 												 Anum_pg_index_indexrelid,
 												 tupDesc, &isnull));
 			(*n_indices)++;
@@ -1005,12 +1004,7 @@ CopyReadNewline(FILE *fp, int *newline)
 {
 	if (!*newline)
 	{
-#ifdef COPY_DEBUG
-		elog(NOTICE, "CopyReadNewline: line %d - extra fields ignored",
-			 lineno);
-#else
-		elog(NOTICE, "CopyReadNewline: line - extra fields ignored");
-#endif
+		elog(NOTICE, "CopyReadNewline: line %d - extra fields ignored", lineno);
 		while (!feof(fp) && (getc(fp) != '\n'));
 	}
 	*newline = 0;
@@ -1125,7 +1119,7 @@ CopyReadAttribute(FILE *fp, bool *isnull, char *delim)
 				case '.':
 					c = getc(fp);
 					if (c != '\n')
-						elog(WARN, "CopyReadAttribute - end of record marker corrupted");
+						elog(ERROR, "CopyReadAttribute - end of record marker corrupted");
 					return (NULL);
 					break;
 			}
@@ -1143,7 +1137,7 @@ CopyReadAttribute(FILE *fp, bool *isnull, char *delim)
 		if (!done)
 			attribute[i++] = c;
 		if (i == EXT_ATTLEN - 1)
-			elog(WARN, "CopyReadAttribute - attribute length too long");
+			elog(ERROR, "CopyReadAttribute - attribute length too long");
 	}
 	attribute[i] = '\0';
 	return (&attribute[0]);
@@ -1166,6 +1160,7 @@ CopyAttributeOut(FILE *fp, char *string, char *delim)
 			(c == '\\' && !is_array))
 			fputc('\\', fp);
 		else if (c == '\\' && is_array)
+		{
 			if (*(string + 1) == '\\')
 			{
 				/* translate \\ to \\\\ */
@@ -1180,6 +1175,7 @@ CopyAttributeOut(FILE *fp, char *string, char *delim)
 				fputc('\\', fp);
 				fputc('\\', fp);
 			}
+		}
 		fputc(*string, fp);
 	}
 }
@@ -1198,7 +1194,7 @@ CountTuples(Relation relation)
 
 	int			i;
 
-	scandesc = heap_beginscan(relation, 0, NULL, 0, NULL);
+	scandesc = heap_beginscan(relation, 0, false, 0, NULL);
 
 	for (tuple = heap_getnext(scandesc, 0, NULL), i = 0;
 		 tuple != NULL;
