@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/define.c,v 1.16 1997/09/08 21:42:38 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/define.c,v 1.24 1998/02/26 04:30:57 momjian Exp $
  *
  * DESCRIPTION
  *	  The "DefineFoo" routines take the parse tree and pick out the
@@ -45,13 +45,14 @@
 #include <catalog/pg_operator.h>
 #include <catalog/pg_proc.h>
 #include <catalog/pg_type.h>
+#include <catalog/pg_language.h>
 #include <utils/syscache.h>
 #include <fmgr.h>				/* for fmgr */
 #include <utils/builtins.h>		/* prototype for textin() */
 #include <commands/defrem.h>
 #include <optimizer/xfunc.h>
 #include <tcop/dest.h>
-#include <catalog/pg_user.h>
+#include <catalog/pg_shadow.h>
 
 static char *defGetString(DefElem *def);
 static int	defGetTypeLength(DefElem *def);
@@ -68,7 +69,7 @@ case_translate_language_name(const char *input, char *output)
 --------------------------------------------------------------------------*/
 	int			i;
 
-	for (i = 0; i < NAMEDATALEN && input[i] != '\0'; ++i)
+	for (i = 0; i < NAMEDATALEN && input[i]; ++i)
 		output[i] = tolower(input[i]);
 
 	output[i] = '\0';
@@ -93,7 +94,7 @@ compute_return_type(const Node *returnType,
 		TypeName   *setType = (TypeName *) returnType;
 
 		*prorettype_p = setType->name;
-		*returnsSet_p = true;
+		*returnsSet_p = setType->setof;
 	}
 	else
 	{
@@ -139,7 +140,7 @@ compute_full_attributes(const List *parameters, int32 *byte_pct_p,
 			 * we don't have untrusted functions any more. The 4.2
 			 * implementation is lousy anyway so I took it out. -ay 10/94
 			 */
-			elog(WARN, "untrusted function has been decommissioned.");
+			elog(ERROR, "untrusted function has been decommissioned.");
 		}
 		else if (strcasecmp(param->name, "byte_pct") == 0)
 		{
@@ -239,6 +240,8 @@ CreateFunction(ProcedureStmt *stmt, CommandDest dest)
 	bool		canCache;
 	bool		returnsSet;
 
+	bool		lanisPL = false;
+
 	/* The function returns a set of values, as opposed to a singleton. */
 
 
@@ -262,19 +265,62 @@ CreateFunction(ProcedureStmt *stmt, CommandDest dest)
 	}
 	else
 	{
-		elog(WARN,
-			 "Unrecognized language specified in a CREATE FUNCTION: "
-			 "'%s'.  Recognized languages are sql, C, and internal.",
-			 languageName);
+		HeapTuple	languageTuple;
+		Form_pg_language languageStruct;
+
+		/* Lookup the language in the system cache */
+		languageTuple = SearchSysCacheTuple(LANNAME,
+											PointerGetDatum(languageName),
+											0, 0, 0);
+
+		if (!HeapTupleIsValid(languageTuple))
+		{
+
+			elog(ERROR,
+				 "Unrecognized language specified in a CREATE FUNCTION: "
+				 "'%s'.  Recognized languages are sql, C, internal "
+				 "and the created procedural languages.",
+				 languageName);
+		}
+
+		/* Check that this language is a PL */
+		languageStruct = (Form_pg_language) GETSTRUCT(languageTuple);
+		if (!(languageStruct->lanispl))
+		{
+			elog(ERROR,
+				 "Language '%s' isn't defined as PL", languageName);
+		}
+
+		/*
+		 * Functions in untrusted procedural languages are restricted to
+		 * be defined by postgres superusers only
+		 */
+		if (languageStruct->lanpltrusted == false && !superuser())
+		{
+			elog(ERROR, "Only users with Postgres superuser privilege "
+				 "are permitted to create a function in the '%s' "
+				 "language.",
+				 languageName);
+		}
+
+		lanisPL = true;
+
+		/*
+		 * These are meaningless
+		 */
+		perbyte_cpu = percall_cpu = 0;
+		byte_pct = outin_ratio = 100;
+		canCache = false;
 	}
 
 	interpret_AS_clause(languageName, stmt->as, &prosrc_str, &probin_str);
 
-	if (strcmp(languageName, "sql") != 0 && !superuser())
-		elog(WARN,
+	if (strcmp(languageName, "sql") != 0 && lanisPL == false && !superuser())
+		elog(ERROR,
 			 "Only users with Postgres superuser privilege are permitted "
 			 "to create a function "
-			 "in the '%s' language.  Others may use the 'sql' language.",
+			 "in the '%s' language.  Others may use the 'sql' language "
+			 "or the created procedural languages.",
 			 languageName);
 	/* Above does not return. */
 	else
@@ -345,7 +391,7 @@ DefineOperator(char *oprName,
 		{
 			/* see gram.y, must be setof */
 			if (nodeTag(defel->arg) == T_TypeName)
-				elog(WARN, "setof type not implemented for leftarg");
+				elog(ERROR, "setof type not implemented for leftarg");
 
 			if (nodeTag(defel->arg) == T_String)
 			{
@@ -353,14 +399,14 @@ DefineOperator(char *oprName,
 			}
 			else
 			{
-				elog(WARN, "type for leftarg is malformed.");
+				elog(ERROR, "type for leftarg is malformed.");
 			}
 		}
 		else if (!strcasecmp(defel->defname, "rightarg"))
 		{
 			/* see gram.y, must be setof */
 			if (nodeTag(defel->arg) == T_TypeName)
-				elog(WARN, "setof type not implemented for rightarg");
+				elog(ERROR, "setof type not implemented for rightarg");
 
 			if (nodeTag(defel->arg) == T_String)
 			{
@@ -368,7 +414,7 @@ DefineOperator(char *oprName,
 			}
 			else
 			{
-				elog(WARN, "type for rightarg is malformed.");
+				elog(ERROR, "type for rightarg is malformed.");
 			}
 		}
 		else if (!strcasecmp(defel->defname, "procedure"))
@@ -431,7 +477,7 @@ DefineOperator(char *oprName,
 	 */
 	if (functionName == NULL)
 	{
-		elog(WARN, "Define: \"procedure\" unspecified");
+		elog(ERROR, "Define: \"procedure\" unspecified");
 	}
 
 	/* ----------------
@@ -536,16 +582,16 @@ DefineAggregate(char *aggName, List *parameters)
 	 * make sure we have our required definitions
 	 */
 	if (baseType == NULL)
-		elog(WARN, "Define: \"basetype\" unspecified");
+		elog(ERROR, "Define: \"basetype\" unspecified");
 	if (stepfunc1Name != NULL)
 	{
 		if (stepfunc1Type == NULL)
-			elog(WARN, "Define: \"stype1\" unspecified");
+			elog(ERROR, "Define: \"stype1\" unspecified");
 	}
 	if (stepfunc2Name != NULL)
 	{
 		if (stepfunc2Type == NULL)
-			elog(WARN, "Define: \"stype2\" unspecified");
+			elog(ERROR, "Define: \"stype2\" unspecified");
 	}
 
 	/*
@@ -592,7 +638,7 @@ DefineType(char *typeName, List *parameters)
 	 */
 	if (strlen(typeName) >= (NAMEDATALEN - 1))
 	{
-		elog(WARN, "DefineType: type names must be %d characters or less",
+		elog(ERROR, "DefineType: type names must be %d characters or less",
 			 NAMEDATALEN - 1);
 	}
 
@@ -656,7 +702,7 @@ DefineType(char *typeName, List *parameters)
 			}
 			else
 			{
-				elog(WARN, "DefineType: \"%s\" alignment  not recognized",
+				elog(ERROR, "DefineType: \"%s\" alignment  not recognized",
 					 a);
 			}
 		}
@@ -671,9 +717,9 @@ DefineType(char *typeName, List *parameters)
 	 * make sure we have our required definitions
 	 */
 	if (inputName == NULL)
-		elog(WARN, "Define: \"input\" unspecified");
+		elog(ERROR, "Define: \"input\" unspecified");
 	if (outputName == NULL)
-		elog(WARN, "Define: \"output\" unspecified");
+		elog(ERROR, "Define: \"output\" unspecified");
 
 	/* ----------------
 	 *	now have TypeCreate do all the real work.
@@ -687,8 +733,8 @@ DefineType(char *typeName, List *parameters)
 			   delimiter,		/* array element delimiter */
 			   inputName,		/* input procedure */
 			   outputName,		/* output procedure */
-			   sendName,		/* send procedure */
 			   receiveName,		/* receive procedure */
+			   sendName,		/* send procedure */
 			   elemName,		/* element type name */
 			   defaultValue,	/* default type value */
 			   byValue,			/* passed by value */
@@ -709,8 +755,8 @@ DefineType(char *typeName, List *parameters)
 			   DEFAULT_TYPDELIM,/* array element delimiter */
 			   "array_in",		/* input procedure */
 			   "array_out",		/* output procedure */
-			   "array_out",		/* send procedure */
 			   "array_in",		/* receive procedure */
+			   "array_out",		/* send procedure */
 			   typeName,		/* element type name */
 			   defaultValue,	/* default type value */
 			   false,			/* never passed by value */
@@ -723,7 +769,7 @@ static char *
 defGetString(DefElem *def)
 {
 	if (nodeTag(def->arg) != T_String)
-		elog(WARN, "Define: \"%s\" = what?", def->defname);
+		elog(ERROR, "Define: \"%s\" = what?", def->defname);
 	return (strVal(def->arg));
 }
 
@@ -736,6 +782,6 @@ defGetTypeLength(DefElem *def)
 			 !strcasecmp(strVal(def->arg), "variable"))
 		return -1;				/* variable length */
 
-	elog(WARN, "Define: \"%s\" = what?", def->defname);
+	elog(ERROR, "Define: \"%s\" = what?", def->defname);
 	return -1;
 }
