@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/explain.c,v 1.14 1997/09/18 20:20:22 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/explain.c,v 1.18 1998/02/26 04:30:58 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -16,12 +16,11 @@
 
 #include <postgres.h>
 
-#include <parser/catalog_utils.h>
-#include <parser/parse_query.h> /* for MakeTimeRange() */
 #include <nodes/plannodes.h>
 #include <tcop/tcopprot.h>
 #include <lib/stringinfo.h>
 #include <commands/explain.h>
+#include <parser/parse_node.h>
 #include <optimizer/planner.h>
 #include <access/xact.h>
 
@@ -69,7 +68,7 @@ ExplainQuery(Query *query, bool verbose, CommandDest dest)
 	if (plan == NULL)
 		return;
 
-	es = (ExplainState *) malloc(sizeof(ExplainState));
+	es = (ExplainState *) palloc(sizeof(ExplainState));
 	MemSet(es, 0, sizeof(ExplainState));
 
 	es->printCost = true;		/* default */
@@ -104,7 +103,7 @@ ExplainQuery(Query *query, bool verbose, CommandDest dest)
 		elog(NOTICE, "%.*s", ELOG_MAXLEN - 64, s);
 		len -= ELOG_MAXLEN - 64;
 	}
-	free(es);
+	pfree(es);
 }
 
 /*****************************************************************************
@@ -177,8 +176,10 @@ explain_outNode(StringInfo str, Plan *plan, int indent, ExplainState *es)
 			break;
 	}
 
+#if 0
 	for (i = 0; i < indent; i++)
 		appendStringInfo(str, "  ");
+#endif
 
 	appendStringInfo(str, pname);
 	switch (nodeTag(plan))
@@ -204,13 +205,33 @@ explain_outNode(StringInfo str, Plan *plan, int indent, ExplainState *es)
 	}
 	appendStringInfo(str, "\n");
 
+	/* initPlan-s */
+	if (plan->initPlan)
+	{
+		List	   *saved_rtable = es->rtable;
+		List	   *lst;
+
+		for (i = 0; i < indent; i++)
+			appendStringInfo(str, "  ");
+		appendStringInfo(str, "  InitPlan\n");
+		foreach(lst, plan->initPlan)
+		{
+			es->rtable = ((SubPlan *) lfirst(lst))->rtable;
+			for (i = 0; i < indent; i++)
+				appendStringInfo(str, "  ");
+			appendStringInfo(str, "    ->  ");
+			explain_outNode(str, ((SubPlan *) lfirst(lst))->plan, indent + 4, es);
+		}
+		es->rtable = saved_rtable;
+	}
+
 	/* lefttree */
 	if (outerPlan(plan))
 	{
 		for (i = 0; i < indent; i++)
 			appendStringInfo(str, "  ");
-		appendStringInfo(str, "  -> ");
-		explain_outNode(str, outerPlan(plan), indent + 1, es);
+		appendStringInfo(str, "  ->  ");
+		explain_outNode(str, outerPlan(plan), indent + 3, es);
 	}
 
 	/* righttree */
@@ -218,8 +239,28 @@ explain_outNode(StringInfo str, Plan *plan, int indent, ExplainState *es)
 	{
 		for (i = 0; i < indent; i++)
 			appendStringInfo(str, "  ");
-		appendStringInfo(str, "  -> ");
-		explain_outNode(str, innerPlan(plan), indent + 1, es);
+		appendStringInfo(str, "  ->  ");
+		explain_outNode(str, innerPlan(plan), indent + 3, es);
+	}
+
+	/* subPlan-s */
+	if (plan->subPlan)
+	{
+		List	   *saved_rtable = es->rtable;
+		List	   *lst;
+
+		for (i = 0; i < indent; i++)
+			appendStringInfo(str, "  ");
+		appendStringInfo(str, "  SubPlan\n");
+		foreach(lst, plan->subPlan)
+		{
+			es->rtable = ((SubPlan *) lfirst(lst))->rtable;
+			for (i = 0; i < indent; i++)
+				appendStringInfo(str, "  ");
+			appendStringInfo(str, "    ->  ");
+			explain_outNode(str, ((SubPlan *) lfirst(lst))->plan, indent + 4, es);
+		}
+		es->rtable = saved_rtable;
 	}
 	return;
 }
