@@ -26,11 +26,11 @@
 #include "utils/mcxt.h"
 #include "utils/inval.h"
 #include "utils/builtins.h"
+#include "utils/syscache.h"
 
 #ifndef NO_SECURITY
 #include "miscadmin.h"
 #include "utils/acl.h"
-#include "utils/syscache.h"
 #endif
 
 TriggerData *CurrentTriggerData = NULL;
@@ -38,16 +38,15 @@ TriggerData *CurrentTriggerData = NULL;
 void		RelationBuildTriggers(Relation relation);
 void		FreeTriggerDesc(Relation relation);
 
-static void DescribeTrigger(TriggerDesc * trigdesc, Trigger * trigger);
+static void DescribeTrigger(TriggerDesc *trigdesc, Trigger *trigger);
 static HeapTuple
 GetTupleForTrigger(Relation relation, ItemPointer tid,
 				   bool before);
 
-extern void fmgr_info(Oid procedureId, func_ptr * function, int *nargs);
 extern GlobalMemory CacheCxt;
 
 void
-CreateTrigger(CreateTrigStmt * stmt)
+CreateTrigger(CreateTrigStmt *stmt)
 {
 	int16		tgtype;
 	int16		tgattr[8] = {0};
@@ -68,16 +67,16 @@ CreateTrigger(CreateTrigStmt * stmt)
 	int			i;
 
 	if (IsSystemRelationName(stmt->relname))
-		elog(WARN, "CreateTrigger: can't create trigger for system relation %s", stmt->relname);
+		elog(ERROR, "CreateTrigger: can't create trigger for system relation %s", stmt->relname);
 
 #ifndef NO_SECURITY
 	if (!pg_ownercheck(GetPgUserName(), stmt->relname, RELNAME))
-		elog(WARN, "%s: %s", stmt->relname, aclcheck_error_strings[ACLCHECK_NOT_OWNER]);
+		elog(ERROR, "%s: %s", stmt->relname, aclcheck_error_strings[ACLCHECK_NOT_OWNER]);
 #endif
 
 	rel = heap_openr(stmt->relname);
 	if (!RelationIsValid(rel))
-		elog(WARN, "CreateTrigger: there is no relation %s", stmt->relname);
+		elog(ERROR, "CreateTrigger: there is no relation %s", stmt->relname);
 
 	RelationSetLockForWrite(rel);
 
@@ -87,29 +86,29 @@ CreateTrigger(CreateTrigStmt * stmt)
 	if (stmt->row)
 		TRIGGER_SETT_ROW(tgtype);
 	else
-		elog (WARN, "CreateTrigger: STATEMENT triggers are unimplemented, yet");
-	
+		elog(ERROR, "CreateTrigger: STATEMENT triggers are unimplemented, yet");
+
 	for (i = 0; i < 3 && stmt->actions[i]; i++)
 	{
 		switch (stmt->actions[i])
 		{
 			case 'i':
 				if (TRIGGER_FOR_INSERT(tgtype))
-					elog(WARN, "CreateTrigger: double INSERT event specified");
+					elog(ERROR, "CreateTrigger: double INSERT event specified");
 				TRIGGER_SETT_INSERT(tgtype);
 				break;
 			case 'd':
 				if (TRIGGER_FOR_DELETE(tgtype))
-					elog(WARN, "CreateTrigger: double DELETE event specified");
+					elog(ERROR, "CreateTrigger: double DELETE event specified");
 				TRIGGER_SETT_DELETE(tgtype);
 				break;
 			case 'u':
 				if (TRIGGER_FOR_UPDATE(tgtype))
-					elog(WARN, "CreateTrigger: double UPDATE event specified");
+					elog(ERROR, "CreateTrigger: double UPDATE event specified");
 				TRIGGER_SETT_UPDATE(tgtype);
 				break;
 			default:
-				elog(WARN, "CreateTrigger: unknown event specified");
+				elog(ERROR, "CreateTrigger: unknown event specified");
 				break;
 		}
 	}
@@ -119,13 +118,13 @@ CreateTrigger(CreateTrigStmt * stmt)
 	RelationSetLockForWrite(tgrel);
 	ScanKeyEntryInitialize(&key, 0, Anum_pg_trigger_tgrelid,
 						   ObjectIdEqualRegProcedure, rel->rd_id);
-	tgscan = heap_beginscan(tgrel, 0, NowTimeQual, 1, &key);
+	tgscan = heap_beginscan(tgrel, 0, false, 1, &key);
 	while (tuple = heap_getnext(tgscan, 0, (Buffer *) NULL), PointerIsValid(tuple))
 	{
 		Form_pg_trigger pg_trigger = (Form_pg_trigger) GETSTRUCT(tuple);
 
 		if (namestrcmp(&(pg_trigger->tgname), stmt->trigname) == 0)
-			elog(WARN, "CreateTrigger: trigger %s already defined on relation %s",
+			elog(ERROR, "CreateTrigger: trigger %s already defined on relation %s",
 				 stmt->trigname, stmt->relname);
 		else
 			found++;
@@ -139,10 +138,25 @@ CreateTrigger(CreateTrigStmt * stmt)
 	if (!HeapTupleIsValid(tuple) ||
 		((Form_pg_proc) GETSTRUCT(tuple))->prorettype != 0 ||
 		((Form_pg_proc) GETSTRUCT(tuple))->pronargs != 0)
-		elog(WARN, "CreateTrigger: function %s () does not exist", stmt->funcname);
+		elog(ERROR, "CreateTrigger: function %s () does not exist", stmt->funcname);
 
 	if (((Form_pg_proc) GETSTRUCT(tuple))->prolang != ClanguageId)
-		elog(WARN, "CreateTrigger: only C functions are supported");
+	{
+		HeapTuple	langTup;
+
+		langTup = SearchSysCacheTuple(LANOID,
+			ObjectIdGetDatum(((Form_pg_proc) GETSTRUCT(tuple))->prolang),
+									  0, 0, 0);
+		if (!HeapTupleIsValid(langTup))
+		{
+			elog(ERROR, "CreateTrigger: cache lookup for PL failed");
+		}
+
+		if (((Form_pg_language) GETSTRUCT(langTup))->lanispl == false)
+		{
+			elog(ERROR, "CreateTrigger: only C and PL functions are supported");
+		}
+	}
 
 	MemSet(nulls, ' ', Natts_pg_trigger * sizeof(char));
 
@@ -159,10 +173,10 @@ CreateTrigger(CreateTrigStmt * stmt)
 
 		foreach(le, stmt->args)
 		{
-			char   *ar = (char *) lfirst(le);
+			char	   *ar = (char *) lfirst(le);
 
-			len += strlen(ar) + 4;
-			for ( ; *ar; ar++)
+			len += strlen(ar) + VARHDRSZ;
+			for (; *ar; ar++)
 			{
 				if (*ar == '\\')
 					len++;
@@ -172,9 +186,9 @@ CreateTrigger(CreateTrigStmt * stmt)
 		args[0] = 0;
 		foreach(le, stmt->args)
 		{
-			char   *s = (char *) lfirst(le);
-			char   *d = args + strlen(args);
-			
+			char	   *s = (char *) lfirst(le);
+			char	   *d = args + strlen(args);
+
 			while (*s)
 			{
 				if (*s == '\\')
@@ -212,7 +226,7 @@ CreateTrigger(CreateTrigStmt * stmt)
 	if (!PointerIsValid(tuple))
 	{
 		heap_close(relrdesc);
-		elog(WARN, "CreateTrigger: relation %s not found in pg_class", stmt->relname);
+		elog(ERROR, "CreateTrigger: relation %s not found in pg_class", stmt->relname);
 	}
 	((Form_pg_class) GETSTRUCT(tuple))->reltriggers = found + 1;
 	RelationInvalidateHeapTuple(relrdesc, tuple);
@@ -235,7 +249,7 @@ CreateTrigger(CreateTrigStmt * stmt)
 }
 
 void
-DropTrigger(DropTrigStmt * stmt)
+DropTrigger(DropTrigStmt *stmt)
 {
 	Relation	rel;
 	Relation	tgrel;
@@ -251,12 +265,12 @@ DropTrigger(DropTrigStmt * stmt)
 
 #ifndef NO_SECURITY
 	if (!pg_ownercheck(GetPgUserName(), stmt->relname, RELNAME))
-		elog(WARN, "%s: %s", stmt->relname, aclcheck_error_strings[ACLCHECK_NOT_OWNER]);
+		elog(ERROR, "%s: %s", stmt->relname, aclcheck_error_strings[ACLCHECK_NOT_OWNER]);
 #endif
 
 	rel = heap_openr(stmt->relname);
 	if (!RelationIsValid(rel))
-		elog(WARN, "DropTrigger: there is no relation %s", stmt->relname);
+		elog(ERROR, "DropTrigger: there is no relation %s", stmt->relname);
 
 	RelationSetLockForWrite(rel);
 
@@ -264,7 +278,7 @@ DropTrigger(DropTrigStmt * stmt)
 	RelationSetLockForWrite(tgrel);
 	ScanKeyEntryInitialize(&key, 0, Anum_pg_trigger_tgrelid,
 						   ObjectIdEqualRegProcedure, rel->rd_id);
-	tgscan = heap_beginscan(tgrel, 0, NowTimeQual, 1, &key);
+	tgscan = heap_beginscan(tgrel, 0, false, 1, &key);
 	while (tuple = heap_getnext(tgscan, 0, (Buffer *) NULL), PointerIsValid(tuple))
 	{
 		Form_pg_trigger pg_trigger = (Form_pg_trigger) GETSTRUCT(tuple);
@@ -278,7 +292,7 @@ DropTrigger(DropTrigStmt * stmt)
 			found++;
 	}
 	if (tgfound == 0)
-		elog(WARN, "DropTrigger: there is no trigger %s on relation %s",
+		elog(ERROR, "DropTrigger: there is no trigger %s on relation %s",
 			 stmt->trigname, stmt->relname);
 	if (tgfound > 1)
 		elog(NOTICE, "DropTrigger: found (and deleted) %d trigger %s on relation %s",
@@ -293,7 +307,7 @@ DropTrigger(DropTrigStmt * stmt)
 	if (!PointerIsValid(tuple))
 	{
 		heap_close(relrdesc);
-		elog(WARN, "DropTrigger: relation %s not found in pg_class", stmt->relname);
+		elog(ERROR, "DropTrigger: relation %s not found in pg_class", stmt->relname);
 	}
 	((Form_pg_class) GETSTRUCT(tuple))->reltriggers = found;
 	RelationInvalidateHeapTuple(relrdesc, tuple);
@@ -329,7 +343,7 @@ RelationRemoveTriggers(Relation rel)
 	ScanKeyEntryInitialize(&key, 0, Anum_pg_trigger_tgrelid,
 						   ObjectIdEqualRegProcedure, rel->rd_id);
 
-	tgscan = heap_beginscan(tgrel, 0, NowTimeQual, 1, &key);
+	tgscan = heap_beginscan(tgrel, 0, false, 1, &key);
 
 	while (tup = heap_getnext(tgscan, 0, (Buffer *) NULL), PointerIsValid(tup))
 		heap_delete(tgrel, &tup->t_ctid);
@@ -380,12 +394,12 @@ RelationBuildTriggers(Relation relation)
 			break;
 
 		iptr = &indexRes->heap_iptr;
-		tuple = heap_fetch(tgrel, NowTimeQual, iptr, &buffer);
+		tuple = heap_fetch(tgrel, false, iptr, &buffer);
 		pfree(indexRes);
 		if (!HeapTupleIsValid(tuple))
 			continue;
 		if (found == ntrigs)
-			elog(WARN, "RelationBuildTriggers: unexpected record found for rel %.*s",
+			elog(ERROR, "RelationBuildTriggers: unexpected record found for rel %.*s",
 				 NAMEDATALEN, relation->rd_rel->relname.data);
 
 		pg_trigger = (Form_pg_trigger) GETSTRUCT(tuple);
@@ -398,7 +412,7 @@ RelationBuildTriggers(Relation relation)
 
 		build->tgname = nameout(&(pg_trigger->tgname));
 		build->tgfoid = pg_trigger->tgfoid;
-		build->tgfunc = NULL;
+		build->tgfunc.fn_addr = NULL;
 		build->tgtype = pg_trigger->tgtype;
 		build->tgnargs = pg_trigger->tgnargs;
 		memcpy(build->tgattr, &(pg_trigger->tgattr), 8 * sizeof(int16));
@@ -406,7 +420,7 @@ RelationBuildTriggers(Relation relation)
 											 Anum_pg_trigger_tgargs,
 											 tgrel->rd_att, &isnull);
 		if (isnull)
-			elog(WARN, "RelationBuildTriggers: tgargs IS NULL for rel %.*s",
+			elog(ERROR, "RelationBuildTriggers: tgargs IS NULL for rel %.*s",
 				 NAMEDATALEN, relation->rd_rel->relname.data);
 		if (build->tgnargs > 0)
 		{
@@ -417,7 +431,7 @@ RelationBuildTriggers(Relation relation)
 												 Anum_pg_trigger_tgargs,
 												 tgrel->rd_att, &isnull);
 			if (isnull)
-				elog(WARN, "RelationBuildTriggers: tgargs IS NULL for rel %.*s",
+				elog(ERROR, "RelationBuildTriggers: tgargs IS NULL for rel %.*s",
 					 NAMEDATALEN, relation->rd_rel->relname.data);
 			p = (char *) VARDATA(val);
 			build->tgargs = (char **) palloc(build->tgnargs * sizeof(char *));
@@ -436,7 +450,7 @@ RelationBuildTriggers(Relation relation)
 	}
 
 	if (found < ntrigs)
-		elog(WARN, "RelationBuildTriggers: %d record not found for rel %.*s",
+		elog(ERROR, "RelationBuildTriggers: %d record not found for rel %.*s",
 			 ntrigs - found,
 			 NAMEDATALEN, relation->rd_rel->relname.data);
 
@@ -505,7 +519,7 @@ FreeTriggerDesc(Relation relation)
 }
 
 static void
-DescribeTrigger(TriggerDesc * trigdesc, Trigger * trigger)
+DescribeTrigger(TriggerDesc *trigdesc, Trigger *trigger)
 {
 	uint16	   *n;
 	Trigger  ***t,
@@ -578,6 +592,24 @@ DescribeTrigger(TriggerDesc * trigdesc, Trigger * trigger)
 
 }
 
+static HeapTuple
+ExecCallTriggerFunc(Trigger *trigger)
+{
+
+	if (trigger->tgfunc.fn_addr == NULL)
+	{
+		fmgr_info(trigger->tgfoid, &trigger->tgfunc);
+	}
+
+	if (trigger->tgfunc.fn_plhandler != NULL)
+	{
+		return (HeapTuple) (*(trigger->tgfunc.fn_plhandler))
+			(&trigger->tgfunc);
+	}
+
+	return (HeapTuple) ((*fmgr_faddr(&trigger->tgfunc)) ());
+}
+
 HeapTuple
 ExecBRInsertTriggers(Relation rel, HeapTuple trigtuple)
 {
@@ -586,7 +618,6 @@ ExecBRInsertTriggers(Relation rel, HeapTuple trigtuple)
 	Trigger   **trigger = rel->trigdesc->tg_before_row[TRIGGER_EVENT_INSERT];
 	HeapTuple	newtuple = trigtuple;
 	HeapTuple	oldtuple;
-	int			nargs;
 	int			i;
 
 	SaveTriggerData = (TriggerData *) palloc(sizeof(TriggerData));
@@ -599,9 +630,7 @@ ExecBRInsertTriggers(Relation rel, HeapTuple trigtuple)
 		CurrentTriggerData = SaveTriggerData;
 		CurrentTriggerData->tg_trigtuple = oldtuple = newtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		newtuple = (HeapTuple) ((*(trigger[i]->tgfunc)) ());
+		newtuple = ExecCallTriggerFunc(trigger[i]);
 		if (newtuple == NULL)
 			break;
 		else if (oldtuple != newtuple && oldtuple != trigtuple)
@@ -618,7 +647,6 @@ ExecARInsertTriggers(Relation rel, HeapTuple trigtuple)
 	TriggerData *SaveTriggerData;
 	int			ntrigs = rel->trigdesc->n_after_row[TRIGGER_EVENT_INSERT];
 	Trigger   **trigger = rel->trigdesc->tg_after_row[TRIGGER_EVENT_INSERT];
-	int			nargs;
 	int			i;
 
 	SaveTriggerData = (TriggerData *) palloc(sizeof(TriggerData));
@@ -630,9 +658,7 @@ ExecARInsertTriggers(Relation rel, HeapTuple trigtuple)
 		CurrentTriggerData = SaveTriggerData;
 		CurrentTriggerData->tg_trigtuple = trigtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		(void) ((*(trigger[i]->tgfunc)) ());
+		ExecCallTriggerFunc(trigger[i]);
 	}
 	CurrentTriggerData = NULL;
 	pfree(SaveTriggerData);
@@ -647,7 +673,6 @@ ExecBRDeleteTriggers(Relation rel, ItemPointer tupleid)
 	Trigger   **trigger = rel->trigdesc->tg_before_row[TRIGGER_EVENT_DELETE];
 	HeapTuple	trigtuple;
 	HeapTuple	newtuple = NULL;
-	int			nargs;
 	int			i;
 
 	trigtuple = GetTupleForTrigger(rel, tupleid, true);
@@ -664,9 +689,7 @@ ExecBRDeleteTriggers(Relation rel, ItemPointer tupleid)
 		CurrentTriggerData = SaveTriggerData;
 		CurrentTriggerData->tg_trigtuple = trigtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		newtuple = (HeapTuple) ((*(trigger[i]->tgfunc)) ());
+		newtuple = ExecCallTriggerFunc(trigger[i]);
 		if (newtuple == NULL)
 			break;
 	}
@@ -684,7 +707,6 @@ ExecARDeleteTriggers(Relation rel, ItemPointer tupleid)
 	int			ntrigs = rel->trigdesc->n_after_row[TRIGGER_EVENT_DELETE];
 	Trigger   **trigger = rel->trigdesc->tg_after_row[TRIGGER_EVENT_DELETE];
 	HeapTuple	trigtuple;
-	int			nargs;
 	int			i;
 
 	trigtuple = GetTupleForTrigger(rel, tupleid, false);
@@ -700,9 +722,7 @@ ExecARDeleteTriggers(Relation rel, ItemPointer tupleid)
 		CurrentTriggerData = SaveTriggerData;
 		CurrentTriggerData->tg_trigtuple = trigtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		(void) ((*(trigger[i]->tgfunc)) ());
+		ExecCallTriggerFunc(trigger[i]);
 	}
 	CurrentTriggerData = NULL;
 	pfree(SaveTriggerData);
@@ -719,7 +739,6 @@ ExecBRUpdateTriggers(Relation rel, ItemPointer tupleid, HeapTuple newtuple)
 	HeapTuple	trigtuple;
 	HeapTuple	oldtuple;
 	HeapTuple	intuple = newtuple;
-	int			nargs;
 	int			i;
 
 	trigtuple = GetTupleForTrigger(rel, tupleid, true);
@@ -736,9 +755,7 @@ ExecBRUpdateTriggers(Relation rel, ItemPointer tupleid, HeapTuple newtuple)
 		CurrentTriggerData->tg_trigtuple = trigtuple;
 		CurrentTriggerData->tg_newtuple = oldtuple = newtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		newtuple = (HeapTuple) ((*(trigger[i]->tgfunc)) ());
+		newtuple = ExecCallTriggerFunc(trigger[i]);
 		if (newtuple == NULL)
 			break;
 		else if (oldtuple != newtuple && oldtuple != intuple)
@@ -757,7 +774,6 @@ ExecARUpdateTriggers(Relation rel, ItemPointer tupleid, HeapTuple newtuple)
 	int			ntrigs = rel->trigdesc->n_after_row[TRIGGER_EVENT_UPDATE];
 	Trigger   **trigger = rel->trigdesc->tg_after_row[TRIGGER_EVENT_UPDATE];
 	HeapTuple	trigtuple;
-	int			nargs;
 	int			i;
 
 	trigtuple = GetTupleForTrigger(rel, tupleid, false);
@@ -773,9 +789,7 @@ ExecARUpdateTriggers(Relation rel, ItemPointer tupleid, HeapTuple newtuple)
 		CurrentTriggerData->tg_trigtuple = trigtuple;
 		CurrentTriggerData->tg_newtuple = newtuple;
 		CurrentTriggerData->tg_trigger = trigger[i];
-		if (trigger[i]->tgfunc == NULL)
-			fmgr_info(trigger[i]->tgfoid, &(trigger[i]->tgfunc), &nargs);
-		(void) ((*(trigger[i]->tgfunc)) ());
+		ExecCallTriggerFunc(trigger[i]);
 	}
 	CurrentTriggerData = NULL;
 	pfree(SaveTriggerData);
@@ -794,7 +808,7 @@ GetTupleForTrigger(Relation relation, ItemPointer tid, bool before)
 	b = ReadBuffer(relation, ItemPointerGetBlockNumber(tid));
 
 	if (!BufferIsValid(b))
-		elog(WARN, "GetTupleForTrigger: failed ReadBuffer");
+		elog(ERROR, "GetTupleForTrigger: failed ReadBuffer");
 
 	dp = (PageHeader) BufferGetPage(b);
 	lp = PageGetItemId(dp, ItemPointerGetOffsetNumber(tid));
@@ -813,11 +827,11 @@ GetTupleForTrigger(Relation relation, ItemPointer tid, bool before)
 		}
 
 		HeapTupleSatisfies(lp, relation, b, dp,
-						   NowTimeQual, 0, (ScanKey) NULL, tuple);
+						   false, 0, (ScanKey) NULL, tuple);
 		if (!tuple)
 		{
 			ReleaseBuffer(b);
-			elog(WARN, "GetTupleForTrigger: (am)invalid tid");
+			elog(ERROR, "GetTupleForTrigger: (am)invalid tid");
 		}
 	}
 
