@@ -10,7 +10,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/version.c,v 1.7 1997/09/08 02:22:18 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/version.c,v 1.11 1998/01/05 16:39:07 momjian Exp $
  *
  * NOTES
  *	  At the point the version is defined, 2 physical relations are created
@@ -30,6 +30,7 @@
 #include <utils/builtins.h>
 #include <commands/version.h>
 #include <access/xact.h>		/* for GetCurrentXactStartTime */
+#include <parser/parse_node.h>
 #include <tcop/tcopprot.h>
 
 #define MAX_QUERY_LEN 1024
@@ -94,7 +95,7 @@ eval_as_new_xact(char *query)
 	 * CommitTransactionCommand(); StartTransactionCommand();
 	 */
 	CommandCounterIncrement();
-	pg_eval(query, (char **) NULL, (Oid *) NULL, 0);
+	pg_exec_query(query, (char **) NULL, (Oid *) NULL, 0);
 }
 
 #endif
@@ -156,7 +157,7 @@ VersionCreate(char *vname, char *bname)
 	sprintf(query_buf, "SELECT * INTO TABLE %s from %s where 1 =2",
 			vname, bname);
 
-	pg_eval(query_buf, (char **) NULL, (Oid *) NULL, 0);
+	pg_exec_query(query_buf, (char **) NULL, (Oid *) NULL, 0);
 
 	/*
 	 * Creating the ``v_added'' relation
@@ -194,7 +195,7 @@ setAttrList(char *bname)
 	rdesc = heap_openr(bname);
 	if (rdesc == NULL)
 	{
-		elog(WARN, "Unable to expand all -- amopenr failed ");
+		elog(ERROR, "Unable to expand all -- amopenr failed ");
 		return;
 	}
 	maxattrs = RelationGetNumberOfAttributes(rdesc);
