@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execAmi.c,v 1.8 1997/09/08 21:42:52 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execAmi.c,v 1.20 1998/02/27 16:11:26 vadim Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -37,6 +37,15 @@
 #include "executor/nodeIndexscan.h"
 #include "executor/nodeSort.h"
 #include "executor/nodeTee.h"
+#include "executor/nodeMaterial.h"
+#include "executor/nodeNestloop.h"
+#include "executor/nodeHashjoin.h"
+#include "executor/nodeHash.h"
+#include "executor/nodeAgg.h"
+#include "executor/nodeResult.h"
+#include "executor/nodeUnique.h"
+#include "executor/nodeMergejoin.h"
+#include "executor/nodeSubplan.h"
 #include "executor/execdebug.h"
 #include "optimizer/internal.h" /* for _TEMP_RELATION_ID_ */
 #include "access/genam.h"
@@ -45,7 +54,7 @@
 
 static Pointer
 ExecBeginScan(Relation relation, int nkeys, ScanKey skeys,
-			  bool isindex, ScanDirection dir, TimeQual time_range);
+			  bool isindex, ScanDirection dir);
 static Relation ExecOpenR(Oid relationOid, bool isindex);
 
 /* ----------------------------------------------------------------
@@ -70,7 +79,6 @@ ExecOpenScanR(Oid relOid,
 			  ScanKey skeys,
 			  bool isindex,
 			  ScanDirection dir,
-			  TimeQual timeRange,
 			  Relation *returnRelation, /* return */
 			  Pointer *returnScanDesc)	/* return */
 {
@@ -89,8 +97,7 @@ ExecOpenScanR(Oid relOid,
 							 nkeys,
 							 skeys,
 							 isindex,
-							 dir,
-							 timeRange);
+							 dir);
 
 	if (returnRelation != NULL)
 		*returnRelation = relation;
@@ -146,8 +153,7 @@ ExecBeginScan(Relation relation,
 			  int nkeys,
 			  ScanKey skeys,
 			  bool isindex,
-			  ScanDirection dir,
-			  TimeQual time_range)
+			  ScanDirection dir)
 {
 	Pointer		scanDesc;
 
@@ -172,7 +178,7 @@ ExecBeginScan(Relation relation,
 	{
 		scanDesc = (Pointer) heap_beginscan(relation,
 											ScanDirectionIsBackward(dir),
-											time_range,
+											false,
 											nkeys,
 											skeys);
 	}
@@ -290,35 +296,97 @@ ExecCloseR(Plan *node)
 void
 ExecReScan(Plan *node, ExprContext *exprCtxt, Plan *parent)
 {
+
+	if (node->chgParam != NULL) /* Wow! */
+	{
+		List	   *lst;
+
+		foreach(lst, node->initPlan)
+		{
+			Plan	   *splan = ((SubPlan *) lfirst(lst))->plan;
+
+			if (splan->extParam != NULL)		/* don't care about child
+												 * locParam */
+				SetChangedParamList(splan, node->chgParam);
+			if (splan->chgParam != NULL)
+				ExecReScanSetParamPlan((SubPlan *) lfirst(lst), node);
+		}
+		foreach(lst, node->subPlan)
+		{
+			Plan	   *splan = ((SubPlan *) lfirst(lst))->plan;
+
+			if (splan->extParam != NULL)
+				SetChangedParamList(splan, node->chgParam);
+		}
+		/* Well. Now set chgParam for left/right trees. */
+		if (node->lefttree != NULL)
+			SetChangedParamList(node->lefttree, node->chgParam);
+		if (node->righttree != NULL)
+			SetChangedParamList(node->righttree, node->chgParam);
+	}
+
 	switch (nodeTag(node))
 	{
-			case T_SeqScan:
+		case T_SeqScan:
 			ExecSeqReScan((SeqScan *) node, exprCtxt, parent);
-			return;
+			break;
 
 		case T_IndexScan:
 			ExecIndexReScan((IndexScan *) node, exprCtxt, parent);
-			return;
+			break;
 
 		case T_Material:
+			ExecMaterialReScan((Material *) node, exprCtxt, parent);
+			break;
 
-			/*
-			 * the first call to ExecReScan should have no effect because
-			 * everything is initialized properly already.	the following
-			 * calls will be handled by ExecSeqReScan() because the nodes
-			 * below the Material node have already been materialized into
-			 * a temp relation.
-			 */
-			return;
+		case T_NestLoop:
+			ExecReScanNestLoop((NestLoop *) node, exprCtxt, parent);
+			break;
 
+		case T_HashJoin:
+			ExecReScanHashJoin((HashJoin *) node, exprCtxt, parent);
+			break;
+
+		case T_Hash:
+			ExecReScanHash((Hash *) node, exprCtxt, parent);
+			break;
+
+		case T_Agg:
+			ExecReScanAgg((Agg *) node, exprCtxt, parent);
+			break;
+
+		case T_Result:
+			ExecReScanResult((Result *) node, exprCtxt, parent);
+			break;
+
+		case T_Unique:
+			ExecReScanUnique((Unique *) node, exprCtxt, parent);
+			break;
+
+		case T_Sort:
+			ExecReScanSort((Sort *) node, exprCtxt, parent);
+			break;
+
+		case T_MergeJoin:
+			ExecReScanMergeJoin((MergeJoin *) node, exprCtxt, parent);
+			break;
+
+/*
+ * Tee is never used
 		case T_Tee:
 			ExecTeeReScan((Tee *) node, exprCtxt, parent);
 			break;
-
+ */
 		default:
-			elog(WARN, "ExecReScan: not a seqscan or indexscan node.");
+			elog(ERROR, "ExecReScan: node type %u not supported", nodeTag(node));
 			return;
 	}
+
+	if (node->chgParam != NULL)
+	{
+		freeList(node->chgParam);
+		node->chgParam = NULL;
+	}
 }
 
 /* ----------------------------------------------------------------
@@ -368,7 +436,7 @@ ExecMarkPos(Plan *node)
 			break;
 
 		default:
-			/* elog(DEBUG, "ExecMarkPos: unsupported node type"); */
+			elog(DEBUG, "ExecMarkPos: node type %u not supported", nodeTag(node));
 			break;
 	}
 	return;
@@ -398,7 +466,7 @@ ExecRestrPos(Plan *node)
 			return;
 
 		default:
-			/* elog(DEBUG, "ExecRestrPos: node type not supported"); */
+			elog(DEBUG, "ExecRestrPos: node type %u not supported", nodeTag(node));
 			return;
 	}
 }
@@ -414,7 +482,7 @@ ExecRestrPos(Plan *node)
  *		  accessMtd -- access methods used to access the created relation.
  *		  relation	-- optional. Either an index to the range table or
  *					   negative number indicating a temporary relation.
- *					   A temporary relation is assume is this field is absent.
+ *					   A temporary relation is assume if this field is absent.
  * ----------------------------------------------------------------
  */
 
@@ -446,12 +514,10 @@ ExecCreatR(TupleDesc tupType,
 */
 
 		/*
-		 * heap_creatr creates a name if the argument to heap_creatr is
+		 * heap_create creates a name if the argument to heap_create is
 		 * '\0 '
 		 */
-		relDesc = heap_creatr("",
-							  DEFAULT_SMGR,
-							  tupType);
+		relDesc = heap_create("", tupType);
 	}
 	else
 	{
