@@ -26,7 +26,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execMain.c,v 1.26 1997/09/18 20:20:29 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execMain.c,v 1.45 1998/02/27 08:43:52 vadim Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -42,6 +42,7 @@
 #include "utils/palloc.h"
 #include "utils/acl.h"
 #include "utils/syscache.h"
+#include "utils/tqual.h"
 #include "parser/parsetree.h"	/* rt_fetch() */
 #include "storage/bufmgr.h"
 #include "storage/lmgr.h"
@@ -53,12 +54,12 @@
 #include "catalog/heap.h"
 #include "commands/trigger.h"
 
+void
+ExecCheckPerms(CmdType operation, int resultRelation, List *rangeTable,
+			   Query *parseTree);
 
 
 /* decls for local routines only used within this module */
-static void
-ExecCheckPerms(CmdType operation, int resultRelation, List *rangeTable,
-			   Query *parseTree);
 static TupleDesc
 InitPlan(CmdType operation, Query *parseTree,
 		 Plan *plan, EState *estate);
@@ -88,14 +89,14 @@ static int	queryLimit = ALL_TUPLES;
 #undef ALL_TUPLES
 #define ALL_TUPLES queryLimit
 
-#ifdef NOT_USED
+int			ExecutorLimit(int limit);
+
 int
 ExecutorLimit(int limit)
 {
 	return queryLimit = limit;
 }
 
-#endif
 #endif
 
 /* ----------------------------------------------------------------
@@ -117,6 +118,13 @@ ExecutorStart(QueryDesc *queryDesc, EState *estate)
 	/* sanity checks */
 	Assert(queryDesc != NULL);
 
+	if (queryDesc->plantree->nParamExec > 0)
+	{
+		estate->es_param_exec_vals = (ParamExecData *)
+			palloc(queryDesc->plantree->nParamExec * sizeof(ParamExecData));
+		memset(estate->es_param_exec_vals, 0, queryDesc->plantree->nParamExec * sizeof(ParamExecData));
+	}
+
 	result = InitPlan(queryDesc->operation,
 					  queryDesc->parsetree,
 					  queryDesc->plantree,
@@ -183,31 +191,6 @@ ExecutorRun(QueryDesc *queryDesc, EState *estate, int feature, int count)
 	estate->es_processed = 0;
 	estate->es_lastoid = InvalidOid;
 
-#if 0
-
-	/*
-	 * It doesn't work in common case (i.g. if function has a aggregate).
-	 * Now we store parameter values before ExecutorStart. - vadim
-	 * 01/22/97
-	 */
-#ifdef INDEXSCAN_PATCH
-
-	/*
-	 * If the plan is an index scan and some of the scan key are function
-	 * arguments rescan the indices after the parameter values have been
-	 * stored in the execution state.  DZ - 27-8-1996
-	 */
-	if ((nodeTag(plan) == T_IndexScan) &&
-		(((IndexScan *) plan)->indxstate->iss_RuntimeKeyInfo != NULL))
-	{
-		ExprContext *econtext;
-
-		econtext = ((IndexScan *) plan)->scan.scanstate->cstate.cs_ExprContext;
-		ExecIndexReScan((IndexScan *) plan, econtext, plan);
-	}
-#endif
-#endif
-
 	switch (feature)
 	{
 
@@ -290,14 +273,7 @@ ExecutorEnd(QueryDesc *queryDesc, EState *estate)
 	BufferRefCountRestore(estate->es_refcount);
 }
 
-/* ===============================================================
- * ===============================================================
-						 static routines follow
- * ===============================================================
- * ===============================================================
- */
-
-static void
+void
 ExecCheckPerms(CmdType operation,
 			   int resultRelation,
 			   List *rangeTable,
@@ -323,16 +299,28 @@ ExecCheckPerms(CmdType operation,
 	{
 		RangeTblEntry *rte = lfirst(lp);
 
+		if (rte->skipAcl)
+		{
+
+			/*
+			 * This happens if the access to this table is due to a view
+			 * query rewriting - the rewrite handler checked the
+			 * permissions against the view owner, so we just skip this
+			 * entry.
+			 */
+			continue;
+		}
+
 		relid = rte->relid;
 		htp = SearchSysCacheTuple(RELOID,
 								  ObjectIdGetDatum(relid),
 								  0, 0, 0);
 		if (!HeapTupleIsValid(htp))
-			elog(WARN, "ExecCheckPerms: bogus RT relid: %d",
+			elog(ERROR, "ExecCheckPerms: bogus RT relid: %d",
 				 relid);
-		strNcpy(rname.data,
+		StrNCpy(rname.data,
 				((Form_pg_class) GETSTRUCT(htp))->relname.data,
-				NAMEDATALEN - 1);
+				NAMEDATALEN);
 		if (i == resultRelation)
 		{						/* this is the result relation */
 			qvars = pull_varnos(parseTree->qual);
@@ -361,7 +349,7 @@ ExecCheckPerms(CmdType operation,
 					opstr = "write";
 					break;
 				default:
-					elog(WARN, "ExecCheckPerms: bogus operation %d",
+					elog(ERROR, "ExecCheckPerms: bogus operation %d",
 						 operation);
 			}
 		}
@@ -377,10 +365,17 @@ ExecCheckPerms(CmdType operation,
 	}
 	if (!ok)
 	{
-		elog(WARN, "%s: %s", rname.data, aclcheck_error_strings[aclcheck_result]);
+		elog(ERROR, "%s: %s", rname.data, aclcheck_error_strings[aclcheck_result]);
 	}
 }
 
+/* ===============================================================
+ * ===============================================================
+						 static routines follow
+ * ===============================================================
+ * ===============================================================
+ */
+
 
 /* ----------------------------------------------------------------
  *		InitPlan
@@ -447,7 +442,7 @@ InitPlan(CmdType operation, Query *parseTree, Plan *plan, EState *estate)
 		resultRelationDesc = heap_open(resultRelationOid);
 
 		if (resultRelationDesc->rd_rel->relkind == RELKIND_SEQUENCE)
-			elog(WARN, "You can't change sequence relation %s",
+			elog(ERROR, "You can't change sequence relation %s",
 				 resultRelationDesc->rd_rel->relname.data);
 
 		/*
@@ -547,7 +542,6 @@ InitPlan(CmdType operation, Query *parseTree, Plan *plan, EState *estate)
 	if (operation == CMD_SELECT)
 	{
 		char	   *intoName;
-		char		archiveMode;
 		Oid			intoRelationId;
 		TupleDesc	tupdesc;
 
@@ -561,31 +555,17 @@ InitPlan(CmdType operation, Query *parseTree, Plan *plan, EState *estate)
 			{
 				/* ----------------
 				 *	create the "into" relation
-				 *
-				 *	note: there is currently no way for the user to
-				 *		  specify the desired archive mode of the
-				 *		  "into" relation...
 				 * ----------------
 				 */
 				intoName = parseTree->into;
-				archiveMode = 'n';
 
 				/*
 				 * have to copy tupType to get rid of constraints
 				 */
 				tupdesc = CreateTupleDescCopy(tupType);
 
-				/* fixup to prevent zero-length columns in create */
-				setVarAttrLenForCreateTable(tupdesc, targetList, rangeTable);
+				intoRelationId = heap_create_with_catalog(intoName, tupdesc);
 
-				intoRelationId = heap_create(intoName,
-											 intoName,	/* not used */
-											 archiveMode,
-											 DEFAULT_SMGR,
-											 tupdesc);
-#ifdef NOT_USED					/* it's copy ... */
-				resetVarAttrLenForCreateTable(tupdesc);
-#endif
 				FreeTupleDesc(tupdesc);
 
 				/* ----------------
@@ -788,10 +768,10 @@ ExecutePlan(EState *estate,
 										  "ctid",
 										  &datum,
 										  &isNull))
-					elog(WARN, "ExecutePlan: NO (junk) `ctid' was found!");
+					elog(ERROR, "ExecutePlan: NO (junk) `ctid' was found!");
 
 				if (isNull)
-					elog(WARN, "ExecutePlan: (junk) `ctid' is NULL!");
+					elog(ERROR, "ExecutePlan: (junk) `ctid' is NULL!");
 
 				tupleid = (ItemPointer) DatumGetPointer(datum);
 				tuple_ctid = *tupleid;	/* make sure we don't free the
@@ -1245,6 +1225,7 @@ ExecReplace(TupleTableSlot *slot,
 		ExecARUpdateTriggers(resultRelationDesc, tupleid, tuple);
 }
 
+#if 0
 static HeapTuple
 ExecAttrDefault(Relation rel, HeapTuple tuple)
 {
@@ -1267,6 +1248,7 @@ ExecAttrDefault(Relation rel, HeapTuple tuple)
 	econtext->ecxt_relation = NULL;		/* relation */
 	econtext->ecxt_relid = 0;	/* relid */
 	econtext->ecxt_param_list_info = NULL;		/* param list info */
+	econtext->ecxt_param_exec_vals = NULL;		/* exec param values */
 	econtext->ecxt_range_table = NULL;	/* range table */
 	for (i = 0; i < ndef; i++)
 	{
@@ -1310,6 +1292,8 @@ ExecAttrDefault(Relation rel, HeapTuple tuple)
 
 }
 
+#endif
+
 static char *
 ExecRelCheck(Relation rel, HeapTuple tuple)
 {
@@ -1330,13 +1314,10 @@ ExecRelCheck(Relation rel, HeapTuple tuple)
 	slot->ttc_buffer = InvalidBuffer;
 	slot->ttc_whichplan = -1;
 	rte->relname = nameout(&(rel->rd_rel->relname));
-	rte->timeRange = NULL;
 	rte->refname = rte->relname;
 	rte->relid = rel->rd_id;
 	rte->inh = false;
-	rte->archive = false;
 	rte->inFromCl = true;
-	rte->timeQual = NULL;
 	rtlist = lcons(rte, NIL);
 	econtext->ecxt_scantuple = slot;	/* scan tuple slot */
 	econtext->ecxt_innertuple = NULL;	/* inner tuple slot */
@@ -1344,6 +1325,7 @@ ExecRelCheck(Relation rel, HeapTuple tuple)
 	econtext->ecxt_relation = rel;		/* relation */
 	econtext->ecxt_relid = 0;	/* relid */
 	econtext->ecxt_param_list_info = NULL;		/* param list info */
+	econtext->ecxt_param_exec_vals = NULL;		/* exec param values */
 	econtext->ecxt_range_table = rtlist;		/* range table */
 
 	for (i = 0; i < ncheck; i++)
@@ -1375,8 +1357,10 @@ ExecConstraints(char *caller, Relation rel, HeapTuple tuple)
 
 	Assert(rel->rd_att->constr);
 
+#if 0
 	if (rel->rd_att->constr->num_defval > 0)
 		newtuple = tuple = ExecAttrDefault(rel, tuple);
+#endif
 
 	if (rel->rd_att->constr->has_not_null)
 	{
@@ -1385,7 +1369,7 @@ ExecConstraints(char *caller, Relation rel, HeapTuple tuple)
 		for (attrChk = 1; attrChk <= rel->rd_att->natts; attrChk++)
 		{
 			if (rel->rd_att->attrs[attrChk - 1]->attnotnull && heap_attisnull(tuple, attrChk))
-				elog(WARN, "%s: Fail to add null value in not null attribute %s",
+				elog(ERROR, "%s: Fail to add null value in not null attribute %s",
 				  caller, rel->rd_att->attrs[attrChk - 1]->attname.data);
 		}
 	}
@@ -1395,7 +1379,7 @@ ExecConstraints(char *caller, Relation rel, HeapTuple tuple)
 		char	   *failed;
 
 		if ((failed = ExecRelCheck(rel, tuple)) != NULL)
-			elog(WARN, "%s: rejected due to CHECK constraint %s", caller, failed);
+			elog(ERROR, "%s: rejected due to CHECK constraint %s", caller, failed);
 	}
 
 	return (newtuple);
