@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execQual.c,v 1.19 1997/09/27 14:37:10 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execQual.c,v 1.29 1998/04/07 18:10:46 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -46,6 +46,7 @@
 #include "executor/execdebug.h"
 #include "executor/execFlatten.h"
 #include "executor/functions.h"
+#include "executor/nodeSubplan.h"
 #include "access/heapam.h"
 #include "utils/memutils.h"
 #include "utils/builtins.h"
@@ -153,7 +154,7 @@ ExecEvalArrayRef(ArrayRef *arrayRef,
 				return (Datum) NULL;
 		}
 		if (i != j)
-			elog(WARN,
+			elog(ERROR,
 				 "ExecEvalArrayRef: upper and lower indices mismatch");
 		lIndex = lower.indx;
 	}
@@ -202,8 +203,7 @@ ExecEvalArrayRef(ArrayRef *arrayRef,
 static Datum
 ExecEvalAggreg(Aggreg *agg, ExprContext *econtext, bool *isNull)
 {
-
-	*isNull = econtext->ecxt_nulls[agg->aggno];
+	*isNull = econtext->ecxt_nulls[agg->aggno];	
 	return econtext->ecxt_values[agg->aggno];
 }
 
@@ -305,11 +305,11 @@ ExecEvalVar(Var *variable, ExprContext *econtext, bool *isNull)
 		return (Datum) tempSlot;
 	}
 
-	result = heap_getattr(heapTuple, /* tuple containing attribute */
-						  buffer,	/* buffer associated with tuple */
-						  attnum,	/* attribute number of desired attribute */
-					 	  tuple_type,/* tuple descriptor of tuple */
-						  isNull);	/* return: is attribute null? */
+	result = heap_getattr(heapTuple,	/* tuple containing attribute */
+						  attnum,		/* attribute number of desired
+										 * attribute */
+						  tuple_type,	/* tuple descriptor of tuple */
+						  isNull);		/* return: is attribute null? */
 
 	/* ----------------
 	 *	return null if att is null
@@ -380,14 +380,23 @@ ExecEvalParam(Param *expression, ExprContext *econtext, bool *isNull)
 {
 
 	char	   *thisParameterName;
-	int			thisParameterKind;
-	AttrNumber	thisParameterId;
+	int			thisParameterKind = expression->paramkind;
+	AttrNumber	thisParameterId = expression->paramid;
 	int			matchFound;
 	ParamListInfo paramList;
 
+	if (thisParameterKind == PARAM_EXEC)
+	{
+		ParamExecData *prm = &(econtext->ecxt_param_exec_vals[thisParameterId]);
+
+		if (prm->execPlan != NULL)
+			ExecSetParamPlan(prm->execPlan);
+		Assert(prm->execPlan == NULL);
+		*isNull = prm->isnull;
+		return (prm->value);
+	}
+
 	thisParameterName = expression->paramname;
-	thisParameterKind = expression->paramkind;
-	thisParameterId = expression->paramid;
 	paramList = econtext->ecxt_param_list_info;
 
 	*isNull = false;
@@ -434,7 +443,7 @@ ExecEvalParam(Param *expression, ExprContext *econtext, bool *isNull)
 						 */
 						if (strcmp(paramList->name, thisParameterName) != 0)
 						{
-							elog(WARN,
+							elog(ERROR,
 								 "ExecEvalParam: new/old params with same id & diff names");
 						}
 					}
@@ -444,7 +453,7 @@ ExecEvalParam(Param *expression, ExprContext *econtext, bool *isNull)
 					/*
 					 * oops! this is not supposed to happen!
 					 */
-					elog(WARN, "ExecEvalParam: invalid paramkind %d",
+					elog(ERROR, "ExecEvalParam: invalid paramkind %d",
 						 thisParameterKind);
 			}
 			if (!matchFound)
@@ -461,7 +470,7 @@ ExecEvalParam(Param *expression, ExprContext *econtext, bool *isNull)
 		 * ooops! we couldn't find this parameter in the parameter list.
 		 * Signal an error
 		 */
-		elog(WARN, "ExecEvalParam: Unknown value for parameter %s",
+		elog(ERROR, "ExecEvalParam: Unknown value for parameter %s",
 			 thisParameterName);
 	}
 
@@ -506,10 +515,7 @@ ExecEvalParam(Param *expression, ExprContext *econtext, bool *isNull)
  *		to use this.  Ex: overpaid(EMP) might call GetAttributeByNum().
  * ----------------
  */
-/*
- * This gets called from external functions, so don't make it static
- * or remove it
- */
+/* static but gets called from external functions */
 char *
 GetAttributeByNum(TupleTableSlot *slot,
 				  AttrNumber attrno,
@@ -518,13 +524,13 @@ GetAttributeByNum(TupleTableSlot *slot,
 	Datum		retval;
 
 	if (!AttributeNumberIsValid(attrno))
-		elog(WARN, "GetAttributeByNum: Invalid attribute number");
+		elog(ERROR, "GetAttributeByNum: Invalid attribute number");
 
 	if (!AttrNumberIsForUserDefinedAttr(attrno))
-		elog(WARN, "GetAttributeByNum: cannot access system attributes here");
+		elog(ERROR, "GetAttributeByNum: cannot access system attributes here");
 
 	if (isNull == (bool *) NULL)
-		elog(WARN, "GetAttributeByNum: a NULL isNull flag was passed");
+		elog(ERROR, "GetAttributeByNum: a NULL isNull flag was passed");
 
 	if (TupIsNull(slot))
 	{
@@ -533,7 +539,6 @@ GetAttributeByNum(TupleTableSlot *slot,
 	}
 
 	retval = heap_getattr(slot->val,
-						  slot->ttc_buffer,
 						  attrno,
 						  slot->ttc_tupleDescriptor,
 						  isNull);
@@ -544,7 +549,7 @@ GetAttributeByNum(TupleTableSlot *slot,
 
 /* XXX char16 name for catalogs */
 #ifdef NOT_USED
-char	   *
+char *
 att_by_num(TupleTableSlot *slot,
 		   AttrNumber attrno,
 		   bool *isNull)
@@ -554,7 +559,7 @@ att_by_num(TupleTableSlot *slot,
 
 #endif
 
-char	   *
+char *
 GetAttributeByName(TupleTableSlot *slot, char *attname, bool *isNull)
 {
 	AttrNumber	attrno;
@@ -565,10 +570,10 @@ GetAttributeByName(TupleTableSlot *slot, char *attname, bool *isNull)
 	int			i;
 
 	if (attname == NULL)
-		elog(WARN, "GetAttributeByName: Invalid attribute name");
+		elog(ERROR, "GetAttributeByName: Invalid attribute name");
 
 	if (isNull == (bool *) NULL)
-		elog(WARN, "GetAttributeByName: a NULL isNull flag was passed");
+		elog(ERROR, "GetAttributeByName: a NULL isNull flag was passed");
 
 	if (TupIsNull(slot))
 	{
@@ -592,10 +597,9 @@ GetAttributeByName(TupleTableSlot *slot, char *attname, bool *isNull)
 	}
 
 	if (attrno == InvalidAttrNumber)
-		elog(WARN, "GetAttributeByName: attribute %s not found", attname);
+		elog(ERROR, "GetAttributeByName: attribute %s not found", attname);
 
 	retval = heap_getattr(slot->val,
-						  slot->ttc_buffer,
 						  attrno,
 						  tupdesc,
 						  isNull);
@@ -606,7 +610,7 @@ GetAttributeByName(TupleTableSlot *slot, char *attname, bool *isNull)
 
 /* XXX char16 name for catalogs */
 #ifdef NOT_USED
-char	   *
+char *
 att_by_name(TupleTableSlot *slot, char *attname, bool *isNull)
 {
 	return (GetAttributeByName(slot, attname, isNull));
@@ -643,6 +647,8 @@ ExecEvalFuncArgs(FunctionCachePtr fcache,
 						 econtext,
 						 &argIsNull,
 						 argIsDone);
+
+
 		if (!(*argIsDone))
 		{
 			Assert(i == 0);
@@ -704,7 +710,7 @@ ExecMakeFunctionResult(Node *node,
 		bool		argDone;
 
 		if (fcache->nargs > MAXFMGRARGS)
-			elog(WARN, "ExecMakeFunctionResult: too many arguments");
+			elog(ERROR, "ExecMakeFunctionResult: too many arguments");
 
 		/*
 		 * If the setArg in the fcache is set we have an argument
@@ -834,8 +840,7 @@ ExecMakeFunctionResult(Node *node,
 			if (fcache->nullVect[i] == true)
 				*isNull = true;
 
-		return ((Datum) fmgr_c(fcache->func, fcache->foid, fcache->nargs,
-							   (FmgrValues *) argv, isNull));
+		return ((Datum) fmgr_c(&fcache->func, (FmgrValues *) argv, isNull));
 	}
 }
 
@@ -1047,19 +1052,21 @@ ExecEvalOr(Expr *orExpr, ExprContext *econtext, bool *isNull)
 		if (*isNull)
 		{
 			IsNull = *isNull;
+
 			/*
-			 * Many functions don't (or can't!) check is an argument
-			 * NULL or NOT_NULL and may return TRUE (1) with *isNull TRUE
-			 * (an_int4_column <> 1: int4ne returns TRUE for NULLs).
-			 * Not having time to fix function manager I want to fix
-			 * OR: if we had 'x <> 1 OR x isnull' then TRUE, TRUE were
-			 * returned by 'x <> 1' for NULL ... but ExecQualClause say
-			 * that qualification *fails* if isnull is TRUE for all values
-			 * returned by ExecEvalExpr. So, force this rule here: if isnull 
-			 * is TRUE then clause failed. Note: nullvalue() & nonnullvalue() 
-			 * always set isnull to FALSE for NULLs.	- vadim 09/22/97
+			 * Many functions don't (or can't!) check is an argument NULL
+			 * or NOT_NULL and may return TRUE (1) with *isNull TRUE
+			 * (an_int4_column <> 1: int4ne returns TRUE for NULLs). Not
+			 * having time to fix function manager I want to fix OR: if we
+			 * had 'x <> 1 OR x isnull' then TRUE, TRUE were returned by
+			 * 'x <> 1' for NULL ... but ExecQualClause say that
+			 * qualification *fails* if isnull is TRUE for all values
+			 * returned by ExecEvalExpr. So, force this rule here: if
+			 * isnull is TRUE then clause failed. Note: nullvalue() &
+			 * nonnullvalue() always set isnull to FALSE for NULLs.    -
+			 * vadim 09/22/97
 			 */
-		    const_value = 0;
+			const_value = 0;
 		}
 
 		/* ----------------
@@ -1239,14 +1246,17 @@ ExecEvalExpr(Node *expression,
 					case NOT_EXPR:
 						retDatum = (Datum) ExecEvalNot(expr, econtext, isNull);
 						break;
+					case SUBPLAN_EXPR:
+						retDatum = (Datum) ExecSubPlan((SubPlan *) expr->oper, expr->args, econtext);
+						break;
 					default:
-						elog(WARN, "ExecEvalExpr: unknown expression type");
+						elog(ERROR, "ExecEvalExpr: unknown expression type %d", expr->opType);
 						break;
 				}
 				break;
 			}
 		default:
-			elog(WARN, "ExecEvalExpr: unknown expression type");
+			elog(ERROR, "ExecEvalExpr: unknown expression type %d", nodeTag(expression));
 			break;
 	}
 
@@ -1347,8 +1357,11 @@ ExecQual(List *qual, ExprContext *econtext)
 	 * ----------------
 	 */
 	result = false;
+
 	foreach(clause, qual)
 	{
+	 
+	  
 		result = ExecQualClause((Node *) lfirst(clause), econtext);
 		if (result == true)
 			break;
