@@ -14,7 +14,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execTuples.c,v 1.10 1997/09/18 20:20:32 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execTuples.c,v 1.17 1998/02/26 04:31:14 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -125,11 +125,11 @@
 #undef ExecStoreTuple
 
 #include "access/tupdesc.h"
+#include "catalog/pg_type.h"
+#include "parser/parse_type.h"
+#include "storage/bufmgr.h"
 #include "utils/palloc.h"
 #include "utils/lsyscache.h"
-#include "storage/bufmgr.h"
-#include "parser/catalog_utils.h"
-#include "catalog/pg_type.h"
 
 static TupleTableSlot *NodeGetResultTupleSlot(Plan *node);
 
@@ -315,7 +315,7 @@ ExecAllocTableSlot(TupleTable table)
 		 * table->size =  newsize;
 		 */
 		elog(NOTICE, "Plan requires more slots than are available");
-		elog(WARN, "send mail to your local executor guru to fix this");
+		elog(ERROR, "send mail to your local executor guru to fix this");
 	}
 
 	/* ----------------
@@ -859,7 +859,7 @@ NodeGetResultTupleSlot(Plan *node)
 			 *	  should never get here
 			 * ----------------
 			 */
-			elog(WARN, "NodeGetResultTupleSlot: node not yet supported: %d ",
+			elog(ERROR, "NodeGetResultTupleSlot: node not yet supported: %d ",
 				 nodeTag(node));
 
 			return NULL;
@@ -888,8 +888,6 @@ NodeGetResultTupleSlot(Plan *node)
  *		the node's inner or outer subplan easily without having
  *		to inspect the subplan.. -cim 10/16/89
  *
- *		Assume that for existential nodes, we get the targetlist out
- *		of the right node's targetlist
  * ----------------------------------------------------------------
  */
 
@@ -988,7 +986,8 @@ ExecTypeFromTL(List *targetList)
 							   resdom->resno,
 							   resdom->resname,
 			/* fix for SELECT NULL ... */
-						  get_id_typname(restype ? restype : UNKNOWNOID),
+							   (restype ? restype : UNKNOWNOID),
+							   resdom->restypmod,
 							   0,
 							   false);
 
@@ -1021,7 +1020,8 @@ ExecTypeFromTL(List *targetList)
 			TupleDescInitEntry(typeInfo,
 							   fjRes->resno,
 							   fjRes->resname,
-							   get_id_typname(restype),
+							   restype,
+							   fjRes->restypmod,
 							   0,
 							   false);
 /*
@@ -1044,7 +1044,8 @@ ExecTypeFromTL(List *targetList)
 				TupleDescInitEntry(typeInfo,
 								   fjRes->resno,
 								   fjRes->resname,
-								   get_id_typname(restype),
+								   restype,
+								   fjRes->restypmod,
 								   0,
 								   false);
 
