@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execUtils.c,v 1.19 1997/09/18 20:20:35 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execUtils.c,v 1.30 1998/02/26 04:31:15 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -51,6 +51,7 @@
 #include "optimizer/clauses.h"
 #include "utils/palloc.h"
 #include "utils/mcxt.h"
+#include "utils/lsyscache.h"
 #include "commands/command.h"
 #include "catalog/index.h"
 #include "catalog/catname.h"
@@ -179,11 +180,6 @@ void
 ExecAssignExprContext(EState *estate, CommonState *commonstate)
 {
 	ExprContext *econtext;
-	ParamListInfo paraminfo;
-	List	   *rangeTable;
-
-	paraminfo = estate->es_param_list_info;
-	rangeTable = estate->es_range_table;
 
 	econtext = makeNode(ExprContext);
 	econtext->ecxt_scantuple = NULL;	/* scan tuple slot */
@@ -191,8 +187,9 @@ ExecAssignExprContext(EState *estate, CommonState *commonstate)
 	econtext->ecxt_outertuple = NULL;	/* outer tuple slot */
 	econtext->ecxt_relation = NULL;		/* relation */
 	econtext->ecxt_relid = 0;	/* relid */
-	econtext->ecxt_param_list_info = paraminfo; /* param list info */
-	econtext->ecxt_range_table = rangeTable;	/* range table */
+	econtext->ecxt_param_list_info = estate->es_param_list_info;
+	econtext->ecxt_param_exec_vals = estate->es_param_exec_vals;
+	econtext->ecxt_range_table = estate->es_range_table;		/* range table */
 
 	commonstate->cs_ExprContext = econtext;
 }
@@ -499,7 +496,7 @@ ExecSetTypeInfo(int index,
 	 */
 	att = typeInfo[index];
 	if (att == NULL)
-		elog(WARN, "ExecSetTypeInfo: trying to assign through NULL");
+		elog(ERROR, "ExecSetTypeInfo: trying to assign through NULL");
 
 	/* ----------------
 	 *	assign values to the tuple descriptor, being careful not
@@ -514,7 +511,7 @@ ExecSetTypeInfo(int index,
 	att->attrelid = 0;			/* dummy value */
 
 	if (attName != (char *) NULL)
-		strNcpy(att->attname.data, attName, NAMEDATALEN - 1);
+		StrNCpy(att->attname.data, attName, NAMEDATALEN);
 	else
 		MemSet(att->attname.data, 0, NAMEDATALEN);
 
@@ -530,6 +527,7 @@ ExecSetTypeInfo(int index,
 	att->attproc = 0;			/* dummy value */
 	att->attnelems = 0;			/* dummy value */
 	att->attcacheoff = -1;
+	att->atttypmod = -1;
 	att->attisset = false;
 	att->attalign = attalign;
 }
@@ -756,7 +754,7 @@ ExecOpenIndices(Oid resultRelationOid,
 	 */
 	indexSd = heap_beginscan(indexRd,	/* scan desc */
 							 false,		/* scan backward flag */
-							 NowTimeQual,		/* time qual */
+							 false,		/* see self */
 							 1, /* number scan keys */
 							 &key);		/* scan keys */
 
@@ -1178,75 +1176,24 @@ ExecInsertIndexTuples(TupleTableSlot *slot,
 		pfree(econtext);
 }
 
-/* ----------------------------------------------------------------
- * setVarAttrLenForCreateTable -
- *	  called when we do a SELECT * INTO TABLE tab
- *	  needed for attributes that have a defined length, like bpchar and
- *	  varchar
- * ----------------------------------------------------------------
- */
 void
-setVarAttrLenForCreateTable(TupleDesc tupType, List *targetList,
-							List *rangeTable)
+SetChangedParamList(Plan *node, List *newchg)
 {
-	List	   *tl;
-	TargetEntry *tle;
-	Node	   *expr;
-	int			varno;
+	List	   *nl;
 
-	tl = targetList;
-
-	for (varno = 0; varno < tupType->natts; varno++)
+	foreach(nl, newchg)
 	{
-		tle = lfirst(tl);
+		int			paramId = lfirsti(nl);
 
-		if (tupType->attrs[varno]->atttypid == BPCHAROID ||
-			tupType->attrs[varno]->atttypid == VARCHAROID)
-		{
-			expr = tle->expr;
-			if (expr && IsA(expr, Var))
-			{
-				Var		   *var;
-				RangeTblEntry *rtentry;
-				Relation	rd;
-
-				var = (Var *) expr;
-				rtentry = rt_fetch(var->varnoold, rangeTable);
-				rd = heap_open(rtentry->relid);
-				/* set length to that defined in relation */
-				tupType->attrs[varno]->attlen =
-					(*rd->rd_att->attrs[var->varoattno - 1]).attlen;
-				heap_close(rd);
-			}
-			else
-				elog(WARN, "setVarAttrLenForCreateTable: can't get length for variable-length field");
-		}
-		tl = lnext(tl);
+		/* if this node doesn't depend on a param ... */
+		if (!intMember(paramId, node->extParam) &&
+			!intMember(paramId, node->locParam))
+			continue;
+		/* if this param is already in list of changed ones ... */
+		if (intMember(paramId, node->chgParam))
+			continue;
+		/* else - add this param to the list */
+		node->chgParam = lappendi(node->chgParam, paramId);
 	}
+
 }
-
-
-#ifdef NOT_USED					/* look at execMain.c */
-/* ----------------------------------------------------------------
- * resetVarAttrLenForCreateTable -
- *	  called when we do a SELECT * INTO TABLE tab
- *	  needed for attributes that have a defined length, like bpchar and
- *	  varchar
- *	  resets length to -1 for those types
- * ----------------------------------------------------------------
- */
-void
-resetVarAttrLenForCreateTable(TupleDesc tupType)
-{
-	int			varno;
-
-	for (varno = 0; varno < tupType->natts; varno++)
-	{
-		if (tupType->attrs[varno]->atttypid == BPCHAROID ||
-			tupType->attrs[varno]->atttypid == VARCHAROID)
-			/* set length to original -1 */
-			tupType->attrs[varno]->attlen = -1;
-	}
-}
-
-#endif
