@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/functions.c,v 1.12 1997/09/18 20:20:37 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/functions.c,v 1.16 1998/02/13 03:26:44 vadim Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -21,7 +21,6 @@
 #include "nodes/plannodes.h"
 
 #include "catalog/pg_proc.h"
-#include "parser/parse_query.h"
 #include "tcop/pquery.h"
 #include "tcop/tcopprot.h"
 #include "tcop/utility.h"
@@ -83,7 +82,7 @@ ProjectAttribute(TupleDesc TD,
 	AttrNumber	attrno = attrVar->varattno;
 
 
-	val = heap_getattr(tup, InvalidBuffer, attrno, TD, isnullP);
+	val = heap_getattr(tup, attrno, TD, isnullP);
 	if (*isnullP)
 		return (Datum) NULL;
 
@@ -114,7 +113,7 @@ init_execution_state(FunctionCachePtr fcache,
 
 
 	planTree_list = (List *)
-		pg_plan(fcache->src, fcache->argOidVect, nargs, &queryTree_list, None);
+		pg_parse_and_plan(fcache->src, fcache->argOidVect, nargs, &queryTree_list, None);
 
 	for (i = 0; i < queryTree_list->len; i++)
 	{
@@ -304,8 +303,6 @@ postquel_execute(execution_state *es,
 	TupleTableSlot *slot;
 	Datum		value;
 
-#ifdef INDEXSCAN_PATCH
-
 	/*
 	 * It's more right place to do it (before
 	 * postquel_start->ExecutorStart). Now
@@ -314,17 +311,12 @@ postquel_execute(execution_state *es,
 	 */
 	if (fcache->nargs > 0)
 		postquel_sub_params(es, fcache->nargs, args, fcache->nullVect);
-#endif
 
 	if (es->status == F_EXEC_START)
 	{
 		postquel_start(es);
 		es->status = F_EXEC_RUN;
 	}
-#ifndef INDEXSCAN_PATCH
-	if (fcache->nargs > 0)
-		postquel_sub_params(es, fcache->nargs, args, fcache->nullVect);
-#endif
 
 	slot = postquel_getnext(es);
 
