@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeHashjoin.c,v 1.8 1997/09/08 21:43:12 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeHashjoin.c,v 1.11 1998/02/26 04:31:26 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,21 +17,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-
-
 #include "postgres.h"
-#include "storage/bufmgr.h"		/* for BLCKSZ */
-#include "storage/fd.h"			/* for SEEK_ */
-#include "executor/executor.h"
+
 #include "executor/execdebug.h"
+#include "executor/executor.h"
 #include "executor/nodeHash.h"
 #include "executor/nodeHashjoin.h"
-
 #include "optimizer/clauses.h"	/* for get_leftop */
 
-
-#include "utils/palloc.h"
-
 static TupleTableSlot *
 			ExecHashJoinOuterGetTuple(Plan *node, Plan *parent, HashJoinState *hjstate);
 
@@ -161,6 +154,9 @@ ExecHashJoin(HashJoin *node)
 		curbatch = 0;
 		node->hashdone = true;
 	}
+	else if (hashtable == NULL)
+		return (NULL);
+
 	nbatch = hashtable->nbatch;
 	outerbatches = hjstate->hj_OuterBatches;
 	if (nbatch > 0 && outerbatches == NULL)
@@ -813,7 +809,7 @@ ExecHashJoinGetBatch(int bucketno, HashJoinTable hashtable, int nbatch)
  * ----------------------------------------------------------------
  */
 
-char	   *
+char *
 ExecHashJoinSaveTuple(HeapTuple heapTuple,
 					  char *buffer,
 					  File file,
@@ -848,3 +844,45 @@ ExecHashJoinSaveTuple(HeapTuple heapTuple,
 
 	return position;
 }
+
+void
+ExecReScanHashJoin(HashJoin *node, ExprContext *exprCtxt, Plan *parent)
+{
+	HashJoinState *hjstate = node->hashjoinstate;
+
+	if (!node->hashdone)
+		return;
+
+	node->hashdone = false;
+
+	/*
+	 * Unfortunately, currently we have to destroy hashtable in all
+	 * cases...
+	 */
+	if (hjstate->hj_HashTable)
+	{
+		ExecHashTableDestroy(hjstate->hj_HashTable);
+		hjstate->hj_HashTable = NULL;
+	}
+	hjstate->hj_CurBucket = (HashBucket) NULL;
+	hjstate->hj_CurTuple = (HeapTuple) NULL;
+	hjstate->hj_CurOTuple = (OverflowTuple) NULL;
+	hjstate->hj_InnerHashKey = (Var *) NULL;
+	hjstate->hj_OuterBatches = (File *) NULL;
+	hjstate->hj_InnerBatches = (File *) NULL;
+	hjstate->hj_OuterReadPos = (char *) NULL;
+	hjstate->hj_OuterReadBlk = (int) 0;
+
+	hjstate->jstate.cs_OuterTupleSlot = (TupleTableSlot *) NULL;
+	hjstate->jstate.cs_TupFromTlist = (bool) false;
+
+	/*
+	 * if chgParam of subnodes is not null then plans will be re-scanned
+	 * by first ExecProcNode.
+	 */
+	if (((Plan *) node)->lefttree->chgParam == NULL)
+		ExecReScan(((Plan *) node)->lefttree, exprCtxt, (Plan *) node);
+	if (((Plan *) node)->righttree->chgParam == NULL)
+		ExecReScan(((Plan *) node)->righttree, exprCtxt, (Plan *) node);
+
+}
