@@ -15,7 +15,7 @@
  *		ExecEndTee
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeTee.c,v 1.9 1997/09/08 21:43:20 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeTee.c,v 1.16 1998/02/26 04:31:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -167,11 +167,8 @@ ExecInitTee(Tee *node, EState *currentEstate, Plan *parent)
 		if (RelationIsValid(r))
 			bufferRel = heap_openr(teeState->tee_bufferRelname);
 		else
-			bufferRel = heap_open(heap_create(teeState->tee_bufferRelname,
-											   /* FIX */ NULL,
-											  'n',
-											  DEFAULT_SMGR,
-											  tupType));
+			bufferRel = heap_open(
+								  heap_create_with_catalog(teeState->tee_bufferRelname, tupType));
 	}
 	else
 	{
@@ -179,11 +176,8 @@ ExecInitTee(Tee *node, EState *currentEstate, Plan *parent)
 				"ttemp_%d",		/* 'ttemp' for 'tee' temporary */
 				newoid());
 /*		bufferRel = ExecCreatR(len, tupType, _TEMP_RELATION_ID); */
-		bufferRel = heap_open(heap_create(teeState->tee_bufferRelname,
-										  NULL, /* XXX */
-										  'n',
-										  DEFAULT_SMGR,
-										  tupType));
+		bufferRel = heap_open(
+		 heap_create_with_catalog(teeState->tee_bufferRelname, tupType));
 	}
 
 	teeState->tee_bufferRel = bufferRel;
@@ -252,7 +246,7 @@ initTeeScanDescs(Tee *node)
 	{
 		teeState->tee_leftScanDesc = heap_beginscan(bufferRel,
 											ScanDirectionIsBackward(dir),
-													NowTimeQual,		/* time qual */
+													false,		/* seeself */
 													0,	/* num scan keys */
 													NULL		/* scan keys */
 			);
@@ -261,7 +255,7 @@ initTeeScanDescs(Tee *node)
 	{
 		teeState->tee_rightScanDesc = heap_beginscan(bufferRel,
 											ScanDirectionIsBackward(dir),
-													 NowTimeQual,		/* time qual */
+													 false,		/* seeself */
 													 0, /* num scan keys */
 													 NULL		/* scan keys */
 			);
@@ -335,7 +329,7 @@ ExecTee(Tee *node, Plan *parent)
 	}
 	else
 	{
-		elog(WARN, "A Tee node can only be executed from its left or right parent\n");
+		elog(ERROR, "A Tee node can only be executed from its left or right parent\n");
 		return NULL;
 	}
 
@@ -525,7 +519,7 @@ ExecEndTee(Tee *node, Plan *parent)
 		bufferRel = teeState->tee_bufferRel;
 		if (bufferRel)
 		{
-			heap_destroyr(bufferRel);
+			heap_destroy(bufferRel);
 			teeState->tee_bufferRel = NULL;
 			if (teeState->tee_mcxt)
 			{
