@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeUnique.c,v 1.11 1997/09/12 04:07:44 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeUnique.c,v 1.17 1998/02/26 04:31:34 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -187,26 +187,35 @@ ExecUnique(Unique *node)
 			char	   *val1,
 					   *val2;
 
-			attr1 = heap_getattr(slot->val, InvalidBuffer,
+			attr1 = heap_getattr(slot->val,
 								 uniqueAttrNum, tupDesc, &isNull1);
-			attr2 = heap_getattr(resultTupleSlot->val, InvalidBuffer,
+			attr2 = heap_getattr(resultTupleSlot->val,
 								 uniqueAttrNum, tupDesc, &isNull2);
 
 			if (isNull1 == isNull2)
 			{
 				if (isNull1)	/* both are null, they are equal */
 					continue;
-				val1 = fmgr(typoutput, attr1, gettypelem(tupDesc->attrs[uniqueAttrNum - 1]->atttypid));
-				val2 = fmgr(typoutput, attr2, gettypelem(tupDesc->attrs[uniqueAttrNum - 1]->atttypid));
+				val1 = fmgr(typoutput, attr1,
+				 gettypelem(tupDesc->attrs[uniqueAttrNum - 1]->atttypid),
+							tupDesc->attrs[uniqueAttrNum - 1]->atttypmod);
+				val2 = fmgr(typoutput, attr2,
+				 gettypelem(tupDesc->attrs[uniqueAttrNum - 1]->atttypid),
+							tupDesc->attrs[uniqueAttrNum - 1]->atttypmod);
 
 				/*
 				 * now, val1 and val2 are ascii representations so we can
 				 * use strcmp for comparison
 				 */
 				if (strcmp(val1, val2) == 0)	/* they are equal */
+				{
+					pfree(val1);
+					pfree(val2);
 					continue;
-				else
-					break;
+				}
+				pfree(val1);
+				pfree(val2);
+				break;
 			}
 			else
 /* one is null and the other isn't, they aren't equal */
@@ -346,3 +355,20 @@ ExecEndUnique(Unique *node)
 	ExecEndNode(outerPlan((Plan *) node), (Plan *) node);
 	ExecClearTuple(uniquestate->cs_ResultTupleSlot);
 }
+
+
+void
+ExecReScanUnique(Unique *node, ExprContext *exprCtxt, Plan *parent)
+{
+	UniqueState *uniquestate = node->uniquestate;
+
+	ExecClearTuple(uniquestate->cs_ResultTupleSlot);
+
+	/*
+	 * if chgParam of subnode is not null then plan will be re-scanned by
+	 * first ExecProcNode.
+	 */
+	if (((Plan *) node)->lefttree->chgParam == NULL)
+		ExecReScan(((Plan *) node)->lefttree, exprCtxt, (Plan *) node);
+
+}
