@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/lib/stringinfo.c,v 1.5 1997/09/08 02:23:05 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/lib/stringinfo.c,v 1.9 1998/01/07 21:03:04 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -36,14 +36,14 @@ makeStringInfo()
 	res = (StringInfo) palloc(sizeof(StringInfoData));
 	if (res == NULL)
 	{
-		elog(WARN, "makeStringInfo: Out of memory!");
+		elog(ERROR, "makeStringInfo: Out of memory!");
 	}
 
 	size = 100;
 	res->data = palloc(size);
 	if (res->data == NULL)
 	{
-		elog(WARN,
+		elog(ERROR,
 		   "makeStringInfo: Out of memory! (%ld bytes requested)", size);
 	}
 	res->maxlen = size;
@@ -76,7 +76,9 @@ appendStringInfo(StringInfo str, char *buffer)
 				newlen;
 	char	   *s;
 
-	Assert((str != NULL));
+	Assert(str != NULL);
+	if (buffer == NULL)
+		buffer = "<>";
 
 	/*
 	 * do we have enough space to append the new string? (don't forget to
@@ -103,7 +105,7 @@ appendStringInfo(StringInfo str, char *buffer)
 		s = palloc(newlen);
 		if (s == NULL)
 		{
-			elog(WARN,
+			elog(ERROR,
 				 "appendStringInfo: Out of memory (%d bytes requested)",
 				 newlen);
 		}
