@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/libpq/be-fsstubs.c,v 1.15 1997/09/08 21:43:41 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/libpq/be-fsstubs.c,v 1.19 1998/01/07 21:03:12 momjian Exp $
  *
  * NOTES
  *	  This should be moved to a more appropriate place.  It is here
@@ -100,12 +100,12 @@ lo_close(int fd)
 
 	if (fd >= MAX_LOBJ_FDS)
 	{
-		elog(WARN, "lo_close: large obj descriptor (%d) out of range", fd);
+		elog(ERROR, "lo_close: large obj descriptor (%d) out of range", fd);
 		return -2;
 	}
 	if (cookies[fd] == NULL)
 	{
-		elog(WARN, "lo_close: invalid large obj descriptor (%d)", fd);
+		elog(ERROR, "lo_close: invalid large obj descriptor (%d)", fd);
 		return -3;
 	}
 #if FSDB
@@ -150,7 +150,7 @@ lo_lseek(int fd, int offset, int whence)
 
 	if (fd >= MAX_LOBJ_FDS)
 	{
-		elog(WARN, "lo_seek: large obj descriptor (%d) out of range", fd);
+		elog(ERROR, "lo_seek: large obj descriptor (%d) out of range", fd);
 		return -2;
 	}
 
@@ -200,12 +200,12 @@ lo_tell(int fd)
 {
 	if (fd >= MAX_LOBJ_FDS)
 	{
-		elog(WARN, "lo_tell: large object descriptor (%d) out of range", fd);
+		elog(ERROR, "lo_tell: large object descriptor (%d) out of range", fd);
 		return -2;
 	}
 	if (cookies[fd] == NULL)
 	{
-		elog(WARN, "lo_tell: invalid large object descriptor (%d)", fd);
+		elog(ERROR, "lo_tell: invalid large object descriptor (%d)", fd);
 		return -3;
 	}
 	return inv_tell(cookies[fd]);
@@ -227,9 +227,9 @@ loread(int fd, int len)
 	struct varlena *retval;
 	int			totalread = 0;
 
-	retval = (struct varlena *) palloc(sizeof(int32) + len);
+	retval = (struct varlena *) palloc(VARHDRSZ + len);
 	totalread = lo_read(fd, VARDATA(retval), len);
-	VARSIZE(retval) = totalread + sizeof(int32);
+	VARSIZE(retval) = totalread + VARHDRSZ;
 
 	return retval;
 }
@@ -240,7 +240,7 @@ lowrite(int fd, struct varlena * wbuf)
 	int			totalwritten;
 	int			bytestowrite;
 
-	bytestowrite = VARSIZE(wbuf) - sizeof(int32);
+	bytestowrite = VARSIZE(wbuf) - VARHDRSZ;
 	totalwritten = lo_write(fd, VARDATA(wbuf), bytestowrite);
 	return totalwritten;
 }
@@ -269,11 +269,11 @@ lo_import(text *filename)
 	/*
 	 * open the file to be read in
 	 */
-	strNcpy(fnamebuf, VARDATA(filename), VARSIZE(filename) - VARHDRSZ);
+	StrNCpy(fnamebuf, VARDATA(filename), VARSIZE(filename) - VARHDRSZ + 1);
 	fd = open(fnamebuf, O_RDONLY, 0666);
 	if (fd < 0)
 	{							/* error */
-		elog(WARN, "be_lo_import: can't open unix file\"%s\"\n",
+		elog(ERROR, "be_lo_import: can't open unix file\"%s\"\n",
 			 fnamebuf);
 	}
 
@@ -283,7 +283,7 @@ lo_import(text *filename)
 	lobj = inv_create(INV_READ | INV_WRITE);
 	if (lobj == NULL)
 	{
-		elog(WARN, "lo_import: can't create inv object for \"%s\"",
+		elog(ERROR, "lo_import: can't create inv object for \"%s\"",
 			 fnamebuf);
 	}
 
@@ -301,7 +301,7 @@ lo_import(text *filename)
 		tmp = inv_write(lobj, buf, nbytes);
 		if (tmp < nbytes)
 		{
-			elog(WARN, "lo_import: error while reading \"%s\"",
+			elog(ERROR, "lo_import: error while reading \"%s\"",
 				 fnamebuf);
 		}
 	}
@@ -335,7 +335,7 @@ lo_export(Oid lobjId, text *filename)
 	lobj = inv_open(lobjId, INV_READ);
 	if (lobj == NULL)
 	{
-		elog(WARN, "lo_export: can't open inv object %d",
+		elog(ERROR, "lo_export: can't open inv object %d",
 			 lobjId);
 	}
 
@@ -343,12 +343,12 @@ lo_export(Oid lobjId, text *filename)
 	 * open the file to be written to
 	 */
 	oumask = umask((mode_t) 0);
-	strNcpy(fnamebuf, VARDATA(filename), VARSIZE(filename) - VARHDRSZ);
+	StrNCpy(fnamebuf, VARDATA(filename), VARSIZE(filename) - VARHDRSZ + 1);
 	fd = open(fnamebuf, O_CREAT | O_WRONLY, 0666);
 	umask(oumask);
 	if (fd < 0)
 	{							/* error */
-		elog(WARN, "lo_export: can't open unix file\"%s\"",
+		elog(ERROR, "lo_export: can't open unix file\"%s\"",
 			 fnamebuf);
 	}
 
@@ -360,7 +360,7 @@ lo_export(Oid lobjId, text *filename)
 		tmp = write(fd, buf, nbytes);
 		if (tmp < nbytes)
 		{
-			elog(WARN, "lo_export: error while writing \"%s\"",
+			elog(ERROR, "lo_export: error while writing \"%s\"",
 				 fnamebuf);
 		}
 	}
