@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/libpq/be-pqexec.c,v 1.7 1997/09/08 21:43:43 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/libpq/be-pqexec.c,v 1.15 1998/02/26 04:31:45 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -27,7 +27,7 @@
 #include <tcop/fastpath.h>
 #include <tcop/tcopprot.h>
 #include <lib/dllist.h>
-#include <libpq/libpq-be.h>
+#include <libpq/libpq.h>
 #include <fmgr.h>
 #include <utils/exc.h>
 #include <utils/builtins.h>
@@ -58,7 +58,7 @@ static char *strmake(char *str, int len);
  *		This code scavanged from HandleFunctionRequest() in tcop/fastpath.h
  * ----------------
  */
-char	   *
+char *
 PQfn(int fnid,
 	 int *result_buf,			/* can't use void, dec compiler barfs */
 	 int result_len,
@@ -80,9 +80,9 @@ PQfn(int fnid,
 		{
 			arg[i] = (char *) args[i].u.ptr;
 		}
-		else if (args[i].len > 4)
+		else if (args[i].len > sizeof(int4))
 		{
-			elog(WARN, "arg_length of argument %d too long", i);
+			elog(ERROR, "arg_length of argument %d too long", i);
 		}
 		else
 		{
@@ -125,11 +125,11 @@ PQfn(int fnid,
  *		If a query is does not return tuples, return "C query-command".
  *		If there is an error: return "E error-message".
  *
- *		Note: if we get a serious error or an elog(WARN), then PQexec never
+ *		Note: if we get a serious error or an elog(ERROR), then PQexec never
  *		returns because the system longjmp's back to the main loop.
  * ----------------
  */
-char	   *
+char *
 PQexec(char *query)
 {
 	PortalEntry *entry = NULL;
@@ -143,11 +143,11 @@ PQexec(char *query)
 	be_portalpush(entry);
 
 	/* ----------------
-	 *	pg_eval_dest will put the query results in a portal which will
+	 *	pg_exec_query_dest will put the query results in a portal which will
 	 *	end up on the top of the portal stack.
 	 * ----------------
 	 */
-	pg_eval_dest(query, (char **) NULL, (Oid *) NULL, 0, Local);
+	pg_exec_query_dest(query, (char **) NULL, (Oid *) NULL, 0, Local);
 
 	/* ----------------
 	 *	pop the portal off the portal stack and return the
@@ -211,7 +211,7 @@ pqtest_PQexec(char *q)
 		case 'P':
 			a = PQparray(&res[1]);
 			if (a == NULL)
-				elog(WARN, "pqtest_PQexec: PQparray could not find portal %s",
+				elog(ERROR, "pqtest_PQexec: PQparray could not find portal %s",
 					 res);
 
 			t = PQntuples(a);
@@ -241,7 +241,7 @@ strmake(char *str, int len)
 		len = strlen(str);
 
 	newstr = (char *) palloc((unsigned) len + 1);
-	strNcpy(newstr, str, len);
+	StrNCpy(newstr, str, len + 1);
 	newstr[len] = (char) 0;
 	return newstr;
 }
@@ -357,7 +357,7 @@ pqtest_PQfn(char *q)
 		v = atoi(fields[j]);
 		if (v != 0 || (v == 0 && fields[j][0] == '0'))
 		{
-			pqargs[k].len = 4;
+			pqargs[k].len = sizeof(int4);
 			pqargs[k].u.integer = v;
 			printf("pqtest_PQfn: arg %d is int %d\n", k, v);	/* debug */
 		}
@@ -373,7 +373,7 @@ pqtest_PQfn(char *q)
 	 *	call PQfn
 	 * ----------------
 	 */
-	pqres = PQfn(f, &res, 4, 1, pqargs, i - 1);
+	pqres = PQfn(f, &res, sizeof(int4), 1, pqargs, i - 1);
 	printf("pqtest_PQfn: pqres is %s\n", pqres);		/* debug */
 
 	/* ----------------
