@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/main/main.c,v 1.9 1997/09/08 02:23:24 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/main/main.c,v 1.16 1998/04/11 21:14:47 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -15,6 +15,13 @@
 #include <string.h>
 #include <unistd.h>
 
+#if defined(alpha) && !defined(linux)
+#include <sys/sysinfo.h>
+#include <machine/hal_sysinfo.h>
+#define ASSEMBLER
+#include <sys/proc.h>
+#undef ASSEMBLER
+#endif
 #include "postgres.h"
 #ifdef USE_LOCALE
 #include <locale.h>
@@ -22,7 +29,6 @@
 #include "miscadmin.h"
 #include "bootstrap/bootstrap.h"/* for BootstrapMain() */
 #include "tcop/tcopprot.h"		/* for PostgresMain() */
-#include "port-protos.h"		/* for init_address_fixup() */
 
 #define NOROOTEXEC "\
 \n\"root\" execution of the PostgreSQL backend is not permitted\n\n\
@@ -36,6 +42,17 @@ main(int argc, char *argv[])
 {
 	int			len;
 
+#if defined(alpha)
+#ifdef NOFIXADE
+	int			buffer[] = {SSIN_UACPROC, UAC_SIGBUS};
+
+#endif							/* NOFIXADE */
+#ifdef NOPRINTADE
+	int			buffer[] = {SSIN_UACPROC, UAC_NOPRINT};
+
+#endif							/* NOPRINTADE */
+#endif
+
 #ifdef USE_LOCALE
 	setlocale(LC_CTYPE, "");	/* take locale information from an
 								 * environment */
@@ -48,7 +65,19 @@ main(int argc, char *argv[])
 	 * Must be first so that the bootstrap code calls it, too. (Only
 	 * needed on some RISC architectures.)
 	 */
-	init_address_fixup();
+
+#if defined(ultrix4)
+	syscall(SYS_sysmips, MIPS_FIXADE, 0, NULL, NULL, NULL);
+#endif
+
+#if defined(alpha)
+	if (setsysinfo(SSI_NVPAIRS, buffer, 1, (caddr_t) NULL,
+				   (unsigned long) NULL) < 0)
+	{
+		elog(NOTICE, "setsysinfo failed: %d\n", errno);
+	}
+#endif
+
 #endif							/* NOFIXADE || NOPRINTADE */
 
 	/*
