@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/copyfuncs.c,v 1.13 1997/09/19 06:52:49 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/copyfuncs.c,v 1.41 1998/02/26 04:32:04 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,7 +22,6 @@
 #include "nodes/parsenodes.h"
 #include "nodes/primnodes.h"
 #include "nodes/relation.h"
-#include "parser/parse_query.h"
 
 #include "utils/syscache.h"
 #include "utils/builtins.h"		/* for namecpy */
@@ -36,7 +35,7 @@
  *	  this copy function only copies the "lcons-cells" of the list but not
  *	  its contents. (good for list of pointers as well as list of integers).
  */
-List	   *
+List *
 listCopy(List *list)
 {
 	List	   *newlist = NIL;
@@ -80,14 +79,26 @@ listCopy(List *list)
 static void
 CopyPlanFields(Plan *from, Plan *newnode)
 {
+	extern List *SS_pull_subplan(void *expr);
+
 	newnode->cost = from->cost;
 	newnode->plan_size = from->plan_size;
 	newnode->plan_width = from->plan_width;
+	newnode->plan_tupperpage = from->plan_tupperpage;
 	newnode->state = from->state;
 	newnode->targetlist = copyObject(from->targetlist);
 	newnode->qual = copyObject(from->qual);
 	newnode->lefttree = copyObject(from->lefttree);
 	newnode->righttree = copyObject(from->righttree);
+	newnode->extParam = listCopy(from->extParam);
+	newnode->locParam = listCopy(from->locParam);
+	newnode->chgParam = listCopy(from->chgParam);
+	Node_Copy(from, newnode, initPlan);
+	if (from->subPlan != NULL)
+		newnode->subPlan = SS_pull_subplan(newnode->qual);
+	else
+		newnode->subPlan = NULL;
+	newnode->nParamExec = from->nParamExec;
 }
 
 /* ----------------
@@ -109,24 +120,6 @@ _copyPlan(Plan *from)
 }
 
 
-/* ----------------
- *		_copyExistential
- * ----------------
- */
-static Existential *
-_copyExistential(Existential *from)
-{
-	Existential *newnode = makeNode(Existential);
-
-	/* ----------------
-	 *	copy node superclass fields
-	 * ----------------
-	 */
-	CopyPlanFields(from, newnode);
-
-	return newnode;
-}
-
 /* ----------------
  *		_copyResult
  * ----------------
@@ -172,6 +165,7 @@ _copyAppend(Append *from)
 	 * ----------------
 	 */
 	Node_Copy(from, newnode, unionplans);
+	Node_Copy(from, newnode, unionrts);
 	newnode->unionrelid = from->unionrelid;
 	Node_Copy(from, newnode, unionrtentries);
 	Node_Copy(from, newnode, unionstate);
@@ -209,7 +203,7 @@ _copyScan(Scan *from)
 	 * ----------------
 	 */
 	CopyPlanFields((Plan *) from, (Plan *) newnode);
-	CopyScanFields(from, newnode);
+	CopyScanFields((Scan *) from, (Scan *) newnode);
 
 	return newnode;
 }
@@ -329,7 +323,6 @@ static MergeJoin *
 _copyMergeJoin(MergeJoin *from)
 {
 	MergeJoin  *newnode = makeNode(MergeJoin);
-	List	   *newlist;
 
 	/* ----------------
 	 *	copy node superclass fields
@@ -345,7 +338,6 @@ _copyMergeJoin(MergeJoin *from)
 	Node_Copy(from, newnode, mergeclauses);
 
 	newnode->mergesortop = from->mergesortop;
-	newlist = NIL;
 
 	newnode->mergerightorder = (Oid *) palloc(sizeof(Oid) * 2);
 	newnode->mergerightorder[0] = from->mergerightorder[0];
@@ -477,6 +469,8 @@ _copySort(Sort *from)
 	 * ----------------
 	 */
 	Node_Copy(from, newnode, sortstate);
+	Node_Copy(from, newnode, psortstate);
+	newnode->cleaned = from->cleaned;
 
 	return newnode;
 }
@@ -490,14 +484,13 @@ static Group *
 _copyGroup(Group *from)
 {
 	Group	   *newnode = makeNode(Group);
-	
+
 	CopyPlanFields((Plan *) from, (Plan *) newnode);
-	CopyTempFields((Temp *) from, (Temp *) newnode);
-	
+
 	newnode->tuplePerGroup = from->tuplePerGroup;
 	newnode->numCols = from->numCols;
-	newnode->grpColIdx = palloc (from->numCols * sizeof (AttrNumber));
-	memcpy (newnode->grpColIdx, from->grpColIdx, from->numCols * sizeof (AttrNumber));
+	newnode->grpColIdx = palloc(from->numCols * sizeof(AttrNumber));
+	memcpy(newnode->grpColIdx, from->grpColIdx, from->numCols * sizeof(AttrNumber));
 	Node_Copy(from, newnode, grpstate);
 
 	return newnode;
@@ -511,23 +504,30 @@ static Agg *
 _copyAgg(Agg *from)
 {
 	Agg		   *newnode = makeNode(Agg);
-	int			i;
 
 	CopyPlanFields((Plan *) from, (Plan *) newnode);
-	CopyTempFields((Temp *) from, (Temp *) newnode);
-
-	newnode->numAgg = from->numAgg;
-	newnode->aggs = malloc(sizeof(Aggreg *));
-	for (i = 0; i < from->numAgg; i++)
-	{
-		newnode->aggs[i] = copyObject(from->aggs[i]);
-	}
 
+	Node_Copy(from, newnode, aggs);
 	Node_Copy(from, newnode, aggstate);
 
 	return newnode;
 }
 
+/* ---------------
+ *	_copyGroupClause
+ * --------------
+ */
+static GroupClause *
+_copyGroupClause(GroupClause *from)
+{
+	GroupClause *newnode = makeNode(GroupClause);
+
+	Node_Copy(from, newnode, entry);
+	newnode->grpOpoid = from->grpOpoid;
+
+	return newnode;
+}
+
 
 /* ----------------
  *		_copyUnique
@@ -549,6 +549,11 @@ _copyUnique(Unique *from)
 	 *	copy remainder of node
 	 * ----------------
 	 */
+	if (newnode->uniqueAttr)
+		newnode->uniqueAttr = pstrdup(from->uniqueAttr);
+	else
+		newnode->uniqueAttr = NULL;
+	newnode->uniqueAttrNum = from->uniqueAttrNum;
 	Node_Copy(from, newnode, uniquestate);
 
 	return newnode;
@@ -584,6 +589,22 @@ _copyHash(Hash *from)
 	return newnode;
 }
 
+static SubPlan *
+_copySubPlan(SubPlan *from)
+{
+	SubPlan    *newnode = makeNode(SubPlan);
+
+	Node_Copy(from, newnode, plan);
+	newnode->plan_id = from->plan_id;
+	Node_Copy(from, newnode, rtable);
+	newnode->setParam = listCopy(from->setParam);
+	newnode->parParam = listCopy(from->parParam);
+	Node_Copy(from, newnode, sublink);
+	newnode->shutdown = from->shutdown;
+
+	return newnode;
+}
+
 /* ****************************************************************
  *					   primnodes.h copy functions
  * ****************************************************************
@@ -600,16 +621,10 @@ _copyResdom(Resdom *from)
 
 	newnode->resno = from->resno;
 	newnode->restype = from->restype;
-	newnode->reslen = from->reslen;
+	newnode->restypmod = from->restypmod;
 
 	if (from->resname != NULL)
-	{
-		newnode->resname = palloc(strlen(from->resname) + 1);
-		strcpy(newnode->resname, from->resname);
-	}
-	else
-		newnode->resname = (char *) NULL;
-
+		newnode->resname = pstrdup(from->resname);
 	newnode->reskey = from->reskey;
 	newnode->reskeyop = from->reskeyop;
 	newnode->resjunk = from->resjunk;
@@ -634,14 +649,12 @@ _copyFjoin(Fjoin *from)
 
 	newnode->fj_results = (DatumPtr)
 		palloc((from->fj_nNodes) * sizeof(Datum));
-
-	newnode->fj_alwaysDone = (BoolPtr)
-		palloc((from->fj_nNodes) * sizeof(bool));
-
 	memmove(from->fj_results,
 			newnode->fj_results,
 			(from->fj_nNodes) * sizeof(Datum));
 
+	newnode->fj_alwaysDone = (BoolPtr)
+		palloc((from->fj_nNodes) * sizeof(bool));
 	memmove(from->fj_alwaysDone,
 			newnode->fj_alwaysDone,
 			(from->fj_nNodes) * sizeof(bool));
@@ -688,6 +701,8 @@ _copyVar(Var *from)
 	newnode->varno = from->varno;
 	newnode->varattno = from->varattno;
 	newnode->vartype = from->vartype;
+	newnode->vartypmod = from->vartypmod;
+	newnode->varlevelsup = from->varlevelsup;
 
 	newnode->varnoold = from->varnoold;
 	newnode->varoattno = from->varoattno;
@@ -745,6 +760,8 @@ _copyConst(Const *from)
 	/* ----------------
 	 *	XXX super cheesy hack until parser/planner
 	 *	puts in the right values here.
+	 *
+	 *	But I like cheese.
 	 * ----------------
 	 */
 	if (!from->constisnull && cached_type != from->consttype)
@@ -817,7 +834,7 @@ _copyConst(Const *from)
 				 */
 				int			length;
 
-				length = *((int *) from->constvalue);
+				length = VARSIZE(from->constvalue);
 				newnode->constvalue = PointerGetDatum(palloc(length));
 				memmove((char *) newnode->constvalue,
 						(char *) from->constvalue, length);
@@ -830,6 +847,8 @@ _copyConst(Const *from)
 	}
 	newnode->constisnull = from->constisnull;
 	newnode->constbyval = from->constbyval;
+	newnode->constisset = from->constisset;
+	newnode->constiscast = from->constiscast;
 
 	return newnode;
 }
@@ -851,12 +870,7 @@ _copyParam(Param *from)
 	newnode->paramid = from->paramid;
 
 	if (from->paramname != NULL)
-	{
 		newnode->paramname = pstrdup(from->paramname);
-	}
-	else
-		newnode->paramname = (char *) NULL;
-
 	newnode->paramtype = from->paramtype;
 	Node_Copy(from, newnode, param_tlist);
 
@@ -903,10 +917,31 @@ _copyAggreg(Aggreg *from)
 	newnode->aggname = pstrdup(from->aggname);
 	newnode->basetype = from->basetype;
 	newnode->aggtype = from->aggtype;
-
 	Node_Copy(from, newnode, target);
-
 	newnode->aggno = from->aggno;
+	newnode->usenulls = from->usenulls;
+
+	return newnode;
+}
+
+/* ----------------
+ *		_copySubLink
+ * ----------------
+ */
+static SubLink *
+_copySubLink(SubLink *from)
+{
+	SubLink    *newnode = makeNode(SubLink);
+
+	/* ----------------
+	 *	copy remainder of node
+	 * ----------------
+	 */
+	newnode->subLinkType = from->subLinkType;
+	newnode->useor = from->useor;
+	Node_Copy(from, newnode, lefthand);
+	Node_Copy(from, newnode, oper);
+	Node_Copy(from, newnode, subselect);
 
 	return newnode;
 }
@@ -940,9 +975,9 @@ _copyArrayRef(ArrayRef *from)
 	 *	copy remainder of node
 	 * ----------------
 	 */
-	newnode->refelemtype = from->refelemtype;
 	newnode->refattrlength = from->refattrlength;
 	newnode->refelemlength = from->refelemlength;
+	newnode->refelemtype = from->refelemtype;
 	newnode->refelembyval = from->refelembyval;
 
 	Node_Copy(from, newnode, refupperindexpr);
@@ -985,14 +1020,11 @@ _copyRel(Rel *from)
 	newnode->tuples = from->tuples;
 	newnode->size = from->size;
 	newnode->width = from->width;
-	newnode->indproc = from->indproc;
-
 	Node_Copy(from, newnode, targetlist);
 	Node_Copy(from, newnode, pathlist);
 	Node_Copy(from, newnode, unorderedpath);
 	Node_Copy(from, newnode, cheapestpath);
 	newnode->pruneable = from->pruneable;
-	newnode->relam = from->relam;
 
 	if (from->classlist)
 	{
@@ -1000,9 +1032,7 @@ _copyRel(Rel *from)
 			;
 		newnode->classlist = (Oid *) palloc(sizeof(Oid) * (len + 1));
 		for (i = 0; i < len; i++)
-		{
 			newnode->classlist[i] = from->classlist[i];
-		}
 		newnode->classlist[len] = 0;
 	}
 
@@ -1012,21 +1042,21 @@ _copyRel(Rel *from)
 			;
 		newnode->indexkeys = (int *) palloc(sizeof(int) * (len + 1));
 		for (i = 0; i < len; i++)
-		{
 			newnode->indexkeys[i] = from->indexkeys[i];
-		}
 		newnode->indexkeys[len] = 0;
 	}
 
+	newnode->relam = from->relam;
+	newnode->indproc = from->indproc;
+	Node_Copy(from, newnode, indpred);
+
 	if (from->ordering)
 	{
 		for (len = 0; from->ordering[len] != 0; len++)
 			;
 		newnode->ordering = (Oid *) palloc(sizeof(Oid) * (len + 1));
 		for (i = 0; i < len; i++)
-		{
 			newnode->ordering[i] = from->ordering[i];
-		}
 		newnode->ordering[len] = 0;
 	}
 
@@ -1073,15 +1103,9 @@ CopyPathFields(Path *from, Path *newnode)
 			newnode->p_ordering.ord.sortop =
 				(Oid *) palloc(sizeof(Oid) * (len + 1));
 			for (i = 0; i < len; i++)
-			{
 				newnode->p_ordering.ord.sortop[i] = ordering[i];
-			}
 			newnode->p_ordering.ord.sortop[len] = 0;
 		}
-		else
-		{
-			newnode->p_ordering.ord.sortop = NULL;
-		}
 	}
 	else
 	{
@@ -1141,9 +1165,7 @@ _copyIndexPath(IndexPath *from)
 			;
 		newnode->indexkeys = (int *) palloc(sizeof(int) * (len + 1));
 		for (i = 0; i < len; i++)
-		{
 			newnode->indexkeys[i] = from->indexkeys[i];
-		}
 		newnode->indexkeys[len] = 0;
 	}
 
@@ -1367,6 +1389,7 @@ _copyHInfo(HInfo *from)
 	 *	copy remainder of node
 	 * ----------------
 	 */
+	CopyJoinMethodFields((JoinMethod *) from, (JoinMethod *) newnode);
 	newnode->hashop = from->hashop;
 
 	return newnode;
@@ -1385,6 +1408,7 @@ _copyMInfo(MInfo *from)
 	 *	copy remainder of node
 	 * ----------------
 	 */
+	CopyJoinMethodFields((JoinMethod *) from, (JoinMethod *) newnode);
 	Node_Copy(from, newnode, m_ordering);
 
 	return newnode;
@@ -1432,15 +1456,17 @@ _copyStream(Stream *from)
 	newnode->pathptr = from->pathptr;
 	newnode->cinfo = from->cinfo;
 	newnode->clausetype = from->clausetype;
-	newnode->groupup = from->groupup;
-	newnode->groupcost = from->groupcost;
-	newnode->groupsel = from->groupsel;
+
 	newnode->upstream = (StreamPtr) NULL;		/* only copy nodes
 												 * downwards! */
 	Node_Copy(from, newnode, downstream);
 	if (newnode->downstream)
 		((Stream *) newnode->downstream)->upstream = (Stream *) newnode;
 
+	newnode->groupup = from->groupup;
+	newnode->groupcost = from->groupcost;
+	newnode->groupsel = from->groupsel;
+
 	return newnode;
 }
 
@@ -1465,26 +1491,15 @@ _copyRangeTblEntry(RangeTblEntry *from)
 {
 	RangeTblEntry *newnode = makeNode(RangeTblEntry);
 
-	memcpy(newnode, from, sizeof(RangeTblEntry));
 	if (from->relname)
 		newnode->relname = pstrdup(from->relname);
 	if (from->refname)
 		newnode->refname = pstrdup(from->refname);
-	if (from->timeRange)
-	{
-		newnode->timeRange = makeNode(TimeRange);
-		if (from->timeRange->startDate)
-			newnode->timeRange->startDate = pstrdup(from->timeRange->startDate);
-		else
-			newnode->timeRange->startDate = NULL;
-		if (from->timeRange->endDate)
-			newnode->timeRange->endDate = pstrdup(from->timeRange->endDate);
-		else
-			newnode->timeRange->endDate = NULL;
-		newnode->timeQual = makeTimeRange(newnode->timeRange->startDate,
-										  newnode->timeRange->endDate,
-						((newnode->timeRange->endDate == NULL) ? 0 : 1));
-	}
+	newnode->relid = from->relid;
+	newnode->inh = from->inh;
+	newnode->inFromCl = from->inFromCl;
+	newnode->skipAcl = from->skipAcl;
+
 
 	return newnode;
 }
@@ -1517,16 +1532,11 @@ _copyTypeName(TypeName *from)
 	TypeName   *newnode = makeNode(TypeName);
 
 	if (from->name)
-	{
 		newnode->name = pstrdup(from->name);
-	}
-	else
-	{
-		from->name = (char *) 0;
-	}
+	newnode->timezone = from->timezone;
 	newnode->setof = from->setof;
+	newnode->typmod = from->typmod;
 	Node_Copy(from, newnode, arrayBounds);
-	newnode->typlen = from->typlen;
 
 	return newnode;
 }
@@ -1537,40 +1547,43 @@ _copyQuery(Query *from)
 	Query	   *newnode = makeNode(Query);
 
 	newnode->commandType = from->commandType;
-	newnode->resultRelation = from->resultRelation;
-	/* probably should dup this string instead of just pointing */
-	/* to the old one  --djm */
-	if (from->into)
-	{
-		newnode->into = pstrdup(from->into);
-	}
-	else
-	{
-		newnode->into = (char *) 0;
-	}
-	newnode->isPortal = from->isPortal;
-	Node_Copy(from, newnode, rtable);
 	if (from->utilityStmt && nodeTag(from->utilityStmt) == T_NotifyStmt)
 	{
 		NotifyStmt *from_notify = (NotifyStmt *) from->utilityStmt;
 		NotifyStmt *n = makeNode(NotifyStmt);
-		int			length = strlen(from_notify->relname);
 
-		n->relname = palloc(length + 1);
-		strcpy(n->relname, from_notify->relname);
+		n->relname = pstrdup(from_notify->relname);
 		newnode->utilityStmt = (Node *) n;
 	}
+	newnode->resultRelation = from->resultRelation;
+	if (from->into)
+		newnode->into = pstrdup(from->into);
+	newnode->isPortal = from->isPortal;
+	newnode->isBinary = from->isBinary;
+	newnode->unionall = from->unionall;
 	if (from->uniqueFlag)
-	{
-		newnode->uniqueFlag = (char *) palloc(strlen(from->uniqueFlag) + 1);
-		strcpy(newnode->uniqueFlag, from->uniqueFlag);
-	}
-	else
-		newnode->uniqueFlag = NULL;
+		newnode->uniqueFlag = pstrdup(from->uniqueFlag);
 	Node_Copy(from, newnode, sortClause);
+	Node_Copy(from, newnode, rtable);
 	Node_Copy(from, newnode, targetList);
 	Node_Copy(from, newnode, qual);
 
+	Node_Copy(from, newnode, groupClause);
+	Node_Copy(from, newnode, havingQual);
+
+	newnode->hasAggs = from->hasAggs;
+	newnode->hasSubLinks = from->hasSubLinks;
+
+	if (from->unionClause)
+	{
+		List	   *ulist,
+				   *temp_list = NIL;
+
+		foreach(ulist, from->unionClause)
+			temp_list = lappend(temp_list, copyObject(lfirst(ulist)));
+		newnode->unionClause = temp_list;
+	}
+
 	return newnode;
 }
 
@@ -1613,7 +1626,7 @@ _copyValue(Value *from)
  *		recursively copies its items.
  * ----------------
  */
-void	   *
+void *
 copyObject(void *from)
 {
 	void	   *retval;
@@ -1629,9 +1642,6 @@ copyObject(void *from)
 		case T_Plan:
 			retval = _copyPlan(from);
 			break;
-		case T_Existential:
-			retval = _copyExistential(from);
-			break;
 		case T_Result:
 			retval = _copyResult(from);
 			break;
@@ -1674,12 +1684,18 @@ copyObject(void *from)
 		case T_Agg:
 			retval = _copyAgg(from);
 			break;
+		case T_GroupClause:
+			retval = _copyGroupClause(from);
+			break;
 		case T_Unique:
 			retval = _copyUnique(from);
 			break;
 		case T_Hash:
 			retval = _copyHash(from);
 			break;
+		case T_SubPlan:
+			retval = _copySubPlan(from);
+			break;
 
 			/*
 			 * PRIMITIVE NODES
@@ -1717,6 +1733,9 @@ copyObject(void *from)
 		case T_Aggreg:
 			retval = _copyAggreg(from);
 			break;
+		case T_SubLink:
+			retval = _copySubLink(from);
+			break;
 
 			/*
 			 * RELATION NODES
