@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/print.c,v 1.9 1997/09/08 21:44:10 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/print.c,v 1.14 1998/01/20 05:03:54 momjian Exp $
  *
  * HISTORY
  *	  AUTHOR			DATE			MAJOR EVENT
@@ -26,11 +26,11 @@
 #include "nodes/parsenodes.h"
 #include "nodes/print.h"
 #include "parser/parsetree.h"
-#include "parser/catalog_utils.h"
 #include "access/heapam.h"
 #include "utils/lsyscache.h"
 #include "nodes/nodes.h"
 #include "nodes/plannodes.h"
+#include "parser/parse_relation.h"
 #include "optimizer/clauses.h"
 
 static char *plannode_type(Plan *p);
@@ -164,7 +164,7 @@ print_expr(Node *expr, List *rtable)
 {
 	if (expr == NULL)
 	{
-		printf("nil");
+		printf("<>");
 		return;
 	}
 
@@ -187,15 +187,11 @@ print_expr(Node *expr, List *rtable)
 				break;
 			default:
 				{
-					Relation	r;
-
 					rt = rt_fetch(var->varno, rtable);
 					relname = rt->relname;
-					r = heap_openr(relname);
 					if (rt->refname)
 						relname = rt->refname;	/* table renamed */
-					attname = getAttrName(r, var->varattno);
-					heap_close(r);
+					attname = get_attname(rt->relid, var->varattno);
 				}
 				break;
 		}
@@ -304,9 +300,6 @@ plannode_type(Plan *p)
 			case T_Plan:
 			return "PLAN";
 			break;
-		case T_Existential:
-			return "EXISTENTIAL";
-			break;
 		case T_Result:
 			return "RESULT";
 			break;
@@ -391,14 +384,14 @@ print_plan_recursive(Plan *p, Query *parsetree, int indentLevel, char *label)
 		RangeTblEntry *rte;
 
 		rte = rt_fetch(((Scan *) p)->scanrelid, parsetree->rtable);
-		strNcpy(extraInfo, rte->relname, NAMEDATALEN - 1);
+		StrNCpy(extraInfo, rte->relname, NAMEDATALEN);
 	}
 	else if (IsA(p, IndexScan))
 	{
-		strNcpy(extraInfo,
+		StrNCpy(extraInfo,
 		   ((RangeTblEntry *) (nth(((IndexScan *) p)->scan.scanrelid - 1,
 								   parsetree->rtable)))->relname,
-				NAMEDATALEN - 1);
+				NAMEDATALEN);
 	}
 	else
 		extraInfo[0] = '\0';
