@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/readfuncs.c,v 1.9 1997/09/08 21:44:11 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/readfuncs.c,v 1.28 1998/02/26 04:32:17 momjian Exp $
  *
  * NOTES
  *	  Most of the read functions for plan nodes are tested. (In fact, they
@@ -83,43 +83,132 @@ _readQuery()
 	token = lsptok(NULL, &length);		/* get the commandType */
 	local_node->commandType = atoi(token);
 
-	token = lsptok(NULL, &length);		/* skip the :utility */
+	token = lsptok(NULL, &length);		/* skip :utility */
+	/* we can't get create or index here, can we? */
+
 	token = lsptok(NULL, &length);		/* get the notify name if any */
-	if (token[0] == '"' && token[1] == '"')
+	if (length == 0)
 		local_node->utilityStmt = NULL;
 	else
 	{
 		NotifyStmt *n = makeNode(NotifyStmt);
 
 		n->relname = palloc(length + 1);
-		strNcpy(n->relname, token, length);
+		StrNCpy(n->relname, token, length + 1);
 		local_node->utilityStmt = (Node *) n;
 	}
 
-	token = lsptok(NULL, &length);		/* skip the :resrel */
+	token = lsptok(NULL, &length);		/* skip the :resultRelation */
 	token = lsptok(NULL, &length);		/* get the resultRelation */
 	local_node->resultRelation = atoi(token);
 
-	token = lsptok(NULL, &length);		/* skip :rtable */
-	local_node->rtable = nodeRead(true);
+	token = lsptok(NULL, &length);		/* skip :into */
+	token = lsptok(NULL, &length);		/* get into */
+	if (length == 0)
+		local_node->into = NULL;
+	else
+	{
+		local_node->into = palloc(length + 1);
+		StrNCpy(local_node->into, token, length + 1);
+	}
 
-	token = lsptok(NULL, &length);		/* skip the :unique */
-	token = lsptok(NULL, &length);		/* get the uniqueFlag */
-/*	  local_node->uniqueFlag = (bool)atoi(token); */
-	if (token[0] == '"' && token[1] == '"')		/* non-unique */
+	token = lsptok(NULL, &length);		/* skip :isPortal */
+	token = lsptok(NULL, &length);		/* get isPortal */
+	local_node->isPortal = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* skip :isBinary */
+	token = lsptok(NULL, &length);		/* get isBinary */
+	local_node->isBinary = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* skip :unionall */
+	token = lsptok(NULL, &length);		/* get unionall */
+	local_node->unionall = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* skip :uniqueFlag */
+	token = lsptok(NULL, &length);		/* get uniqueFlag */
+	if (length == 0)
 		local_node->uniqueFlag = NULL;
 	else
 	{
 		local_node->uniqueFlag = palloc(length + 1);
-		strNcpy(local_node->uniqueFlag, token, length);
+		StrNCpy(local_node->uniqueFlag, token, length + 1);
 	}
 
+	token = lsptok(NULL, &length);		/* skip :sortClause */
+	local_node->sortClause = nodeRead(true);
+
+	token = lsptok(NULL, &length);		/* skip :rtable */
+	local_node->rtable = nodeRead(true);
+
 	token = lsptok(NULL, &length);		/* skip :targetlist */
 	local_node->targetList = nodeRead(true);
 
 	token = lsptok(NULL, &length);		/* skip :qual */
 	local_node->qual = nodeRead(true);
 
+	token = lsptok(NULL, &length);		/* skip :groupClause */
+	local_node->groupClause = nodeRead(true);
+
+	token = lsptok(NULL, &length);		/* skip :havingQual */
+	local_node->havingQual = nodeRead(true);
+
+	token = lsptok(NULL, &length);		/* skip the :hasAggs */
+	token = lsptok(NULL, &length);		/* get hasAggs */
+	local_node->hasAggs = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* skip the :hasSubLinks */
+	token = lsptok(NULL, &length);		/* get hasSubLinks */
+	local_node->hasSubLinks = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* skip :unionClause */
+	local_node->unionClause = nodeRead(true);
+
+	return (local_node);
+}
+
+/* ----------------
+ *		_readSortClause
+ * ----------------
+ */
+static SortClause *
+_readSortClause()
+{
+	SortClause *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(SortClause);
+
+	token = lsptok(NULL, &length);		/* skip the :resdom */
+	local_node->resdom = nodeRead(true);
+
+	token = lsptok(NULL, &length);		/* skip :opoid */
+	token = lsptok(NULL, &length);		/* get opoid */
+	local_node->opoid = strtoul(token, NULL, 10);
+
+	return (local_node);
+}
+
+/* ----------------
+ *		_readGroupClause
+ * ----------------
+ */
+static GroupClause *
+_readGroupClause()
+{
+	GroupClause *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(GroupClause);
+
+	token = lsptok(NULL, &length);		/* skip the :entry */
+	local_node->entry = nodeRead(true);
+
+	token = lsptok(NULL, &length);		/* skip :grpOpoid */
+	token = lsptok(NULL, &length);		/* get grpOpoid */
+	local_node->grpOpoid = strtoul(token, NULL, 10);
+
 	return (local_node);
 }
 
@@ -148,7 +237,7 @@ _getPlan(Plan *node)
 	token = lsptok(NULL, &length);		/* eat the :state stuff */
 	token = lsptok(NULL, &length);		/* now get the state */
 
-	if (!strncmp(token, "nil", 3))
+	if (length == 0)
 	{
 		node->state = (EState *) NULL;
 	}
@@ -217,24 +306,6 @@ _readResult()
 	return (local_node);
 }
 
-/* ----------------
- *		_readExistential
- *
- *		Existential nodes are only used by the planner.
- * ----------------
- */
-static Existential *
-_readExistential()
-{
-	Existential *local_node;
-
-	local_node = makeNode(Existential);
-
-	_getPlan((Plan *) local_node);
-
-	return (local_node);
-}
-
 /* ----------------
  *		_readAppend
  *
@@ -256,9 +327,12 @@ _readAppend()
 	token = lsptok(NULL, &length);		/* eat :unionplans */
 	local_node->unionplans = nodeRead(true);	/* now read it */
 
+	token = lsptok(NULL, &length);		/* eat :unionrts */
+	local_node->unionrts = nodeRead(true);		/* now read it */
+
 	token = lsptok(NULL, &length);		/* eat :unionrelid */
 	token = lsptok(NULL, &length);		/* get unionrelid */
-	local_node->unionrelid = atoi(token);
+	local_node->unionrelid = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :unionrtentries */
 	local_node->unionrtentries = nodeRead(true);		/* now read it */
@@ -364,7 +438,7 @@ _readHashJoin()
 
 	token = lsptok(NULL, &length);		/* eat :hashjoinop */
 	token = lsptok(NULL, &length);		/* get hashjoinop */
-	local_node->hashjoinop = atoi(token);
+	local_node->hashjoinop = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :hashjointable */
 	token = lsptok(NULL, &length);		/* eat hashjointable */
@@ -405,7 +479,7 @@ _getScan(Scan *node)
 
 	token = lsptok(NULL, &length);		/* eat :scanrelid */
 	token = lsptok(NULL, &length);		/* get scanrelid */
-	node->scanrelid = atoi(token);
+	node->scanrelid = strtoul(token, NULL, 10);
 }
 
 /* ----------------
@@ -537,9 +611,8 @@ _readAgg()
 	local_node = makeNode(Agg);
 	_getPlan((Plan *) local_node);
 
-	token = lsptok(NULL, &length);		/* eat :numagg */
-	token = lsptok(NULL, &length);		/* get numagg */
-	local_node->numAgg = atoi(token);
+	token = lsptok(NULL, &length);		/* eat :agg */
+	local_node->aggs = nodeRead(true);	/* now read it */
 
 	return (local_node);
 }
@@ -633,35 +706,24 @@ _readResdom()
 	token = lsptok(NULL, &length);		/* get restype */
 	local_node->restype = atol(token);
 
-	token = lsptok(NULL, &length);		/* eat :reslen */
-	token = lsptok(NULL, &length);		/* get reslen */
-	local_node->reslen = atoi(token);
+	token = lsptok(NULL, &length);		/* eat :restypmod */
+	token = lsptok(NULL, &length);		/* get restypmod */
+	local_node->restypmod = atoi(token);
 
 	token = lsptok(NULL, &length);		/* eat :resname */
 	token = lsptok(NULL, &length);		/* get the name */
 
-	if (!strncmp(token, "\"null\"", 5))
-	{
+	if (length == 0)
 		local_node->resname = NULL;
-	}
 	else
 	{
-
-		/*
-		 * Peel off ""'s, then make a true copy.
-		 */
-
-		token++;
-		token[length - 2] = '\0';
-
-		local_node->resname = palloc(length);
-		strcpy(local_node->resname, token);
-		token[length - 2] = '\"';
+		local_node->resname = (char *) palloc(length + 1);
+		StrNCpy(local_node->resname, token, length + 1);
 	}
 
 	token = lsptok(NULL, &length);		/* eat :reskey */
 	token = lsptok(NULL, &length);		/* get reskey */
-	local_node->reskey = atoi(token);
+	local_node->reskey = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :reskeyop */
 	token = lsptok(NULL, &length);		/* get reskeyop */
@@ -715,6 +777,10 @@ _readExpr()
 	{
 		local_node->opType = NOT_EXPR;
 	}
+	else if (!strncmp(token, "subp", 4))
+	{
+		local_node->opType = SUBPLAN_EXPR;
+	}
 
 	token = lsptok(NULL, &length);		/* eat :oper */
 	local_node->oper = nodeRead(true);
@@ -742,7 +808,7 @@ _readVar()
 
 	token = lsptok(NULL, &length);		/* eat :varno */
 	token = lsptok(NULL, &length);		/* get varno */
-	local_node->varno = atoi(token);
+	local_node->varno = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :varattno */
 	token = lsptok(NULL, &length);		/* get varattno */
@@ -752,6 +818,14 @@ _readVar()
 	token = lsptok(NULL, &length);		/* get vartype */
 	local_node->vartype = (Oid) atol(token);
 
+	token = lsptok(NULL, &length);		/* eat :vartypmod */
+	token = lsptok(NULL, &length);		/* get vartypmod */
+	local_node->vartypmod = atoi(token);
+
+	token = lsptok(NULL, &length);		/* eat :varlevelsup */
+	token = lsptok(NULL, &length);		/* get varlevelsup */
+	local_node->varlevelsup = (Oid) atol(token);
+
 	token = lsptok(NULL, &length);		/* eat :varnoold */
 	token = lsptok(NULL, &length);		/* get varnoold */
 	local_node->varnoold = (Oid) atol(token);
@@ -780,7 +854,7 @@ _readArray()
 
 	token = lsptok(NULL, &length);		/* eat :arrayelemtype */
 	token = lsptok(NULL, &length);		/* get arrayelemtype */
-	local_node->arrayelemtype = (Oid) atoi(token);
+	local_node->arrayelemtype = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :arrayelemlength */
 	token = lsptok(NULL, &length);		/* get arrayelemlength */
@@ -822,7 +896,7 @@ _readArrayRef()
 
 	token = lsptok(NULL, &length);		/* eat :refelemtype */
 	token = lsptok(NULL, &length);		/* get refelemtype */
-	local_node->refelemtype = (Oid) atoi(token);
+	local_node->refelemtype = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* eat :refattrlength */
 	token = lsptok(NULL, &length);		/* get refattrlength */
@@ -873,7 +947,7 @@ _readConst()
 
 	token = lsptok(NULL, &length);		/* get :constlen */
 	token = lsptok(NULL, &length);		/* now read it */
-	local_node->constlen = atoi(token);
+	local_node->constlen = strtol(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* get :constisnull */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1033,11 +1107,13 @@ _readParam()
 
 	token = lsptok(NULL, &length);		/* get :paramname */
 	token = lsptok(NULL, &length);		/* now read it */
-	token++;					/* skip the first `"' */
-	token[length - 2] = '\0';	/* this is the 2nd `"' */
-
-	local_node->paramname = pstrdup(token);
-	token[length - 2] = '\"';	/* restore the 2nd `"' */
+	if (length == 0)
+		local_node->paramname = NULL;
+	else
+	{
+		local_node->paramname = (char *) palloc(length + 1);
+		StrNCpy(local_node->paramname, token, length + 1);
+	}
 
 	token = lsptok(NULL, &length);		/* get :paramtype */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1067,7 +1143,7 @@ _readAggreg()
 	token = lsptok(NULL, &length);		/* eat :aggname */
 	token = lsptok(NULL, &length);		/* get aggname */
 	local_node->aggname = (char *) palloc(length + 1);
-	strNcpy(local_node->aggname, token, length);
+	StrNCpy(local_node->aggname, token, length + 1);
 
 	token = lsptok(NULL, &length);		/* eat :basetype */
 	token = lsptok(NULL, &length);		/* get basetype */
@@ -1077,12 +1153,51 @@ _readAggreg()
 	token = lsptok(NULL, &length);		/* get aggtype */
 	local_node->aggtype = (Oid) atol(token);
 
+	token = lsptok(NULL, &length);		/* eat :target */
+	local_node->target = nodeRead(true);		/* now read it */
+
 	token = lsptok(NULL, &length);		/* eat :aggno */
 	token = lsptok(NULL, &length);		/* get aggno */
 	local_node->aggno = atoi(token);
 
-	token = lsptok(NULL, &length);		/* eat :target */
-	local_node->target = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* eat :usenulls */
+	token = lsptok(NULL, &length);		/* get usenulls */
+	local_node->usenulls = (token[0] == 't') ? true : false;
+
+	return (local_node);
+}
+
+/* ----------------
+ *		_readSubLink
+ *
+ *	SubLink is a subclass of Node
+ * ----------------
+ */
+static SubLink *
+_readSubLink()
+{
+	SubLink    *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(SubLink);
+
+	token = lsptok(NULL, &length);		/* eat :subLinkType */
+	token = lsptok(NULL, &length);		/* get subLinkType */
+	local_node->subLinkType = atoi(token);
+
+	token = lsptok(NULL, &length);		/* eat :useor */
+	token = lsptok(NULL, &length);		/* get useor */
+	local_node->useor = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* eat :lefthand */
+	local_node->lefthand = nodeRead(true);		/* now read it */
+
+	token = lsptok(NULL, &length);		/* eat :oper */
+	local_node->oper = nodeRead(true);	/* now read it */
+
+	token = lsptok(NULL, &length);		/* eat :subselect */
+	local_node->subselect = nodeRead(true);		/* now read it */
 
 	return (local_node);
 }
@@ -1233,7 +1348,7 @@ _readTargetEntry()
 }
 
 /* ----------------
- *		_readTargetEntry
+ *		_readRangeTblEntry
  * ----------------
  */
 static RangeTblEntry *
@@ -1247,52 +1362,39 @@ _readRangeTblEntry()
 
 	token = lsptok(NULL, &length);		/* eat :relname */
 	token = lsptok(NULL, &length);		/* get :relname */
-	if (!strncmp(token, "\"null\"", 5))
-	{
+	if (length == 0)
 		local_node->relname = NULL;
-	}
 	else
 	{
-
-		/*
-		 * Peel off ""'s, then make a true copy.
-		 */
-
-		token++;
-		token[length - 2] = '\0';
-
-		local_node->relname = (char *) palloc(NAMEDATALEN);
-		strcpy(local_node->relname, token);
-		token[length - 2] = '\"';
+		local_node->relname = (char *) palloc(length + 1);
+		StrNCpy(local_node->relname, token, length + 1);
 	}
 
-	token = lsptok(NULL, &length);		/* eat :inh */
-	token = lsptok(NULL, &length);		/* get :inh */
-	local_node->inh = atoi(token);
-
 	token = lsptok(NULL, &length);		/* eat :refname */
 	token = lsptok(NULL, &length);		/* get :refname */
-	if (!strncmp(token, "\"null\"", 5))
-	{
+	if (length == 0)
 		local_node->refname = NULL;
-	}
 	else
 	{
-
-		/*
-		 * Peel off ""'s, then make a true copy.
-		 */
-
-		token++;
-		token[length - 2] = '\0';
-
-		local_node->refname = (char *) pstrdup(token);
-		token[length - 2] = '\"';
+		local_node->refname = (char *) palloc(length + 1);
+		StrNCpy(local_node->refname, token, length + 1);
 	}
 
 	token = lsptok(NULL, &length);		/* eat :relid */
 	token = lsptok(NULL, &length);		/* get :relid */
-	local_node->relid = atoi(token);
+	local_node->relid = strtoul(token, NULL, 10);
+
+	token = lsptok(NULL, &length);		/* eat :inh */
+	token = lsptok(NULL, &length);		/* get :inh */
+	local_node->inh = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* eat :inFromCl */
+	token = lsptok(NULL, &length);		/* get :inFromCl */
+	local_node->inFromCl = (token[0] == 't') ? true : false;
+
+	token = lsptok(NULL, &length);		/* eat :skipAcl */
+	token = lsptok(NULL, &length);		/* get :skipAcl */
+	local_node->skipAcl = (token[0] == 't') ? true : false;
 
 	return (local_node);
 }
@@ -1617,7 +1719,7 @@ _readOrderKey()
 	token = lsptok(NULL, &length);		/* get :array_index */
 	token = lsptok(NULL, &length);		/* now read it */
 
-	local_node->array_index = atoi(token);
+	local_node->array_index = strtoul(token, NULL, 10);
 
 	return (local_node);
 }
@@ -1779,7 +1881,7 @@ _readHInfo()
 	token = lsptok(NULL, &length);		/* get :hashop */
 	token = lsptok(NULL, &length);		/* now read it */
 
-	local_node->hashop = atoi(token);
+	local_node->hashop = strtoul(token, NULL, 10);
 
 	token = lsptok(NULL, &length);		/* get :jmkeys */
 	local_node->jmethod.jmkeys = nodeRead(true);		/* now read it */
@@ -1867,7 +1969,7 @@ _readIter()
  * The string passed to parsePlanString must be null-terminated.
  * ----------------
  */
-Node	   *
+Node *
 parsePlanString(void)
 {
 	char	   *token;
@@ -1884,10 +1986,6 @@ parsePlanString(void)
 	{
 		return_value = _readResult();
 	}
-	else if (!strncmp(token, "EXISTENTIAL", 11))
-	{
-		return_value = _readExistential();
-	}
 	else if (!strncmp(token, "APPEND", 6))
 	{
 		return_value = _readAppend();
@@ -1932,6 +2030,10 @@ parsePlanString(void)
 	{
 		return_value = _readAggreg();
 	}
+	else if (!strncmp(token, "SUBLINK", 6))
+	{
+		return_value = _readSubLink();
+	}
 	else if (!strncmp(token, "AGG", 3))
 	{
 		return_value = _readAgg();
@@ -2053,9 +2155,17 @@ parsePlanString(void)
 	{
 		return_value = _readQuery();
 	}
+	else if (!strncmp(token, "SORTCLAUSE", 10))
+	{
+		return_value = _readSortClause();
+	}
+	else if (!strncmp(token, "GROUPCLAUSE", 10))
+	{
+		return_value = _readGroupClause();
+	}
 	else
 	{
-		elog(WARN, "badly formatted planstring \"%.10s\"...\n", token);
+		elog(ERROR, "badly formatted planstring \"%.10s\"...\n", token);
 	}
 
 	return ((Node *) return_value);
@@ -2094,7 +2204,7 @@ readDatum(Oid type)
 	{
 		if (length > sizeof(Datum))
 		{
-			elog(WARN, "readValue: byval & length = %d", length);
+			elog(ERROR, "readValue: byval & length = %d", length);
 		}
 		s = (char *) (&res);
 		for (i = 0; i < sizeof(Datum); i++)
@@ -2122,7 +2232,7 @@ readDatum(Oid type)
 	token = lsptok(NULL, &tokenLength); /* skip the ']' */
 	if (token[0] != ']')
 	{
-		elog(WARN, "readValue: ']' expected, length =%d", length);
+		elog(ERROR, "readValue: ']' expected, length =%d", length);
 	}
 
 	return (res);
