@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/path/predmig.c,v 1.6 1997/09/08 21:45:07 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/path/predmig.c,v 1.10 1998/03/30 16:46:55 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -47,7 +47,6 @@
 #include "optimizer/cost.h"
 #include "optimizer/keys.h"
 #include "optimizer/tlist.h"
-#include "lib/qsort.h"
 
 #define is_clause(node) (get_cinfo(node))		/* a stream node
 												 * represents a clause
@@ -78,7 +77,7 @@ static int	xfunc_stream_compare(void *arg1, void *arg2);
 static bool xfunc_check_stream(Stream node);
 static bool xfunc_in_stream(Stream node, Stream stream);
 
-/* -----------------   MAIN FUNCTIONS		------------------------ */
+/* -----------------   MAIN FUNCTIONS	  ------------------------ */
 /*
 ** xfunc_do_predmig
 **	  wrapper for Predicate Migration.	It calls xfunc_predmig until no
@@ -133,7 +132,7 @@ xfunc_predmig(JoinPath pathnode,/* root of the join tree */
 	/* sanity check */
 	if ((!streamroot && laststream) ||
 		(streamroot && !laststream))
-		elog(WARN, "called xfunc_predmig with bad inputs");
+		elog(ERROR, "called xfunc_predmig with bad inputs");
 	if (streamroot)
 		Assert(xfunc_check_stream(streamroot));
 
@@ -333,7 +332,7 @@ xfunc_prdmig_pullup(Stream origstream, Stream pullme, JoinPath joinpath)
 		 orignode = (Stream) get_downstream(orignode))
 		 /* empty body in for loop */ ;
 	if (!orignode)
-		elog(WARN, "Didn't find matching node in original stream");
+		elog(ERROR, "Didn't find matching node in original stream");
 
 
 	/* pull up this node as far as it should go */
@@ -495,7 +494,7 @@ xfunc_form_groups(Query *queryInfo, Stream root, Stream bottom)
 }
 
 
-/* -------------------		 UTILITY FUNCTIONS	   ------------------------- */
+/* -------------------		   UTILITY FUNCTIONS	 ------------------------- */
 
 /*
  ** xfunc_free_stream --
@@ -698,7 +697,7 @@ xfunc_stream_qsort(Stream root, Stream bottom)
 		nodearray[i] = tmp;
 
 	/* sort the array */
-	pg_qsort(nodearray, num, sizeof(LispValue), xfunc_stream_compare);
+	qsort(nodearray, num, sizeof(LispValue), xfunc_stream_compare);
 
 	/* paste together the array elements */
 	output = nodearray[num - 1];
@@ -765,7 +764,7 @@ xfunc_stream_compare(void *arg1, void *arg2)
 	}
 }
 
-/* ------------------  DEBUGGING ROUTINES	---------------------------- */
+/* ------------------  DEBUGGING ROUTINES ---------------------------- */
 
 /*
  ** Make sure all pointers in stream make sense.  Make sure no joins are
@@ -790,14 +789,14 @@ xfunc_check_stream(Stream node)
 	{
 		if ((Stream) get_upstream((Stream) get_downstream(temp)) != temp)
 		{
-			elog(WARN, "bad pointers in stream");
+			elog(ERROR, "bad pointers in stream");
 			return (false);
 		}
 		if (!is_clause(temp))
 		{
 			if ((tmp = xfunc_num_relids(temp)) >= numrelids)
 			{
-				elog(WARN, "Joins got reordered!");
+				elog(ERROR, "Joins got reordered!");
 				return (false);
 			}
 			numrelids = tmp;
