@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/planmain.c,v 1.8 1997/09/25 12:21:15 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/planmain.c,v 1.21 1998/03/31 23:30:49 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,7 +22,9 @@
 #include "nodes/makefuncs.h"
 
 #include "optimizer/planmain.h"
+#include "optimizer/subselect.h"
 #include "optimizer/internal.h"
+#include "optimizer/prep.h"
 #include "optimizer/paths.h"
 #include "optimizer/clauses.h"
 #include "optimizer/keys.h"
@@ -41,7 +43,7 @@
 static Plan *subplanner(Query *root, List *flat_tlist, List *qual);
 static Result *make_result(List *tlist, Node *resconstantqual, Plan *subplan);
 
-static Plan *
+extern Plan *
 make_groupPlan(List **tlist, bool tuplePerGroup,
 			   List *groupClause, Plan *subplan);
 
@@ -62,17 +64,27 @@ make_groupPlan(List **tlist, bool tuplePerGroup,
  *
  *	  Returns a query plan.
  */
-Plan	   *
+Plan *
 query_planner(Query *root,
 			  int command_type,
 			  List *tlist,
 			  List *qual)
 {
 	List	   *constant_qual = NIL;
-	List	   *flattened_tlist = NIL;
+	List	   *var_only_tlist = NIL;
 	List	   *level_tlist = NIL;
-	Plan	   *subplan = (Plan *) NULL;
-	Agg		   *aggplan = NULL;
+	Plan	   *subplan = NULL;
+
+	if (PlannerQueryLevel > 1)
+	{
+		/* should copy be made ? */
+		tlist = (List *) SS_replace_correlation_vars((Node *) tlist);
+		qual = (List *) SS_replace_correlation_vars((Node *) qual);
+	}
+	if (root->hasSubLinks)
+		qual = (List *) SS_process_sublinks((Node *) qual);
+
+	qual = cnfify((Expr *) qual, true);
 
 	/*
 	 * A command without a target list or qualification is an error,
@@ -110,16 +122,12 @@ query_planner(Query *root,
 	 * Create a target list that consists solely of (resdom var) target
 	 * list entries, i.e., contains no arbitrary expressions.
 	 */
-	flattened_tlist = flatten_tlist(tlist);
-	if (flattened_tlist)
-	{
-		level_tlist = flattened_tlist;
-	}
+	var_only_tlist = flatten_tlist(tlist);
+	if (var_only_tlist)
+		level_tlist = var_only_tlist;
 	else
-	{
 		/* from old code. the logic is beyond me. - ay 2/95 */
 		level_tlist = tlist;
-	}
 
 	/*
 	 * A query may have a non-variable target list and a non-variable
@@ -127,7 +135,7 @@ query_planner(Query *root,
 	 * all-new tuples, or - the query is a replace (a scan must still be
 	 * done in this case).
 	 */
-	if (flattened_tlist == NULL && qual == NULL)
+	if (var_only_tlist == NULL && qual == NULL)
 	{
 
 		switch (command_type)
@@ -174,67 +182,25 @@ query_planner(Query *root,
 
 	set_tlist_references(subplan);
 
-	/*
-	 * If we have a GROUP BY clause, insert a group node (with the
-	 * appropriate sort node.)
-	 */
-	if (root->groupClause != NULL)
-	{
-		bool		tuplePerGroup;
-
-		/*
-		 * decide whether how many tuples per group the Group node needs
-		 * to return. (Needs only one tuple per group if no aggregate is
-		 * present. Otherwise, need every tuple from the group to do the
-		 * aggregation.)
-		 */
-		tuplePerGroup = (root->qry_aggs) ? TRUE : FALSE;
-
-		subplan =
-			make_groupPlan(&tlist, tuplePerGroup, root->groupClause, subplan);
-
-	}
-
-	/*
-	 * If aggregate is present, insert the agg node
-	 */
-	if (root->qry_aggs)
-	{
-		aggplan = make_agg(tlist, root->qry_numAgg, root->qry_aggs);
-		aggplan->plan.lefttree = subplan;
-
-		/*
-		 * set the varno/attno entries to the appropriate references to
-		 * the result tuple of the subplans. (We need to set those in the
-		 * array of aggreg's in the Agg node also. Even though they're
-		 * pointers, after a few dozen's of copying, they're not the same
-		 * as those in the target list.)
-		 */
-		set_agg_tlist_references(aggplan);
-		set_agg_agglist_references(aggplan);
-
-		subplan = (Plan *) aggplan;
-
-		tlist = aggplan->plan.targetlist;
-	}
-
 	/*
 	 * Build a result node linking the plan if we have constant quals
 	 */
 	if (constant_qual)
 	{
-		Plan	   *plan;
-
-		plan = (Plan *) make_result(tlist,
-									(Node *) constant_qual,
-									subplan);
+		subplan = (Plan *) make_result((!root->hasAggs &&
+									    !root->groupClause &&
+									    !root->havingQual)
+									   ? tlist : subplan->targetlist,
+									   (Node *) constant_qual,
+									   subplan);
 
 		/*
 		 * Change all varno's of the Result's node target list.
 		 */
-		set_result_tlist_references((Result *) plan);
+		if (!root->hasAggs && !root->groupClause && !root->havingQual)
+			set_tlist_references(subplan);
 
-		return (plan);
+		return subplan;
 	}
 
 	/*
@@ -250,19 +216,23 @@ query_planner(Query *root,
 	 * aggregates fixing only other entries (i.e. - GroupBy-ed and so
 	 * fixed by make_groupPlan).	 - vadim 04/05/97
 	 */
-	if (root->groupClause == NULL && aggplan == NULL)
+	else
 	{
-		subplan->targetlist = flatten_tlist_vars(tlist,
-												 subplan->targetlist);
+		if (!root->hasAggs && !root->groupClause && !root->havingQual)
+			subplan->targetlist = flatten_tlist_vars(tlist,
+													 subplan->targetlist);
+		return subplan;
 	}
 
+#ifdef NOT_USED
+
 	/*
 	 * Destructively modify the query plan's targetlist to add fjoin lists
 	 * to flatten functions that return sets of base types
 	 */
 	subplan->targetlist = generate_fjoin(subplan->targetlist);
+#endif
 
-	return (subplan);
 }
 
 /*
@@ -361,8 +331,10 @@ make_result(List *tlist,
 	Result	   *node = makeNode(Result);
 	Plan	   *plan = &node->plan;
 
+#ifdef NOT_USED
 	tlist = generate_fjoin(tlist);
-	plan->cost = 0.0;
+#endif
+	plan->cost = (subplan ? subplan->cost : 0);
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->lefttree = subplan;
@@ -377,7 +349,7 @@ make_result(List *tlist,
  *
  *****************************************************************************/
 
-static Plan *
+Plan *
 make_groupPlan(List **tlist,
 			   bool tuplePerGroup,
 			   List *groupClause,
@@ -411,9 +383,9 @@ make_groupPlan(List **tlist,
 	 */
 	foreach(sl, sort_tlist)
 	{
-		Resdom		   *resdom = NULL;
-		TargetEntry	   *te = (TargetEntry *) lfirst(sl);
-		int				keyno = 0;
+		Resdom	   *resdom = NULL;
+		TargetEntry *te = (TargetEntry *) lfirst(sl);
+		int			keyno = 0;
 
 		foreach(gl, groupClause)
 		{
@@ -452,7 +424,7 @@ make_groupPlan(List **tlist,
 
 	if (length(glc) != 0)
 	{
-		elog(WARN, "group attribute disappeared from target list");
+		elog(ERROR, "group attribute disappeared from target list");
 	}
 
 	/*
@@ -509,7 +481,8 @@ make_groupPlan(List **tlist,
 		else
 			te->expr = (Node *) makeVar(1, resdom->resno,
 										resdom->restype,
-										-1, resdom->resno);
+										resdom->restypmod,
+										0, -1, resdom->resno);
 	}
 
 	sortplan = make_sort(sort_tlist,
