@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/setrefs.c,v 1.8 1997/09/08 21:45:28 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/setrefs.c,v 1.21 1998/04/15 15:29:44 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -44,9 +44,10 @@ replace_subclause_joinvar_refs(List *clauses,
 							   List *outer_tlist, List *inner_tlist);
 static Var *replace_joinvar_refs(Var *var, List *outer_tlist, List *inner_tlist);
 static List *tlist_temp_references(Oid tempid, List *tlist);
-static void replace_result_clause(List *clause, List *subplanTargetList);
+static void replace_result_clause(Node *clause, List *subplanTargetList);
 static bool OperandIsInner(Node *opnd, int inner_relid);
-static void replace_agg_clause(Node *expr, List *targetlist);
+static List *replace_agg_clause(Node *expr, List *targetlist);
+static Node *del_agg_clause(Node *clause);
 
 /*****************************************************************************
  *
@@ -200,7 +201,7 @@ set_temp_tlist_references(Temp *temp)
 	}
 	else
 	{
-		elog(WARN, "calling set_temp_tlist_references with empty lefttree");
+		elog(ERROR, "calling set_temp_tlist_references with empty lefttree");
 	}
 }
 
@@ -217,7 +218,7 @@ set_temp_tlist_references(Temp *temp)
  * Returns the new join clauses.
  *
  */
-List	   *
+List *
 join_references(List *clauses,
 				List *outer_tlist,
 				List *inner_tlist)
@@ -243,7 +244,7 @@ join_references(List *clauses,
  * Returns the new list of clauses.
  *
  */
-List	   *
+List *
 index_outerjoin_references(List *inner_indxqual,
 						   List *outer_tlist,
 						   Index inner_relid)
@@ -328,6 +329,15 @@ replace_clause_joinvar_refs(Expr *clause,
 	{
 		return ((List *) clause);
 	}
+	else if (and_clause((Node *) clause))
+	{
+		List	   *andclause =
+		replace_subclause_joinvar_refs(((Expr *) clause)->args,
+									   outer_tlist,
+									   inner_tlist);
+
+		return ((List *) make_andclause(andclause));
+	}
 	else if (or_clause((Node *) clause))
 	{
 		List	   *orclause =
@@ -397,7 +407,21 @@ replace_clause_joinvar_refs(Expr *clause,
 									   leftvar,
 									   rightvar));
 	}
+	else if (is_subplan(clause))
+	{
+		((Expr *) clause)->args =
+			replace_subclause_joinvar_refs(((Expr *) clause)->args,
+										   outer_tlist,
+										   inner_tlist);
+		((SubPlan *) ((Expr *) clause)->oper)->sublink->oper =
+			replace_subclause_joinvar_refs(((SubPlan *) ((Expr *) clause)->oper)->sublink->oper,
+										   outer_tlist,
+										   inner_tlist);
+		return ((List *) clause);
+	}
 	/* shouldn't reach here */
+	elog(ERROR, "replace_clause_joinvar_refs: unsupported clause %d",
+		 nodeTag(clause));
 	return NULL;
 }
 
@@ -432,6 +456,8 @@ replace_joinvar_refs(Var *var, List *outer_tlist, List *inner_tlist)
 		return (makeVar(OUTER,
 						outer_resdom->resno,
 						var->vartype,
+						var->vartypmod,
+						0,
 						var->varnoold,
 						var->varoattno));
 	}
@@ -445,6 +471,8 @@ replace_joinvar_refs(Var *var, List *outer_tlist, List *inner_tlist)
 			return (makeVar(INNER,
 							inner_resdom->resno,
 							var->vartype,
+							var->vartypmod,
+							0,
 							var->varnoold,
 							var->varoattno));
 		}
@@ -489,6 +517,8 @@ tlist_temp_references(Oid tempid,
 					   (Node *) makeVar(tempid,
 										xtl->resdom->resno,
 										xtl->resdom->restype,
+										xtl->resdom->restypmod,
+										0,
 										tempid,
 										oattno));
 
@@ -528,13 +558,9 @@ set_result_tlist_references(Result *resultNode)
 	 */
 	subplan = ((Plan *) resultNode)->lefttree;
 	if (subplan != NULL)
-	{
 		subplanTargetList = subplan->targetlist;
-	}
 	else
-	{
 		subplanTargetList = NIL;
-	}
 
 	/*
 	 * now for traverse all the entris of the target list. These should be
@@ -546,7 +572,7 @@ set_result_tlist_references(Result *resultNode)
 	{
 		entry = (TargetEntry *) lfirst(t);
 		expr = (Expr *) get_expr(entry);
-		replace_result_clause((List *) expr, subplanTargetList);
+		replace_result_clause((Node *) expr, subplanTargetList);
 	}
 }
 
@@ -560,16 +586,15 @@ set_result_tlist_references(Result *resultNode)
  *
  */
 static void
-replace_result_clause(List *clause,
+replace_result_clause(Node *clause,
 					  List *subplanTargetList)	/* target list of the
 												 * subplan */
 {
 	List	   *t;
-	List	   *subClause;
-	TargetEntry *subplanVar;
 
 	if (IsA(clause, Var))
 	{
+		TargetEntry *subplanVar;
 
 		/*
 		 * Ha! A Var node!
@@ -583,15 +608,20 @@ replace_result_clause(List *clause,
 		((Var *) clause)->varno = (Index) OUTER;
 		((Var *) clause)->varattno = subplanVar->resdom->resno;
 	}
-	else if (is_funcclause((Node *) clause))
+	else if (IsA(clause, Aggreg))
 	{
+		replace_result_clause(((Aggreg *) clause)->target, subplanTargetList);
+	}
+	else if (is_funcclause(clause))
+	{
+		List	   *subExpr;
 
 		/*
 		 * This is a function. Recursively call this routine for its
 		 * arguments...
 		 */
-		subClause = ((Expr *) clause)->args;
-		foreach(t, subClause)
+		subExpr = ((Expr *) clause)->args;
+		foreach(t, subExpr)
 		{
 			replace_result_clause(lfirst(t), subplanTargetList);
 		}
@@ -604,32 +634,31 @@ replace_result_clause(List *clause,
 		 * This is an arrayref. Recursively call this routine for its
 		 * expression and its index expression...
 		 */
-		subClause = aref->refupperindexpr;
-		foreach(t, subClause)
+		foreach(t, aref->refupperindexpr)
 		{
 			replace_result_clause(lfirst(t), subplanTargetList);
 		}
-		subClause = aref->reflowerindexpr;
-		foreach(t, subClause)
+		foreach(t, aref->reflowerindexpr)
 		{
 			replace_result_clause(lfirst(t), subplanTargetList);
 		}
-		replace_result_clause((List *) aref->refexpr,
+		replace_result_clause(aref->refexpr,
 							  subplanTargetList);
-		replace_result_clause((List *) aref->refassgnexpr,
+		replace_result_clause(aref->refassgnexpr,
 							  subplanTargetList);
 	}
-	else if (is_opclause((Node *) clause))
+	else if (is_opclause(clause))
 	{
+		Node	   *subNode;
 
 		/*
 		 * This is an operator. Recursively call this routine for both its
 		 * left and right operands
 		 */
-		subClause = (List *) get_leftop((Expr *) clause);
-		replace_result_clause(subClause, subplanTargetList);
-		subClause = (List *) get_rightop((Expr *) clause);
-		replace_result_clause(subClause, subplanTargetList);
+		subNode = (Node *) get_leftop((Expr *) clause);
+		replace_result_clause(subNode, subplanTargetList);
+		subNode = (Node *) get_rightop((Expr *) clause);
+		replace_result_clause(subNode, subplanTargetList);
 	}
 	else if (IsA(clause, Param) ||IsA(clause, Const))
 	{
@@ -641,7 +670,7 @@ replace_result_clause(List *clause,
 		/*
 		 * Ooops! we can not handle that!
 		 */
-		elog(WARN, "replace_result_clause: Can not handle this tlist!\n");
+		elog(ERROR, "replace_result_clause: Can not handle this tlist!\n");
 	}
 }
 
@@ -684,13 +713,16 @@ OperandIsInner(Node *opnd, int inner_relid)
  *	  changes the target list of an Agg node so that it points to
  *	  the tuples returned by its left tree subplan.
  *
+ *	We now also generate a linked list of Aggreg pointers for Agg.
+ *
  */
-void
+List *
 set_agg_tlist_references(Agg *aggNode)
 {
 	List	   *aggTargetList;
 	List	   *subplanTargetList;
 	List	   *tl;
+	List	   *aggreg_list = NIL;
 
 	aggTargetList = aggNode->plan.targetlist;
 	subplanTargetList = aggNode->plan.lefttree->targetlist;
@@ -699,34 +731,21 @@ set_agg_tlist_references(Agg *aggNode)
 	{
 		TargetEntry *tle = lfirst(tl);
 
-		replace_agg_clause(tle->expr, subplanTargetList);
+		aggreg_list = nconc(
+		  replace_agg_clause(tle->expr, subplanTargetList), aggreg_list);
 	}
+	return aggreg_list;
 }
 
-void
-set_agg_agglist_references(Agg *aggNode)
-{
-	List	   *subplanTargetList;
-	Aggreg	  **aggs;
-	int			i;
-
-	aggs = aggNode->aggs;
-	subplanTargetList = aggNode->plan.lefttree->targetlist;
-
-	for (i = 0; i < aggNode->numAgg; i++)
-	{
-		replace_agg_clause(aggs[i]->target, subplanTargetList);
-	}
-}
-
-static void
+static List *
 replace_agg_clause(Node *clause, List *subplanTargetList)
 {
 	List	   *t;
-	TargetEntry *subplanVar;
+	List	   *agg_list = NIL;
 
 	if (IsA(clause, Var))
 	{
+		TargetEntry *subplanVar;
 
 		/*
 		 * Ha! A Var node!
@@ -738,6 +757,8 @@ replace_agg_clause(Node *clause, List *subplanTargetList)
 		 *
 		 */
 		((Var *) clause)->varattno = subplanVar->resdom->resno;
+
+		return NIL;
 	}
 	else if (is_funcclause(clause))
 	{
@@ -748,12 +769,15 @@ replace_agg_clause(Node *clause, List *subplanTargetList)
 		 */
 		foreach(t, ((Expr *) clause)->args)
 		{
-			replace_agg_clause(lfirst(t), subplanTargetList);
+			agg_list = nconc(agg_list,
+					   replace_agg_clause(lfirst(t), subplanTargetList));
 		}
+		return agg_list;
 	}
 	else if (IsA(clause, Aggreg))
 	{
-		replace_agg_clause(((Aggreg *) clause)->target, subplanTargetList);
+		return lcons(clause,
+					 replace_agg_clause(((Aggreg *) clause)->target, subplanTargetList));
 	}
 	else if (IsA(clause, ArrayRef))
 	{
@@ -765,14 +789,20 @@ replace_agg_clause(Node *clause, List *subplanTargetList)
 		 */
 		foreach(t, aref->refupperindexpr)
 		{
-			replace_agg_clause(lfirst(t), subplanTargetList);
+			agg_list = nconc(agg_list,
+					   replace_agg_clause(lfirst(t), subplanTargetList));
 		}
 		foreach(t, aref->reflowerindexpr)
 		{
-			replace_agg_clause(lfirst(t), subplanTargetList);
+			agg_list = nconc(agg_list,
+					   replace_agg_clause(lfirst(t), subplanTargetList));
 		}
-		replace_agg_clause(aref->refexpr, subplanTargetList);
-		replace_agg_clause(aref->refassgnexpr, subplanTargetList);
+		agg_list = nconc(agg_list,
+				   replace_agg_clause(aref->refexpr, subplanTargetList));
+		agg_list = nconc(agg_list,
+			  replace_agg_clause(aref->refassgnexpr, subplanTargetList));
+
+		return agg_list;
 	}
 	else if (is_opclause(clause))
 	{
@@ -785,13 +815,18 @@ replace_agg_clause(Node *clause, List *subplanTargetList)
 		Node	   *right = (Node *) get_rightop((Expr *) clause);
 
 		if (left != (Node *) NULL)
-			replace_agg_clause(left, subplanTargetList);
+			agg_list = nconc(agg_list,
+							 replace_agg_clause(left, subplanTargetList));
 		if (right != (Node *) NULL)
-			replace_agg_clause(right, subplanTargetList);
+			agg_list = nconc(agg_list,
+						   replace_agg_clause(right, subplanTargetList));
+
+		return agg_list;
 	}
 	else if (IsA(clause, Param) ||IsA(clause, Const))
 	{
 		/* do nothing! */
+		return NIL;
 	}
 	else
 	{
@@ -799,7 +834,210 @@ replace_agg_clause(Node *clause, List *subplanTargetList)
 		/*
 		 * Ooops! we can not handle that!
 		 */
-		elog(WARN, "replace_agg_clause: Can not handle this tlist!\n");
+		elog(ERROR, "replace_agg_clause: Can not handle this tlist!\n");
+		return NIL;
 	}
-
 }
+
+
+/*
+ * del_agg_tlist_references
+ *	  Remove the Agg nodes from the target list
+ *	  We do this so inheritance only does aggregates in the upper node
+ */
+void
+del_agg_tlist_references(List *tlist)
+{
+	List	   *tl;
+
+	foreach(tl, tlist)
+	{
+		TargetEntry *tle = lfirst(tl);
+
+		tle->expr = del_agg_clause(tle->expr);
+	}
+}
+
+static Node *
+del_agg_clause(Node *clause)
+{
+	List	   *t;
+
+	if (IsA(clause, Var))
+	{
+		return clause;
+	}
+	else if (is_funcclause(clause))
+	{
+
+		/*
+		 * This is a function. Recursively call this routine for its
+		 * arguments...
+		 */
+		foreach(t, ((Expr *) clause)->args)
+		{
+			lfirst(t) = del_agg_clause(lfirst(t));
+		}
+	}
+	else if (IsA(clause, Aggreg))
+	{
+
+		/* here is the real action, to remove the Agg node */
+		return del_agg_clause(((Aggreg *) clause)->target);
+
+	}
+	else if (IsA(clause, ArrayRef))
+	{
+		ArrayRef   *aref = (ArrayRef *) clause;
+
+		/*
+		 * This is an arrayref. Recursively call this routine for its
+		 * expression and its index expression...
+		 */
+		foreach(t, aref->refupperindexpr)
+		{
+			lfirst(t) = del_agg_clause(lfirst(t));
+		}
+		foreach(t, aref->reflowerindexpr)
+		{
+			lfirst(t) = del_agg_clause(lfirst(t));
+		}
+		aref->refexpr = del_agg_clause(aref->refexpr);
+		aref->refassgnexpr = del_agg_clause(aref->refassgnexpr);
+	}
+	else if (is_opclause(clause))
+	{
+
+		/*
+		 * This is an operator. Recursively call this routine for both its
+		 * left and right operands
+		 */
+		Node	   *left = (Node *) get_leftop((Expr *) clause);
+		Node	   *right = (Node *) get_rightop((Expr *) clause);
+
+		if (left != (Node *) NULL)
+			left = del_agg_clause(left);
+		if (right != (Node *) NULL)
+			right = del_agg_clause(right);
+	}
+	else if (IsA(clause, Param) ||IsA(clause, Const))
+	{
+		return clause;
+	}
+	else
+	{
+
+		/*
+		 * Ooops! we can not handle that!
+		 */
+		elog(ERROR, "del_agg_clause: Can not handle this tlist!\n");
+	}
+	return NULL;
+}
+
+
+List *
+check_having_qual_for_aggs(Node *clause, List *subplanTargetList)
+{
+	List	   *t;
+	List	   *agg_list = NIL;
+
+	if (IsA(clause, Var))
+	{
+	  TargetEntry *subplanVar;
+	  
+	  /*
+	   * Ha! A Var node!
+	   */
+	  subplanVar = match_varid((Var *) clause, subplanTargetList);
+	  
+	  /*
+	   * Change the varno & varattno fields of the var node.
+	   *
+	   */
+	  ((Var *) clause)->varattno = subplanVar->resdom->resno;
+	  return NIL;
+	}
+	else if (is_funcclause(clause) || not_clause(clause) || 
+		 or_clause(clause) || and_clause(clause))
+	{
+
+		/*
+		 * This is a function. Recursively call this routine for its
+		 * arguments...
+		 */
+		foreach(t, ((Expr *) clause)->args)
+		{
+			agg_list = nconc(agg_list,
+					   check_having_qual_for_aggs(lfirst(t), subplanTargetList));
+		}
+		return agg_list;
+	}
+	else if (IsA(clause, Aggreg))
+	{
+		return lcons(clause,
+		    check_having_qual_for_aggs(((Aggreg *) clause)->target, subplanTargetList));
+		
+	}
+	else if (IsA(clause, ArrayRef))
+	{
+		ArrayRef   *aref = (ArrayRef *) clause;
+
+		/*
+		 * This is an arrayref. Recursively call this routine for its
+		 * expression and its index expression...
+		 */
+		foreach(t, aref->refupperindexpr)
+		{
+			agg_list = nconc(agg_list,
+					 check_having_qual_for_aggs(lfirst(t), subplanTargetList));
+		}
+		foreach(t, aref->reflowerindexpr)
+		{
+			agg_list = nconc(agg_list,
+					 check_having_qual_for_aggs(lfirst(t), subplanTargetList));
+		}
+		agg_list = nconc(agg_list,
+				 check_having_qual_for_aggs(aref->refexpr, subplanTargetList));
+		agg_list = nconc(agg_list,
+				 check_having_qual_for_aggs(aref->refassgnexpr, subplanTargetList));
+
+		return agg_list;
+	}
+	else if (is_opclause(clause))
+	{
+
+		/*
+		 * This is an operator. Recursively call this routine for both its
+		 * left and right operands
+		 */
+		Node	   *left = (Node *) get_leftop((Expr *) clause);
+		Node	   *right = (Node *) get_rightop((Expr *) clause);
+
+		if (left != (Node *) NULL)
+			agg_list = nconc(agg_list,
+					 check_having_qual_for_aggs(left, subplanTargetList));
+		if (right != (Node *) NULL)
+			agg_list = nconc(agg_list,
+					 check_having_qual_for_aggs(right, subplanTargetList));
+
+		return agg_list;
+	}
+	else if (IsA(clause, Param) ||IsA(clause, Const))
+	{
+		/* do nothing! */
+		return NIL;
+	}
+	else
+	{
+
+		/*
+		 * Ooops! we can not handle that!
+		 */
+		elog(ERROR, "check_having_qual_for_aggs: Can not handle this having_qual!\n");
+		return NIL;
+	}
+}
+
+
+
