@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/prepqual.c,v 1.6 1997/09/08 21:45:31 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/prepqual.c,v 1.8 1998/02/26 04:32:57 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -36,42 +36,6 @@ static List *qualcleanup(Expr *qual);
 static List *remove_ands(Expr *qual);
 static List *remove_duplicates(List *list);
 
-/*
- * preprocess-qualification--
- *	  Driver routine for modifying the parse tree qualification.
- *
- * Returns the new base qualification and the existential qualification
- * in existentialQualPtr.
- *
- *	XXX right now, update_clauses() does nothing so
- *		preprocess-qualification simply converts the qual in conjunctive
- *		normal form  (see cnfify() below )
- */
-List	   *
-preprocess_qualification(Expr *qual, List *tlist, List **existentialQualPtr)
-{
-	List	   *cnf_qual = cnfify(qual, true);
-
-/*
-	List *existential_qual =
-		update_clauses(intCons(_query_result_relation_,
-								update_relations(tlist)),
-					   cnf_qual,
-					   _query_command_type_);
-	if (existential_qual) {
-		*existentialQualPtr = existential_qual;
-		return set_difference(cnf_qual, existential_qual);
-	} else {
-		*existentialQualPtr = NIL;
-		return cnf_qual;
-	}
-*/
-	/* update_clauses() is not working right now */
-	*existentialQualPtr = NIL;
-	return cnf_qual;
-
-}
-
 /*****************************************************************************
  *
  *		CNF CONVERSION ROUTINES
@@ -100,7 +64,7 @@ preprocess_qualification(Expr *qual, List *tlist, List **existentialQualPtr)
  *		and from the rule manager (removeAndFlag = false).
  *
  */
-List	   *
+List *
 cnfify(Expr *qual, bool removeAndFlag)
 {
 	Expr	   *newqual = NULL;
@@ -607,27 +571,6 @@ remove_ands(Expr *qual)
 	}
 }
 
-/*****************************************************************************
- *
- *		EXISTENTIAL QUALIFICATIONS
- *
- *****************************************************************************/
-
-/*
- * update-relations--
- *	  Returns the range table indices (i.e., varnos) for all relations which
- *	  are referenced in the target list.
- *
- */
-#ifdef NOT_USED
-static List *
-update_relations(List *tlist)
-{
-	return (NIL);
-}
-
-#endif
-
 /*****************************************************************************
  *
  *
