@@ -7,13 +7,14 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/preptlist.c,v 1.5 1997/09/08 21:45:36 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/preptlist.c,v 1.10 1998/02/26 04:33:02 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include <string.h>
 #include "postgres.h"
 
+#include "catalog/pg_type.h"
 #include "nodes/pg_list.h"
 #include "nodes/relation.h"
 #include "nodes/primnodes.h"
@@ -24,9 +25,9 @@
 #include "utils/builtins.h"
 #include "utils/lsyscache.h"
 #include "utils/palloc.h"
+#include "parser/parse_type.h"
 
 #include "parser/parsetree.h"	/* for getrelid() */
-#include "parser/catalog_utils.h"
 
 #include "optimizer/internal.h"
 #include "optimizer/prep.h"
@@ -54,7 +55,7 @@ new_relation_targetlist(Oid relid, Index rt_index,
  *
  *	  Returns the new targetlist.
  */
-List	   *
+List *
 preprocess_targetlist(List *tlist,
 					  int command_type,
 					  Index result_relation,
@@ -107,13 +108,13 @@ preprocess_targetlist(List *tlist,
 
 		resdom = makeResdom(length(t_list) + 1,
 							27,
-							6,
+							-1,
 							"ctid",
 							0,
 							0,
 							1);
 
-		var = makeVar(result_relation, -1, 27, result_relation, -1);
+		var = makeVar(result_relation, -1, 27, -1, 0, result_relation, -1);
 
 		ctid = makeNode(TargetEntry);
 		ctid->resdom = resdom;
@@ -259,31 +260,24 @@ new_relation_targetlist(Oid relid, Index rt_index, NodeTag node_type)
 	AttrNumber	attno;
 	List	   *t_list = NIL;
 	char	   *attname;
+	int			typlen;
 	Oid			atttype = 0;
-	int16		typlen = 0;
 	bool		attisset = false;
 
-/*	  Oid type_id; */
-/*	  type_id = RelationIdGetTypeId(relid); */
-
 	for (attno = 1; attno <= get_relnatts(relid); attno++)
 	{
-		attname = get_attname( /* type_id, */ relid, attno);
-		atttype = get_atttype( /* type_id, */ relid, attno);
+		attname = get_attname(relid, attno);
+		atttype = get_atttype(relid, attno);
 
 		/*
 		 * Since this is an append or replace, the size of any set
 		 * attribute is the size of the OID used to represent it.
 		 */
-		attisset = get_attisset( /* type_id, */ relid, attname);
+		attisset = get_attisset(relid, attname);
 		if (attisset)
-		{
-			typlen = tlen(type("oid"));
-		}
+			typlen = typeLen(typeidType(OIDOID));
 		else
-		{
 			typlen = get_typlen(atttype);
-		}
 
 		switch (node_type)
 		{
@@ -303,14 +297,14 @@ new_relation_targetlist(Oid relid, Index rt_index, NodeTag node_type)
 									  temp,
 									  (Datum) typedefault,
 								(typedefault == (struct varlena *) NULL),
-					/* XXX this is bullshit */
+					/* XXX ? */
 									  false,
 									  false,	/* not a set */
 									  false);
 
 					temp3 = MakeTLE(makeResdom(attno,
 											   atttype,
-											   typlen,
+											   -1,
 											   attname,
 											   0,
 											   (Oid) 0,
@@ -325,11 +319,13 @@ new_relation_targetlist(Oid relid, Index rt_index, NodeTag node_type)
 					TargetEntry *temp_list = NULL;
 
 					temp_var =
-						makeVar(rt_index, attno, atttype, rt_index, attno);
+						makeVar(rt_index, attno, atttype,
+								get_atttypmod(relid, attno),
+								0, rt_index, attno);
 
 					temp_list = MakeTLE(makeResdom(attno,
 												   atttype,
-												   typlen,
+											 get_atttypmod(relid, attno),
 												   attname,
 												   0,
 												   (Oid) 0,
