@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/clauses.c,v 1.12 1997/09/25 12:48:15 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/clauses.c,v 1.16 1998/02/26 04:33:11 momjian Exp $
  *
  * HISTORY
  *	  AUTHOR			DATE			MAJOR EVENT
@@ -22,6 +22,7 @@
 #include "nodes/primnodes.h"
 #include "nodes/relation.h"
 #include "nodes/parsenodes.h"
+#include "nodes/plannodes.h"
 #include "nodes/makefuncs.h"
 #include "nodes/nodeFuncs.h"
 
@@ -37,7 +38,7 @@
 static bool agg_clause(Node *clause);
 
 
-Expr	   *
+Expr *
 make_clause(int type, Node *oper, List *args)
 {
 	if (type == AND_EXPR || type == OR_EXPR || type == NOT_EXPR ||
@@ -57,6 +58,7 @@ make_clause(int type, Node *oper, List *args)
 	}
 	else
 	{
+		elog(ERROR, "make_clause: unsupported type %d", type);
 		/* will this ever happen? translated from lispy C code - ay 10/94 */
 		return ((Expr *) args);
 	}
@@ -92,7 +94,7 @@ is_opclause(Node *clause)
  *	  operand (if it is non-null).
  *
  */
-Expr	   *
+Expr *
 make_opclause(Oper *op, Var *leftop, Var *rightop)
 {
 	Expr	   *expr = makeNode(Expr);
@@ -111,7 +113,7 @@ make_opclause(Oper *op, Var *leftop, Var *rightop)
  *		or (op expr)
  * NB: it is assumed (for now) that all expr must be Var nodes
  */
-Var		   *
+Var *
 get_leftop(Expr *clause)
 {
 	if (clause->args != NULL)
@@ -126,7 +128,7 @@ get_leftop(Expr *clause)
  * Returns the right operand in a clause of the form (op expr expr).
  *
  */
-Var		   *
+Var *
 get_rightop(Expr *clause)
 {
 	if (clause->args != NULL && lnext(clause->args) != NULL)
@@ -171,7 +173,7 @@ is_funcclause(Node *clause)
  * arguments.
  *
  */
-Expr	   *
+Expr *
 make_funcclause(Func *func, List *funcargs)
 {
 	Expr	   *expr = makeNode(Expr);
@@ -207,7 +209,7 @@ or_clause(Node *clause)
  * Creates an 'or' clause given a list of its subclauses.
  *
  */
-Expr	   *
+Expr *
 make_orclause(List *orclauses)
 {
 	Expr	   *expr = makeNode(Expr);
@@ -243,7 +245,7 @@ not_clause(Node *clause)
  * Create a 'not' clause given the expression to be negated.
  *
  */
-Expr	   *
+Expr *
 make_notclause(Expr *notclause)
 {
 	Expr	   *expr = makeNode(Expr);
@@ -261,7 +263,7 @@ make_notclause(Expr *notclause)
  * Retrieve the clause within a 'not' clause
  *
  */
-Expr	   *
+Expr *
 get_notclausearg(Expr *notclause)
 {
 	return (lfirst(notclause->args));
@@ -292,7 +294,7 @@ and_clause(Node *clause)
  * Create an 'and' clause given its arguments in a list.
  *
  */
-Expr	   *
+Expr *
 make_andclause(List *andclauses)
 {
 	Expr	   *expr = makeNode(Expr);
@@ -320,7 +322,7 @@ make_andclause(List *andclauses)
  * quals as the return value.
  *
  */
-List	   *
+List *
 pull_constant_clauses(List *quals, List **constantQual)
 {
 	List	   *q;
@@ -376,6 +378,7 @@ clause_relids_vars(Node *clause, List **relids, List **vars)
 		{
 			Var		   *in_list = (Var *) lfirst(vi);
 
+			Assert(var->varlevelsup == 0);
 			if (in_list->varno == var->varno &&
 				in_list->varattno == var->varattno)
 				break;
@@ -431,7 +434,7 @@ contains_not(Node *clause)
 	if (not_clause(clause))
 		return (true);
 
-	if (or_clause(clause))
+	if (or_clause(clause) || and_clause(clause))
 	{
 		List	   *a;
 
@@ -491,6 +494,7 @@ qual_clause_p(Node *clause)
 	if (!is_opclause(clause))
 		return false;
 
+	/* How about Param-s ?	- vadim 02/03/98 */
 	if (IsA(get_leftop((Expr *) clause), Var) &&
 		IsA(get_rightop((Expr *) clause), Const))
 	{
@@ -518,7 +522,7 @@ fix_opid(Node *clause)
 	{
 		;
 	}
-	else if (or_clause(clause))
+	else if (or_clause(clause) || and_clause(clause))
 	{
 		fix_opids(((Expr *) clause)->args);
 	}
@@ -549,6 +553,17 @@ fix_opid(Node *clause)
 	{
 		fix_opid(((Aggreg *) clause)->target);
 	}
+	else if (is_subplan(clause) &&
+			 ((SubPlan *) ((Expr *) clause)->oper)->sublink->subLinkType != EXISTS_SUBLINK)
+	{
+		List	   *lst;
+
+		foreach(lst, ((SubPlan *) ((Expr *) clause)->oper)->sublink->oper)
+		{
+			replace_opid((Oper *) ((Expr *) lfirst(lst))->oper);
+			fix_opid((Node *) get_leftop((Expr *) lfirst(lst)));
+		}
+	}
 
 }
 
@@ -559,7 +574,7 @@ fix_opid(Node *clause)
  * Returns its argument.
  *
  */
-List	   *
+List *
 fix_opids(List *clauses)
 {
 	List	   *clause;
@@ -626,23 +641,20 @@ get_relattval(Node *clause,
 			*flag = (_SELEC_CONSTANT_RIGHT_ | _SELEC_NOT_CONSTANT_);
 
 		}
-#ifdef INDEXSCAN_PATCH
 	}
 	else if (is_opclause(clause) && IsA(left, Var) &&IsA(right, Param))
 	{
-		/* Function parameter used as index scan arg.  DZ - 27-8-1996 */
 		*relid = left->varno;
 		*attno = left->varattno;
 		*constval = 0;
 		*flag = (_SELEC_NOT_CONSTANT_);
-#endif
 	}
 	else if (is_opclause(clause) &&
 			 is_funcclause((Node *) left) &&
 			 IsA(right, Const))
 	{
-		List	   *vars = pull_var_clause((Node*)left);
-		
+		List	   *vars = pull_var_clause((Node *) left);
+
 		*relid = ((Var *) lfirst(vars))->varno;
 		*attno = InvalidAttrNumber;
 		*constval = ((Const *) right)->constvalue;
@@ -652,8 +664,8 @@ get_relattval(Node *clause,
 			 is_funcclause((Node *) right) &&
 			 IsA(left, Const))
 	{
-		List	   *vars = pull_var_clause((Node*)right);
-		
+		List	   *vars = pull_var_clause((Node *) right);
+
 		*relid = ((Var *) lfirst(vars))->varno;
 		*attno = InvalidAttrNumber;
 		*constval = ((Const *) left)->constvalue;
@@ -679,16 +691,13 @@ get_relattval(Node *clause,
 			*constval = 0;
 			*flag = (_SELEC_NOT_CONSTANT_);
 		}
-#ifdef INDEXSCAN_PATCH
 	}
 	else if (is_opclause(clause) && IsA(right, Var) &&IsA(left, Param))
 	{
-		/* ...And here... - vadim 01/22/97 */
 		*relid = right->varno;
 		*attno = right->varattno;
 		*constval = 0;
 		*flag = (_SELEC_NOT_CONSTANT_);
-#endif
 	}
 	else
 	{
