@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/tlist.c,v 1.7 1997/09/08 21:45:55 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/tlist.c,v 1.12 1998/02/26 04:33:22 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,7 +26,6 @@
 #include "optimizer/clauses.h"
 
 #include "nodes/makefuncs.h"
-#include "parser/catalog_utils.h"
 
 static Node *flatten_tlistentry(Node *tlistentry, List *flat_tlist);
 
@@ -69,7 +68,7 @@ tlistentry_member(Var *var, List *targetlist)
  * REQUIRES: "test" operates on lispval unions,
  *
  */
-Expr	   *
+Expr *
 matching_tlvar(Var *var, List *targetlist)
 {
 	TargetEntry *tlentry;
@@ -109,6 +108,8 @@ add_tl_element(Rel *rel, Var *var)
 		Var		   *newvar = makeVar(var->varno,
 									 var->varattno,
 									 var->vartype,
+									 var->vartypmod,
+									 var->varlevelsup,
 									 var->varno,
 									 var->varoattno);
 
@@ -137,7 +138,7 @@ create_tl_element(Var *var, int resdomno)
 	tlelement->resdom =
 		makeResdom(resdomno,
 				   var->vartype,
-				   get_typlen(var->vartype),
+				   var->vartypmod,
 				   NULL,
 				   (Index) 0,
 				   (Oid) 0,
@@ -152,7 +153,7 @@ create_tl_element(Var *var, int resdomno)
  *	  Returns the targetlist elements from a relation tlist.
  *
  */
-List	   *
+List *
 get_actual_tlist(List *tlist)
 {
 
@@ -199,7 +200,7 @@ get_actual_tlist(List *tlist)
  * Returns the resdom entry of the matching var node.
  *
  */
-Resdom	   *
+Resdom *
 tlist_member(Var *var, List *tlist)
 {
 	List	   *i = NIL;
@@ -229,7 +230,7 @@ tlist_member(Var *var, List *tlist)
 /*
  *	 Routine to get the resdom out of a targetlist.
  */
-Resdom	   *
+Resdom *
 tlist_resdom(List *tlist, Resdom *resnode)
 {
 	Resdom	   *resdom = (Resdom *) NULL;
@@ -272,6 +273,7 @@ match_varid(Var *test_var, List *tlist)
 
 	type_var = (Oid) test_var->vartype;
 
+	Assert(test_var->varlevelsup == 0);
 	foreach(tl, tlist)
 	{
 		TargetEntry *entry;
@@ -287,6 +289,7 @@ match_varid(Var *test_var, List *tlist)
 		 * we test the original varno (instead of varno which might be
 		 * changed to INNER/OUTER.
 		 */
+		Assert(tlvar->varlevelsup == 0);
 		if (tlvar->varnoold == test_var->varnoold &&
 			tlvar->varoattno == test_var->varoattno)
 		{
@@ -310,7 +313,7 @@ match_varid(Var *test_var, List *tlist)
  * Returns the resulting target list.
  *
  */
-List	   *
+List *
 new_unsorted_tlist(List *targetlist)
 {
 	List	   *new_targetlist = (List *) copyObject((Node *) targetlist);
@@ -338,7 +341,7 @@ new_unsorted_tlist(List *targetlist)
  * Returns a new target list.
  *
  */
-List	   *
+List *
 copy_vars(List *target, List *source)
 {
 	List	   *result = NIL;
@@ -366,7 +369,7 @@ copy_vars(List *target, List *source)
  * Returns the "flattened" new target list.
  *
  */
-List	   *
+List *
 flatten_tlist(List *tlist)
 {
 	int			last_resdomno = 1;
@@ -397,7 +400,7 @@ flatten_tlist(List *tlist)
 
 			r = makeResdom(last_resdomno,
 						   var->vartype,
-						   get_typlen(var->vartype),
+						   var->vartypmod,
 						   NULL,
 						   (Index) 0,
 						   (Oid) 0,
@@ -422,7 +425,7 @@ flatten_tlist(List *tlist)
  * Returns the modified actual target list.
  *
  */
-List	   *
+List *
 flatten_tlist_vars(List *full_tlist, List *flat_tlist)
 {
 	List	   *x = NIL;
@@ -553,7 +556,7 @@ MakeTLE(Resdom *resdom, Node *expr)
 	return rt;
 }
 
-Var		   *
+Var *
 get_expr(TargetEntry *tle)
 {
 	Assert(tle != NULL);
@@ -590,7 +593,7 @@ AddGroupAttrToTlist(List *tlist, List *grpCl)
 
 			r = makeResdom(last_resdomno,
 						   var->vartype,
-						   get_typlen(var->vartype),
+						   var->vartypmod,
 						   NULL,
 						   (Index) 0,
 						   (Oid) 0,
