@@ -7,21 +7,19 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/parser/keywords.c,v 1.19 1997/09/24 17:49:56 thomas Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/parser/keywords.c,v 1.36 1998/03/18 16:50:21 thomas Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include <ctype.h>
 #include <string.h>
+
 #include "postgres.h"
-
-#include "nodes/pg_list.h"
 #include "nodes/parsenodes.h"
+#include "nodes/pg_list.h"
 #include "parse.h"
-#include "utils/elog.h"
 #include "parser/keywords.h"
-#include "parser/dbcommands.h"	/* createdb, destroydb stop_vacuum */
-
+#include "utils/elog.h"
 
 /*
  * List of (keyword-name, keyword-token-value) pairs.
@@ -32,7 +30,7 @@
 static ScanKeyword ScanKeywords[] = {
 	/* name					value			*/
 	{"abort", ABORT_TRANS},
-	{"acl", ACL},
+	{"action", ACTION},
 	{"add", ADD},
 	{"after", AFTER},
 	{"aggregate", AGGREGATE},
@@ -40,10 +38,8 @@ static ScanKeyword ScanKeywords[] = {
 	{"alter", ALTER},
 	{"analyze", ANALYZE},
 	{"and", AND},
-	{"append", APPEND},
-	{"archIve", ARCHIVE},		/* XXX crooked: I < _ */
-	{"arch_store", ARCH_STORE},
-	{"archive", ARCHIVE},		/* XXX crooked: i > _ */
+	{"any", ANY},
+	{"archive", ARCHIVE},
 	{"as", AS},
 	{"asc", ASC},
 	{"backward", BACKWARD},
@@ -53,25 +49,32 @@ static ScanKeyword ScanKeywords[] = {
 	{"binary", BINARY},
 	{"both", BOTH},
 	{"by", BY},
+	{"cache", CACHE},
+	{"cascade", CASCADE},
 	{"cast", CAST},
-	{"change", CHANGE},
+	{"char", CHAR},
 	{"character", CHARACTER},
 	{"check", CHECK},
 	{"close", CLOSE},
 	{"cluster", CLUSTER},
+	{"collate", COLLATE},
 	{"column", COLUMN},
 	{"commit", COMMIT},
 	{"constraint", CONSTRAINT},
 	{"copy", COPY},
 	{"create", CREATE},
+	{"createdb", CREATEDB},
+	{"createuser", CREATEUSER},
 	{"cross", CROSS},
 	{"current", CURRENT},
 	{"current_date", CURRENT_DATE},
 	{"current_time", CURRENT_TIME},
 	{"current_timestamp", CURRENT_TIMESTAMP},
+	{"current_user", CURRENT_USER},
 	{"cursor", CURSOR},
+	{"cycle", CYCLE},
 	{"database", DATABASE},
-	{"day", DAYINTERVAL},
+	{"day", DAY_P},
 	{"decimal", DECIMAL},
 	{"declare", DECLARE},
 	{"default", DEFAULT},
@@ -82,28 +85,32 @@ static ScanKeyword ScanKeywords[] = {
 	{"do", DO},
 	{"double", DOUBLE},
 	{"drop", DROP},
+	{"each", EACH},
 	{"end", END_TRANS},
 	{"execute", EXECUTE},
 	{"exists", EXISTS},
 	{"explain", EXPLAIN},
 	{"extend", EXTEND},
 	{"extract", EXTRACT},
+	{"false", FALSE_P},
 	{"fetch", FETCH},
 	{"float", FLOAT},
 	{"for", FOR},
+	{"foreign", FOREIGN},
 	{"forward", FORWARD},
 	{"from", FROM},
 	{"full", FULL},
 	{"function", FUNCTION},
 	{"grant", GRANT},
 	{"group", GROUP},
+	{"handler", HANDLER},
 	{"having", HAVING},
-	{"heavy", HEAVY},
-	{"hour", HOURINTERVAL},
+	{"hour", HOUR_P},
 	{"in", IN},
+	{"increment", INCREMENT},
 	{"index", INDEX},
 	{"inherits", INHERITS},
-	{"inner", INNERJOIN},
+	{"inner", INNER_P},
 	{"insert", INSERT},
 	{"instead", INSTEAD},
 	{"interval", INTERVAL},
@@ -111,26 +118,36 @@ static ScanKeyword ScanKeywords[] = {
 	{"is", IS},
 	{"isnull", ISNULL},
 	{"join", JOIN},
+	{"key", KEY},
+	{"lancompiler", LANCOMPILER},
 	{"language", LANGUAGE},
 	{"leading", LEADING},
 	{"left", LEFT},
-	{"light", LIGHT},
 	{"like", LIKE},
 	{"listen", LISTEN},
 	{"load", LOAD},
 	{"local", LOCAL},
-	{"merge", MERGE},
-	{"minute", MINUTEINTERVAL},
-	{"month", MONTHINTERVAL},
+	{"location", LOCATION},
+	{"lock", LOCK_P},
+	{"match", MATCH},
+	{"maxvalue", MAXVALUE},
+	{"minute", MINUTE_P},
+	{"minvalue", MINVALUE},
+	{"month", MONTH_P},
 	{"move", MOVE},
+	{"national", NATIONAL},
 	{"natural", NATURAL},
+	{"nchar", NCHAR},
 	{"new", NEW},
+	{"no", NO},
+	{"nocreatedb", NOCREATEDB},
+	{"nocreateuser", NOCREATEUSER},
 	{"none", NONE},
 	{"not", NOT},
 	{"nothing", NOTHING},
 	{"notify", NOTIFY},
 	{"notnull", NOTNULL},
-	{"null", PNULL},
+	{"null", NULL_P},
 	{"numeric", NUMERIC},
 	{"oids", OIDS},
 	{"on", ON},
@@ -138,32 +155,36 @@ static ScanKeyword ScanKeywords[] = {
 	{"option", OPTION},
 	{"or", OR},
 	{"order", ORDER},
-	{"outer", OUTERJOIN},
+	{"outer", OUTER_P},
+	{"partial", PARTIAL},
+	{"password", PASSWORD},
 	{"position", POSITION},
 	{"precision", PRECISION},
+	{"primary", PRIMARY},
 	{"privileges", PRIVILEGES},
+	{"procedural", PROCEDURAL},
 	{"procedure", PROCEDURE},
 	{"public", PUBLIC},
-	{"purge", PURGE},
 	{"recipe", RECIPE},
+	{"references", REFERENCES},
 	{"rename", RENAME},
-	{"replace", REPLACE},
 	{"reset", RESET},
-	{"retrieve", RETRIEVE},
 	{"returns", RETURNS},
 	{"revoke", REVOKE},
 	{"right", RIGHT},
 	{"rollback", ROLLBACK},
+	{"row", ROW},
 	{"rule", RULE},
-	{"second", SECONDINTERVAL},
+	{"second", SECOND_P},
 	{"select", SELECT},
 	{"sequence", SEQUENCE},
 	{"set", SET},
 	{"setof", SETOF},
 	{"show", SHOW},
+	{"start", START},
+	{"statement", STATEMENT},
 	{"stdin", STDIN},
 	{"stdout", STDOUT},
-	{"store", STORE},
 	{"substring", SUBSTRING},
 	{"table", TABLE},
 	{"time", TIME},
@@ -172,13 +193,19 @@ static ScanKeyword ScanKeywords[] = {
 	{"transaction", TRANSACTION},
 	{"trigger", TRIGGER},
 	{"trim", TRIM},
-	{"type", P_TYPE},
+	{"true", TRUE_P},
+	{"trusted", TRUSTED},
+	{"type", TYPE_P},
 	{"union", UNION},
 	{"unique", UNIQUE},
+	{"until", UNTIL},
 	{"update", UPDATE},
+	{"user", USER},
 	{"using", USING},
 	{"vacuum", VACUUM},
+	{"valid", VALID},
 	{"values", VALUES},
+	{"varchar", VARCHAR},
 	{"varying", VARYING},
 	{"verbose", VERBOSE},
 	{"version", VERSION},
@@ -186,7 +213,7 @@ static ScanKeyword ScanKeywords[] = {
 	{"where", WHERE},
 	{"with", WITH},
 	{"work", WORK},
-	{"year", YEARINTERVAL},
+	{"year", YEAR_P},
 	{"zone", ZONE},
 };
 
@@ -212,22 +239,3 @@ ScanKeywordLookup(char *text)
 
 	return (NULL);
 }
-
-#ifdef NOT_USED
-char	   *
-AtomValueGetString(int atomval)
-{
-	ScanKeyword *low = &ScanKeywords[0];
-	ScanKeyword *high = endof(ScanKeywords) - 1;
-	int			keyword_list_length = (high - low);
-	int			i;
-
-	for (i = 0; i < keyword_list_length; i++)
-		if (ScanKeywords[i].value == atomval)
-			return (ScanKeywords[i].name);
-
-	elog(WARN, "AtomGetString called with bogus atom # : %d", atomval);
-	return (NULL);
-}
-
-#endif
