@@ -36,12 +36,6 @@ putenv(char *name)
 	environ[size] = NULL;
 }
 
-char	   *
-strdup(const char *string)
-{
-	return strcpy(malloc(strlen(string) + 1), string);
-}
-
 #ifndef _POSIX_SOURCE
 int
 sigaddset(int *set, int signo)
@@ -56,7 +50,7 @@ sigemptyset(int *set)
 	return (*set = 0);
 }
 
-char	   *
+char *
 getcwd(char *buf, size_t size)
 {
 	return getwd(buf);
