@@ -10,7 +10,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/postmaster/postmaster.c,v 1.58 1997/09/18 20:21:08 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/postmaster/postmaster.c,v 1.76 1998/03/30 16:47:11 momjian Exp $
  *
  * NOTES
  *
@@ -47,6 +47,7 @@
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
 
 #if !defined(NO_UNISTD_H)
 #include <unistd.h>
@@ -59,7 +60,6 @@
 #include <sys/socket.h>
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
-#define MAXINT		   INT_MAX
 #else
 #include <values.h>
 #endif
@@ -78,6 +78,7 @@
 #include "libpq/auth.h"
 #include "libpq/pqcomm.h"
 #include "libpq/pqsignal.h"
+#include "libpq/crypt.h"
 #include "miscadmin.h"
 #include "version.h"
 #include "lib/dllist.h"
@@ -85,7 +86,10 @@
 #include "utils/mcxt.h"
 #include "storage/proc.h"
 #include "utils/elog.h"
+#ifndef HAVE_GETHOSTNAME
 #include "port-protos.h"		/* For gethostname() */
+#endif
+#include "storage/fd.h"
 
 #if defined(DBX_VERSION)
 #define FORK() (0)
@@ -97,7 +101,12 @@
 #endif
 #endif
 
-#define LINGER_TIME 3
+#if !defined(MAXINT)
+#define MAXINT		   INT_MAX
+#endif
+
+#define INVALID_SOCK	(-1)
+#define ARGV_SIZE	64
 
  /*
   * Max time in seconds for socket to linger (close() to block) waiting
@@ -123,6 +132,37 @@ static Dllist *PortList;
 
 static short PostPortName = -1;
 static short ActiveBackends = FALSE;
+
+ /*
+  * This is a boolean indicating that there is at least one backend that
+  * is accessing the current shared memory and semaphores. Between the
+  * time that we start up, or throw away shared memory segments and start
+  * over, and the time we generate the next backend (because we received a
+  * connection request), it is false. Other times, it is true.
+  */
+static short shmem_seq = 0;
+
+ /*
+  * This is a sequence number that indicates how many times we've had to
+  * throw away the shared memory and start over because we doubted its
+  * integrity.	It starts off at zero and is incremented every time we
+  * start over.  We use this to ensure that we use a new IPC shared memory
+  * key for the new shared memory segment in case the old segment isn't
+  * entirely gone yet.
+  *
+  * The sequence actually cycles back to 0 after 9, so pathologically there
+  * could be an IPC failure if 10 sets of backends are all stuck and won't
+  * release IPC resources.
+  */
+
+static IpcMemoryKey ipc_key;
+
+ /*
+  * This is the base IPC shared memory key.  Other keys are generated by
+  * adding to this.
+  */
+
+
 static int	NextBackendId = MAXINT;		/* XXX why? */
 static char *progname = (char *) NULL;
 
@@ -131,7 +171,8 @@ static char *progname = (char *) NULL;
  */
 static char Execfile[MAXPATHLEN] = "";
 
-static int	ServerSock = INVALID_SOCK;	/* stream socket server */
+static int	ServerSock_INET = INVALID_SOCK;		/* stream socket server */
+static int	ServerSock_UNIX = INVALID_SOCK;		/* stream socket server */
 
 /*
  * Set by the -o option
@@ -148,28 +189,32 @@ static char ExtraOptions[ARGV_SIZE] = "";
 static int	Reinit = 1;
 static int	SendStop = 0;
 
-static int	MultiplexedBackends = 0;
-static int	MultiplexedBackendPort;
+static int	NetServer = 0;		/* if not zero, postmaster listen for
+								 * non-local connections */
 
 /*
  * postmaster.c - function prototypes
  */
 static void pmdaemonize(void);
-static void
-ConnStartup(Port *port, int *status,
-			char *errormsg, const int errormsg_len);
-static int	ConnCreate(int serverFd, int *newFdP);
+static Port *ConnCreate(int serverFd);
 static void reset_shared(short port);
 static void pmdie(SIGNAL_ARGS);
 static void reaper(SIGNAL_ARGS);
 static void dumpstatus(SIGNAL_ARGS);
 static void CleanupProc(int pid, int exitstatus);
-static int	DoExec(StartupInfo *packet, int portFd);
+static int	DoExec(Port *port);
 static void ExitPostmaster(int status);
 static void usage(const char *);
 static int	ServerLoop(void);
-static int	BackendStartup(StartupInfo *packet, Port *port, int *pidPtr);
-static void send_error_reply(Port *port, const char *errormsg);
+static int	BackendStartup(Port *port);
+static void readStartupPacket(char *arg, PacketLen len, char *pkt);
+static int	initMasks(fd_set *rmask, fd_set *wmask);
+static void RandomSalt(char *salt);
+
+#ifdef CYR_RECODE
+void		GetCharSetByHost(char *, int, char *);
+
+#endif
 
 extern char *optarg;
 extern int	optind,
@@ -196,7 +241,7 @@ checkDataDir(const char *DataDir, bool *DataDirOK)
 
 		sprintf(path, "%s%cbase%ctemplate1%cpg_class",
 				DataDir, SEP_CHAR, SEP_CHAR, SEP_CHAR);
-		fp = fopen(path, "r");
+		fp = AllocateFile(path, "r");
 		if (fp == NULL)
 		{
 			fprintf(stderr, "%s does not find the database system.  "
@@ -212,7 +257,7 @@ checkDataDir(const char *DataDir, bool *DataDirOK)
 
 			/* reason ValidatePgVersion failed.  NULL if didn't */
 
-			fclose(fp);
+			FreeFile(fp);
 
 			ValidatePgVersion(DataDir, &reason);
 			if (reason)
@@ -244,8 +289,8 @@ PostmasterMain(int argc, char *argv[])
 	char	   *hostName;
 	int			status;
 	int			silentflag = 0;
-	char		hostbuf[MAXHOSTNAMELEN];
 	bool		DataDirOK;		/* We have a usable PGDATA value */
+	char		hostbuf[MAXHOSTNAMELEN];
 
 	progname = argv[0];
 
@@ -267,13 +312,12 @@ PostmasterMain(int argc, char *argv[])
 	DataDir = getenv("PGDATA"); /* default value */
 
 	opterr = 0;
-	while ((opt = getopt(argc, argv, "a:B:b:D:dm:Mno:p:Ss")) != EOF)
+	while ((opt = getopt(argc, argv, "a:B:b:D:dim:Mno:p:Ss")) != EOF)
 	{
 		switch (opt)
 		{
 			case 'a':
-				/* Set the authentication system. */
-				be_setauthsvc(optarg);
+				/* Can no longer set authentication method. */
 				break;
 			case 'B':
 
@@ -315,9 +359,11 @@ PostmasterMain(int argc, char *argv[])
 				else
 					DebugLvl = 1;
 				break;
+			case 'i':
+				NetServer = 1;
+				break;
 			case 'm':
-				MultiplexedBackends = 1;
-				MultiplexedBackendPort = atoi(optarg);
+				/* Multiplexed backends no longer supported. */
 				break;
 			case 'M':
 
@@ -387,11 +433,20 @@ PostmasterMain(int argc, char *argv[])
 		exit(1);
 	}
 
-
-	status = StreamServerPort(hostName, PostPortName, &ServerSock);
+	if (NetServer)
+	{
+		status = StreamServerPort(hostName, PostPortName, &ServerSock_INET);
+		if (status != STATUS_OK)
+		{
+			fprintf(stderr, "%s: cannot create INET stream port\n",
+					progname);
+			exit(1);
+		}
+	}
+	status = StreamServerPort(NULL, PostPortName, &ServerSock_UNIX);
 	if (status != STATUS_OK)
 	{
-		fprintf(stderr, "%s: cannot create stream port\n",
+		fprintf(stderr, "%s: cannot create UNIX stream port\n",
 				progname);
 		exit(1);
 	}
@@ -431,7 +486,7 @@ pmdaemonize(void)
 	int			i;
 
 	if (fork())
-		exit(0);
+		_exit(0);
 /* GH: If there's no setsid(), we hopefully don't need silent mode.
  * Until there's a better solution.
  */
@@ -454,12 +509,11 @@ static void
 usage(const char *progname)
 {
 	fprintf(stderr, "usage: %s [options..]\n", progname);
-	fprintf(stderr, "\t-a authsys\tdo/do not permit use of an authentication system\n");
 	fprintf(stderr, "\t-B nbufs\tset number of shared buffers\n");
 	fprintf(stderr, "\t-b backend\tuse a specific backend server executable\n");
 	fprintf(stderr, "\t-d [1|2|3]\tset debugging level\n");
 	fprintf(stderr, "\t-D datadir\tset data directory\n");
-	fprintf(stderr, "\t-m \tstart up multiplexing backends\n");
+	fprintf(stderr, "\t-i \tlisten on TCP/IP sockets as well as Unix domain socket\n");
 	fprintf(stderr, "\t-n\t\tdon't reinitialize shared memory after abnormal exit\n");
 	fprintf(stderr, "\t-o option\tpass 'option' to each backend servers\n");
 	fprintf(stderr, "\t-p port\t\tspecify port for postmaster to listen on\n");
@@ -471,15 +525,10 @@ usage(const char *progname)
 static int
 ServerLoop(void)
 {
-	int			serverFd = ServerSock;
-	fd_set		rmask,
-				basemask;
-	int			nSockets,
-				nSelected,
-				status,
-				newFd;
-	Dlelem	   *next,
-			   *curr;
+	fd_set		readmask,
+				writemask;
+	int			nSockets;
+	Dlelem	   *curr;
 
 	/*
 	 * GH: For !HAVE_SIGPROCMASK (NEXTSTEP), TRH implemented an
@@ -494,9 +543,7 @@ ServerLoop(void)
 
 #endif
 
-	nSockets = ServerSock + 1;
-	FD_ZERO(&basemask);
-	FD_SET(ServerSock, &basemask);
+	nSockets = initMasks(&readmask, &writemask);
 
 #ifdef HAVE_SIGPROCMASK
 	sigprocmask(0, 0, &oldsigmask);
@@ -505,17 +552,20 @@ ServerLoop(void)
 #endif
 	for (;;)
 	{
+		Port	   *port;
+		fd_set		rmask,
+					wmask;
+
 #ifdef HAVE_SIGPROCMASK
 		sigprocmask(SIG_SETMASK, &oldsigmask, 0);
 #else
 		sigsetmask(orgsigmask);
 #endif
-		newFd = -1;
-		memmove((char *) &rmask, (char *) &basemask, sizeof(fd_set));
-		if ((nSelected = select(nSockets, &rmask,
-								(fd_set *) NULL,
-								(fd_set *) NULL,
-								(struct timeval *) NULL)) < 0)
+
+		memmove((char *) &rmask, (char *) &readmask, sizeof(fd_set));
+		memmove((char *) &wmask, (char *) &writemask, sizeof(fd_set));
+		if (select(nSockets, &rmask, &wmask, (fd_set *) NULL,
+				   (struct timeval *) NULL) < 0)
 		{
 			if (errno == EINTR)
 				continue;
@@ -535,313 +585,206 @@ ServerLoop(void)
 #else
 		sigblock(sigmask(SIGCHLD));		/* XXX[TRH] portability */
 #endif
-		if (DebugLvl > 1)
-		{
-			fprintf(stderr, "%s: ServerLoop: %d sockets pending\n",
-					progname, nSelected);
-		}
 
 		/* new connection pending on our well-known port's socket */
-		if (FD_ISSET(ServerSock, &rmask))
-		{
 
-			/*
-			 * connect and make an addition to PortList.  If the
-			 * connection dies and we notice it, just forget about the
-			 * whole thing.
-			 */
-			if (ConnCreate(serverFd, &newFd) == STATUS_OK)
-			{
-				if (newFd >= nSockets)
-					nSockets = newFd + 1;
-				FD_SET(newFd, &rmask);
-				FD_SET(newFd, &basemask);
-				if (DebugLvl)
-					fprintf(stderr, "%s: ServerLoop: connect on %d\n",
-							progname, newFd);
-			}
-			--nSelected;
-			FD_CLR(ServerSock, &rmask);
-		}
+		if (ServerSock_UNIX != INVALID_SOCK &&
+			FD_ISSET(ServerSock_UNIX, &rmask) &&
+			(port = ConnCreate(ServerSock_UNIX)) != NULL)
+			PacketReceiveSetup(&port->pktInfo,
+							   readStartupPacket,
+							   (char *) port);
 
-		if (DebugLvl > 1)
-		{
-			fprintf(stderr, "%s: ServerLoop:\tnSelected=%d\n",
-					progname, nSelected);
-			curr = DLGetHead(PortList);
-			while (curr)
-			{
-				Port	   *port = DLE_VAL(curr);
+		if (ServerSock_INET != INVALID_SOCK &&
+			FD_ISSET(ServerSock_INET, &rmask) &&
+			(port = ConnCreate(ServerSock_INET)) != NULL)
+			PacketReceiveSetup(&port->pktInfo,
+							   readStartupPacket,
+							   (char *) port);
 
-				fprintf(stderr, "%s: ServerLoop:\t\tport %d%s pending\n",
-						progname, port->sock,
-						FD_ISSET(port->sock, &rmask)
-						? "" :
-						" not");
-				curr = DLGetSucc(curr);
-			}
-		}
+		/* Build up new masks for select(). */
+
+		nSockets = initMasks(&readmask, &writemask);
 
 		curr = DLGetHead(PortList);
 
 		while (curr)
 		{
 			Port	   *port = (Port *) DLE_VAL(curr);
-			int			lastbytes = port->nBytes;
+			int			status = STATUS_OK;
+			Dlelem	   *next;
 
-			if (FD_ISSET(port->sock, &rmask) && port->sock != newFd)
+			if (FD_ISSET(port->sock, &rmask))
 			{
 				if (DebugLvl > 1)
-					fprintf(stderr, "%s: ServerLoop:\t\thandling %d\n",
+					fprintf(stderr, "%s: ServerLoop:\t\thandling reading %d\n",
 							progname, port->sock);
-				--nSelected;
+
+				if (PacketReceiveFragment(&port->pktInfo, port->sock) != STATUS_OK)
+					status = STATUS_ERROR;
+			}
+
+			if (FD_ISSET(port->sock, &wmask))
+			{
+				if (DebugLvl > 1)
+					fprintf(stderr, "%s: ServerLoop:\t\thandling writing %d\n",
+							progname, port->sock);
+
+				if (PacketSendFragment(&port->pktInfo, port->sock) != STATUS_OK)
+					status = STATUS_ERROR;
+			}
+
+			/* Get this before the connection might be closed. */
+
+			next = DLGetSucc(curr);
+
+			/*
+			 * If there is no error and no outstanding data transfer going
+			 * on, then the authentication handshake must be complete to
+			 * the postmaster's satisfaction.  So, start the backend.
+			 */
+
+			if (status == STATUS_OK && port->pktInfo.state == Idle)
+			{
 
 				/*
-				 * Read the incoming packet into its packet buffer. Read
-				 * the connection id out of the packet so we know who the
-				 * packet is from.
+				 * If the backend start fails then keep the connection
+				 * open to report it.  Otherwise, pretend there is an
+				 * error to close the connection which will now be managed
+				 * by the backend.
 				 */
-				status = PacketReceive(port, &port->buf, NON_BLOCKING);
-				switch (status)
-				{
-					case STATUS_OK:
-						{
-							int			CSstatus;		/* Completion status of
-														 * ConnStartup */
-							char		errormsg[200];	/* error msg from
-														 * ConnStartup */
 
-							ConnStartup(port, &CSstatus, errormsg, sizeof(errormsg));
+				if (BackendStartup(port) != STATUS_OK)
+					PacketSendError(&port->pktInfo,
+									"Backend startup failed");
+				else
+					status = STATUS_ERROR;
+			}
 
-							if (CSstatus == STATUS_ERROR)
-								send_error_reply(port, errormsg);
-							ActiveBackends = TRUE;
-						}
-						/* FALLTHROUGH */
-					case STATUS_INVALID:
-						if (DebugLvl)
-							fprintf(stderr, "%s: ServerLoop:\t\tdone with %d\n",
-									progname, port->sock);
-						break;
-					case STATUS_BAD_PACKET:
+			/* Close the connection if required. */
 
-						/*
-						 * This is a bogus client, kill the connection and
-						 * forget the whole thing.
-						 */
-						if (DebugLvl)
-							fprintf(stderr, "%s: ServerLoop:\t\tbad packet format (reported packet size of %d read on port %d\n", progname, port->nBytes, port->sock);
-						break;
-					case STATUS_NOT_DONE:
-						if (DebugLvl)
-							fprintf(stderr, "%s: ServerLoop:\t\tpartial packet (%d bytes actually read) on %d\n",
-									progname, port->nBytes, port->sock);
-
-						/*
-						 * If we've received at least a PacketHdr's worth
-						 * of data and we're still receiving data each
-						 * time we read, we're ok.  If the client gives us
-						 * less than a PacketHdr at the beginning, just
-						 * kill the connection and forget about the whole
-						 * thing.
-						 */
-						if (lastbytes < port->nBytes)
-						{
-							if (DebugLvl)
-								fprintf(stderr, "%s: ServerLoop:\t\tpartial packet on %d ok\n",
-										progname, port->sock);
-							curr = DLGetSucc(curr);
-							continue;
-						}
-						break;
-					case STATUS_ERROR:	/* system call error - die */
-						fprintf(stderr, "%s: ServerLoop:\t\terror receiving packet\n",
-								progname);
-						return (STATUS_ERROR);
-				}
-				FD_CLR(port->sock, &basemask);
+			if (status != STATUS_OK)
+			{
 				StreamClose(port->sock);
-				next = DLGetSucc(curr);
 				DLRemove(curr);
 				free(port);
 				DLFreeElem(curr);
-				curr = next;
-				continue;
-			}
-			curr = DLGetSucc(curr);
-		}
-		Assert(nSelected == 0);
-	}
-}
-
-
-
-/*
-	ConnStartup: get the startup packet from the front end (client),
-	authenticate the user, and start up a backend.
-
-	If all goes well, return *status == STATUS_OK.
-	Otherwise, return *status == STATUS_ERROR and return a text string
-	explaining why in the "errormsg_len" bytes at "errormsg",
-*/
-
-static void
-ConnStartup(Port *port, int *status,
-			char *errormsg, const int errormsg_len)
-{
-	MsgType		msgType;
-	char		namebuf[NAMEDATALEN];
-	int			pid;
-	PacketBuf  *p;
-	StartupInfo sp;
-	char	   *tmp;
-
-	p = &port->buf;
-
-	sp.database[0] = '\0';
-	sp.user[0] = '\0';
-	sp.options[0] = '\0';
-	sp.execFile[0] = '\0';
-	sp.tty[0] = '\0';
-
-	tmp = p->data;
-	strncpy(sp.database, tmp, sizeof(sp.database));
-	tmp += sizeof(sp.database);
-	strncpy(sp.user, tmp, sizeof(sp.user));
-	tmp += sizeof(sp.user);
-	strncpy(sp.options, tmp, sizeof(sp.options));
-	tmp += sizeof(sp.options);
-	strncpy(sp.execFile, tmp, sizeof(sp.execFile));
-	tmp += sizeof(sp.execFile);
-	strncpy(sp.tty, tmp, sizeof(sp.tty));
-
-	msgType = (MsgType) ntohl(port->buf.msgtype);
-
-	strNcpy(namebuf, sp.user, NAMEDATALEN - 1);
-	if (!namebuf[0])
-	{
-		strncpy(errormsg,
-				"No Postgres username specified in startup packet.",
-				errormsg_len);
-		*status = STATUS_ERROR;
-	}
-	else
-	{
-		if (be_recvauth(msgType, port, namebuf, &sp) != STATUS_OK)
-		{
-			char		buffer[200 + sizeof(namebuf)];
-
-			sprintf(buffer,
-					"Failed to authenticate client as Postgres user '%s' "
-					"using %s: %s",
-			  namebuf, name_of_authentication_type(msgType), PQerrormsg);
-			strncpy(errormsg, buffer, errormsg_len);
-			*status = STATUS_ERROR;
-		}
-		else
-		{
-			if (BackendStartup(&sp, port, &pid) != STATUS_OK)
-			{
-				strncpy(errormsg, "Startup (fork) of backend failed.",
-						errormsg_len);
-				*status = STATUS_ERROR;
 			}
 			else
 			{
-				errormsg[0] = '\0';		/* just for robustness */
-				*status = STATUS_OK;
+				/* Set the masks for this connection. */
+
+				if (nSockets <= port->sock)
+					nSockets = port->sock + 1;
+
+				if (port->pktInfo.state == WritingPacket)
+					FD_SET(port->sock, &writemask);
+				else
+					FD_SET(port->sock, &readmask);
 			}
+
+			curr = next;
 		}
 	}
-	if (*status == STATUS_ERROR)
-		fprintf(stderr, "%s: ConnStartup: %s\n", progname, errormsg);
 }
 
 
 /*
-	 send_error_reply: send a reply to the front end telling it that
-	 the connection was a bust, and why.
+ * Initialise the read and write masks for select() for the well-known ports
+ * we are listening on.  Return the number of sockets to listen on.
+ */
 
-	 "port" tells to whom and how to send the reply.  "errormsg" is
-	 the string of text telling what the problem was.
+static int
+initMasks(fd_set *rmask, fd_set *wmask)
+{
+	int			nsocks = -1;
 
-	 It should be noted that we're executing a pretty messy protocol
-	 here.	The postmaster does not reply when the connection is
-	 successful, but rather just hands the connection off to the
-	 backend and the backend waits for a query from the frontend.
-	 Thus, the frontend is not expecting any reply in regards to the
-	 connect request.
+	FD_ZERO(rmask);
+	FD_ZERO(wmask);
 
-	 But when the connection fails, we send this reply that starts
-	 with "E".	The frontend only gets this reply when it sends its
-	 first query and waits for the reply.  Nobody receives that query,
-	 but this reply is already in the pipe, so that's what the
-	 frontend sees.
+	if (ServerSock_UNIX != INVALID_SOCK)
+	{
+		FD_SET(ServerSock_UNIX, rmask);
 
-	 Note that the backend closes the socket immediately after sending
-	 the reply, so to give the frontend a fighting chance to see the
-	 error info, we set the socket to linger up to 3 seconds waiting
-	 for the frontend to retrieve the message.	That's all the delay
-	 we can afford, since we have other clients to serve and the
-	 postmaster will be blocked the whole time.  Also, if there is no
-	 message space in the socket for the reply (shouldn't be a
-	 problem) the postmaster will block until the frontend reads the
-	 reply.
+		if (ServerSock_UNIX > nsocks)
+			nsocks = ServerSock_UNIX;
+	}
 
-*/
+	if (ServerSock_INET != INVALID_SOCK)
+	{
+		FD_SET(ServerSock_INET, rmask);
+
+		if (ServerSock_INET > nsocks)
+			nsocks = ServerSock_INET;
+	}
+
+	return (nsocks + 1);
+}
+
+
+/*
+ * Called when the startup packet has been read.
+ */
 
 static void
-send_error_reply(Port *port, const char *errormsg)
+readStartupPacket(char *arg, PacketLen len, char *pkt)
 {
-	int			rc;				/* return code from sendto */
-	char	   *reply;
+	Port	   *port;
+	StartupPacket *si;
+
+	port = (Port *) arg;
+	si = (StartupPacket *) pkt;
 
 	/*
-	 * The literal reply string we put into the socket.  This is a pointer
-	 * to storage we malloc.
-	 */
-	const struct linger linger_parm = {true, LINGER_TIME};
-
-	/*
-	 * A parameter for setsockopt() that tells it to have close() block
-	 * for a while waiting for the frontend to read its outstanding
-	 * messages.
+	 * Get the parameters from the startup packet as C strings.  The
+	 * packet destination was cleared first so a short packet has zeros
+	 * silently added and a long packet is silently truncated.
 	 */
 
-	reply = malloc(strlen(errormsg) + 10);
+	StrNCpy(port->database, si->database, sizeof(port->database) - 1);
+	StrNCpy(port->user, si->user, sizeof(port->user) - 1);
+	StrNCpy(port->options, si->options, sizeof(port->options) - 1);
+	StrNCpy(port->tty, si->tty, sizeof(port->tty) - 1);
 
-	sprintf(reply, "E%s", errormsg);
+	/* The database defaults to the user name. */
 
-	rc = send(port->sock, (Addr) reply, strlen(reply) + 1, /* flags */ 0);
-	if (rc < 0)
-		fprintf(stderr,
-				"%s: ServerLoop:\t\t"
-				"Failed to send error reply to front end\n",
-				progname);
-	else if (rc < strlen(reply) + 1)
-		fprintf(stderr,
-				"%s: ServerLoop:\t\t"
-				"Only partial error reply sent to front end.\n",
-				progname);
+	if (port->database[0] == '\0')
+		StrNCpy(port->database, si->user, sizeof(port->database) - 1);
 
-	free(reply);
+	/* Check we can handle the protocol the frontend is using. */
 
-	/*
-	 * Now we have to make sure frontend has a chance to see what we just
-	 * wrote.
-	 */
-	rc = setsockopt(port->sock, SOL_SOCKET, SO_LINGER,
-					&linger_parm, sizeof(linger_parm));
+	port->proto = ntohl(si->protoVersion);
+
+	if (PG_PROTOCOL_MAJOR(port->proto) < PG_PROTOCOL_MAJOR(PG_PROTOCOL_EARLIEST) ||
+		PG_PROTOCOL_MAJOR(port->proto) > PG_PROTOCOL_MAJOR(PG_PROTOCOL_LATEST) ||
+		(PG_PROTOCOL_MAJOR(port->proto) == PG_PROTOCOL_MAJOR(PG_PROTOCOL_LATEST) &&
+		 PG_PROTOCOL_MINOR(port->proto) > PG_PROTOCOL_MINOR(PG_PROTOCOL_LATEST)))
+	{
+		PacketSendError(&port->pktInfo, "Unsupported frontend protocol.");
+		return;
+	}
+
+	/* Check a user name was given. */
+
+	if (port->user[0] == '\0')
+	{
+		PacketSendError(&port->pktInfo,
+					"No Postgres username specified in startup packet.");
+		return;
+	}
+
+	/* Start the authentication itself. */
+
+	be_recvauth(port);
 }
 
 
 /*
  * ConnCreate -- create a local connection data structure
  */
-static int
-ConnCreate(int serverFd, int *newFdP)
+static Port *
+ConnCreate(int serverFd)
 {
-	int			status;
 	Port	   *port;
 
 
@@ -852,18 +795,20 @@ ConnCreate(int serverFd, int *newFdP)
 		ExitPostmaster(1);
 	}
 
-	if ((status = StreamConnection(serverFd, port)) != STATUS_OK)
+	if (StreamConnection(serverFd, port) != STATUS_OK)
 	{
 		StreamClose(port->sock);
 		free(port);
+		port = NULL;
 	}
 	else
 	{
 		DLAddHead(PortList, DLNewElem(port));
-		*newFdP = port->sock;
+		RandomSalt(port->salt);
+		port->pktInfo.state = Idle;
 	}
 
-	return (status);
+	return port;
 }
 
 /*
@@ -872,11 +817,12 @@ ConnCreate(int serverFd, int *newFdP)
 static void
 reset_shared(short port)
 {
-	IPCKey		key;
-
-	key = SystemPortAddressCreateIPCKey((SystemPortAddress) port);
-	CreateSharedMemoryAndSemaphores(key);
+	ipc_key = port * 1000 + shmem_seq * 100;
+	CreateSharedMemoryAndSemaphores(ipc_key);
 	ActiveBackends = FALSE;
+	shmem_seq += 1;
+	if (shmem_seq >= 10)
+		shmem_seq -= 10;
 }
 
 /*
@@ -1040,16 +986,23 @@ CleanupProc(int pid,
  *
  */
 static int
-BackendStartup(StartupInfo *packet,		/* client's startup packet */
-			   Port *port,
-			   int *pidPtr)
+BackendStartup(Port *port)
 {
 	Backend    *bn;				/* for backend cleanup */
 	int			pid,
 				i;
-	static char envEntry[4][2 * ARGV_SIZE];
 
-	for (i = 0; i < 4; ++i)
+#ifdef CYR_RECODE
+#define NR_ENVIRONMENT_VBL 6
+	char		ChTable[80];
+
+#else
+#define NR_ENVIRONMENT_VBL 5
+#endif
+
+	static char envEntry[NR_ENVIRONMENT_VBL][2 * ARGV_SIZE];
+
+	for (i = 0; i < NR_ENVIRONMENT_VBL; ++i)
 	{
 		MemSet(envEntry[i], 0, 2 * ARGV_SIZE);
 	}
@@ -1062,13 +1015,25 @@ BackendStartup(StartupInfo *packet,		/* client's startup packet */
 	putenv(envEntry[0]);
 	sprintf(envEntry[1], "POSTID=%d", NextBackendId);
 	putenv(envEntry[1]);
-	sprintf(envEntry[2], "PG_USER=%s", packet->user);
+	sprintf(envEntry[2], "PG_USER=%s", port->user);
 	putenv(envEntry[2]);
 	if (!getenv("PGDATA"))
 	{
 		sprintf(envEntry[3], "PGDATA=%s", DataDir);
 		putenv(envEntry[3]);
 	}
+	sprintf(envEntry[4], "IPC_KEY=%d", ipc_key);
+	putenv(envEntry[4]);
+
+#ifdef CYR_RECODE
+	GetCharSetByHost(ChTable, port->raddr.in.sin_addr.s_addr, DataDir);
+	if (*ChTable != '\0')
+	{
+		sprintf(envEntry[5], "PG_RECODETABLE=%s", ChTable);
+		putenv(envEntry[5]);
+	}
+#endif
+
 	if (DebugLvl > 2)
 	{
 		char	  **p;
@@ -1084,7 +1049,7 @@ BackendStartup(StartupInfo *packet,		/* client's startup packet */
 
 	if ((pid = FORK()) == 0)
 	{							/* child */
-		if (DoExec(packet, port->sock))
+		if (DoExec(port))
 			fprintf(stderr, "%s child[%d]: BackendStartup: execv failed\n",
 					progname, pid);
 		/* use _exit to keep from double-flushing stdio */
@@ -1101,8 +1066,7 @@ BackendStartup(StartupInfo *packet,		/* client's startup packet */
 
 	if (DebugLvl)
 		fprintf(stderr, "%s: BackendStartup: pid %d user %s db %s socket %d\n",
-				progname, pid, packet->user,
-		 (packet->database[0] == '\0' ? packet->user : packet->database),
+				progname, pid, port->user, port->database,
 				port->sock);
 
 	/* adjust backend counter */
@@ -1123,10 +1087,7 @@ BackendStartup(StartupInfo *packet,		/* client's startup packet */
 	bn->pid = pid;
 	DLAddHead(BackendList, DLNewElem(bn));
 
-	if (MultiplexedBackends)
-		MultiplexedBackendPort++;
-
-	*pidPtr = pid;
+	ActiveBackends = TRUE;
 
 	return (STATUS_OK);
 }
@@ -1173,19 +1134,19 @@ split_opts(char **argv, int *argcp, char *s)
  *		If execv() fails, return status.
  */
 static int
-DoExec(StartupInfo *packet, int portFd)
+DoExec(Port *port)
 {
 	char		execbuf[MAXPATHLEN];
 	char		portbuf[ARGV_SIZE];
-	char		mbbuf[ARGV_SIZE];
 	char		debugbuf[ARGV_SIZE];
 	char		ttybuf[ARGV_SIZE + 1];
+	char		protobuf[ARGV_SIZE + 1];
 	char		argbuf[(2 * ARGV_SIZE) + 1];
 
 	/*
 	 * each argument takes at least three chars, so we can't have more
 	 * than ARGV_SIZE arguments in (2 * ARGV_SIZE) chars (i.e.,
-	 * packet->options plus ExtraOptions)...
+	 * port->options plus ExtraOptions)...
 	 */
 	char	   *av[ARGV_SIZE];
 	char		dbbuf[ARGV_SIZE + 1];
@@ -1215,32 +1176,28 @@ DoExec(StartupInfo *packet, int portFd)
 		av[ac++] = "-Q";
 
 	/* Pass the requested debugging output file */
-	if (packet->tty[0])
+	if (port->tty[0])
 	{
-		strncpy(ttybuf, packet->tty, ARGV_SIZE);
+		strncpy(ttybuf, port->tty, ARGV_SIZE);
 		av[ac++] = "-o";
 		av[ac++] = ttybuf;
 	}
 
-	/* tell the multiplexed backend to start on a certain port */
-	if (MultiplexedBackends)
-	{
-		sprintf(mbbuf, "-m %d", MultiplexedBackendPort);
-		av[ac++] = mbbuf;
-	}
 	/* Tell the backend the descriptor of the fe/be socket */
-	sprintf(portbuf, "-P%d", portFd);
+	sprintf(portbuf, "-P%d", port->sock);
 	av[ac++] = portbuf;
 
-	strNcpy(argbuf, packet->options, ARGV_SIZE);
+	StrNCpy(argbuf, port->options, ARGV_SIZE);
 	strncat(argbuf, ExtraOptions, ARGV_SIZE);
 	argbuf[(2 * ARGV_SIZE)] = '\0';
 	split_opts(av, &ac, argbuf);
 
-	if (packet->database[0])
-		strNcpy(dbbuf, packet->database, ARGV_SIZE);
-	else
-		strNcpy(dbbuf, packet->user, NAMEDATALEN - 1);
+	/* Tell the backend what protocol the frontend is using. */
+
+	sprintf(protobuf, "-v %u", port->proto);
+	av[ac++] = protobuf;
+
+	StrNCpy(dbbuf, port->database, ARGV_SIZE);
 	av[ac++] = dbbuf;
 
 	av[ac] = (char *) NULL;
@@ -1248,7 +1205,7 @@ DoExec(StartupInfo *packet, int portFd)
 	if (DebugLvl > 1)
 	{
 		fprintf(stderr, "%s child[%ld]: execv(",
-				progname, (long) getpid());
+				progname, (long) MyProcPid);
 		for (i = 0; i < ac; ++i)
 			fprintf(stderr, "%s, ", av[i]);
 		fprintf(stderr, ")\n");
@@ -1269,8 +1226,10 @@ ExitPostmaster(int status)
 	 * Not sure of the semantics here.	When the Postmaster dies, should
 	 * the backends all be killed? probably not.
 	 */
-	if (ServerSock != INVALID_SOCK)
-		close(ServerSock);
+	if (ServerSock_INET != INVALID_SOCK)
+		close(ServerSock_INET);
+	if (ServerSock_UNIX != INVALID_SOCK)
+		close(ServerSock_UNIX);
 	exitpg(status);
 }
 
@@ -1284,10 +1243,51 @@ dumpstatus(SIGNAL_ARGS)
 		Port	   *port = DLE_VAL(curr);
 
 		fprintf(stderr, "%s: dumpstatus:\n", progname);
-		fprintf(stderr, "\tsock %d: nBytes=%d, laddr=0x%lx, raddr=0x%lx\n",
-				port->sock, port->nBytes,
-				(long int) port->laddr.sin_addr.s_addr,
-				(long int) port->raddr.sin_addr.s_addr);
+		fprintf(stderr, "\tsock %d\n", port->sock);
 		curr = DLGetSucc(curr);
 	}
 }
+
+/*
+ * CharRemap
+ */
+static char
+CharRemap(long int ch)
+{
+
+	if (ch < 0)
+		ch = -ch;
+
+	ch = ch % 62;
+	if (ch < 26)
+		return ('A' + ch);
+
+	ch -= 26;
+	if (ch < 26)
+		return ('a' + ch);
+
+	ch -= 26;
+	return ('0' + ch);
+}
+
+/*
+ * RandomSalt
+ */
+static void
+RandomSalt(char *salt)
+{
+
+	static bool initialized = false;
+
+	if (!initialized)
+	{
+		time_t		now;
+
+		now = time(NULL);
+		srandom((unsigned int) now);
+		initialized = true;
+	}
+
+	*salt = CharRemap(random());
+	*(salt + 1) = CharRemap(random());
+}
