@@ -7,28 +7,28 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/rewrite/rewriteDefine.c,v 1.6 1997/09/08 21:46:35 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/rewrite/rewriteDefine.c,v 1.14 1998/01/13 04:04:12 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include <stdio.h>
 #include <string.h>
+
 #include "postgres.h"
 
-#include "utils/rel.h"			/* for Relation stuff */
 #include "access/heapam.h"		/* access methods like amopenr */
-#include "utils/builtins.h"
-#include "utils/elog.h"			/* for elog */
-#include "utils/palloc.h"
-#include "utils/lsyscache.h"	/* for get_typlen */
-#include "nodes/pg_list.h"		/* for Lisp support */
 #include "nodes/parsenodes.h"
-#include "parser/catalog_utils.h"
+#include "nodes/pg_list.h"		/* for Lisp support */
+#include "parser/parse_relation.h"
 #include "rewrite/locks.h"
 #include "rewrite/rewriteDefine.h"
 #include "rewrite/rewriteRemove.h"
 #include "rewrite/rewriteSupport.h"
 #include "tcop/tcopprot.h"
+#include "utils/builtins.h"
+#include "utils/lsyscache.h"	/* for get_typlen */
+#include "utils/rel.h"			/* for Relation stuff */
+
 
 Oid			LastOidProcessed = InvalidOid;
 
@@ -38,7 +38,7 @@ Oid			LastOidProcessed = InvalidOid;
  *
  * should this be smaller?
  */
-#define RULE_PLAN_SIZE 8192
+#define RULE_PLAN_SIZE BLCKSZ
 
 static void
 strcpyq(char *dest, char *source)
@@ -97,7 +97,7 @@ InsertRule(char *rulname,
 	eventrel = heap_openr(evobj);
 	if (eventrel == NULL)
 	{
-		elog(WARN, "rules cannot be defined on relations not in schema");
+		elog(ERROR, "rules cannot be defined on relations not in schema");
 	}
 	eventrel_oid = RelationGetRelationId(eventrel);
 
@@ -107,35 +107,35 @@ InsertRule(char *rulname,
 	if (evslot == NULL)
 		evslot_index = -1;
 	else
-		evslot_index = varattno(eventrel, (char *) evslot);
+		evslot_index = attnameAttNum(eventrel, (char *) evslot);
 	heap_close(eventrel);
 
 	if (evinstead)
 		is_instead = "t";
 
 	if (evqual == NULL)
-		evqual = "nil";
+		evqual = "<>";
 
 	if (IsDefinedRewriteRule(rulname))
-		elog(WARN, "Attempt to insert rule '%s' failed: already exists",
+		elog(ERROR, "Attempt to insert rule '%s' failed: already exists",
 			 rulname);
 	strcpyq(actionbuf, actiontree);
 	strcpyq(qualbuf, evqual);
 
 	template = "INSERT INTO pg_rewrite \
-(rulename, ev_type, ev_class, ev_attr, action, ev_qual, is_instead) VALUES \
+(rulename, ev_type, ev_class, ev_attr, ev_action, ev_qual, is_instead) VALUES \
 ('%s', %d::char, %d::oid, %d::int2, '%s'::text, '%s'::text, \
  '%s'::bool);";
 	if (strlen(template) + strlen(rulname) + strlen(actionbuf) +
 		strlen(qualbuf) + 20 /* fudge fac */ > RULE_PLAN_SIZE)
 	{
-		elog(WARN, "DefineQueryRewrite: rule plan string too big.");
+		elog(ERROR, "DefineQueryRewrite: rule plan string too big.");
 	}
 	sprintf(rulebuf, template,
 			rulname, evtype, eventrel_oid, evslot_index, actionbuf,
 			qualbuf, is_instead);
 
-	pg_eval(rulebuf, (char **) NULL, (Oid *) NULL, 0);
+	pg_exec_query(rulebuf, (char **) NULL, (Oid *) NULL, 0);
 
 	return (LastOidProcessed);
 }
@@ -155,12 +155,12 @@ ValidateRule(int event_type,
 	if (((event_type == CMD_INSERT) || (event_type == CMD_DELETE)) &&
 		eslot_string)
 	{
-		elog(WARN,
+		elog(ERROR,
 		"rules not allowed for insert or delete events to an attribute");
 	}
 
 	if (event_qual && !*action && is_instead)
-		elog(WARN,
+		elog(ERROR,
 		"event_quals on 'instead nothing' rules not currently supported");
 
 #if 0
@@ -210,7 +210,7 @@ DefineQueryRewrite(RuleStmt *stmt)
 	event_relation = heap_openr(event_obj->relname);
 	if (event_relation == NULL)
 	{
-		elog(WARN, "virtual relations not supported yet");
+		elog(ERROR, "virtual relations not supported yet");
 	}
 	ev_relid = RelationGetRelationId(event_relation);
 
@@ -221,8 +221,8 @@ DefineQueryRewrite(RuleStmt *stmt)
 	}
 	else
 	{
-		event_attno = varattno(event_relation, eslot_string);
-		event_attype = att_typeid(event_relation, event_attno);
+		event_attno = attnameAttNum(event_relation, eslot_string);
+		event_attype = attnumTypeId(event_relation, event_attno);
 	}
 	heap_close(event_relation);
 
@@ -244,7 +244,7 @@ DefineQueryRewrite(RuleStmt *stmt)
 							eslot_string,
 							event_qualP,
 							true,
-							"nil");
+							"<>");
 		prs2_addToRelation(ev_relid, ruleId, event_type, event_attno, TRUE,
 						   event_qual, NIL);
 
@@ -264,7 +264,7 @@ DefineQueryRewrite(RuleStmt *stmt)
 
 		/* what is the max size of type text? XXX -- glass */
 		if (length(action) > 15)
-			elog(WARN, "max # of actions exceeded");
+			elog(ERROR, "max # of actions exceeded");
 		prs2_addToRelation(ev_relid, ruleId, event_type, event_attno,
 						   is_instead, event_qual, action);
 	}
