@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/buffer/bufmgr.c,v 1.26 1997/09/22 07:13:56 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/buffer/bufmgr.c,v 1.36 1998/04/05 21:04:22 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -100,7 +100,7 @@ static void BufferSync(void);
 static int	BufferReplace(BufferDesc *bufHdr, bool bufferLockHeld);
 
 /* not static but used by vacuum only ... */
-int BlowawayRelationBuffers(Relation rdesc, BlockNumber block);
+int			BlowawayRelationBuffers(Relation rdesc, BlockNumber block);
 
 /* ---------------------------------------------------
  * RelationGetBufferWithBuffer
@@ -275,7 +275,7 @@ ReadBufferWithBufferLock(Relation reln,
 		{
 			/* new buffers are zero-filled */
 			MemSet((char *) MAKE_PTR(bufHdr->data), 0, BLCKSZ);
-			smgrextend(bufHdr->bufsmgr, reln,
+			smgrextend(DEFAULT_SMGR, reln,
 					   (char *) MAKE_PTR(bufHdr->data));
 		}
 		return (BufferDescriptorGetBuffer(bufHdr));
@@ -290,12 +290,12 @@ ReadBufferWithBufferLock(Relation reln,
 	{
 		/* new buffers are zero-filled */
 		MemSet((char *) MAKE_PTR(bufHdr->data), 0, BLCKSZ);
-		status = smgrextend(bufHdr->bufsmgr, reln,
+		status = smgrextend(DEFAULT_SMGR, reln,
 							(char *) MAKE_PTR(bufHdr->data));
 	}
 	else
 	{
-		status = smgrread(bufHdr->bufsmgr, reln, blockNum,
+		status = smgrread(DEFAULT_SMGR, reln, blockNum,
 						  (char *) MAKE_PTR(bufHdr->data));
 	}
 
@@ -372,7 +372,7 @@ BufferAlloc(Relation reln,
 	if (blockNum == P_NEW)
 	{
 		newblock = TRUE;
-		blockNum = smgrnblocks(reln->rd_rel->relsmgr, reln);
+		blockNum = smgrnblocks(DEFAULT_SMGR, reln);
 	}
 
 	INIT_BUFFERTAG(&newTag, reln, blockNum);
@@ -444,7 +444,7 @@ BufferAlloc(Relation reln,
 
 		/*
 		 * But it can return buf == NULL if we are in aborting transaction
-		 * now and so elog(WARN,...) in GetFreeBuffer will not abort
+		 * now and so elog(ERROR,...) in GetFreeBuffer will not abort
 		 * again.
 		 */
 		if (buf == NULL)
@@ -643,10 +643,7 @@ BufferAlloc(Relation reln,
 
 	/* record the database name and relation name for this buffer */
 	strcpy(buf->sb_relname, reln->rd_rel->relname.data);
-	strcpy(buf->sb_dbname, GetDatabaseName());
-
-	/* remember which storage manager is responsible for it */
-	buf->bufsmgr = reln->rd_rel->relsmgr;
+	strcpy(buf->sb_dbname, DatabaseName);
 
 	INIT_BUFFERTAG(&(buf->tag), reln, blockNum);
 	if (!BufTableInsert(buf))
@@ -830,12 +827,14 @@ FlushBuffer(Buffer buffer, bool release)
 	bufHdr->flags &= ~BM_JUST_DIRTIED;
 	SpinRelease(BufMgrLock);
 
-	status = smgrflush(bufHdr->bufsmgr, bufrel, bufHdr->tag.blockNum,
+	status = smgrflush(DEFAULT_SMGR, bufrel, bufHdr->tag.blockNum,
 					   (char *) MAKE_PTR(bufHdr->data));
 
+	RelationDecrementReferenceCount(bufrel);
+
 	if (status == SM_FAIL)
 	{
-		elog(WARN, "FlushBuffer: cannot flush block %u of the relation %s",
+		elog(ERROR, "FlushBuffer: cannot flush block %u of the relation %s",
 			 bufHdr->tag.blockNum, bufHdr->sb_relname);
 		return (STATUS_ERROR);
 	}
@@ -1010,7 +1009,7 @@ BufferSync()
 					UnpinBuffer(bufHdr);
 					if (bufHdr->flags & BM_IO_ERROR)
 					{
-						elog(WARN, "BufferSync: write error %u for %s",
+						elog(ERROR, "BufferSync: write error %u for %s",
 							 bufHdr->tag.blockNum, bufHdr->sb_relname);
 					}
 					if (reln != (Relation) NULL)
@@ -1036,14 +1035,14 @@ BufferSync()
 #endif							/* OPTIMIZE_SINGLE */
 				if (reln == (Relation) NULL)
 				{
-					status = smgrblindwrt(bufHdr->bufsmgr, bufHdr->sb_dbname,
+					status = smgrblindwrt(DEFAULT_SMGR, bufHdr->sb_dbname,
 									   bufHdr->sb_relname, bufdb, bufrel,
 										  bufHdr->tag.blockNum,
 										(char *) MAKE_PTR(bufHdr->data));
 				}
 				else
 				{
-					status = smgrwrite(bufHdr->bufsmgr, reln,
+					status = smgrwrite(DEFAULT_SMGR, reln,
 									   bufHdr->tag.blockNum,
 									   (char *) MAKE_PTR(bufHdr->data));
 				}
@@ -1055,7 +1054,7 @@ BufferSync()
 				if (status == SM_FAIL)
 				{
 					bufHdr->flags |= BM_IO_ERROR;
-					elog(WARN, "BufferSync: cannot write %u for %s",
+					elog(ERROR, "BufferSync: cannot write %u for %s",
 						 bufHdr->tag.blockNum, bufHdr->sb_relname);
 				}
 				BufferFlushCount++;
@@ -1065,15 +1064,8 @@ BufferSync()
 				 * were flushing it out we must not clear DIRTY flag -
 				 * vadim 01/17/97
 				 */
-				if (bufHdr->flags & BM_JUST_DIRTIED)
-				{
-					elog(NOTICE, "BufferSync: content of block %u (%s) changed while flushing",
-						 bufHdr->tag.blockNum, bufHdr->sb_relname);
-				}
-				else
-				{
+				if (!(bufHdr->flags & BM_JUST_DIRTIED))
 					bufHdr->flags &= ~BM_DIRTY;
-				}
 				if (reln != (Relation) NULL)
 					RelationDecrementReferenceCount(reln);
 			}
@@ -1205,7 +1197,7 @@ ResetBufferUsage()
 void
 ResetBufferPool()
 {
-	register int i;
+	int			i;
 
 	for (i = 1; i <= NBuffers; i++)
 	{
@@ -1233,7 +1225,7 @@ ResetBufferPool()
 int
 BufferPoolCheckLeak()
 {
-	register int i;
+	int			i;
 	int			error = 0;
 
 	for (i = 1; i <= NBuffers; i++)
@@ -1380,19 +1372,22 @@ BufferReplace(BufferDesc *bufHdr, bool bufferLockHeld)
 
 	if (reln != (Relation) NULL)
 	{
-		status = smgrflush(bufHdr->bufsmgr, reln, bufHdr->tag.blockNum,
+		status = smgrflush(DEFAULT_SMGR, reln, bufHdr->tag.blockNum,
 						   (char *) MAKE_PTR(bufHdr->data));
 	}
 	else
 	{
 
 		/* blind write always flushes */
-		status = smgrblindwrt(bufHdr->bufsmgr, bufHdr->sb_dbname,
+		status = smgrblindwrt(DEFAULT_SMGR, bufHdr->sb_dbname,
 							  bufHdr->sb_relname, bufdb, bufrel,
 							  bufHdr->tag.blockNum,
 							  (char *) MAKE_PTR(bufHdr->data));
 	}
 
+	if (reln != (Relation) NULL)
+		RelationDecrementReferenceCount(reln);
+
 	if (status == SM_FAIL)
 		return (FALSE);
 
@@ -1415,7 +1410,7 @@ RelationGetNumberOfBlocks(Relation relation)
 {
 	return
 	((relation->rd_islocal) ? relation->rd_nblocks :
-	 smgrnblocks(relation->rd_rel->relsmgr, relation));
+	 smgrnblocks(DEFAULT_SMGR, relation));
 }
 
 /*
@@ -1449,7 +1444,7 @@ BufferGetBlock(Buffer buffer)
 void
 ReleaseRelationBuffers(Relation rdesc)
 {
-	register int i;
+	int			i;
 	int			holding = 0;
 	BufferDesc *buf;
 
@@ -1506,7 +1501,7 @@ ReleaseRelationBuffers(Relation rdesc)
 void
 DropBuffers(Oid dbid)
 {
-	register int i;
+	int			i;
 	BufferDesc *buf;
 
 	SpinAcquire(BufMgrLock);
@@ -1537,6 +1532,7 @@ PrintBufferDescs()
 	if (IsUnderPostmaster)
 	{
 		SpinAcquire(BufMgrLock);
+#if 0
 		for (i = 0; i < NBuffers; ++i, ++buf)
 		{
 			elog(NOTICE, "[%02d] (freeNext=%d, freePrev=%d, relname=%s, \
@@ -1545,6 +1541,7 @@ blockNum=%d, flags=0x%x, refcount=%d %d)",
 				 buf->sb_relname, buf->tag.blockNum, buf->flags,
 				 buf->refcount, PrivateRefCount[i]);
 		}
+#endif
 		SpinRelease(BufMgrLock);
 	}
 	else
@@ -1590,7 +1587,7 @@ blockNum=%d, flags=0x%x, refcount=%d %d)\n",
 void
 BufferPoolBlowaway()
 {
-	register int i;
+	int			i;
 
 	BufferSync();
 	for (i = 1; i <= NBuffers; i++)
@@ -1611,9 +1608,9 @@ BufferPoolBlowaway()
  *
  *		This function blowaway all the pages with blocknumber >= passed
  *		of a relation in the buffer pool. Used by vacuum before truncation...
- *		
+ *
  *		Returns: 0 - Ok, -1 - DIRTY, -2 - PINNED
- *		
+ *
  *		XXX currently it sequentially searches the buffer pool, should be
  *		changed to more clever ways of searching.
  * --------------------------------------------------------------------
@@ -1621,21 +1618,30 @@ BufferPoolBlowaway()
 int
 BlowawayRelationBuffers(Relation rdesc, BlockNumber block)
 {
-	register int	i;
-	BufferDesc	   *buf;
+	int			i;
+	BufferDesc *buf;
 
 	if (rdesc->rd_islocal)
 	{
 		for (i = 0; i < NLocBuffer; i++)
 		{
 			buf = &LocalBufferDescriptors[i];
-			if (buf->tag.relId.relId == rdesc->rd_id && 
+			if (buf->tag.relId.relId == rdesc->rd_id &&
 				buf->tag.blockNum >= block)
 			{
 				if (buf->flags & BM_DIRTY)
+				{
+					elog(NOTICE, "BlowawayRelationBuffers(%s (local), %u): block %u is dirty",
+						 rdesc->rd_rel->relname.data, block, buf->tag.blockNum);
 					return (-1);
+				}
 				if (LocalRefCount[i] > 0)
+				{
+					elog(NOTICE, "BlowawayRelationBuffers(%s (local), %u): block %u is referenced (%d)",
+						 rdesc->rd_rel->relname.data, block,
+						 buf->tag.blockNum, LocalRefCount[i]);
 					return (-2);
+				}
 				buf->tag.relId.relId = InvalidOid;
 			}
 		}
@@ -1647,16 +1653,22 @@ BlowawayRelationBuffers(Relation rdesc, BlockNumber block)
 	{
 		buf = &BufferDescriptors[i];
 		if (buf->tag.relId.dbId == MyDatabaseId &&
-			buf->tag.relId.relId == rdesc->rd_id && 
+			buf->tag.relId.relId == rdesc->rd_id &&
 			buf->tag.blockNum >= block)
 		{
 			if (buf->flags & BM_DIRTY)
 			{
+				elog(NOTICE, "BlowawayRelationBuffers(%s, %u): block %u is dirty (private %d, last %d, global %d)",
+					 buf->sb_relname, block, buf->tag.blockNum,
+					 PrivateRefCount[i], LastRefCount[i], buf->refcount);
 				SpinRelease(BufMgrLock);
 				return (-1);
 			}
 			if (!(buf->flags & BM_FREE))
 			{
+				elog(NOTICE, "BlowawayRelationBuffers(%s, %u): block %u is referenced (private %d, last %d, global %d)",
+					 buf->sb_relname, block, buf->tag.blockNum,
+					 PrivateRefCount[i], LastRefCount[i], buf->refcount);
 				SpinRelease(BufMgrLock);
 				return (-2);
 			}
@@ -1818,14 +1830,10 @@ refcount = %ld, file: %s, line: %d\n",
 
 _bm_trace(Oid dbId, Oid relId, int blkNo, int bufNo, int allocType)
 {
-	static int	mypid = 0;
 	long		start,
 				cur;
 	bmtrace    *tb;
 
-	if (mypid == 0)
-		mypid = getpid();
-
 	start = *CurTraceBuf;
 
 	if (start > 0)
@@ -1861,7 +1869,7 @@ _bm_trace(Oid dbId, Oid relId, int blkNo, int bufNo, int allocType)
 
 okay:
 	tb = &TraceBuf[start];
-	tb->bmt_pid = mypid;
+	tb->bmt_pid = MyProcPid;
 	tb->bmt_buf = bufNo;
 	tb->bmt_dbid = dbId;
 	tb->bmt_relid = relId;
