@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/ipc/shmem.c,v 1.14 1997/09/18 20:21:30 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/ipc/shmem.c,v 1.18 1998/04/06 17:27:25 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -276,7 +276,7 @@ InitShmem(unsigned int key, unsigned int size)
  *		of space.  Has to return a real pointer in order
  *		to be compatable with malloc().
  */
-long	   *
+long *
 ShmemAlloc(unsigned long size)
 {
 	unsigned long tmpFree;
@@ -338,7 +338,7 @@ ShmemIsValid(unsigned long addr)
  * table at once.  Use SpinAlloc() to create a spinlock
  * for the structure before creating the structure itself.
  */
-HTAB	   *
+HTAB *
 ShmemInitHash(char *name,		/* table string name for binding */
 			  long init_size,	/* initial size */
 			  long max_size,	/* max size of the table */
@@ -418,7 +418,7 @@ ShmemPIDLookup(int pid, SHMEM_OFFSET *locationPtr)
 	{
 
 		SpinRelease(BindingLock);
-		elog(WARN, "ShmemInitPID: BindingTable corrupted");
+		elog(ERROR, "ShmemInitPID: BindingTable corrupted");
 		return (FALSE);
 
 	}
@@ -469,7 +469,7 @@ ShmemPIDDestroy(int pid)
 	if (!result)
 	{
 
-		elog(WARN, "ShmemPIDDestroy: PID table corrupted");
+		elog(ERROR, "ShmemPIDDestroy: PID table corrupted");
 		return (INVALID_OFFSET);
 
 	}
@@ -496,7 +496,7 @@ ShmemPIDDestroy(int pid)
  *		the object is already in the binding table (hence, already
  *		initialized).
  */
-long	   *
+long *
 ShmemInitStruct(char *name, unsigned long size, bool *foundPtr)
 {
 	BindingEnt *result,
@@ -511,7 +511,7 @@ ShmemInitStruct(char *name, unsigned long size, bool *foundPtr)
 	if (!BindingTable)
 	{
 		/* Assert() is a macro now. substitutes inside quotes. */
-#ifndef NO_ASSERT_CHECKING
+#ifdef USE_ASSERT_CHECKING
 		char	   *strname = "BindingTable";
 
 #endif
@@ -555,7 +555,7 @@ ShmemInitStruct(char *name, unsigned long size, bool *foundPtr)
 
 		SpinRelease(BindingLock);
 
-		elog(WARN, "ShmemInitStruct: Binding Table corrupted");
+		elog(ERROR, "ShmemInitStruct: Binding Table corrupted");
 		return (NULL);
 
 	}
@@ -640,6 +640,6 @@ TransactionIdIsInProgress(TransactionId xid)
 	}
 
 	SpinRelease(BindingLock);
-	elog(WARN, "TransactionIdIsInProgress: BindingTable corrupted");
+	elog(ERROR, "TransactionIdIsInProgress: BindingTable corrupted");
 	return (false);
 }
