@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/lmgr.c,v 1.8 1997/09/08 21:47:19 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/lmgr.c,v 1.11 1998/01/07 21:05:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,7 +26,6 @@
 #include "access/htup.h"
 #include "access/relscan.h"
 #include "access/skey.h"
-#include "utils/tqual.h"
 #include "access/xact.h"
 
 #include "storage/block.h"
@@ -435,7 +434,7 @@ RelationUnsetLockForRead(Relation relation)
 	 */
 	if (!LockInfoIsValid(linfo))
 	{
-		elog(WARN,
+		elog(ERROR,
 			 "Releasing a lock on %s with invalid lock information",
 			 RelationGetRelationName(relation));
 	}
@@ -528,7 +527,7 @@ RelationUnsetLockForWrite(Relation relation)
 
 	if (!LockInfoIsValid(linfo))
 	{
-		elog(WARN,
+		elog(ERROR,
 			 "Releasing a lock on %s with invalid lock information",
 			 RelationGetRelationName(relation));
 	}
@@ -770,7 +769,7 @@ RelationUnsetSingleWLockPage(Relation relation,
 		return;
 
 	if (!LockInfoIsValid(relation->lockInfo))
-		elog(WARN,
+		elog(ERROR,
 			 "Releasing a lock on %s with invalid lock information",
 			 RelationGetRelationName(relation));
 
@@ -817,7 +816,7 @@ RelationUnsetSingleRLockPage(Relation relation,
 		return;
 
 	if (!LockInfoIsValid(relation->lockInfo))
-		elog(WARN,
+		elog(ERROR,
 			 "Releasing a lock on %s with invalid lock information",
 			 RelationGetRelationName(relation));
 
