@@ -7,12 +7,12 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/lock.c,v 1.16 1997/09/18 20:21:35 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/lock.c,v 1.26 1998/02/26 04:36:07 momjian Exp $
  *
  * NOTES
  *	  Outside modules can create a lock table and acquire/release
  *	  locks.  A lock table is a shared memory hash table.  When
- *	  a process tries to acquire a lock of a type that conflicts
+ *	  a process tries to acquire a lock of a type that conflictRs
  *	  with existing locks, it is put to sleep using the routines
  *	  in storage/lmgr/proc.c.
  *
@@ -37,7 +37,9 @@
 #include <unistd.h>
 
 #include "postgres.h"
+#include "miscadmin.h"
 #include "storage/shmem.h"
+#include "storage/sinvaladt.h"
 #include "storage/spin.h"
 #include "storage/proc.h"
 #include "storage/lock.h"
@@ -51,7 +53,7 @@
 static int
 WaitOnLock(LOCKTAB *ltable, LockTableId tableId, LOCK *lock,
 		   LOCKT lockt);
-		   
+
 /*#define LOCK_MGR_DEBUG*/
 
 #ifndef LOCK_MGR_DEBUG
@@ -78,7 +80,7 @@ static char *lock_types[] = {
 	if ((lockDebug >= 1) && (tag->relId >= lock_debug_oid_min)) \
 		elog(DEBUG, \
 			 "%s: pid (%d) rel (%d) dbid (%d) tid (%d,%d) type (%s)",where, \
-			 getpid(),\
+			 MyProcPid,\
 			 tag->relId, tag->dbId, \
 			 ((tag->tupleId.ip_blkid.bi_hi<<16)+\
 			  tag->tupleId.ip_blkid.bi_lo),\
@@ -93,7 +95,7 @@ static char *lock_types[] = {
 		elog(DEBUG, \
 			 "%s: pid (%d) rel (%d) dbid (%d) tid (%d,%d) nHolding (%d) "\
 			 "holders (%d,%d,%d,%d,%d) type (%s)",where, \
-			 getpid(),\
+			 MyProcPid,\
 			 lock->tag.relId, lock->tag.dbId, \
 			 ((lock->tag.tupleId.ip_blkid.bi_hi<<16)+\
 			  lock->tag.tupleId.ip_blkid.bi_lo),\
@@ -114,7 +116,7 @@ static char *lock_types[] = {
 			 "%s: pid (%d) xid (%d) pid (%d) lock (%x) nHolding (%d) "\
 			 "holders (%d,%d,%d,%d,%d)",\
 			 where,\
-			 getpid(),\
+			 MyProcPid,\
 			 xidentP->tag.xid,\
 			 xidentP->tag.pid,\
 			 xidentP->tag.lock,\
@@ -541,7 +543,7 @@ LockAcquire(LockTableId tableId, LOCKTAG *lockName, LOCKT lockt)
 	 * word alignment and ensures hashing consistency).
 	 * ------------------
 	 */
-	MemSet(&item, 0, XID_TAGSIZE); /* must clear padding, needed */
+	MemSet(&item, 0, XID_TAGSIZE);		/* must clear padding, needed */
 	TransactionIdStore(myXid, &item.tag.xid);
 	item.tag.lock = MAKE_OFFSET(lock);
 #if 0
@@ -551,7 +553,7 @@ LockAcquire(LockTableId tableId, LOCKTAG *lockName, LOCKT lockt)
 #ifdef USER_LOCKS
 	if (is_user_lock)
 	{
-		item.tag.pid = getpid();
+		item.tag.pid = MyProcPid;
 		item.tag.xid = myXid = 0;
 #ifdef USER_LOCKS_DEBUG
 		elog(NOTICE, "LockAcquire: user lock xid [%d,%d,%d]",
@@ -709,6 +711,20 @@ LockResolveConflicts(LOCKTAB *ltable,
 		result->nHolding = 0;
 	}
 
+	{
+		/* ------------------------
+		 * If someone with a greater priority is waiting for the lock,
+		 * do not continue and share the lock, even if we can.	bjm
+		 * ------------------------
+		 */
+		int			myprio = ltable->ctl->prio[lockt];
+		PROC_QUEUE *waitQueue = &(lock->waitProcs);
+		PROC	   *topproc = (PROC *) MAKE_PTR(waitQueue->links.prev);
+
+		if (waitQueue->size && topproc->prio > myprio)
+			return STATUS_FOUND;
+	}
+
 	/* ----------------------------
 	 * first check for global conflicts: If no locks conflict
 	 * with mine, then I get the lock.
@@ -740,7 +756,7 @@ LockResolveConflicts(LOCKTAB *ltable,
 	tmpMask = 2;
 	for (i = 1; i <= nLockTypes; i++, tmpMask <<= 1)
 	{
-		if (lock->activeHolders[i] - myHolders[i])
+		if (lock->activeHolders[i] != myHolders[i])
 		{
 			bitmask |= tmpMask;
 		}
@@ -802,7 +818,7 @@ WaitOnLock(LOCKTAB *ltable, LockTableId tableId, LOCK *lock, LOCKT lockt)
 		lock->holders[lockt]--;
 		LOCK_DUMP_AUX("WaitOnLock: aborting on lock", lock, lockt);
 		SpinRelease(ltable->ctl->masterLock);
-		elog(WARN, "WaitOnLock: error on wakeup - Aborting this transaction");
+		elog(ERROR, "WaitOnLock: error on wakeup - Aborting this transaction");
 	}
 
 	LOCK_DUMP_AUX("WaitOnLock: wakeup on lock", lock, lockt);
@@ -887,7 +903,8 @@ LockRelease(LockTableId tableId, LOCKTAG *lockName, LOCKT lockt)
 #endif
 
 	/*
-	 * let the caller print its own error message, too. Do not elog(WARN).
+	 * let the caller print its own error message, too. Do not
+	 * elog(ERROR).
 	 */
 	if (!lock)
 	{
@@ -962,7 +979,7 @@ LockRelease(LockTableId tableId, LOCKTAG *lockName, LOCKT lockt)
 #ifdef USER_LOCKS
 	if (is_user_lock)
 	{
-		item.tag.pid = getpid();
+		item.tag.pid = MyProcPid;
 		item.tag.xid = 0;
 #ifdef USER_LOCKS_DEBUG
 		elog(NOTICE, "LockRelease: user lock xid [%d,%d,%d]",
@@ -1140,14 +1157,12 @@ LockReleaseAll(LockTableId tableId, SHM_QUEUE *lockQueue)
 
 #ifdef USER_LOCKS
 	int			is_user_lock_table,
-				my_pid,
 				count,
 				nskip;
 
 	is_user_lock_table = (tableId == 0);
-	my_pid = getpid();
 #ifdef USER_LOCKS_DEBUG
-	elog(NOTICE, "LockReleaseAll: tableId=%d, pid=%d", tableId, my_pid);
+	elog(NOTICE, "LockReleaseAll: tableId=%d, pid=%d", tableId, MyProcPid);
 #endif
 	if (is_user_lock_table)
 	{
@@ -1189,7 +1204,7 @@ LockReleaseAll(LockTableId tableId, SHM_QUEUE *lockQueue)
 		done = (xidLook->queue.next == end);
 		lock = (LOCK *) MAKE_PTR(xidLook->tag.lock);
 
-		LOCK_PRINT("ReleaseAll", (&lock->tag), 0);
+		LOCK_PRINT("LockReleaseAll", (&lock->tag), 0);
 
 #ifdef USER_LOCKS
 
@@ -1213,7 +1228,7 @@ LockReleaseAll(LockTableId tableId, SHM_QUEUE *lockQueue)
 				nskip++;
 				goto next_item;
 			}
-			if (xidLook->tag.pid != my_pid)
+			if (xidLook->tag.pid != MyProcPid)
 			{
 				/* This should never happen */
 #ifdef USER_LOCKS_DEBUG
@@ -1295,11 +1310,7 @@ LockReleaseAll(LockTableId tableId, SHM_QUEUE *lockQueue)
 			|| !found)
 		{
 			SpinRelease(masterLock);
-#ifdef USER_LOCKS
 			elog(NOTICE, "LockReleaseAll: xid table corrupted");
-#else
-			elog(NOTICE, "LockReplace: xid table corrupted");
-#endif
 			return (FALSE);
 		}
 
@@ -1317,11 +1328,7 @@ LockReleaseAll(LockTableId tableId, SHM_QUEUE *lockQueue)
 			if ((!lock) || (!found))
 			{
 				SpinRelease(masterLock);
-#ifdef USER_LOCKS
 				elog(NOTICE, "LockReleaseAll: cannot remove lock from HTAB");
-#else
-				elog(NOTICE, "LockReplace: cannot remove lock from HTAB");
-#endif
 				return (FALSE);
 			}
 		}
@@ -1403,6 +1410,184 @@ LockingDisabled()
 	return LockingIsDisabled;
 }
 
+/*
+ * DeadlockCheck -- Checks for deadlocks for a given process
+ *
+ * We can't block on user locks, so no sense testing for deadlock
+ * because there is no blocking, and no timer for the block.
+ *
+ * This code takes a list of locks a process holds, and the lock that
+ * the process is sleeping on, and tries to find if any of the processes
+ * waiting on its locks hold the lock it is waiting for.  If no deadlock
+ * is found, it goes on to look at all the processes waiting on their locks.
+ *
+ * We have already locked the master lock before being called.
+ */
+bool
+DeadLockCheck(SHM_QUEUE *lockQueue, LOCK *findlock, bool skip_check)
+{
+	int			done;
+	XIDLookupEnt *xidLook = NULL;
+	XIDLookupEnt *tmp = NULL;
+	SHMEM_OFFSET end = MAKE_OFFSET(lockQueue);
+	LOCK	   *lock;
+
+	LOCKTAB    *ltable;
+	XIDLookupEnt *result,
+				item;
+	HTAB	   *xidTable;
+	bool		found;
+
+	static PROC *checked_procs[MaxBackendId];
+	static int	nprocs;
+	static bool MyNHolding;
+
+	/* initialize at start of recursion */
+	if (skip_check)
+	{
+		checked_procs[0] = MyProc;
+		nprocs = 1;
+
+		ltable = AllTables[1];
+		xidTable = ltable->xidHash;
+
+		MemSet(&item, 0, XID_TAGSIZE);
+		TransactionIdStore(MyProc->xid, &item.tag.xid);
+		item.tag.lock = MAKE_OFFSET(findlock);
+#if 0
+		item.tag.pid = pid;
+#endif
+
+		if (!(result = (XIDLookupEnt *)
+			  hash_search(xidTable, (Pointer) &item, HASH_FIND, &found)) || !found)
+		{
+			elog(NOTICE, "LockAcquire: xid table corrupted");
+			return true;
+		}
+		MyNHolding = result->nHolding;
+	}
+	if (SHMQueueEmpty(lockQueue))
+		return false;
+
+	SHMQueueFirst(lockQueue, (Pointer *) &xidLook, &xidLook->queue);
+
+	XID_PRINT("DeadLockCheck", xidLook);
+
+	for (;;)
+	{
+		/* ---------------------------
+		 * XXX Here we assume the shared memory queue is circular and
+		 * that we know its internal structure.  Should have some sort of
+		 * macros to allow one to walk it.	mer 20 July 1991
+		 * ---------------------------
+		 */
+		done = (xidLook->queue.next == end);
+		lock = (LOCK *) MAKE_PTR(xidLook->tag.lock);
+
+		LOCK_PRINT("DeadLockCheck", (&lock->tag), 0);
+
+		/*
+		 * This is our only check to see if we found the lock we want.
+		 *
+		 * The lock we are waiting for is already in MyProc->lockQueue so we
+		 * need to skip it here.  We are trying to find it in someone
+		 * else's lockQueue.
+		 */
+		if (lock == findlock && !skip_check)
+			return true;
+
+		{
+			PROC_QUEUE *waitQueue = &(lock->waitProcs);
+			PROC	   *proc;
+			int			i;
+			int			j;
+
+			proc = (PROC *) MAKE_PTR(waitQueue->links.prev);
+			for (i = 0; i < waitQueue->size; i++)
+			{
+				if (proc != MyProc &&
+					lock == findlock && /* skip_check also true */
+					MyNHolding) /* I already hold some lock on it */
+				{
+
+					/*
+					 * For findlock's wait queue, we are interested in
+					 * procs who are blocked waiting for a write-lock on
+					 * the table we are waiting on, and already hold a
+					 * lock on it. We first check to see if there is an
+					 * escalation deadlock, where we hold a readlock and
+					 * want a writelock, and someone else holds readlock
+					 * on the same table, and wants a writelock.
+					 *
+					 * Basically, the test is, "Do we both hold some lock on
+					 * findlock, and we are both waiting in the lock
+					 * queue?"
+					 */
+
+					Assert(skip_check);
+					Assert(MyProc->prio == 2);
+
+					ltable = AllTables[1];
+					xidTable = ltable->xidHash;
+
+					MemSet(&item, 0, XID_TAGSIZE);
+					TransactionIdStore(proc->xid, &item.tag.xid);
+					item.tag.lock = MAKE_OFFSET(findlock);
+#if 0
+					item.tag.pid = pid;
+#endif
+
+					if (!(result = (XIDLookupEnt *)
+						  hash_search(xidTable, (Pointer) &item, HASH_FIND, &found)) || !found)
+					{
+						elog(NOTICE, "LockAcquire: xid table corrupted");
+						return true;
+					}
+					if (result->nHolding)
+						return true;
+				}
+
+				/*
+				 * No sense in looking at the wait queue of the lock we
+				 * are looking for. If lock == findlock, and I got here,
+				 * skip_check must be true too.
+				 */
+				if (lock != findlock)
+				{
+					for (j = 0; j < nprocs; j++)
+						if (checked_procs[j] == proc)
+							break;
+					if (j >= nprocs && lock != findlock)
+					{
+						checked_procs[nprocs++] = proc;
+						Assert(nprocs <= MaxBackendId);
+
+						/*
+						 * For non-MyProc entries, we are looking only
+						 * waiters, not necessarily people who already
+						 * hold locks and are waiting. Now we check for
+						 * cases where we have two or more tables in a
+						 * deadlock.  We do this by continuing to search
+						 * for someone holding a lock
+						 */
+						if (DeadLockCheck(&(proc->lockQueue), findlock, false))
+							return true;
+					}
+				}
+				proc = (PROC *) MAKE_PTR(proc->links.prev);
+			}
+		}
+
+		if (done)
+			break;
+		SHMQueueFirst(&xidLook->queue, (Pointer *) &tmp, &tmp->queue);
+		xidLook = tmp;
+	}
+
+	/* if we got here, no deadlock */
+	return false;
+}
+
 #ifdef DEADLOCK_DEBUG
 /*
  * Dump all locks. Must have already acquired the masterLock.
@@ -1420,13 +1605,12 @@ DumpLocks()
 	SPINLOCK	masterLock;
 	int			nLockTypes;
 	LOCK	   *lock;
-	int			pid,
-				count;
+
+	count;
 	int			tableId = 1;
 	LOCKTAB    *ltable;
 
-	pid = getpid();
-	ShmemPIDLookup(pid, &location);
+	ShmemPIDLookup(MyProcPid, &location);
 	if (location == INVALID_OFFSET)
 		return;
 	proc = (PROC *) MAKE_PTR(location);
