@@ -12,7 +12,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/multi.c,v 1.8 1997/09/18 20:21:38 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/multi.c,v 1.11 1998/01/23 19:53:40 scrappy Exp $
  *
  * NOTES:
  *	 (1) The lock.c module assumes that the caller here is doing
@@ -31,10 +31,10 @@
 
 static bool
 MultiAcquire(LockTableId tableId, LOCKTAG *tag, LOCKT lockt,
-			 LOCK_LEVEL level);
+			 PG_LOCK_LEVEL level);
 static bool
 MultiRelease(LockTableId tableId, LOCKTAG *tag, LOCKT lockt,
-			 LOCK_LEVEL level);
+			 PG_LOCK_LEVEL level);
 
 /*
  * INTENT indicates to higher level that a lower level lock has been
@@ -100,14 +100,14 @@ InitMultiLevelLockm()
 	MultiTableId = tableId;
 	if (!(MultiTableId))
 	{
-		elog(WARN, "InitMultiLockm: couldnt initialize lock table");
+		elog(ERROR, "InitMultiLockm: couldnt initialize lock table");
 	}
 	/* -----------------------
 	 * No short term lock table for now.  -Jeff 15 July 1991
 	 *
 	 * ShortTermTableId = LockTabRename(tableId);
 	 * if (! (ShortTermTableId)) {
-	 *	 elog(WARN,"InitMultiLockm: couldnt rename lock table");
+	 *	 elog(ERROR,"InitMultiLockm: couldnt rename lock table");
 	 * }
 	 * -----------------------
 	 */
@@ -201,7 +201,7 @@ static bool
 MultiAcquire(LockTableId tableId,
 			 LOCKTAG *tag,
 			 LOCKT lockt,
-			 LOCK_LEVEL level)
+			 PG_LOCK_LEVEL level)
 {
 	LOCKT		locks[N_LEVELS];
 	int			i,
@@ -237,7 +237,7 @@ MultiAcquire(LockTableId tableId,
 			locks[2] = lockt;
 			break;
 		default:
-			elog(WARN, "MultiAcquire: bad lock level");
+			elog(ERROR, "MultiAcquire: bad lock level");
 			return (FALSE);
 	}
 
@@ -361,7 +361,7 @@ static bool
 MultiRelease(LockTableId tableId,
 			 LOCKTAG *tag,
 			 LOCKT lockt,
-			 LOCK_LEVEL level)
+			 PG_LOCK_LEVEL level)
 {
 	LOCKT		locks[N_LEVELS];
 	int			i,
@@ -390,7 +390,7 @@ MultiRelease(LockTableId tableId,
 			locks[2] = lockt;
 			break;
 		default:
-			elog(WARN, "MultiRelease: bad lockt");
+			elog(ERROR, "MultiRelease: bad lockt");
 	}
 
 	/*
@@ -435,7 +435,7 @@ MultiRelease(LockTableId tableId,
 			status = LockRelease(tableId, tmpTag, locks[i]);
 			if (!status)
 			{
-				elog(WARN, "MultiRelease: couldn't release after error");
+				elog(ERROR, "MultiRelease: couldn't release after error");
 			}
 		}
 	}
