@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/page/bufpage.c,v 1.11 1997/09/18 20:21:41 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/page/bufpage.c,v 1.16 1998/04/06 02:38:17 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -24,8 +24,6 @@
 #include "utils/memutils.h"
 #include "storage/bufpage.h"
 
-#include "lib/qsort.h"
-
 static void
 PageIndexTupleDeleteAdjustLinePointers(PageHeader phdr,
 									   char *location, Size size);
@@ -98,7 +96,7 @@ PageAddItem(Page page,
 			OffsetNumber offsetNumber,
 			ItemIdFlags flags)
 {
-	register	i;
+	int			i;
 	Size		alignedSize;
 	Offset		lower;
 	Offset		upper;
@@ -134,7 +132,7 @@ PageAddItem(Page page,
 			if (((*itemId).lp_flags & LP_USED) ||
 				((*itemId).lp_len != 0))
 			{
-				elog(WARN, "PageAddItem: tried overwrite of used ItemId");
+				elog(ERROR, "PageAddItem: tried overwrite of used ItemId");
 				return (InvalidOffsetNumber);
 			}
 		}
@@ -259,7 +257,7 @@ struct itemIdSortData
 };
 
 static int
-itemidcompare(void *itemidp1, void *itemidp2)
+itemidcompare(const void *itemidp1, const void *itemidp2)
 {
 	if (((struct itemIdSortData *) itemidp1)->itemiddata.lp_off ==
 		((struct itemIdSortData *) itemidp2)->itemiddata.lp_off)
@@ -330,7 +328,7 @@ PageRepairFragmentation(Page page)
 		}
 
 		/* sort itemIdSortData array... */
-		pg_qsort((char *) itemidbase, nused, sizeof(struct itemIdSortData),
+		qsort((char *) itemidbase, nused, sizeof(struct itemIdSortData),
 				 itemidcompare);
 
 		/* compactify page */
