@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/smgr/smgrtype.c,v 1.5 1997/09/08 21:47:42 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/smgr/smgrtype.c,v 1.8 1998/02/26 04:36:17 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -49,17 +49,17 @@ smgrin(char *s)
 		if (strcmp(s, StorageManager[i].smgr_name) == 0)
 			return ((int2) i);
 	}
-	elog(WARN, "smgrin: illegal storage manager name %s", s);
+	elog(ERROR, "smgrin: illegal storage manager name %s", s);
 	return 0;
 }
 
-char	   *
+char *
 smgrout(int2 i)
 {
 	char	   *s;
 
 	if (i >= NStorageManagers || i < 0)
-		elog(WARN, "Illegal storage manager id %d", i);
+		elog(ERROR, "Illegal storage manager id %d", i);
 
 	s = (char *) palloc(strlen(StorageManager[i].smgr_name) + 1);
 	strcpy(s, StorageManager[i].smgr_name);
