@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/tcop/dest.c,v 1.12 1997/09/08 02:29:42 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/tcop/dest.c,v 1.17 1998/02/26 04:36:24 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -29,7 +29,7 @@
 #include "postgres.h"
 
 #include "access/htup.h"
-#include "libpq/libpq-be.h"
+#include "libpq/libpq.h"
 #include "access/printtup.h"
 #include "utils/portal.h"
 #include "utils/palloc.h"
@@ -192,9 +192,9 @@ NullCommand(CommandDest dest)
 					 nPtr = (PQNotifyList *) SLGetSucc(&nPtr->Node))
 				{
 					pq_putnchar("A", 1);
-					pq_putint(0, 4);
+					pq_putint(0, sizeof(int4));
 					pq_putstr(nPtr->relname);
-					pq_putint(nPtr->be_pid, 4);
+					pq_putint(nPtr->be_pid, sizeof(nPtr->be_pid));
 					PQremoveNotify(nPtr);
 				}
 				pq_flush();
@@ -283,11 +283,10 @@ BeginCommand(char *pname,
 				{
 					pq_putstr(attrs[i]->attname.data);	/* if 16 char name
 														 * oops.. */
-					pq_putint((int) attrs[i]->atttypid, 4);
-					pq_putint(attrs[i]->attlen, 2);
+					pq_putint((int) attrs[i]->atttypid, sizeof(attrs[i]->atttypid));
+					pq_putint(attrs[i]->attlen, sizeof(attrs[i]->attlen));
 				}
 			}
-			pq_flush();
 			break;
 
 		case Local:
