@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/tcop/fastpath.c,v 1.7 1997/09/18 20:21:58 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/tcop/fastpath.c,v 1.15 1998/04/07 18:11:18 momjian Exp $
  *
  * NOTES
  *	  This cruft is the server side of PQfn.
@@ -101,7 +101,7 @@ SendFunctionResult(Oid fid,		/* function id */
 		{						/* by-reference ... */
 			if (retlen < 0)
 			{					/* ... varlena */
-				pq_putint(VARSIZE(retval) - VARHDRSZ, 4);
+				pq_putint(VARSIZE(retval) - VARHDRSZ, VARHDRSZ);
 				pq_putnchar(VARDATA(retval), VARSIZE(retval) - VARHDRSZ);
 			}
 			else
@@ -199,7 +199,7 @@ update_fp_info(Oid func_id, struct fp_info * fip)
 	 * Since the validity of this structure is determined by whether the
 	 * funcid is OK, we clear the funcid here.	It must not be set to the
 	 * correct value until we are about to return with a good struct
-	 * fp_info, since we can be interrupted (i.e., with an elog(WARN,
+	 * fp_info, since we can be interrupted (i.e., with an elog(ERROR,
 	 * ...)) at any time.
 	 */
 	MemSet((char *) fip, 0, (int) sizeof(struct fp_info));
@@ -209,7 +209,7 @@ update_fp_info(Oid func_id, struct fp_info * fip)
 								   0, 0, 0);
 	if (!HeapTupleIsValid(func_htp))
 	{
-		elog(WARN, "update_fp_info: cache lookup for function %d failed",
+		elog(ERROR, "update_fp_info: cache lookup for function %d failed",
 			 func_id);
 	}
 	pp = (Form_pg_proc) GETSTRUCT(func_htp);
@@ -226,7 +226,7 @@ update_fp_info(Oid func_id, struct fp_info * fip)
 										   0, 0, 0);
 			if (!HeapTupleIsValid(type_htp))
 			{
-				elog(WARN, "update_fp_info: bad argument type %d for %d",
+				elog(ERROR, "update_fp_info: bad argument type %d for %d",
 					 argtypes[i], func_id);
 			}
 			tp = (TypeTupleForm) GETSTRUCT(type_htp);
@@ -241,7 +241,7 @@ update_fp_info(Oid func_id, struct fp_info * fip)
 									   0, 0, 0);
 		if (!HeapTupleIsValid(type_htp))
 		{
-			elog(WARN, "update_fp_info: bad return type %d for %d",
+			elog(ERROR, "update_fp_info: bad return type %d for %d",
 				 rettype, func_id);
 		}
 		tp = (TypeTupleForm) GETSTRUCT(type_htp);
@@ -267,7 +267,7 @@ update_fp_info(Oid func_id, struct fp_info * fip)
  *
  * RETURNS:
  *		nothing of significance.
- *		All errors result in elog(WARN,...).
+ *		All errors result in elog(ERROR,...).
  */
 int
 HandleFunctionRequest()
@@ -297,7 +297,7 @@ HandleFunctionRequest()
 
 	if (fip->nargs != nargs)
 	{
-		elog(WARN, "HandleFunctionRequest: actual arguments (%d) != registered arguments (%d)",
+		elog(ERROR, "HandleFunctionRequest: actual arguments (%d) != registered arguments (%d)",
 			 nargs, fip->nargs);
 	}
 
@@ -326,9 +326,11 @@ HandleFunctionRequest()
 			{					/* by-reference ... */
 				if (fip->arglen[i] < 0)
 				{				/* ... varlena */
-					if (!(p = palloc(argsize + VARHDRSZ)))
+					if (!(p = palloc(argsize + VARHDRSZ + 1)))	/* Added +1 to solve
+																 * memory leak - Peter
+																 * 98 Jan 6 */
 					{
-						elog(WARN, "HandleFunctionRequest: palloc failed");
+						elog(ERROR, "HandleFunctionRequest: palloc failed");
 					}
 					VARSIZE(p) = argsize + VARHDRSZ;
 					pq_getnchar(VARDATA(p), 0, argsize);
@@ -336,9 +338,9 @@ HandleFunctionRequest()
 				else
 				{				/* ... fixed */
 					/* XXX cross our fingers and trust "argsize" */
-					if (!(p = palloc(argsize)))
+					if (!(p = palloc(argsize + 1)))
 					{
-						elog(WARN, "HandleFunctionRequest: palloc failed");
+						elog(ERROR, "HandleFunctionRequest: palloc failed");
 					}
 					pq_getnchar(p, 0, argsize);
 				}
