@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/arrayfuncs.c,v 1.20 1997/09/18 20:22:10 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/arrayfuncs.c,v 1.29 1998/03/30 16:47:23 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,9 +45,9 @@
 static int	_ArrayCount(char *str, int dim[], int typdelim);
 static char *
 _ReadArrayStr(char *arrayStr, int nitems, int ndim, int dim[],
-			  func_ptr inputproc, Oid typelem, char typdelim,
-			  int typlen, bool typbyval, char typalign,
-			  int *nbytes);
+			  FmgrInfo *inputproc, Oid typelem, int16 typmod,
+			  char typdelim, int typlen, bool typbyval,
+			  char typalign, int *nbytes);
 
 #ifdef LOARRAY
 static char *
@@ -78,8 +78,8 @@ _LOArrayRange(int st[], int endp[], int bsize, int srcfd,
 static void
 _ReadArray(int st[], int endp[], int bsize, int srcfd, int destfd,
 		   ArrayType *array, int isDestLO, bool *isNull);
-static ArrayCastAndSet(char *src, bool typbyval, int typlen, char *dest);
-static SanityCheckInput(int ndim, int n, int dim[], int lb[], int indx[]);
+static int  ArrayCastAndSet(char *src, bool typbyval, int typlen, char *dest);
+static int  SanityCheckInput(int ndim, int n, int dim[], int lb[], int indx[]);
 static int	array_read(char *destptr, int eltsize, int nitems, char *srcptr);
 static char *array_seek(char *ptr, int eltsize, int nitems);
 
@@ -91,9 +91,10 @@ static char *array_seek(char *ptr, int eltsize, int nitems);
  *		  the internal representation of the input array
  *--------------------------------------------------------------------
  */
-char	   *
+char *
 array_in(char *string,			/* input array in external form */
-		 Oid element_type)		/* type OID of an array element */
+		 Oid element_type,		/* type OID of an array element */
+		 int16 typmod)
 {
 	int			typlen;
 	bool		typbyval,
@@ -105,10 +106,9 @@ array_in(char *string,			/* input array in external form */
 			   *p,
 			   *q,
 			   *r;
-	func_ptr	inputproc;
+	FmgrInfo	inputproc;
 	int			i,
-				nitems,
-				dummy;
+				nitems;
 	int32		nbytes;
 	char	   *dataPtr;
 	ArrayType  *retval = NULL;
@@ -120,7 +120,7 @@ array_in(char *string,			/* input array in external form */
 	system_cache_lookup(element_type, true, &typlen, &typbyval, &typdelim,
 						&typelem, &typinput, &typalign);
 
-	fmgr_info(typinput, &inputproc, &dummy);
+	fmgr_info(typinput, &inputproc);
 
 	string_save = (char *) palloc(strlen(string) + 3);
 	strcpy(string_save, string);
@@ -145,14 +145,14 @@ array_in(char *string,			/* input array in external form */
 			}
 			for (q = p; isdigit(*q); q++);
 			if (*q != ']')
-				elog(WARN, "array_in: missing ']' in array declaration");
+				elog(ERROR, "array_in: missing ']' in array declaration");
 			*q = '\0';
 			dim[ndim] = atoi(p);
 			if ((dim[ndim] < 0) || (lBound[ndim] < 0))
-				elog(WARN, "array_in: array dimensions need to be positive");
+				elog(ERROR, "array_in: array dimensions need to be positive");
 			dim[ndim] = dim[ndim] - lBound[ndim] + 1;
 			if (dim[ndim] < 0)
-				elog(WARN, "array_in: upper_bound cannot be < lower_bound");
+				elog(ERROR, "array_in: upper_bound cannot be < lower_bound");
 			p = q + 1;
 			ndim++;
 		}
@@ -171,7 +171,7 @@ array_in(char *string,			/* input array in external form */
 		}
 		else
 		{
-			elog(WARN, "array_in: Need to specify dimension");
+			elog(ERROR, "array_in: Need to specify dimension");
 		}
 	}
 	else
@@ -179,7 +179,7 @@ array_in(char *string,			/* input array in external form */
 		while (isspace(*p))
 			p++;
 		if (strncmp(p, ASSGN, strlen(ASSGN)))
-			elog(WARN, "array_in: missing assignment operator");
+			elog(ERROR, "array_in: missing assignment operator");
 		p += strlen(ASSGN);
 		while (isspace(*p))
 			p++;
@@ -208,8 +208,8 @@ array_in(char *string,			/* input array in external form */
 	{
 		/* array not a large object */
 		dataPtr =
-			(char *) _ReadArrayStr(p, nitems, ndim, dim, inputproc, typelem,
-								   typdelim, typlen, typbyval, typalign,
+			(char *) _ReadArrayStr(p, nitems, ndim, dim, &inputproc, typelem,
+							typmod, typdelim, typlen, typbyval, typalign,
 								   &nbytes);
 		nbytes += ARR_OVERHEAD(ndim);
 		retval = (ArrayType *) palloc(nbytes);
@@ -249,7 +249,7 @@ array_in(char *string,			/* input array in external form */
 		memmove((char *) ARR_LBOUND(retval), (char *) lBound, ndim * sizeof(int));
 		memmove(ARR_DATA_PTR(retval), dataPtr, bytes);
 #endif
-		elog(WARN, "large object arrays not supported");
+		elog(ERROR, "large object arrays not supported");
 	}
 	pfree(string_save);
 	return ((char *) retval);
@@ -302,7 +302,7 @@ _ArrayCount(char *str, int dim[], int typdelim)
 					 * Signal a premature end of the string.  DZ -
 					 * 2-9-1996
 					 */
-					elog(WARN, "malformed array constant: %s", str);
+					elog(ERROR, "malformed array constant: %s", str);
 					break;
 				case '\"':
 					scanning_string = !scanning_string;
@@ -367,9 +367,10 @@ _ReadArrayStr(char *arrayStr,
 			  int nitems,
 			  int ndim,
 			  int dim[],
-			  func_ptr inputproc,		/* function used for the
+			  FmgrInfo *inputproc,		/* function used for the
 										 * conversion */
 			  Oid typelem,
+			  int16 typmod,
 			  char typdelim,
 			  int typlen,
 			  bool typbyval,
@@ -425,7 +426,7 @@ _ReadArrayStr(char *arrayStr,
 						p++;
 						nest_level++;
 						if (nest_level > ndim)
-							elog(WARN, "array_in: illformed array constant");
+							elog(ERROR, "array_in: illformed array constant");
 						indx[nest_level - 1] = 0;
 						indx[ndim - 1] = 0;
 					}
@@ -460,8 +461,8 @@ _ReadArrayStr(char *arrayStr,
 		}
 		*q = '\0';
 		if (i >= nitems)
-			elog(WARN, "array_in: illformed array constant");
-		values[i] = (*inputproc) (p, typelem);
+			elog(ERROR, "array_in: illformed array constant");
+		values[i] = (*fmgr_faddr(inputproc)) (p, typelem, typmod);
 		p = ++q;
 		if (!eoArray)
 
@@ -544,27 +545,27 @@ _ReadLOArray(char *str,
 		if (!strcmp(word, "-chunk"))
 		{
 			if (str == NULL)
-				elog(WARN, "array_in: access pattern file required");
+				elog(ERROR, "array_in: access pattern file required");
 			str = _AdvanceBy1word(str, &accessfile);
 		}
 		else if (!strcmp(word, "-noreorg"))
 		{
 			if (str == NULL)
-				elog(WARN, "array_in: chunk file required");
+				elog(ERROR, "array_in: chunk file required");
 			str = _AdvanceBy1word(str, &chunkfile);
 		}
 		else
 		{
-			elog(WARN, "usage: <input file> -chunk DEFAULT/<access pattern file> -invert/-native [-noreorg <chunk file>]");
+			elog(ERROR, "usage: <input file> -chunk DEFAULT/<access pattern file> -invert/-native [-noreorg <chunk file>]");
 		}
 	}
 
 	if (inputfile == NULL)
-		elog(WARN, "array_in: missing file name");
+		elog(ERROR, "array_in: missing file name");
 	lobjId = lo_creat(0);
 	*fd = lo_open(lobjId, INV_READ);
 	if (*fd < 0)
-		elog(WARN, "Large object create failed");
+		elog(ERROR, "Large object create failed");
 	retStr = inputfile;
 	*nbytes = strlen(retStr) + 2;
 
@@ -573,7 +574,7 @@ _ReadLOArray(char *str,
 		FILE	   *afd;
 
 		if ((afd = AllocateFile(accessfile, "r")) == NULL)
-			elog(WARN, "unable to open access pattern file");
+			elog(ERROR, "unable to open access pattern file");
 		*chunkFlag = true;
 		retStr = _ChunkArray(*fd, afd, ndim, dim, baseSize, nbytes,
 							 chunkfile);
@@ -612,7 +613,7 @@ _CopyArrayEls(char **values,
  *		  containing the array in its external format.
  *-------------------------------------------------------------------------
  */
-char	   *
+char *
 array_out(ArrayType *v, Oid element_type)
 {
 	int			typlen;
@@ -620,10 +621,10 @@ array_out(ArrayType *v, Oid element_type)
 	char		typdelim;
 	Oid			typoutput,
 				typelem;
-	func_ptr	outputproc;
+	FmgrInfo	outputproc;
 	char		typalign;
 
-	char	   *p,
+	char	   *p, *tmp, 
 			   *retval,
 			  **values,
 				delim[2];
@@ -632,9 +633,9 @@ array_out(ArrayType *v, Oid element_type)
 				i,
 				j,
 				k,
+				l,
 				indx[MAXDIM];
 	bool		dummy_bool;
-	int			dummy_int;
 	int			ndim,
 			   *dim;
 
@@ -649,7 +650,7 @@ array_out(ArrayType *v, Oid element_type)
 
 		/* get a wide string to print to */
 		p = array_dims(v, &dummy_bool);
-		nbytes = strlen(ARR_DATA_PTR(v)) + 4 + *(int *) p;
+		nbytes = strlen(ARR_DATA_PTR(v)) + VARHDRSZ + *(int *) p;
 
 		save_p = (char *) palloc(nbytes);
 
@@ -662,7 +663,7 @@ array_out(ArrayType *v, Oid element_type)
 
 	system_cache_lookup(element_type, false, &typlen, &typbyval,
 						&typdelim, &typelem, &typoutput, &typalign);
-	fmgr_info(typoutput, &outputproc, &dummy_int);
+	fmgr_info(typoutput, &outputproc);
 	sprintf(delim, "%c", typdelim);
 	ndim = ARR_NDIM(v);
 	dim = ARR_DIMS(v);
@@ -688,21 +689,21 @@ array_out(ArrayType *v, Oid element_type)
 			switch (typlen)
 			{
 				case 1:
-					values[i] = (*outputproc) (*p, typelem);
+					values[i] = (*fmgr_faddr(&outputproc)) (*p, typelem);
 					break;
 				case 2:
-					values[i] = (*outputproc) (*(int16 *) p, typelem);
+					values[i] = (*fmgr_faddr(&outputproc)) (*(int16 *) p, typelem);
 					break;
 				case 3:
 				case 4:
-					values[i] = (*outputproc) (*(int32 *) p, typelem);
+					values[i] = (*fmgr_faddr(&outputproc)) (*(int32 *) p, typelem);
 					break;
 			}
 			p += typlen;
 		}
 		else
 		{
-			values[i] = (*outputproc) (p, typelem);
+			values[i] = (*fmgr_faddr(&outputproc)) (p, typelem);
 			if (typlen > 0)
 				p += typlen;
 			else
@@ -713,7 +714,11 @@ array_out(ArrayType *v, Oid element_type)
 			 */
 			overall_length += 2;
 		}
-		overall_length += (strlen(values[i]) + 1);
+		for (tmp=values[i];*tmp;tmp++) {
+			overall_length += 1;
+			if (*tmp=='"') overall_length += 1;
+		}
+		overall_length += 1;
 	}
 
 	/*
@@ -740,7 +745,12 @@ array_out(ArrayType *v, Oid element_type)
 		if (!typbyval)
 		{
 			strcat(p, "\"");
-			strcat(p, values[k]);
+			l=strlen(p);
+			for (tmp=values[k];*tmp;tmp++) {
+				if (*tmp=='"') p[l++]='\\';
+				p[l++]=*tmp;
+				}
+			p[l]='\0';
 			strcat(p, "\"");
 		}
 		else
@@ -770,7 +780,7 @@ array_out(ArrayType *v, Oid element_type)
  *		  returns the dimension of the array pointed to by "v"
  *----------------------------------------------------------------------------
  */
-char	   *
+char *
 array_dims(ArrayType *v, bool *isNull)
 {
 	char	   *p,
@@ -787,18 +797,18 @@ array_dims(ArrayType *v, bool *isNull)
 	/*
 	 * 33 since we assume 15 digits per number + ':' +'[]'
 	 */
-	save_p = p = (char *) palloc(nbytes + 4);
-	MemSet(save_p, 0, nbytes + 4);
+	save_p = p = (char *) palloc(nbytes + VARHDRSZ);
+	MemSet(save_p, 0, nbytes + VARHDRSZ);
 	dimv = ARR_DIMS(v);
 	lb = ARR_LBOUND(v);
-	p += 4;
+	p += VARHDRSZ;
 	for (i = 0; i < ARR_NDIM(v); i++)
 	{
 		sprintf(p, "[%d:%d]", lb[i], dimv[i] + lb[i] - 1);
 		p += strlen(p);
 	}
-	nbytes = strlen(save_p + 4) + 4;
-	memmove(save_p, &nbytes, 4);
+	nbytes = strlen(save_p + VARHDRSZ) + VARHDRSZ;
+	memmove(save_p, &nbytes, VARHDRSZ);
 	return (save_p);
 }
 
@@ -836,7 +846,7 @@ array_ref(ArrayType *array,
 		 * fixed length arrays -- these are assumed to be 1-d
 		 */
 		if (indx[0] * elmlen > arraylen)
-			elog(WARN, "array_ref: array bound exceeded");
+			elog(ERROR, "array_ref: array bound exceeded");
 		retval = (char *) array + indx[0] * elmlen;
 		return _ArrayCast(retval, reftype, elmlen);
 	}
@@ -874,7 +884,7 @@ array_ref(ArrayType *array,
 		}
 		if (*isNull)
 			RETURN_NULL;
-		if (VARSIZE(v) - 4 < elmlen)
+		if (VARSIZE(v) - VARHDRSZ < elmlen)
 			RETURN_NULL;
 		lo_close(fd);
 		retval = (char *) _ArrayCast((char *) VARDATA(v), reftype, elmlen);
@@ -964,7 +974,7 @@ array_clip(ArrayType *array,
 
 	for (i = 0; i < n; i++)
 		if (lowerIndx[i] > upperIndx[i])
-			elog(WARN, "lowerIndex cannot be larger than upperIndx");
+			elog(ERROR, "lowerIndex cannot be larger than upperIndx");
 	mda_get_range(n, span, lowerIndx, upperIndx);
 
 	if (ARR_IS_LO(array))
@@ -980,7 +990,7 @@ array_clip(ArrayType *array,
 					rsize;
 
 		if (len < 0)
-			elog(WARN, "array_clip: array of variable length objects not supported");
+			elog(ERROR, "array_clip: array of variable length objects not supported");
 #ifdef LOARRAY
 		lo_name = (char *) ARR_DATA_PTR(array);
 		if ((fd = LOopen(lo_name, ARR_IS_INV(array) ? INV_READ : O_RDONLY)) < 0)
@@ -1000,22 +1010,22 @@ array_clip(ArrayType *array,
 		{
 			char	   *buff;
 
-			rsize += 4;
+			rsize += VARHDRSZ;
 			buff = palloc(rsize);
 			if (buff)
 				isDestLO = false;
 			if (ARR_IS_CHUNKED(array))
 			{
-				_ReadChunkArray(lowerIndx, upperIndx, len, fd, &(buff[4]),
+				_ReadChunkArray(lowerIndx, upperIndx, len, fd, &(buff[VARHDRSZ]),
 								array, 0, isNull);
 			}
 			else
 			{
-				_ReadArray(lowerIndx, upperIndx, len, fd, (int) &(buff[4]),
+				_ReadArray(lowerIndx, upperIndx, len, fd, (int) &(buff[VARHDRSZ]),
 						   array,
 						   0, isNull);
 			}
-			memmove(buff, &rsize, 4);
+			memmove(buff, &rsize, VARHDRSZ);
 #ifdef LOARRAY
 			if (!*isNull)
 				bytes = LOwrite(newfd, (struct varlena *) buff);
@@ -1023,6 +1033,7 @@ array_clip(ArrayType *array,
 			pfree(buff);
 		}
 		if (isDestLO)
+		{
 			if (ARR_IS_CHUNKED(array))
 			{
 				_ReadChunkArray(lowerIndx, upperIndx, len, fd, (char *) newfd, array,
@@ -1032,6 +1043,7 @@ array_clip(ArrayType *array,
 			{
 				_ReadArray(lowerIndx, upperIndx, len, fd, newfd, array, 1, isNull);
 			}
+		}
 #ifdef LOARRAY
 		LOclose(fd);
 		LOclose(newfd);
@@ -1072,7 +1084,7 @@ array_clip(ArrayType *array,
  *		  returns a pointer to the modified array.
  *-----------------------------------------------------------------------------
  */
-char	   *
+char *
 array_set(ArrayType *array,
 		  int n,
 		  int indx[],
@@ -1098,7 +1110,7 @@ array_set(ArrayType *array,
 		 * fixed length arrays -- these are assumed to be 1-d
 		 */
 		if (indx[0] * elmlen > arraylen)
-			elog(WARN, "array_ref: array bound exceeded");
+			elog(ERROR, "array_ref: array bound exceeded");
 		pos = (char *) array + indx[0] * elmlen;
 		ArrayCastAndSet(dataPtr, (bool) reftype, elmlen, pos);
 		return ((char *) array);
@@ -1110,7 +1122,7 @@ array_set(ArrayType *array,
 
 	if (!SanityCheckInput(ndim, n, dim, lb, indx))
 	{
-		elog(WARN, "array_set: array bound exceeded");
+		elog(ERROR, "array_set: array bound exceeded");
 		return ((char *) array);
 	}
 	offset = GetOffset(n, dim, lb, indx);
@@ -1131,15 +1143,15 @@ array_set(ArrayType *array,
 #endif
 		if (lo_lseek(fd, offset, SEEK_SET) < 0)
 			return ((char *) array);
-		v = (struct varlena *) palloc(elmlen + 4);
-		VARSIZE(v) = elmlen + 4;
+		v = (struct varlena *) palloc(elmlen + VARHDRSZ);
+		VARSIZE(v) = elmlen + VARHDRSZ;
 		ArrayCastAndSet(dataPtr, (bool) reftype, elmlen, VARDATA(v));
 #ifdef LOARRAY
 		n = LOwrite(fd, v);
 #endif
 
 		/*
-		 * if (n < VARSIZE(v) - 4) RETURN_NULL;
+		 * if (n < VARSIZE(v) - VARHDRSZ) RETURN_NULL;
 		 */
 		pfree(v);
 		lo_close(fd);
@@ -1207,7 +1219,7 @@ array_set(ArrayType *array,
  *		  returns a pointer to the modified array.
  *----------------------------------------------------------------------------
  */
-char	   *
+char *
 array_assgn(ArrayType *array,
 			int n,
 			int upperIndx[],
@@ -1225,7 +1237,7 @@ array_assgn(ArrayType *array,
 	if (array == (ArrayType *) NULL)
 		RETURN_NULL;
 	if (len < 0)
-		elog(WARN, "array_assgn:updates on arrays of variable length elements not allowed");
+		elog(ERROR, "array_assgn:updates on arrays of variable length elements not allowed");
 
 	dim = ARR_DIMS(array);
 	lb = ARR_LBOUND(array);
@@ -1239,7 +1251,7 @@ array_assgn(ArrayType *array,
 
 	for (i = 0; i < n; i++)
 		if (lowerIndx[i] > upperIndx[i])
-			elog(WARN, "lowerIndex larger than upperIndx");
+			elog(ERROR, "lowerIndex larger than upperIndx");
 
 	if (ARR_IS_LO(array))
 	{
@@ -1315,7 +1327,7 @@ system_cache_lookup(Oid element_type,
 
 	if (!HeapTupleIsValid(typeTuple))
 	{
-		elog(WARN, "array_out: Cache lookup failed for type %d\n",
+		elog(ERROR, "array_out: Cache lookup failed for type %d\n",
 			 element_type);
 		return;
 	}
@@ -1350,7 +1362,7 @@ _ArrayCast(char *value, bool byval, int len)
 			case 4:
 				return ((Datum) *(int32 *) value);
 			default:
-				elog(WARN, "array_ref: byval and elt len > 4!");
+				elog(ERROR, "array_ref: byval and elt len > 4!");
 				break;
 		}
 	}
@@ -1692,7 +1704,7 @@ _LOtransfer(char **destfd,
 		{
 #ifdef LOARRAY
 			v = (struct varlena *) LOread((int) *srcfd, inc);
-			if (VARSIZE(v) - 4 < inc)
+			if (VARSIZE(v) - VARHDRSZ < inc)
 			{
 				pfree(v);
 				return (-1);
@@ -1723,7 +1735,7 @@ _LOtransfer(char **destfd,
 #undef MAX_READ
 }
 
-char	   *
+char *
 _array_newLO(int *fd, int flag)
 {
 	char	   *p;
@@ -1734,7 +1746,7 @@ _array_newLO(int *fd, int flag)
 	strcpy(saveName, p);
 #ifdef LOARRAY
 	if ((*fd = LOcreat(saveName, 0600, flag)) < 0)
-		elog(WARN, "Large object create failed");
+		elog(ERROR, "Large object create failed");
 #endif
 	return (p);
 }
