@@ -12,7 +12,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/char.c,v 1.11 1997/09/08 21:48:18 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/char.c,v 1.17 1998/04/07 18:11:25 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -40,7 +40,7 @@ charin(char *ch)
 /*
  *		charout			- converts 'x' to "x"
  */
-char	   *
+char *
 charout(int32 ch)
 {
 	char	   *result = (char *) palloc(2);
@@ -75,18 +75,13 @@ cidin(char *s)
  *		NOTE: we must no use 'charout' because cid might be a non
  *		printable character...
  */
-char	   *
+char *
 cidout(int32 c)
 {
 	char	   *result;
 	CommandId	c2;
 
-	/*
-	 * cid is a number between 0 .. 2^16-1, therefore we need at most 6
-	 * chars for the string (5 digits + '\0') NOTE: print it as an
-	 * UNSIGNED int!
-	 */
-	result = palloc(6);
+	result = palloc(12);
 	c2 = (CommandId) c;
 	sprintf(result, "%u", (unsigned) (c2));
 	return (result);
@@ -98,7 +93,7 @@ cidout(int32 c)
  *		Note:
  *				Currently if strlen(s) < 14, the extra chars are nulls
  */
-char	   *
+char *
 char16in(char *s)
 {
 	char	   *result;
@@ -113,7 +108,7 @@ char16in(char *s)
 /*
  *		char16out		- converts internal reprsentation to "..."
  */
-char	   *
+char *
 char16out(char *s)
 {
 	char	   *result = (char *) palloc(17);
@@ -124,7 +119,7 @@ char16out(char *s)
 		result[1] = '\0';
 	}
 	else
-		strNcpy(result, s, 16);
+		StrNCpy(result, s, 17);
 	return (result);
 }
 
@@ -277,12 +272,12 @@ char2in(char *s)
 	return (res);
 }
 
-char	   *
+char *
 char2out(uint16 s)
 {
 	char	   *result = (char *) palloc(3);
 
-	strNcpy(result, (char *) &s, 2);
+	StrNCpy(result, (char *) &s, 3);
 
 	return (result);
 }
@@ -343,13 +338,13 @@ char4in(char *s)
 	return (res);
 }
 
-char	   *
+char *
 char4out(s)
 uint32		s;
 {
 	char	   *result = (char *) palloc(5);
 
-	strNcpy(result, (char *) &s, 4);
+	StrNCpy(result, (char *) &s, 5);
 
 	return (result);
 }
@@ -397,7 +392,7 @@ char4cmp(uint32 a, uint32 b)
 }
 
 /* ============================== char8 ============================== */
-char	   *
+char *
 char8in(char *s)
 {
 	char	   *result;
@@ -410,7 +405,7 @@ char8in(char *s)
 	return (result);
 }
 
-char	   *
+char *
 char8out(char *s)
 {
 	char	   *result = (char *) palloc(9);
@@ -421,7 +416,7 @@ char8out(char *s)
 		result[1] = '\0';
 	}
 	else
-		strNcpy(result, s, 8);
+		StrNCpy(result, s, 9);
 	return (result);
 }
 
