@@ -6,7 +6,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/chunk.c,v 1.10 1997/09/08 21:48:19 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/chunk.c,v 1.16 1998/02/26 04:36:56 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -15,26 +15,21 @@
 #include <fcntl.h>
 
 #include "postgres.h"
-
-#include <libpq/be-fsstubs.h>
-#include "utils/memutils.h"
-#include "libpq/libpq-fs.h"
-
-#include "storage/fd.h"			/* for SEEK_ */
+#include "fmgr.h"
 
 #include "catalog/pg_type.h"
-
-#include "fmgr.h"
-#include "utils/array.h"
-
+#include "libpq/be-fsstubs.h"
+#include "libpq/libpq-fs.h"
 #include "optimizer/internal.h"
+#include "utils/array.h"
+#include "utils/memutils.h"
+
 #ifndef HAVE_MEMMOVE
 #include <regex/utils.h>
 #else
 #include <string.h>
 #endif
 
-
 #define INFTY 500000000
 #define MANY  10000
 #define MAXPAT 20
@@ -75,7 +70,7 @@ GetChunkSize(FILE *fd, int ndim, int dim[MAXDIM], int baseSize,
  *	   information about the chunked file
  *-----------------------------------------------------------------------
  */
-char	   *
+char *
 _ChunkArray(int fd,
 			FILE *afd,
 			int ndim,
@@ -104,7 +99,7 @@ _ChunkArray(int fd,
 	else
 		cfd = LOopen(chunkfile, O_RDONLY);
 	if (cfd < 0)
-		elog(WARN, "Unable to open chunk file");
+		elog(ERROR, "Unable to open chunk file");
 #endif
 
 	strcpy(cInfo.lo_name, chunkfile);
@@ -148,11 +143,11 @@ GetChunkSize(FILE *fd,
 	 */
 	fscanf(fd, "%d", &N);
 	if (N > MAXPAT)
-		elog(WARN, "array_in: too many access pattern elements");
+		elog(ERROR, "array_in: too many access pattern elements");
 	for (i = 0; i < N; i++)
 		for (j = 0; j < ndim + 1; j++)
 			if (fscanf(fd, "%d ", &(A[i][j])) == EOF)
-				elog(WARN, "array_in: bad access pattern input");
+				elog(ERROR, "array_in: bad access pattern input");
 
 	/*
 	 * estimate chunk size
@@ -196,7 +191,7 @@ _FindBestChunk(int size,
 		 * compute the number of page fetches for a given chunk size (d[])
 		 * and access pattern (A[][])
 		 */
-		register int i,
+		int			i,
 					j,
 					nc;
 
@@ -229,7 +224,7 @@ _FindBestChunk(int size,
 static int
 get_next(int d[], int k, int C, int dmax[])
 {
-	register int i,
+	int			i,
 				j,
 				temp;
 
@@ -271,7 +266,8 @@ get_next(int d[], int k, int C, int dmax[])
 }
 
 #ifdef LOARRAY
-static char a_chunk[BLCKSZ + 4];/* 4 since a_chunk is in varlena format */
+static char a_chunk[BLCKSZ + VARHDRSZ]; /* VARHDRSZ since a_chunk is in
+										 * varlena format */
 
 #endif
 
@@ -318,8 +314,8 @@ _ConvertToChunkFile(int n,
 		csize *= C[i];
 	}
 	csize *= baseSize;
-	temp = csize + 4;
-	memmove(a_chunk, &temp, 4);
+	temp = csize + VARHDRSZ;
+	memmove(a_chunk, &temp, VARHDRSZ);
 
 	mda_get_prod(n, dim, PX);
 	mda_get_offset_values(n, dist, PX, C);
@@ -327,7 +323,7 @@ _ConvertToChunkFile(int n,
 		;
 	do
 	{
-		read_chunk(chunk_no, C, &(a_chunk[4]), srcfd, n, baseSize, PX, dist);
+		read_chunk(chunk_no, C, &(a_chunk[VARHDRSZ]), srcfd, n, baseSize, PX, dist);
 		write_chunk((struct varlena *) a_chunk, destfd);
 	} while (next_tuple(n, chunk_no, max_chunks) != -1);
 }
@@ -413,12 +409,12 @@ seek_and_read(int pos, int size, char buff[], int fp, int from)
 
 	/* Assuming only one file */
 	if (lo_lseek(fp, pos, from) < 0)
-		elog(WARN, "File seek error");
+		elog(ERROR, "File seek error");
 #ifdef LOARRAY
 	v = (struct varlena *) LOread(fp, size);
 #endif
-	if (VARSIZE(v) - 4 < size)
-		elog(WARN, "File read error");
+	if (VARSIZE(v) - VARHDRSZ < size)
+		elog(ERROR, "File read error");
 	memmove(buff, VARDATA(v), size);
 	pfree(v);
 	return (1);
