@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/not_in.c,v 1.6 1997/09/12 04:08:21 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/not_in.c,v 1.8 1998/01/31 04:38:45 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -72,7 +72,7 @@ int4notin(int16 not_in_arg, char *relation_and_attr)
 	/* the last argument should be a ScanKey, not an integer! - jolly */
 	/* it looks like the arguments are out of order, too */
 	/* but skeyData is never initialized! does this work?? - ay 2/95 */
-	scan_descriptor = heap_beginscan(relation_to_scan, false, NULL, 0,
+	scan_descriptor = heap_beginscan(relation_to_scan, false, false, 0,
 									 &skeyData);
 
 	retval = true;
@@ -83,7 +83,6 @@ int4notin(int16 not_in_arg, char *relation_and_attr)
 		 current_tuple = heap_getnext(scan_descriptor, 0, NULL))
 	{
 		value = heap_getattr(current_tuple,
-							 InvalidBuffer,
 							 (AttrNumber) attrid,
 							 RelationGetTupleDescriptor(relation_to_scan),
 							 &dummy);
