@@ -10,13 +10,16 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/numutils.c,v 1.16 1997/09/18 20:22:15 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/numutils.c,v 1.23 1998/02/26 04:37:14 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include <stdio.h>				/* for sprintf() */
 #include <errno.h>
 #include <math.h>
+#ifdef HAVE_LIMITS
+#include <limits.h>
+#endif
 #include "postgres.h"
 #include "utils/builtins.h"		/* where the declarations go */
 #ifndef HAVE_MEMMOVE
@@ -24,7 +27,25 @@
 #else
 #include <string.h>
 #endif
-#include <port-protos.h>		/* ecvt(), fcvt() */
+
+#ifndef INT_MAX
+#define INT_MAX (0x7FFFFFFFL)
+#endif
+#ifndef INT_MIN
+#define INT_MIN (-0x80000000L)
+#endif
+#ifndef SHRT_MAX
+#define SHRT_MAX (0x7FFF)
+#endif
+#ifndef SHRT_MIN
+#define SHRT_MIN (-0x8000)
+#endif
+#ifndef SCHAR_MAX
+#define SCHAR_MAX (0x7F)
+#endif
+#ifndef SCHAR_MIN
+#define SCHAR_MIN (-0x80)
+#endif
 
 int32
 pg_atoi(char *s, int size, int c)
@@ -37,53 +58,53 @@ pg_atoi(char *s, int size, int c)
 	errno = 0;
 	l = strtol(s, &badp, 10);
 	if (errno)					/* strtol must set ERANGE */
-		elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+		elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 	if (badp && *badp && (*badp != c))
-		elog(WARN, "pg_atoi: error in \"%s\": can\'t parse \"%s\"", s, badp);
+		elog(ERROR, "pg_atoi: error in \"%s\": can\'t parse \"%s\"", s, badp);
 
 	switch (size)
 	{
 		case sizeof(int32):
 #ifdef HAS_LONG_LONG
 			/* won't get ERANGE on these with 64-bit longs... */
-			if (l < -0x80000000L)
+			if (l < INT_MIN)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
-			if (l > 0x7fffffffL)
+			if (l > INT_MAX)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
 #endif							/* HAS_LONG_LONG */
 			break;
 		case sizeof(int16):
-			if (l < -0x8000)
+			if (l < SHRT_MIN)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
-			if (l > 0x7fff)
+			if (l > SHRT_MAX)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
 			break;
 		case sizeof(int8):
-			if (l < -0x80)
+			if (l < SCHAR_MIN)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
-			if (l > 0x7f)
+			if (l > SCHAR_MAX)
 			{
 				errno = ERANGE;
-				elog(WARN, "pg_atoi: error reading \"%s\": %m", s);
+				elog(ERROR, "pg_atoi: error reading \"%s\": %m", s);
 			}
 			break;
 		default:
-			elog(WARN, "pg_atoi: invalid result size: %d", size);
+			elog(ERROR, "pg_atoi: invalid result size: %d", size);
 	}
 	return ((int32) l);
 }
@@ -163,9 +184,9 @@ ftoa(double value, char *ascii, int width, int prec1, char format)
 #else
 	auto int	expon;
 	auto int	sign;
-	register int avail = 0;
-	register char *a = NULL;
-	register char *p = NULL;
+	int			avail = 0;
+	char	   *a = NULL;
+	char	   *p = NULL;
 	char		mode;
 	int			lowercase;
 	int			prec;
@@ -312,7 +333,7 @@ frac_out:
 	*a = 0;
 	avail = a - ascii;
 	return (avail);
-#endif							/* !BSD44_derived */
+#endif
 }
 
 #endif
@@ -349,13 +370,13 @@ frac_out:
 int
 atof1(char *str, double *val)
 {
-	register char *p;
+	char	   *p;
 	double		v;
 	double		fact;
 	int			minus;
-	register char c;
+	char		c;
 	int			expon;
-	register int gotmant;
+	int			gotmant;
 
 	v = 0.0;
 	p = str;
