@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/regproc.c,v 1.7 1997/09/08 02:30:58 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/regproc.c,v 1.17 1998/04/07 18:11:34 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,7 +45,7 @@ regprocin(char *proname)
 	proc = heap_openr(ProcedureRelationName);
 	if (!RelationIsValid(proc))
 	{
-		elog(WARN, "regprocin: could not open %s",
+		elog(ERROR, "regprocin: could not open %s",
 			 ProcedureRelationName);
 		return (0);
 	}
@@ -55,11 +55,11 @@ regprocin(char *proname)
 						   (RegProcedure) F_CHAR16EQ,
 						   (Datum) proname);
 
-	procscan = heap_beginscan(proc, 0, NowTimeQual, 1, &key);
+	procscan = heap_beginscan(proc, 0, false, 1, &key);
 	if (!HeapScanIsValid(procscan))
 	{
 		heap_close(proc);
-		elog(WARN, "regprocin: could not being scan of %s",
+		elog(ERROR, "regprocin: could not being scan of %s",
 			 ProcedureRelationName);
 		return (0);
 	}
@@ -68,7 +68,6 @@ regprocin(char *proname)
 	{
 		case 1:
 			result = (RegProcedure) heap_getattr(proctup,
-												 InvalidBuffer,
 												 ObjectIdAttributeNumber,
 										RelationGetTupleDescriptor(proc),
 												 &isnull);
@@ -91,7 +90,7 @@ regprocin(char *proname)
 /*
  *		regprocout		- converts proid to "proname"
  */
-char	   *
+char *
 regprocout(RegProcedure proid)
 {
 	Relation	proc;
@@ -104,7 +103,7 @@ regprocout(RegProcedure proid)
 	proc = heap_openr(ProcedureRelationName);
 	if (!RelationIsValid(proc))
 	{
-		elog(WARN, "regprocout: could not open %s",
+		elog(ERROR, "regprocout: could not open %s",
 			 ProcedureRelationName);
 		return (0);
 	}
@@ -114,11 +113,11 @@ regprocout(RegProcedure proid)
 						   (RegProcedure) F_INT4EQ,
 						   (Datum) proid);
 
-	procscan = heap_beginscan(proc, 0, NowTimeQual, 1, &key);
+	procscan = heap_beginscan(proc, 0, false, 1, &key);
 	if (!HeapScanIsValid(procscan))
 	{
 		heap_close(proc);
-		elog(WARN, "regprocin: could not being scan of %s",
+		elog(ERROR, "regprocout: could not being scan of %s",
 			 ProcedureRelationName);
 		return (0);
 	}
@@ -129,11 +128,11 @@ regprocout(RegProcedure proid)
 			bool		isnull;
 
 		case 1:
-			s = (char *) heap_getattr(proctup, InvalidBuffer, 1,
+			s = (char *) heap_getattr(proctup, 1,
 							  RelationGetTupleDescriptor(proc), &isnull);
 			if (!isnull)
 			{
-				strNcpy(result, s, 16);
+				StrNCpy(result, s, 16);
 				break;
 			}
 			elog(FATAL, "regprocout: null procedure %d", proid);
@@ -150,6 +149,81 @@ regprocout(RegProcedure proid)
 	return (result);
 }
 
+/*
+ *		int8typeout			- converts int8 type oids to "typname" list
+ */
+text *
+oid8types(Oid (*oidArray)[])
+{
+	Relation	type;
+	HeapScanDesc typescan;
+	HeapTuple	typetup;
+	text	   *result;
+	ScanKeyData key;
+	int			num;
+	Oid		   *sp;
+
+	if (oidArray == NULL)
+	{
+		result = (text *) palloc(VARHDRSZ);
+		VARSIZE(result) = 0;
+		return (result);
+	}
+
+	result = (text *) palloc(NAMEDATALEN * 8 + 8 + VARHDRSZ);
+	*VARDATA(result) = '\0';
+	type = heap_openr(TypeRelationName);
+	if (!RelationIsValid(type))
+	{
+		elog(ERROR, "int8typeout: could not open %s",
+			 TypeRelationName);
+		return (0);
+	}
+
+	sp = *oidArray;
+	for (num = 8; num != 0; num--, sp++)
+	{
+		if (*sp != InvalidOid)
+		{
+			ScanKeyEntryInitialize(&key,
+								   (bits16) 0,
+								   (AttrNumber) ObjectIdAttributeNumber,
+								   (RegProcedure) F_INT4EQ,
+								   (Datum) *sp);
+
+			typescan = heap_beginscan(type, 0, false, 1, &key);
+			if (!HeapScanIsValid(typescan))
+			{
+				heap_close(type);
+				elog(ERROR, "int8typeout: could not being scan of %s",
+					 TypeRelationName);
+				return (0);
+			}
+			typetup = heap_getnext(typescan, 0, (Buffer *) NULL);
+			if (HeapTupleIsValid(typetup))
+			{
+				char	   *s;
+				bool		isnull;
+
+				s = (char *) heap_getattr(typetup, 1,
+							  RelationGetTupleDescriptor(type), &isnull);
+				if (!isnull)
+				{
+					StrNCpy(VARDATA(result) + strlen(VARDATA(result)), s, 16);
+					strcat(VARDATA(result), " ");
+				}
+				else
+					elog(FATAL, "int8typeout: null procedure %d", *sp);
+				/* FALLTHROUGH */
+			}
+			heap_endscan(typescan);
+		}
+	}
+	heap_close(type);
+	VARSIZE(result) = strlen(VARDATA(result)) + VARHDRSZ;
+	return (result);
+}
+
 
 /*****************************************************************************
  *	 PUBLIC ROUTINES														 *
