@@ -12,7 +12,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/selfuncs.c,v 1.10 1997/09/08 02:31:00 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/selfuncs.c,v 1.18 1998/02/26 04:37:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,7 +22,6 @@
 #include "postgres.h"
 
 #include "access/heapam.h"
-#include "utils/tqual.h"		/* for NowTimeQual */
 #include "fmgr.h"
 #include "utils/builtins.h"		/* for textout() prototype and where the
 								 * declarations go */
@@ -288,7 +287,7 @@ getattdisbursion(Oid relid, AttrNumber attnum)
 							  0, 0);
 	if (!HeapTupleIsValid(atp))
 	{
-		elog(WARN, "getattdisbursion: no attribute tuple %d %d",
+		elog(ERROR, "getattdisbursion: no attribute tuple %d %d",
 			 relid, attnum);
 		return (0);
 	}
@@ -305,7 +304,7 @@ getattdisbursion(Oid relid, AttrNumber attnum)
 	 */
 	if (!HeapTupleIsValid(atp))
 	{
-		elog(WARN, "getattdisbursion: no relation tuple %d", relid);
+		elog(ERROR, "getattdisbursion: no relation tuple %d", relid);
 		return (0);
 	}
 	ntuples = ((Form_pg_class) GETSTRUCT(atp))->reltuples;
@@ -335,12 +334,12 @@ gethilokey(Oid relid,
 		   char **high,
 		   char **low)
 {
-	register Relation rdesc;
-	register HeapScanDesc sdesc;
+	Relation	rdesc;
+	HeapScanDesc sdesc;
 	static ScanKeyData key[3] = {
-		{0, Anum_pg_statistic_starelid, F_OIDEQ},
-		{0, Anum_pg_statistic_staattnum, F_INT2EQ},
-		{0, Anum_pg_statistic_staop, F_OIDEQ}
+		{0, Anum_pg_statistic_starelid, F_OIDEQ, {0, 0, F_OIDEQ}},
+		{0, Anum_pg_statistic_staattnum, F_INT2EQ, {0, 0, F_INT2EQ}},
+		{0, Anum_pg_statistic_staop, F_OIDEQ, {0, 0, F_OIDEQ}}
 	};
 	bool		isnull;
 	HeapTuple	tuple;
@@ -350,7 +349,7 @@ gethilokey(Oid relid,
 	key[0].sk_argument = ObjectIdGetDatum(relid);
 	key[1].sk_argument = Int16GetDatum((int16) attnum);
 	key[2].sk_argument = ObjectIdGetDatum(opid);
-	sdesc = heap_beginscan(rdesc, 0, NowTimeQual, 3, key);
+	sdesc = heap_beginscan(rdesc, 0, false, 3, key);
 	tuple = heap_getnext(sdesc, 0, (Buffer *) NULL);
 	if (!HeapTupleIsValid(tuple))
 	{
@@ -358,14 +357,13 @@ gethilokey(Oid relid,
 		*low = "n";
 
 		/*
-		 * XXX			elog(WARN, "gethilokey: statistic tuple not
+		 * XXX			elog(ERROR, "gethilokey: statistic tuple not
 		 * found");
 		 */
 		return;
 	}
 	*high = textout((struct varlena *)
 					heap_getattr(tuple,
-								 InvalidBuffer,
 								 Anum_pg_statistic_stahikey,
 								 RelationGetTupleDescriptor(rdesc),
 								 &isnull));
@@ -373,7 +371,6 @@ gethilokey(Oid relid,
 		elog(DEBUG, "gethilokey: high key is null");
 	*low = textout((struct varlena *)
 				   heap_getattr(tuple,
-								InvalidBuffer,
 								Anum_pg_statistic_stalokey,
 								RelationGetTupleDescriptor(rdesc),
 								&isnull));
@@ -418,9 +415,9 @@ btreesel(Oid operatorObjectId,
 	}
 
 	if (!PointerIsValid(result))
-		elog(WARN, "Btree Selectivity: bad pointer");
+		elog(ERROR, "Btree Selectivity: bad pointer");
 	if (*result < 0.0 || *result > 1.0)
-		elog(WARN, "Btree Selectivity: bad value %lf", *result);
+		elog(ERROR, "Btree Selectivity: bad value %lf", *result);
 
 	return (result);
 }
@@ -466,7 +463,7 @@ btreenpage(Oid operatorObjectId,
 							  0, 0, 0);
 	if (!HeapTupleIsValid(atp))
 	{
-		elog(WARN, "btreenpage: no index tuple %d", indexrelid);
+		elog(ERROR, "btreenpage: no index tuple %d", indexrelid);
 		return (0);
 	}
 
@@ -504,7 +501,7 @@ hashsel(Oid operatorObjectId,
 								  0, 0, 0);
 		if (!HeapTupleIsValid(atp))
 		{
-			elog(WARN, "hashsel: no index tuple %d", indexrelid);
+			elog(ERROR, "hashsel: no index tuple %d", indexrelid);
 			return (0);
 		}
 		ntuples = ((Form_pg_class) GETSTRUCT(atp))->reltuples;
@@ -531,9 +528,9 @@ hashsel(Oid operatorObjectId,
 	}
 
 	if (!PointerIsValid(result))
-		elog(WARN, "Hash Table Selectivity: bad pointer");
+		elog(ERROR, "Hash Table Selectivity: bad pointer");
 	if (*result < 0.0 || *result > 1.0)
-		elog(WARN, "Hash Table Selectivity: bad value %lf", *result);
+		elog(ERROR, "Hash Table Selectivity: bad value %lf", *result);
 
 	return (result);
 
@@ -560,7 +557,7 @@ hashnpage(Oid operatorObjectId,
 							  0, 0, 0);
 	if (!HeapTupleIsValid(atp))
 	{
-		elog(WARN, "hashsel: no index tuple %d", indexrelid);
+		elog(ERROR, "hashsel: no index tuple %d", indexrelid);
 		return (0);
 	}
 
