@@ -51,11 +51,11 @@ timestamp_out(time_t timestamp)
 			EncodeDateTime(tm, fsec, &tz, &tzn, USE_ISO_DATES, buf);
 			break;
 	}
- 
+
 	result = palloc(strlen(buf) + 1);
 	strcpy(result, buf);
 	return result;
-} /* timestamp_out() */
+}	/* timestamp_out() */
 
 time_t
 now(void)
@@ -69,56 +69,47 @@ now(void)
 bool
 timestampeq(time_t t1, time_t t2)
 {
-	return(abstimeeq(t1,t2));
+	return (abstimeeq(t1, t2));
 }
 
 bool
 timestampne(time_t t1, time_t t2)
 {
-	return(abstimene(t1,t2));
+	return (abstimene(t1, t2));
 }
 
 bool
 timestamplt(time_t t1, time_t t2)
 {
-	return(abstimelt(t1,t2));
+	return (abstimelt(t1, t2));
 }
 
 bool
 timestampgt(time_t t1, time_t t2)
 {
-	return(abstimegt(t1,t2));
+	return (abstimegt(t1, t2));
 }
 
 bool
 timestample(time_t t1, time_t t2)
 {
-	return(abstimele(t1,t2));
+	return (abstimele(t1, t2));
 }
 
 bool
 timestampge(time_t t1, time_t t2)
 {
-	return(abstimege(t1,t2));
+	return (abstimege(t1, t2));
 }
 
-DateTime *
+DateTime   *
 timestamp_datetime(time_t timestamp)
 {
-	DateTime   *result;
+	return (abstime_datetime((AbsoluteTime) timestamp));
+}	/* timestamp_datetime() */
 
-	double		fsec = 0;
-	struct tm  *tm;
-
-	if (!PointerIsValid(result = PALLOCTYPE(DateTime)))
-		elog(WARN, "Memory allocation failed, can't convert timestamp to datetime", NULL);
-
-	tm = localtime((time_t *) &timestamp);
-	tm->tm_year += 1900;
-	tm->tm_mon += 1;
-
-	if (tm2datetime(tm, fsec, NULL, result) != 0)
-		elog(WARN, "Unable to convert timestamp to datetime", timestamp_out(timestamp));
-
-	return (result);
-} /* timestamp_datetime() */
+time_t
+datetime_timestamp(DateTime *datetime)
+{
+	return ((AbsoluteTime) datetime_abstime(datetime));
+}	/* datetime_timestamp() */
