@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/varlena.c,v 1.21 1997/09/08 21:48:42 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/varlena.c,v 1.32 1998/03/15 08:07:01 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -31,22 +31,22 @@
  *
  *		Non-printable characters must be passed as '\nnn' (octal) and are
  *		converted to internal form.  '\' must be passed as '\\'.
- *		elog(WARN, ...) if bad form.
+ *		elog(ERROR, ...) if bad form.
  *
  *		BUGS:
  *				The input is scaned twice.
  *				The error checking of input is minimal.
  */
-struct varlena *
+text *
 byteain(char *inputText)
 {
-	register char *tp;
-	register char *rp;
-	register int byte;
-	struct varlena *result;
+	char	   *tp;
+	char	   *rp;
+	int			byte;
+	text	   *result;
 
 	if (inputText == NULL)
-		elog(WARN, "Bad input string for type bytea");
+		elog(ERROR, "Bad input string for type bytea");
 
 	for (byte = 0, tp = inputText; *tp != '\0'; byte++)
 		if (*tp++ == '\\')
@@ -56,11 +56,11 @@ byteain(char *inputText)
 			else if (!isdigit(*tp++) ||
 					 !isdigit(*tp++) ||
 					 !isdigit(*tp++))
-				elog(WARN, "Bad input string for type bytea");
+				elog(ERROR, "Bad input string for type bytea");
 		}
 	tp = inputText;
-	byte += sizeof(int32);		/* varlena? */
-	result = (struct varlena *) palloc(byte);
+	byte += VARHDRSZ;
+	result = (text *) palloc(byte);
 	result->vl_len = byte;		/* varlena? */
 	rp = result->vl_dat;
 	while (*tp != '\0')
@@ -77,28 +77,6 @@ byteain(char *inputText)
 	return (result);
 }
 
-/*
- * Shoves a bunch of memory pointed at by bytes into varlena.
- * BUGS:  Extremely unportable as things shoved can be string
- * representations of structs, etc.
- */
-#ifdef NOT_USED
-struct varlena *
-shove_bytes(unsigned char *stuff, int len)
-{
-	struct varlena *result;
-
-	result = (struct varlena *) palloc(len + sizeof(int32));
-	result->vl_len = len;
-	memmove(result->vl_dat,
-			stuff + sizeof(int32),
-			len - sizeof(int32));
-	return (result);
-}
-
-#endif
-
-
 /*
  *		byteaout		- converts to printable representation of byte array
  *
@@ -107,15 +85,16 @@ shove_bytes(unsigned char *stuff, int len)
  *
  *		NULL vlena should be an error--returning string with NULL for now.
  */
-char	   *
-byteaout(struct varlena * vlena)
+char *
+byteaout(text *vlena)
 {
-	register char *vp;
-	register char *rp;
-	register int val;			/* holds unprintable chars */
+	char	   *result;
+
+	char	   *vp;
+	char	   *rp;
+	int			val;			/* holds unprintable chars */
 	int			i;
 	int			len;
-	static char *result;
 
 	if (vlena == NULL)
 	{
@@ -126,16 +105,16 @@ byteaout(struct varlena * vlena)
 	}
 	vp = vlena->vl_dat;
 	len = 1;					/* empty string has 1 char */
-	for (i = vlena->vl_len - sizeof(int32); i != 0; i--, vp++)	/* varlena? */
+	for (i = vlena->vl_len - VARHDRSZ; i != 0; i--, vp++)
 		if (*vp == '\\')
 			len += 2;
 		else if (isascii(*vp) && isprint(*vp))
 			len++;
 		else
-			len += 4;
+			len += VARHDRSZ;
 	rp = result = (char *) palloc(len);
 	vp = vlena->vl_dat;
-	for (i = vlena->vl_len - sizeof(int32); i != 0; i--)		/* varlena? */
+	for (i = vlena->vl_len - VARHDRSZ; i != 0; i--)
 		if (*vp == '\\')
 		{
 			vp++;
@@ -164,26 +143,33 @@ byteaout(struct varlena * vlena)
 /*
  *		textin			- converts "..." to internal representation
  */
-struct varlena *
+text *
 textin(char *inputText)
 {
-	struct varlena *result;
+	text	   *result;
 	int			len;
 
 	if (inputText == NULL)
 		return (NULL);
+
 	len = strlen(inputText) + VARHDRSZ;
-	result = (struct varlena *) palloc(len);
+	result = (text *) palloc(len);
 	VARSIZE(result) = len;
+
 	memmove(VARDATA(result), inputText, len - VARHDRSZ);
+
+#ifdef CYR_RECODE
+	convertstr(VARDATA(result), len - VARHDRSZ, 0);
+#endif
+
 	return (result);
 }
 
 /*
  *		textout			- converts internal representation to "..."
  */
-char	   *
-textout(struct varlena * vlena)
+char *
+textout(text *vlena)
 {
 	int			len;
 	char	   *result;
@@ -199,6 +185,11 @@ textout(struct varlena * vlena)
 	result = (char *) palloc(len + 1);
 	memmove(result, VARDATA(vlena), len);
 	result[len] = '\0';
+
+#ifdef CYR_RECODE
+	convertstr(result, len, 1);
+#endif
+
 	return (result);
 }
 
@@ -207,23 +198,17 @@ textout(struct varlena * vlena)
 
 /*
  * textlen -
- *	  returns the actual length of a text* (which may be less than
- *	  the VARSIZE of the text*)
+ *	  returns the actual length of a text*
+ *	   (which is less than the VARSIZE of the text*)
  */
-#ifdef NOT_USED
-int
+int32
 textlen(text *t)
 {
-	int			i = 0;
-	int			max = VARSIZE(t) - VARHDRSZ;
-	char	   *ptr = VARDATA(t);
+	if (!PointerIsValid(t))
+		elog(ERROR, "Null input to textlen");
 
-	while (i < max && *ptr++)
-		i++;
-	return i;
-}
-
-#endif
+	return (VARSIZE(t) - VARHDRSZ);
+}	/* textlen() */
 
 /*
  * textcat -
@@ -237,8 +222,7 @@ textlen(text *t)
  * As in previous code, allow concatenation when one string is NULL.
  * Is this OK?
  */
-
-text	   *
+text *
 textcat(text *t1, text *t2)
 {
 	int			len1,
@@ -253,11 +237,16 @@ textcat(text *t1, text *t2)
 	len1 = (PointerIsValid(t1) ? (VARSIZE(t1) - VARHDRSZ) : 0);
 	if (len1 < 0)
 		len1 = 0;
+	while (len1 > 0 && VARDATA(t1)[len1 - 1] == '\0')
+		len1--;
+
 	len2 = (PointerIsValid(t2) ? (VARSIZE(t2) - VARHDRSZ) : 0);
 	if (len2 < 0)
 		len2 = 0;
+	while (len2 > 0 && VARDATA(t2)[len2 - 1] == '\0')
+		len2--;
 
-	result = PALLOC(len = len1 + len2 + VARHDRSZ);
+	result = palloc(len = len1 + len2 + VARHDRSZ);
 
 	/* Fill data field of result string... */
 	ptr = VARDATA(result);
@@ -270,7 +259,57 @@ textcat(text *t1, text *t2)
 	VARSIZE(result) = len;
 
 	return (result);
-}								/* textcat() */
+}	/* textcat() */
+
+/*
+ * text_substr()
+ * Return a substring starting at the specified position.
+ * - thomas 1997-12-31
+ *
+ * Input:
+ *	- string
+ *	- starting position (is one-based)
+ *	- string length
+ *
+ * If the starting position is zero or less, then return the entire string.
+ * XXX Note that this may not be the right behavior:
+ *	if we are calculating the starting position we might want it to start at one.
+ * If the length is less than zero, return the remaining string.
+ *
+ * Note that the arguments operate on octet length,
+ *	so not aware of multi-byte character sets.
+ */
+text *
+text_substr(text *string, int32 m, int32 n)
+{
+	text	   *ret;
+	int			len;
+
+	if ((string == (text *) NULL) || (m <= 0))
+		return string;
+
+	len = VARSIZE(string) - VARHDRSZ;
+
+	/* m will now become a zero-based starting position */
+	if (m > len)
+	{
+		m = 0;
+		n = 0;
+	}
+	else
+	{
+		m--;
+		if (((m + n) > len) || (n < 0))
+			n = (len - m);
+	}
+
+	ret = (text *) palloc(VARHDRSZ + n);
+	VARSIZE(ret) = VARHDRSZ + n;
+
+	memcpy(VARDATA(ret), VARDATA(string) + m, n);
+
+	return ret;
+}	/* text_substr() */
 
 /*
  * textpos -
@@ -279,7 +318,6 @@ textcat(text *t1, text *t2)
  *	  Ref: A Guide To The SQL Standard, Date & Darwen, 1997
  * - thomas 1997-07-27
  */
-
 int32
 textpos(text *t1, text *t2)
 {
@@ -313,17 +351,17 @@ textpos(text *t1, text *t2)
 		p1++;
 	};
 	return (pos);
-}								/* textpos() */
+}	/* textpos() */
 
 /*
  *		texteq			- returns 1 iff arguments are equal
  *		textne			- returns 1 iff arguments are not equal
  */
 bool
-texteq(struct varlena * arg1, struct varlena * arg2)
+texteq(text *arg1, text *arg2)
 {
-	register int len;
-	register char *a1p,
+	int			len;
+	char	   *a1p,
 			   *a2p;
 
 	if (arg1 == NULL || arg2 == NULL)
@@ -343,10 +381,10 @@ texteq(struct varlena * arg1, struct varlena * arg2)
 		if (*a1p++ != *a2p++)
 			return ((bool) 0);
 	return ((bool) 1);
-}								/* texteq() */
+}	/* texteq() */
 
 bool
-textne(struct varlena * arg1, struct varlena * arg2)
+textne(text *arg1, text *arg2)
 {
 	return ((bool) !texteq(arg1, arg2));
 }
@@ -359,7 +397,7 @@ textne(struct varlena * arg1, struct varlena * arg2)
  *	but it appears that most routines (incl. this one) assume not! - tgl 97/04/07
  */
 bool
-text_lt(struct varlena * arg1, struct varlena * arg2)
+text_lt(text *arg1, text *arg2)
 {
 	bool		result;
 
@@ -405,7 +443,7 @@ text_lt(struct varlena * arg1, struct varlena * arg2)
 #endif
 
 	return (result);
-}								/* text_lt() */
+}	/* text_lt() */
 
 /* text_le()
  * Comparison function for text strings.
@@ -415,7 +453,7 @@ text_lt(struct varlena * arg1, struct varlena * arg2)
  *	but it appears that most routines (incl. this one) assume not! - tgl 97/04/07
  */
 bool
-text_le(struct varlena * arg1, struct varlena * arg2)
+text_le(text *arg1, text *arg2)
 {
 	bool		result;
 
@@ -461,16 +499,16 @@ text_le(struct varlena * arg1, struct varlena * arg2)
 #endif
 
 	return (result);
-}								/* text_le() */
+}	/* text_le() */
 
 bool
-text_gt(struct varlena * arg1, struct varlena * arg2)
+text_gt(text *arg1, text *arg2)
 {
 	return ((bool) !text_le(arg1, arg2));
 }
 
 bool
-text_ge(struct varlena * arg1, struct varlena * arg2)
+text_ge(text *arg1, text *arg2)
 {
 	return ((bool) !text_lt(arg1, arg2));
 }
@@ -482,9 +520,9 @@ text_ge(struct varlena * arg1, struct varlena * arg2)
  *-------------------------------------------------------------
  */
 int32
-byteaGetSize(struct varlena * v)
+byteaGetSize(text *v)
 {
-	register int len;
+	int			len;
 
 	len = v->vl_len - sizeof(v->vl_len);
 
@@ -500,7 +538,7 @@ byteaGetSize(struct varlena * v)
  *-------------------------------------------------------------
  */
 int32
-byteaGetByte(struct varlena * v, int32 n)
+byteaGetByte(text *v, int32 n)
 {
 	int			len;
 	int			byte;
@@ -509,7 +547,7 @@ byteaGetByte(struct varlena * v, int32 n)
 
 	if (n >= len)
 	{
-		elog(WARN, "byteaGetByte: index (=%d) out of range [0..%d]",
+		elog(ERROR, "byteaGetByte: index (=%d) out of range [0..%d]",
 			 n, len - 1);
 	}
 
@@ -528,7 +566,7 @@ byteaGetByte(struct varlena * v, int32 n)
  *-------------------------------------------------------------
  */
 int32
-byteaGetBit(struct varlena * v, int32 n)
+byteaGetBit(text *v, int32 n)
 {
 	int			byteNo,
 				bitNo;
@@ -557,17 +595,17 @@ byteaGetBit(struct varlena * v, int32 n)
  *
  *-------------------------------------------------------------
  */
-struct varlena *
-byteaSetByte(struct varlena * v, int32 n, int32 newByte)
+text *
+byteaSetByte(text *v, int32 n, int32 newByte)
 {
 	int			len;
-	struct varlena *res;
+	text	   *res;
 
 	len = byteaGetSize(v);
 
 	if (n >= len)
 	{
-		elog(WARN,
+		elog(ERROR,
 			 "byteaSetByte: index (=%d) out of range [0..%d]",
 			 n, len - 1);
 	}
@@ -575,10 +613,10 @@ byteaSetByte(struct varlena * v, int32 n, int32 newByte)
 	/*
 	 * Make a copy of the original varlena.
 	 */
-	res = (struct varlena *) palloc(VARSIZE(v));
+	res = (text *) palloc(VARSIZE(v));
 	if (res == NULL)
 	{
-		elog(WARN, "byteaSetByte: Out of memory (%d bytes requested)",
+		elog(ERROR, "byteaSetByte: Out of memory (%d bytes requested)",
 			 VARSIZE(v));
 	}
 	memmove((char *) res, (char *) v, VARSIZE(v));
@@ -599,10 +637,10 @@ byteaSetByte(struct varlena * v, int32 n, int32 newByte)
  *
  *-------------------------------------------------------------
  */
-struct varlena *
-byteaSetBit(struct varlena * v, int32 n, int32 newBit)
+text *
+byteaSetBit(text *v, int32 n, int32 newBit)
 {
-	struct varlena *res;
+	text	   *res;
 	int			oldByte,
 				newByte;
 	int			byteNo,
@@ -613,7 +651,7 @@ byteaSetBit(struct varlena * v, int32 n, int32 newBit)
 	 */
 	if (newBit != 0 && newBit != 1)
 	{
-		elog(WARN, "byteaSetByte: new bit must be 0 or 1");
+		elog(ERROR, "byteaSetByte: new bit must be 0 or 1");
 	}
 
 	/*
