@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/catcache.c,v 1.16 1997/09/18 20:22:18 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/catcache.c,v 1.26 1998/04/07 18:11:38 momjian Exp $
  *
  * Notes:
  *		XXX This needs to use exception.h to handle recovery when
@@ -19,18 +19,18 @@
 #include "postgres.h"
 #include "access/heapam.h"
 #include "access/genam.h"
-#include "utils/builtins.h"
 #include "utils/tqual.h"
-#include "storage/bufpage.h"
-#include "access/valid.h"
-#include "miscadmin.h"
+#include "utils/builtins.h"
 #include "utils/portal.h"
 #include "utils/catcache.h"
-#include "fmgr.h"				/* for F_BOOLEQ, etc.  DANGER */
 #include "utils/elog.h"
 #include "utils/palloc.h"
 #include "utils/mcxt.h"
 #include "utils/rel.h"
+#include "storage/bufpage.h"
+#include "access/valid.h"
+#include "miscadmin.h"
+#include "fmgr.h"				/* for F_BOOLEQ, etc.  DANGER */
 #include "catalog/pg_type.h"	/* for OID of int28 type */
 #include "lib/dllist.h"
 
@@ -213,8 +213,8 @@ CatalogCacheInitializeCache(struct catcache * cache,
 				EQPROC(tupdesc->attrs[cache->cc_key[i] - 1]->atttypid);
 
 			fmgr_info(cache->cc_skey[i].sk_procedure,
-					  (func_ptr *) &cache->cc_skey[i].sk_func,
-					  (int *) &cache->cc_skey[i].sk_nargs);
+					  &cache->cc_skey[i].sk_func);
+			cache->cc_skey[i].sk_nargs = cache->cc_skey[i].sk_func.fn_nargs;
 
 			CACHE5_elog(DEBUG, "CatalogCacheInit %16s %d %d %x",
 						&relation->rd_rel->relname,
@@ -287,7 +287,7 @@ CatalogCacheSetId(CatCache *cacheInOutP, int id)
  * ----------------
  */
 static long
-comphash(long l, register char *v)
+comphash(long l, char *v)
 {
 	long		i;
 	NameData	n;
@@ -568,7 +568,7 @@ ResetSystemCache()
 	CACHE1_elog(DEBUG, "ResetSystemCache called");
 	if (DisableCache)
 	{
-		elog(WARN, "ResetSystemCache: Called while cache disabled");
+		elog(ERROR, "ResetSystemCache: Called while cache disabled");
 		return;
 	}
 
@@ -605,7 +605,7 @@ ResetSystemCache()
 				nextelt = DLGetSucc(elt);
 				CatCacheRemoveCTup(cache, elt);
 				if (cache->cc_ntup == -1)
-					elog(WARN, "ResetSystemCache: cc_ntup<0 (software error)");
+					elog(ERROR, "ResetSystemCache: cc_ntup<0 (software error)");
 			}
 		}
 		cache->cc_ntup = 0;		/* in case of WARN error above */
@@ -620,6 +620,29 @@ ResetSystemCache()
 	MemoryContextSwitchTo(oldcxt);
 }
 
+/* --------------------------------
+ *		SystemCacheRelationFlushed
+ *
+ *	RelationFlushRelation() frees some information referenced in the
+ *	cache structures. So we get informed when this is done and arrange
+ *	for the next SearchSysCache() call that this information is setup
+ *	again.
+ * --------------------------------
+ */
+void
+SystemCacheRelationFlushed(Oid relId)
+{
+	struct catcache *cache;
+
+	for (cache = Caches; PointerIsValid(cache); cache = cache->cc_next)
+	{
+		if (cache->relationId == relId)
+		{
+			cache->relationId = InvalidOid;
+		}
+	}
+}
+
 /* --------------------------------
  *		InitIndexedSysCache
  *
@@ -654,7 +677,7 @@ InitSysCache(char *relname,
 			 HeapTuple (*iScanfuncP) ())
 {
 	CatCache   *cp;
-	register int i;
+	int			i;
 	MemoryContext oldcxt;
 
 	char	   *indname;
@@ -839,8 +862,8 @@ SearchSysCache(struct catcache * cache,
 		 elt;
 		 elt = DLGetSucc(elt))
 	{
-		bool res;
-		
+		bool		res;
+
 		ct = (CatCTup *) DLE_VAL(elt);
 		/* ----------------
 		 *	see if the cached tuple matches our key.
@@ -848,10 +871,10 @@ SearchSysCache(struct catcache * cache,
 		 * ----------------
 		 */
 		HeapKeyTest(ct->ct_tup,
-						 cache->cc_tupdesc,
-						 cache->cc_nkeys,
-						 cache->cc_skey,
-						 res);
+					cache->cc_tupdesc,
+					cache->cc_nkeys,
+					cache->cc_skey,
+					res);
 		if (res)
 			break;
 	}
@@ -892,7 +915,7 @@ SearchSysCache(struct catcache * cache,
 
 	if (DisableCache)
 	{
-		elog(WARN, "SearchSysCache: Called while cache disabled");
+		elog(ERROR, "SearchSysCache: Called while cache disabled");
 		return ((HeapTuple) NULL);
 	}
 
@@ -973,7 +996,7 @@ SearchSysCache(struct catcache * cache,
 		 */
 		MemoryContextSwitchTo(oldcxt);
 
-		sd = heap_beginscan(relation, 0, NowTimeQual,
+		sd = heap_beginscan(relation, 0, false,
 							cache->cc_nkeys, cache->cc_skey);
 
 		/* should this buffer be ReleaseBuffer'd?  --djm 8/20/96 */
