@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/fcache.c,v 1.7 1997/09/08 21:48:49 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/fcache.c,v 1.11 1998/02/26 04:37:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -71,7 +71,7 @@ GetDynamicFuncArgType(Var *arg, ExprContext *econtext)
 	tup = SearchSysCacheTuple(TYPNAME, PointerGetDatum(relname),
 							  0, 0, 0);
 	if (!tup)
-		elog(WARN, "Lookup failed on type tuple for class %s",
+		elog(ERROR, "Lookup failed on type tuple for class %s",
 			 relname);
 
 	return tup->t_oid;
@@ -98,16 +98,17 @@ init_fcache(Oid foid,
 	 * ----------------
 	 */
 	retval = (FunctionCachePtr) palloc(sizeof(FunctionCache));
+	memset(retval, 0, sizeof(FunctionCache));
 
 	if (!use_syscache)
-		elog(WARN, "what the ????, init the fcache without the catalogs?");
+		elog(ERROR, "what the ????, init the fcache without the catalogs?");
 
 	procedureTuple = SearchSysCacheTuple(PROOID,
 										 ObjectIdGetDatum(foid),
 										 0, 0, 0);
 
 	if (!HeapTupleIsValid(procedureTuple))
-		elog(WARN,
+		elog(ERROR,
 			 "init_fcache: %s %d",
 			 "Cache lookup failed for procedure", foid);
 
@@ -128,7 +129,7 @@ init_fcache(Oid foid,
 									0, 0, 0);
 
 	if (!HeapTupleIsValid(typeTuple))
-		elog(WARN,
+		elog(ERROR,
 			 "init_fcache: %s %d",
 			 "Cache lookup failed for type",
 			 (procedureStruct)->prorettype);
@@ -282,9 +283,14 @@ init_fcache(Oid foid,
 
 
 	if (retval->language != SQLlanguageId)
-		fmgr_info(foid, &(retval->func), &(retval->nargs));
+	{
+		fmgr_info(foid, &(retval->func));
+		retval->nargs = retval->func.fn_nargs;
+	}
 	else
-		retval->func = (func_ptr) NULL;
+	{
+		retval->func.fn_addr = (func_ptr) NULL;
+	}
 
 
 	return (retval);
@@ -311,6 +317,6 @@ setFcache(Node *node, Oid foid, List *argList, ExprContext *econtext)
 	}
 	else
 	{
-		elog(WARN, "init_fcache: node must be Oper or Func!");
+		elog(ERROR, "init_fcache: node must be Oper or Func!");
 	}
 }
