@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/syscache.c,v 1.9 1997/09/18 20:22:25 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/syscache.c,v 1.16 1998/02/26 04:37:33 momjian Exp $
  *
  * NOTES
  *	  These routines allow the parser/planner/executor to perform
@@ -48,7 +48,7 @@
 #include "catalog/pg_type.h"
 #include "catalog/pg_rewrite.h"
 #include "catalog/pg_aggregate.h"
-#include "catalog/pg_user.h"
+#include "catalog/pg_shadow.h"
 #include "storage/large_object.h"
 #include "catalog/pg_listener.h"
 
@@ -254,22 +254,22 @@ static struct cachedesc cacheinfo[] = {
 		sizeof(FormData_pg_listener),
 		NULL,
 	(ScanFunc) NULL},
-	{UserRelationName,			/* USENAME */
+	{ShadowRelationName,		/* USENAME */
 		1,
-		{Anum_pg_user_usename,
+		{Anum_pg_shadow_usename,
 			0,
 			0,
 		0},
-		sizeof(FormData_pg_user),
+		sizeof(FormData_pg_shadow),
 		NULL,
 	(ScanFunc) NULL},
-	{UserRelationName,			/* USESYSID */
+	{ShadowRelationName,		/* USESYSID */
 		1,
-		{Anum_pg_user_usesysid,
+		{Anum_pg_shadow_usesysid,
 			0,
 			0,
 		0},
-		sizeof(FormData_pg_user),
+		sizeof(FormData_pg_shadow),
 		NULL,
 	(ScanFunc) NULL},
 	{GroupRelationName,			/* GRONAME */
@@ -316,7 +316,16 @@ static struct cachedesc cacheinfo[] = {
 		0},
 		sizeof(FormData_pg_opclass),
 		NULL,
-	(ScanFunc) NULL}
+	(ScanFunc) NULL},
+	{LanguageRelationName,		/* LANOID */
+		1,
+		{ObjectIdAttributeNumber,
+			0,
+			0,
+		0},
+		offsetof(FormData_pg_language, lancompiler),
+		NULL,
+	NULL}
 };
 
 static struct catcache *SysCache[
@@ -362,7 +371,7 @@ InitCatalogCache()
 							 cacheinfo[cacheId].iScanFunc);
 			if (!PointerIsValid((char *) SysCache[cacheId]))
 			{
-				elog(WARN,
+				elog(ERROR,
 					 "InitCatalogCache: Can't init cache %.16s(%d)",
 					 cacheinfo[cacheId].name,
 					 cacheId);
@@ -389,11 +398,11 @@ SearchSysCacheTuple(int cacheId,/* cache selection code */
 					Datum key3,
 					Datum key4)
 {
-	register HeapTuple tp;
+	HeapTuple	tp;
 
 	if (cacheId < 0 || cacheId >= SysCacheSize)
 	{
-		elog(WARN, "SearchSysCacheTuple: Bad cache id %d", cacheId);
+		elog(ERROR, "SearchSysCacheTuple: Bad cache id %d", cacheId);
 		return ((HeapTuple) NULL);
 	}
 
@@ -414,7 +423,7 @@ SearchSysCacheTuple(int cacheId,/* cache selection code */
 							 cacheinfo[cacheId].iScanFunc);
 			if (!PointerIsValid(SysCache[cacheId]))
 			{
-				elog(WARN,
+				elog(ERROR,
 					 "InitCatalogCache: Can't init cache %.16s(%d)",
 					 cacheinfo[cacheId].name,
 					 cacheId);
@@ -460,7 +469,7 @@ SearchSysCacheStruct(int cacheId,		/* cache selection code */
 
 	if (!PointerIsValid(returnStruct))
 	{
-		elog(WARN, "SearchSysCacheStruct: No receiving struct");
+		elog(ERROR, "SearchSysCacheStruct: No receiving struct");
 		return (0);
 	}
 	tp = SearchSysCacheTuple(cacheId, key1, key2, key3, key4);
@@ -480,7 +489,7 @@ SearchSysCacheStruct(int cacheId,		/* cache selection code */
  *
  * [callers all assume this returns a (struct varlena *). -ay 10/94]
  */
-void	   *
+void *
 SearchSysCacheGetAttribute(int cacheId,
 						   AttrNumber attributeNumber,
 						   Datum key1,
@@ -528,14 +537,13 @@ SearchSysCacheGetAttribute(int cacheId,
 	}
 	else
 	{
-		elog(WARN,
+		elog(ERROR,
 			 "SearchSysCacheGetAttribute: Bad attr # %d in %s(%d)",
 			 attributeNumber, cacheName, cacheId);
 		return (NULL);
 	}
 
 	attributeValue = heap_getattr(tp,
-								  (Buffer) 0,
 								  attributeNumber,
 								  RelationGetTupleDescriptor(relation),
 								  &isNull);
@@ -562,7 +570,7 @@ SearchSysCacheGetAttribute(int cacheId,
 		: attributeLength;		/* fixed length */
 
 		tmp = (char *) palloc(size);
-		memmove(tmp, (void *)attributeValue, size);
+		memmove(tmp, (void *) attributeValue, size);
 		returnValue = (void *) tmp;
 	}
 
@@ -583,7 +591,7 @@ SearchSysCacheGetAttribute(int cacheId,
  * [identical to get_typdefault, expecting a (struct varlena *) as ret val.
  *	some day, either of the functions should be removed		 -ay 10/94]
  */
-void	   *
+void *
 TypeDefaultRetrieve(Oid typId)
 {
 	HeapTuple	typeTuple;
