@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/elog.c,v 1.19 1997/09/08 02:31:27 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/elog.c,v 1.27 1998/02/26 04:37:34 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,8 +45,8 @@ elog(int lev, const char *fmt,...)
 	va_list		ap;
 	char		buf[ELOG_MAXLEN],
 				line[ELOG_MAXLEN];
-	register char *bp;
-	register const char *cp;
+	char	   *bp;
+	const char *cp;
 	extern int	errno,
 				sys_nerr;
 
@@ -74,7 +74,7 @@ elog(int lev, const char *fmt,...)
 				i = 0;
 			if (i > 30)
 				i = i % 30;
-			cp = "DEBUG:";
+			cp = "DEBUG:  ";
 			break;
 		case DEBUG:
 			i = ElogDebugIndentLevel;
@@ -82,16 +82,16 @@ elog(int lev, const char *fmt,...)
 				i = 0;
 			if (i > 30)
 				i = i % 30;
-			cp = "DEBUG:";
+			cp = "DEBUG:  ";
 			break;
 		case NOTICE:
-			cp = "NOTICE:";
+			cp = "NOTICE:  ";
 			break;
-		case WARN:
-			cp = "WARN:";
+		case ERROR:
+			cp = "ERROR:  ";
 			break;
 		default:
-			sprintf(line, "FATAL %d:", lev);
+			sprintf(line, "FATAL %d:  ", lev);
 			cp = line;
 	}
 #ifdef ELOG_TIMESTAMPS
@@ -162,16 +162,25 @@ elog(int lev, const char *fmt,...)
 		pq_putstr(line);
 		pq_flush();
 	}
+	if (Pfout == NULL)
+	{
+
+		/*
+		 * There is no socket.	One explanation for this is we are running
+		 * as the Postmaster.  So we'll write the message to stderr.
+		 */
+		fputs(line, stderr);
+	}
 #endif							/* !PG_STANDALONE */
 
-	if (lev == WARN)
+	if (lev == ERROR)
 	{
-		extern int	InWarn;
+		extern int	InError;
 
 		ProcReleaseSpins(NULL); /* get rid of spinlocks we hold */
-		if (!InWarn)
+		if (!InError)
 		{
-			kill(getpid(), 1);	/* abort to traffic cop */
+			kill(MyProcPid, 1); /* abort to traffic cop */
 			pause();
 		}
 
@@ -251,7 +260,7 @@ DebugFileOpen(void)
 	if (fcntl(fd, F_GETFD, 0) < 0)
 	{
 		sprintf(OutputFileName, "%s/pg.errors.%d",
-				DataDir, (int) getpid());
+				DataDir, (int) MyProcPid);
 		fd = open(OutputFileName, O_CREAT | O_APPEND | O_WRONLY, 0666);
 	}
 	if (fd < 0)
