@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/exc.c,v 1.18 1997/09/08 21:49:00 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/exc.c,v 1.20 1998/02/26 04:37:35 momjian Exp $
  *
  * NOTE
  *	  XXX this code needs improvement--check for state violations and
@@ -179,7 +179,7 @@ ExcRaise(Exception *excP,
 		 ExcData data,
 		 ExcMessage message)
 {
-	register ExcFrame *efp;
+	ExcFrame   *efp;
 
 	efp = ExcCurFrameP;
 	if (efp == NULL)
