@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/globals.c,v 1.14 1997/09/18 14:33:46 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/globals.c,v 1.21 1998/02/26 04:37:56 momjian Exp $
  *
  * NOTES
  *	  Globals used all over the place should be declared here and not
@@ -28,19 +28,20 @@
 
 #include <storage/backendid.h>
 #include "access/heapam.h"
-#include "utils/tqual.h"
 #include "storage/sinval.h"
 #include "storage/sinvaladt.h"
 #include "storage/lmgr.h"
 #include "utils/elog.h"
-
+#include "libpq/pqcomm.h"
 #include "catalog/catname.h"
 
+ProtocolVersion FrontendProtocol = PG_PROTOCOL_LATEST;
 int			Portfd = -1;
 int			Noversion = 0;
 int			Quiet = 1;
 
-int			MasterPid;
+int			MyProcPid;
+
 char	   *DataDir;
 
  /*
@@ -107,15 +108,9 @@ char	   *IndexedCatalogNames[] = {
  */
 char	   *SharedSystemRelationNames[] = {
 	DatabaseRelationName,
-	DefaultsRelationName,
-	DemonRelationName,
 	GroupRelationName,
-	HostsRelationName,
 	LogRelationName,
-	MagicRelationName,
-	ServerRelationName,
-	TimeRelationName,
-	UserRelationName,
+	ShadowRelationName,
 	VariableRelationName,
 	0
 };
