@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/sort/psort.c,v 1.26 1997/10/15 06:36:08 vadim Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/sort/psort.c,v 1.39 1998/02/26 04:38:29 momjian Exp $
  *
  * NOTES
  *		Sorts the first relation into the second relation.
@@ -40,56 +40,49 @@
 #include <unistd.h>
 
 #include "postgres.h"
+#include "miscadmin.h"
 
-#include "executor/execdebug.h"
 #include "access/heapam.h"
 #include "access/htup.h"
 #include "access/relscan.h"
 #include "access/skey.h"
-#include "utils/tqual.h"		/* for NowTimeQual */
-
-#include "storage/buf.h"
-#include "storage/bufmgr.h"		/* for BLCKSZ */
-#include "utils/portal.h"		/* for {Start,End}PortalAllocMode */
-#include "utils/elog.h"
-#include "utils/rel.h"
-
+#include "executor/execdebug.h"
+#include "executor/executor.h"
 #include "nodes/execnodes.h"
 #include "nodes/plannodes.h"
-#include "executor/executor.h"
-
-#include "utils/lselect.h"
-#include "utils/psort.h"
-
-#include "miscadmin.h"
+#include "storage/buf.h"
 #include "storage/fd.h"
+#include "utils/lselect.h"
+#include "utils/portal.h"		/* for {Start,End}PortalAllocMode */
+#include "utils/psort.h"
+#include "utils/rel.h"
 
-static bool createfirstrun(Sort * node);
-static bool createrun(Sort * node, FILE * file);
-static void destroytape(FILE * file);
-static void dumptuples(FILE * file, Sort * node);
+static bool createfirstrun(Sort *node);
+static bool createrun(Sort *node, FILE *file);
+static void destroytape(FILE *file);
+static void dumptuples(FILE *file, Sort *node);
 static FILE *gettape(void);
-static void initialrun(Sort * node);
-static void inittapes(Sort * node);
-static void merge(Sort * node, struct tape * dest);
-static FILE *mergeruns(Sort * node);
+static void initialrun(Sort *node);
+static void inittapes(Sort *node);
+static void merge(Sort *node, struct tape * dest);
+static FILE *mergeruns(Sort *node);
 static HeapTuple tuplecopy(HeapTuple tup);
-static int _psort_cmp (HeapTuple *ltup, HeapTuple *rtup);
+static int	_psort_cmp(HeapTuple *ltup, HeapTuple *rtup);
 
 
 
 #define TEMPDIR "./"
 
-/* 
+/*
  * tlenzero used to delimit runs; both vars below must have
  * the same size as HeapTuple->t_len
  */
 static unsigned int tlenzero = 0;
 static unsigned int tlendummy;
 
-static TupleDesc	PsortTupDesc;
-static ScanKey		PsortKeys;		/* used by _psort_cmp */
-static int			PsortNkeys;
+static TupleDesc PsortTupDesc;
+static ScanKey PsortKeys;		/* used by _psort_cmp */
+static int	PsortNkeys;
 
 /*
  * old psort global variables
@@ -134,7 +127,7 @@ static int			PsortNkeys;
  *						  Allocates and initializes sort node's psort state.
  */
 bool
-psort_begin(Sort * node, int nkeys, ScanKey key)
+psort_begin(Sort *node, int nkeys, ScanKey key)
 {
 
 	node->psortstate = (struct Psortstate *) palloc(sizeof(struct Psortstate));
@@ -180,10 +173,10 @@ psort_begin(Sort * node, int nkeys, ScanKey key)
  *				number of allocated tapes
  */
 static void
-inittapes(Sort * node)
+inittapes(Sort *node)
 {
-	register int i;
-	register struct tape *tp;
+	int			i;
+	struct tape *tp;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -207,7 +200,7 @@ inittapes(Sort * node)
 	PS(node)->Tape[0].tp_prev = tp;
 
 	if (PS(node)->TapeRange <= 1)
-		elog(WARN, "inittapes: Could only allocate %d < 3 tapes\n",
+		elog(ERROR, "inittapes: Could only allocate %d < 3 tapes\n",
 			 PS(node)->TapeRange + 1);
 
 	PS(node)->Level = 1;
@@ -252,7 +245,7 @@ inittapes(Sort * node)
 
 #define USEMEM(NODE,AMT)		PS(node)->treeContext.sortMem -= (AMT)
 #define FREEMEM(NODE,AMT)		PS(node)->treeContext.sortMem += (AMT)
-#define LACKMEM(NODE)			(PS(node)->treeContext.sortMem <= MAXBLCKSZ)	/* not accurate */
+#define LACKMEM(NODE)			(PS(node)->treeContext.sortMem <= BLCKSZ)		/* not accurate */
 #define TRACEMEM(FUNC)
 #define TRACEOUT(FUNC, TUP)
 
@@ -281,10 +274,10 @@ inittapes(Sort * node)
  *				Also, perhaps allocate tapes when needed. Split into 2 funcs.
  */
 static void
-initialrun(Sort * node)
+initialrun(Sort *node)
 {
-	/* register struct tuple   *tup; */
-	register struct tape *tp;
+	/* struct tuple   *tup; */
+	struct tape *tp;
 	int			baseruns;		/* D:(a) */
 	int			extrapasses;	/* EOF */
 
@@ -295,22 +288,25 @@ initialrun(Sort * node)
 
 	if (createfirstrun(node))
 	{
-		Assert (PS(node)->using_tape_files);
+		Assert(PS(node)->using_tape_files);
 		extrapasses = 0;
 	}
-	else			/* all tuples fetched */
+	else
+/* all tuples fetched */
 	{
-		if ( !PS(node)->using_tape_files )	/* empty or sorted in memory */
+		if (!PS(node)->using_tape_files)		/* empty or sorted in
+												 * memory */
 			return;
-		/* 
+
+		/*
 		 * if PS(node)->Tuples == NULL then we have single (sorted) run
-		 * which can be used as result grab file! So, we may avoid 
+		 * which can be used as result grab file! So, we may avoid
 		 * mergeruns - it will just copy this run to new file.
 		 */
-		if ( PS(node)->Tuples == NULL )
+		if (PS(node)->Tuples == NULL)
 		{
 			PS(node)->psort_grab_file = PS(node)->Tape->tp_file;
-			rewind (PS(node)->psort_grab_file);
+			rewind(PS(node)->psort_grab_file);
 			return;
 		}
 		extrapasses = 2;
@@ -359,7 +355,7 @@ initialrun(Sort * node)
 }
 
 /*
- *		createfirstrun		- tries to sort tuples in memory using qsort 
+ *		createfirstrun		- tries to sort tuples in memory using qsort
  *						until LACKMEM; if not enough memory then switches
  *						to tape method
  *
@@ -370,12 +366,12 @@ initialrun(Sort * node)
 static bool
 createfirstrun(Sort *node)
 {
-    HeapTuple			tup;
-	bool				foundeor = false;
-	HeapTuple		   *memtuples;
-	int					t_last = -1;
-	int					t_free = 1000;
-	TupleTableSlot	   *cr_slot;
+	HeapTuple	tup;
+	bool		foundeor = false;
+	HeapTuple  *memtuples;
+	int			t_last = -1;
+	int			t_free = 1000;
+	TupleTableSlot *cr_slot;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -383,15 +379,15 @@ createfirstrun(Sort *node)
 	Assert(PS(node)->memtuples == NULL);
 	Assert(PS(node)->tupcount == 0);
 	if (LACKMEM(node))
-		elog (FATAL, "psort: LACKMEM in createfirstrun");
-	
+		elog(FATAL, "psort: LACKMEM in createfirstrun");
+
 	memtuples = palloc(t_free * sizeof(HeapTuple));
-	
+
 	for (;;)
 	{
-		if ( LACKMEM (node) )
+		if (LACKMEM(node))
 			break;
-		
+
 		/*
 		 * About to call ExecProcNode, it can mess up the state if it
 		 * eventually calls another Sort node. So must stow it away here
@@ -406,28 +402,28 @@ createfirstrun(Sort *node)
 			foundeor = true;
 			break;
 		}
-		
+
 		tup = tuplecopy(cr_slot->val);
 		ExecClearTuple(cr_slot);
 
 		IncrProcessed();
 		USEMEM(node, tup->t_len);
 		TRACEMEM(createfirstrun);
-		if ( t_free <= 0 )
+		if (t_free <= 0)
 		{
-		    t_free = 1000;
-			memtuples = repalloc (memtuples, 
-	    				(t_last + t_free + 1) * sizeof (HeapTuple));
+			t_free = 1000;
+			memtuples = repalloc(memtuples,
+							  (t_last + t_free + 1) * sizeof(HeapTuple));
 		}
 		t_last++;
 		t_free--;
 		memtuples[t_last] = tup;
 	}
-	
-	if ( t_last < 0 )			/* empty */
+
+	if (t_last < 0)				/* empty */
 	{
-		Assert (foundeor);
-		pfree (memtuples);
+		Assert(foundeor);
+		pfree(memtuples);
 		return (false);
 	}
 	t_last++;
@@ -435,27 +431,27 @@ createfirstrun(Sort *node)
 	PsortTupDesc = PS(node)->treeContext.tupDesc;
 	PsortKeys = PS(node)->treeContext.scanKeys;
 	PsortNkeys = PS(node)->treeContext.nKeys;
-    qsort (memtuples, t_last, sizeof (HeapTuple), 
-    	(int (*)(const void *,const void *))_psort_cmp);
-	
-	if ( LACKMEM (node) )	/* in-memory sort is impossible */
+	qsort(memtuples, t_last, sizeof(HeapTuple),
+		  (int (*) (const void *, const void *)) _psort_cmp);
+
+	if (LACKMEM(node))			/* in-memory sort is impossible */
 	{
-    	register int t;
-		
-		Assert (!foundeor);
+		int			t;
+
+		Assert(!foundeor);
 		inittapes(node);
 		/* put tuples into leftist tree for createrun */
-		for (t = t_last - 1 ; t >= 0; t--)
+		for (t = t_last - 1; t >= 0; t--)
 			puttuple(&PS(node)->Tuples, memtuples[t], 0, &PS(node)->treeContext);
-		pfree (memtuples);
-		foundeor = !createrun (node, PS(node)->Tape->tp_file);
+		pfree(memtuples);
+		foundeor = !createrun(node, PS(node)->Tape->tp_file);
 	}
 	else
 	{
-		Assert (foundeor);
+		Assert(foundeor);
 		PS(node)->memtuples = memtuples;
 	}
-		
+
 	return (!foundeor);
 }
 
@@ -471,24 +467,24 @@ createfirstrun(Sort *node)
  *				Tuples contains the tuples for the following run upon exit
  */
 static bool
-createrun(Sort * node, FILE * file)
+createrun(Sort *node, FILE *file)
 {
-	register HeapTuple	lasttuple;
-	register HeapTuple	tup;
-	TupleTableSlot	   *cr_slot;
-	HeapTuple		   *memtuples;
-	int					t_last = -1;
-	int					t_free = 1000;
-	bool				foundeor = false;
-	short				junk;
+	HeapTuple	lasttuple;
+	HeapTuple	tup;
+	TupleTableSlot *cr_slot;
+	HeapTuple  *memtuples;
+	int			t_last = -1;
+	int			t_free = 1000;
+	bool		foundeor = false;
+	short		junk;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
-	Assert (PS(node)->using_tape_files);
+	Assert(PS(node)->using_tape_files);
 
 	lasttuple = NULL;
 	memtuples = palloc(t_free * sizeof(HeapTuple));
-	
+
 	for (;;)
 	{
 		while (LACKMEM(node) && PS(node)->Tuples != NULL)
@@ -500,14 +496,14 @@ createrun(Sort * node, FILE * file)
 				TRACEMEM(createrun);
 			}
 			lasttuple = gettuple(&PS(node)->Tuples, &junk,
-										   &PS(node)->treeContext);
+								 &PS(node)->treeContext);
 			PUTTUP(node, lasttuple, file);
 			TRACEOUT(createrun, lasttuple);
 		}
-		
+
 		if (LACKMEM(node))
 			break;
-		
+
 		/*
 		 * About to call ExecProcNode, it can mess up the state if it
 		 * eventually calls another Sort node. So must stow it away here
@@ -535,11 +531,11 @@ createrun(Sort * node, FILE * file)
 		if (lasttuple != NULL && tuplecmp(tup, lasttuple,
 										  &PS(node)->treeContext))
 		{
-			if ( t_free <= 0 )
+			if (t_free <= 0)
 			{
-			    t_free = 1000;
-				memtuples = repalloc (memtuples, 
-		    				(t_last + t_free + 1) * sizeof (HeapTuple));
+				t_free = 1000;
+				memtuples = repalloc(memtuples,
+							  (t_last + t_free + 1) * sizeof(HeapTuple));
 			}
 			t_last++;
 			t_free--;
@@ -556,23 +552,23 @@ createrun(Sort * node, FILE * file)
 	}
 	dumptuples(file, node);
 	ENDRUN(file);				/* delimit the end of the run */
-	
+
 	t_last++;
 	/* put tuples for the next run into leftist tree */
-	if ( t_last >= 1 )
+	if (t_last >= 1)
 	{
-		register int t;
-		
+		int			t;
+
 		PsortTupDesc = PS(node)->treeContext.tupDesc;
 		PsortKeys = PS(node)->treeContext.scanKeys;
 		PsortNkeys = PS(node)->treeContext.nKeys;
-    	qsort (memtuples, t_last, sizeof (HeapTuple), 
-    		(int (*)(const void *,const void *))_psort_cmp);
-		for (t = t_last - 1 ; t >= 0; t--)
+		qsort(memtuples, t_last, sizeof(HeapTuple),
+			  (int (*) (const void *, const void *)) _psort_cmp);
+		for (t = t_last - 1; t >= 0; t--)
 			puttuple(&PS(node)->Tuples, memtuples[t], 0, &PS(node)->treeContext);
 	}
-	
-	pfree (memtuples);
+
+	pfree(memtuples);
 
 	return (!foundeor);
 }
@@ -605,9 +601,9 @@ tuplecopy(HeapTuple tup)
  *				file of tuples in order
  */
 static FILE *
-mergeruns(Sort * node)
+mergeruns(Sort *node)
 {
-	register struct tape *tp;
+	struct tape *tp;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -632,17 +628,17 @@ mergeruns(Sort * node)
  *						  (polyphase merge Alg.D(D5)--Knuth, Vol.3, p271)
  */
 static void
-merge(Sort * node, struct tape * dest)
+merge(Sort *node, struct tape * dest)
 {
-	register HeapTuple tup;
-	register struct tape *lasttp;		/* (TAPE[P]) */
-	register struct tape *tp;
+	HeapTuple	tup;
+	struct tape *lasttp;		/* (TAPE[P]) */
+	struct tape *tp;
 	struct leftist *tuples;
-	FILE		   *destfile;
-	int				times;			/* runs left to merge */
-	int				outdummy;		/* complete dummy runs */
-	short			fromtape;
-	unsigned int	tuplen;
+	FILE	   *destfile;
+	int			times;			/* runs left to merge */
+	int			outdummy;		/* complete dummy runs */
+	short		fromtape;
+	unsigned int tuplen;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -739,15 +735,15 @@ merge(Sort * node, struct tape * dest)
  * dumptuples	- stores all the tuples in tree into file
  */
 static void
-dumptuples(FILE * file, Sort * node)
+dumptuples(FILE *file, Sort *node)
 {
-	register struct leftist *tp;
-	register struct leftist *newp;
+	struct leftist *tp;
+	struct leftist *newp;
 	struct leftist **treep = &PS(node)->Tuples;
 	LeftistContext context = &PS(node)->treeContext;
 	HeapTuple	tup;
 
-	Assert (PS(node)->using_tape_files);
+	Assert(PS(node)->using_tape_files);
 
 	tp = *treep;
 	while (tp != NULL)
@@ -774,19 +770,19 @@ dumptuples(FILE * file, Sort * node)
  *						  a NULL indicating the last tuple has been processed.
  */
 HeapTuple
-psort_grabtuple(Sort * node, bool * should_free)
+psort_grabtuple(Sort *node, bool *should_free)
 {
-	register HeapTuple	tup;
+	HeapTuple	tup;
 
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
 
 	if (PS(node)->using_tape_files == true)
 	{
-		unsigned int	tuplen;
-		
+		unsigned int tuplen;
+
 		*should_free = true;
-		if (ScanDirectionIsForward (node->plan.state->es_direction))
+		if (ScanDirectionIsForward(node->plan.state->es_direction))
 		{
 			if (PS(node)->all_fetched)
 				return NULL;
@@ -797,7 +793,7 @@ psort_grabtuple(Sort * node, bool * should_free)
 				GETTUP(node, tup, tuplen, PS(node)->psort_grab_file);
 
 				/* Update current merged sort file position */
-				PS(node)->psort_current += tuplen + sizeof (tlendummy);
+				PS(node)->psort_current += tuplen + sizeof(tlendummy);
 				return tup;
 			}
 			else
@@ -807,64 +803,72 @@ psort_grabtuple(Sort * node, bool * should_free)
 			}
 		}
 		/* Backward */
-		if (PS(node)->psort_current <= sizeof (tlendummy))
+		if (PS(node)->psort_current <= sizeof(tlendummy))
 			return NULL;
-		/* 
-		 * if all tuples are fetched already then we return last tuple, 
+
+		/*
+		 * if all tuples are fetched already then we return last tuple,
 		 * else - tuple before last returned.
 		 */
 		if (PS(node)->all_fetched)
 		{
-			/* psort_current is pointing to the zero tuplen at the end of file */
-			fseek(PS(node)->psort_grab_file, 
-					PS(node)->psort_current - sizeof (tlendummy), SEEK_SET);
+
+			/*
+			 * psort_current is pointing to the zero tuplen at the end of
+			 * file
+			 */
+			fseek(PS(node)->psort_grab_file,
+				  PS(node)->psort_current - sizeof(tlendummy), SEEK_SET);
 			GETLEN(tuplen, PS(node)->psort_grab_file);
 			if (PS(node)->psort_current < tuplen)
-				elog (FATAL, "psort_grabtuple: too big last tuple len in backward scan");
+				elog(FATAL, "psort_grabtuple: too big last tuple len in backward scan");
 			PS(node)->all_fetched = false;
 		}
 		else
 		{
 			/* move to position of end tlen of prev tuple */
-			PS(node)->psort_current -= sizeof (tlendummy);
+			PS(node)->psort_current -= sizeof(tlendummy);
 			fseek(PS(node)->psort_grab_file, PS(node)->psort_current, SEEK_SET);
-			GETLEN(tuplen, PS(node)->psort_grab_file);	/* get tlen of prev tuple */
+			GETLEN(tuplen, PS(node)->psort_grab_file);	/* get tlen of prev
+														 * tuple */
 			if (tuplen == 0)
-				elog (FATAL, "psort_grabtuple: tuplen is 0 in backward scan");
-			if (PS(node)->psort_current <= tuplen + sizeof (tlendummy))
-			{	/* prev tuple should be first one */
+				elog(FATAL, "psort_grabtuple: tuplen is 0 in backward scan");
+			if (PS(node)->psort_current <= tuplen + sizeof(tlendummy))
+			{					/* prev tuple should be first one */
 				if (PS(node)->psort_current != tuplen)
-					elog (FATAL, "psort_grabtuple: first tuple expected in backward scan");
+					elog(FATAL, "psort_grabtuple: first tuple expected in backward scan");
 				PS(node)->psort_current = 0;
 				fseek(PS(node)->psort_grab_file, PS(node)->psort_current, SEEK_SET);
 				return NULL;
 			}
-			/* 
-			 * Get position of prev tuple. This tuple becomes current tuple
-			 * now and we have to return previous one.
+
+			/*
+			 * Get position of prev tuple. This tuple becomes current
+			 * tuple now and we have to return previous one.
 			 */
 			PS(node)->psort_current -= tuplen;
 			/* move to position of end tlen of prev tuple */
-			fseek(PS(node)->psort_grab_file, 
-					PS(node)->psort_current - sizeof (tlendummy), SEEK_SET);
+			fseek(PS(node)->psort_grab_file,
+				  PS(node)->psort_current - sizeof(tlendummy), SEEK_SET);
 			GETLEN(tuplen, PS(node)->psort_grab_file);
-			if (PS(node)->psort_current < tuplen + sizeof (tlendummy))
-				elog (FATAL, "psort_grabtuple: too big tuple len in backward scan");
+			if (PS(node)->psort_current < tuplen + sizeof(tlendummy))
+				elog(FATAL, "psort_grabtuple: too big tuple len in backward scan");
 		}
-		/* 
-		 * move to prev (or last) tuple start position + sizeof(t_len) 
+
+		/*
+		 * move to prev (or last) tuple start position + sizeof(t_len)
 		 */
 		fseek(PS(node)->psort_grab_file,
-				PS(node)->psort_current - tuplen, SEEK_SET);
+			  PS(node)->psort_current - tuplen, SEEK_SET);
 		tup = (HeapTuple) palloc((unsigned) tuplen);
 		SETTUPLEN(tup, tuplen);
 		GETTUP(node, tup, tuplen, PS(node)->psort_grab_file);
-		return tup;		/* file position is equal to psort_current */
+		return tup;				/* file position is equal to psort_current */
 	}
 	else
 	{
 		*should_free = false;
-		if (ScanDirectionIsForward (node->plan.state->es_direction))
+		if (ScanDirectionIsForward(node->plan.state->es_direction))
 		{
 			if (PS(node)->psort_current < PS(node)->tupcount)
 				return (PS(node)->memtuples[PS(node)->psort_current++]);
@@ -877,15 +881,16 @@ psort_grabtuple(Sort * node, bool * should_free)
 		/* Backward */
 		if (PS(node)->psort_current <= 0)
 			return NULL;
-		/* 
-		 * if all tuples are fetched already then we return last tuple, 
+
+		/*
+		 * if all tuples are fetched already then we return last tuple,
 		 * else - tuple before last returned.
 		 */
-		if (PS(node)->all_fetched)		
+		if (PS(node)->all_fetched)
 			PS(node)->all_fetched = false;
 		else
 		{
-			PS(node)->psort_current--;			/* last returned tuple */
+			PS(node)->psort_current--;	/* last returned tuple */
 			if (PS(node)->psort_current <= 0)
 				return NULL;
 		}
@@ -897,7 +902,7 @@ psort_grabtuple(Sort * node, bool * should_free)
  *		psort_markpos	- saves current position in the merged sort file
  */
 void
-psort_markpos(Sort * node)
+psort_markpos(Sort *node)
 {
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -910,7 +915,7 @@ psort_markpos(Sort * node)
  *						  last saved position
  */
 void
-psort_restorepos(Sort * node)
+psort_restorepos(Sort *node)
 {
 	Assert(node != (Sort *) NULL);
 	Assert(PS(node) != (Psortstate *) NULL);
@@ -925,13 +930,12 @@ psort_restorepos(Sort * node)
  *						  called unless psort_grabtuple has returned a NULL.
  */
 void
-psort_end(Sort * node)
+psort_end(Sort *node)
 {
-	register struct tape *tp;
+	struct tape *tp;
 
 	if (!node->cleaned)
 	{
-		Assert(node != (Sort *) NULL);
 
 		/*
 		 * I'm changing this because if we are sorting a relation with no
@@ -951,12 +955,36 @@ psort_end(Sort * node)
 				(int) ceil((double) PS(node)->BytesWritten / BLCKSZ);
 
 			pfree((void *) node->psortstate);
+			node->psortstate = NULL;
 
 			node->cleaned = TRUE;
 		}
 	}
 }
 
+void
+psort_rescan(Sort *node)
+{
+
+	/*
+	 * If subnode is to be rescanned then free our previous results
+	 */
+	if (((Plan *) node)->lefttree->chgParam != NULL)
+	{
+		psort_end(node);
+		node->cleaned = false;
+	}
+	else if (PS(node) != (Psortstate *) NULL)
+	{
+		PS(node)->all_fetched = false;
+		PS(node)->psort_current = 0;
+		PS(node)->psort_saved = 0;
+		if (PS(node)->using_tape_files == true)
+			rewind(PS(node)->psort_grab_file);
+	}
+
+}
+
 /*
  *		gettape			- handles access temporary files in polyphase merging
  *
@@ -985,7 +1013,7 @@ static struct tapelst *Tapes = NULL;
 static FILE *
 gettape()
 {
-	register struct tapelst *tp;
+	struct tapelst *tp;
 	FILE	   *file;
 	static int	tapeinit = 0;
 	char	   *mktemp();
@@ -995,7 +1023,7 @@ gettape()
 
 	tp = (struct tapelst *) palloc((unsigned) sizeof(struct tapelst));
 
-	sprintf(uniqueName, "%spg_psort.%d.%d", TEMPDIR, (int) getpid(), uniqueFileId);
+	sprintf(uniqueName, "%spg_psort.%d.%d", TEMPDIR, (int) MyProcPid, uniqueFileId);
 	uniqueFileId++;
 
 	tapeinit = 1;
@@ -1006,12 +1034,12 @@ gettape()
 	 * now, copy template with final null into palloc'd space
 	 */
 
-	strNcpy(tp->tl_name, uniqueName, MAXPGPATH-1);
+	StrNCpy(tp->tl_name, uniqueName, MAXPGPATH);
 
 
 	file = AllocateFile(tp->tl_name, "w+");
 	if (file == NULL)
-		elog(WARN, "Open: %s in %s line %d, %s", tp->tl_name,
+		elog(ERROR, "Open: %s in %s line %d, %s", tp->tl_name,
 			 __FILE__, __LINE__, strerror(errno));
 
 	tp->tl_fd = fileno(file);
@@ -1025,10 +1053,10 @@ gettape()
  */
 #ifdef NOT_USED
 static void
-resettape(FILE * file)
+resettape(FILE *file)
 {
-	register struct tapelst *tp;
-	register int fd;
+	struct tapelst *tp;
+	int			fd;
 
 	Assert(PointerIsValid(file));
 
@@ -1036,7 +1064,7 @@ resettape(FILE * file)
 	for (tp = Tapes; tp != NULL && tp->tl_fd != fd; tp = tp->tl_next)
 		;
 	if (tp == NULL)
-		elog(WARN, "resettape: tape not found");
+		elog(ERROR, "resettape: tape not found");
 
 	file = freopen(tp->tl_name, "w+", file);
 	if (file == NULL)
@@ -1057,11 +1085,11 @@ resettape(FILE * file)
  *				Exits instead of returning status, if given invalid tape.
  */
 static void
-destroytape(FILE * file)
+destroytape(FILE *file)
 {
-	register struct tapelst *tp,
+	struct tapelst *tp,
 			   *tq;
-	register int fd;
+	int			fd;
 
 	if ((tp = Tapes) == NULL)
 		elog(FATAL, "destroytape: tape not found");
@@ -1094,40 +1122,40 @@ destroytape(FILE * file)
 }
 
 static int
-_psort_cmp (HeapTuple *ltup, HeapTuple *rtup)
+_psort_cmp(HeapTuple *ltup, HeapTuple *rtup)
 {
-    register Datum	lattr, rattr;
-    int		nkey = 0;
-    int		result = 0;
-    bool	isnull1, isnull2;
-    
-    while ( nkey < PsortNkeys && !result )
-    {
-		lattr = heap_getattr(*ltup, InvalidBuffer,
-				     PsortKeys[nkey].sk_attno, 
-				     PsortTupDesc,
-			    	 &isnull1);
-		rattr = heap_getattr(*rtup, InvalidBuffer,
-				     PsortKeys[nkey].sk_attno, 
-				     PsortTupDesc,
-				     &isnull2);
-		if ( isnull1 )
+	Datum		lattr,
+				rattr;
+	int			nkey;
+	int			result = 0;
+	bool		isnull1,
+				isnull2;
+
+	for (nkey = 0; nkey < PsortNkeys && !result; nkey++)
+	{
+		lattr = heap_getattr(*ltup,
+							 PsortKeys[nkey].sk_attno,
+							 PsortTupDesc,
+							 &isnull1);
+		rattr = heap_getattr(*rtup,
+							 PsortKeys[nkey].sk_attno,
+							 PsortTupDesc,
+							 &isnull2);
+		if (isnull1)
 		{
-	    	if ( isnull2 )
-	    		return (0);
-		    return(1);
+			if (!isnull2)
+				result = 1;
 		}
-		else if ( isnull2 )
-		    return (-1);
-		
-		if (PsortKeys[nkey].sk_flags & SK_COMMUTE)
+		else if (isnull2)
+			result = -1;
+
+		else if (PsortKeys[nkey].sk_flags & SK_COMMUTE)
 		{
-	    	if (!(result = -(long) (*PsortKeys[nkey].sk_func) (rattr, lattr)))
-			result = (long) (*PsortKeys[nkey].sk_func) (lattr, rattr);
+			if (!(result = -(long) (*fmgr_faddr(&PsortKeys[nkey].sk_func)) (rattr, lattr)))
+				result = (long) (*fmgr_faddr(&PsortKeys[nkey].sk_func)) (lattr, rattr);
 		}
-		else if (!(result = -(long) (*PsortKeys[nkey].sk_func) (lattr, rattr)))
-		    result = (long) (*PsortKeys[nkey].sk_func) (rattr, lattr);
-		nkey++;
-    }
-    return (result);
+		else if (!(result = -(long) (*fmgr_faddr(&PsortKeys[nkey].sk_func)) (lattr, rattr)))
+			result = (long) (*fmgr_faddr(&PsortKeys[nkey].sk_func)) (rattr, lattr);
+	}
+	return (result);
 }
