@@ -11,7 +11,7 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: miscadmin.h,v 1.16 1997/09/18 14:42:22 vadim Exp $
+ * $Id: miscadmin.h,v 1.22 1998/04/05 21:04:50 momjian Exp $
  *
  * NOTES
  *	  some of the information in this file will be moved to
@@ -36,10 +36,11 @@ extern int	PostmasterMain(int argc, char *argv[]);
  */
 extern int	Portfd;
 extern int	Noversion;
-extern int	MasterPid;
 extern int	Quiet;
 extern char *DataDir;
 
+extern int	MyProcPid;
+
 extern char OutputFileName[];
 
 /*
@@ -63,6 +64,7 @@ extern short DebugLvl;
  *	USE_POSTGRES_DATES specifies traditional postgres format for output.
  *	USE_ISO_DATES specifies ISO-compliant format for output.
  *	USE_SQL_DATES specified Oracle/Ingres-compliant format for output.
+ *	USE_GERMAN_DATES specifies German-style dd.mm/yyyy date format.
  *
  * DateStyle specifies preference for date formatting for output.
  * EuroDates if client prefers dates interpreted and written w/European conventions.
@@ -78,6 +80,7 @@ extern short DebugLvl;
 #define USE_POSTGRES_DATES		0
 #define USE_ISO_DATES			1
 #define USE_SQL_DATES			2
+#define USE_GERMAN_DATES		3
 
 extern int	DateStyle;
 extern bool EuroDates;
@@ -107,9 +110,15 @@ extern Oid	LastOidProcessed;	/* for query rewrite */
  *			POSTGRES directory path definitions.							 *
  *****************************************************************************/
 
+extern char *DatabaseName;
+extern char *DatabasePath;
+
+/* in utils/misc/database.c */
+extern void GetRawDatabaseInfo(char *name, Oid *owner, Oid *db_id, char *path);
+extern int	GetDatabaseInfo(char *name, Oid *owner, char *path);
+extern char *ExpandDatabasePath(char *path);
+
 /* now in utils/init/miscinit.c */
-extern char *GetDatabasePath(void);
-extern char *GetDatabaseName(void);
 extern void SetDatabaseName(char *name);
 extern void SetDatabasePath(char *path);
 extern char *getpgusername(void);
