@@ -6,7 +6,7 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: acl.h,v 1.11 1997/09/08 21:54:47 momjian Exp $
+ * $Id: acl.h,v 1.17 1998/02/26 04:43:43 momjian Exp $
  *
  * NOTES
  *	  For backward-compatability purposes we have to allow there
@@ -39,7 +39,7 @@ typedef uint32 AclId;
 typedef uint8 AclIdType;
 
 #define ACL_IDTYPE_WORLD		0x00
-#define ACL_IDTYPE_UID			0x01	/* user id - from pg_user */
+#define ACL_IDTYPE_UID			0x01	/* user id - from pg_shadow */
 #define ACL_IDTYPE_GID			0x02	/* group id - from pg_group */
 
 /*
@@ -62,7 +62,7 @@ typedef uint8 AclMode;
 #define ACL_MODECHG_EQL			3
 
 /* change this line if you want to set the default acl permission  */
-#define ACL_WORLD_DEFAULT		(ACL_RD)
+#define ACL_WORLD_DEFAULT		(ACL_NO)
 /* #define		ACL_WORLD_DEFAULT		(ACL_RD|ACL_WR|ACL_AP|ACL_RU) */
 #define ACL_OWNER_DEFAULT		(ACL_RD|ACL_WR|ACL_AP|ACL_RU)
 
@@ -135,8 +135,8 @@ extern char *aclcheck_error_strings[];
 /*
  * routines used internally (parser, etc.)
  */
-extern Acl *aclownerdefault(AclId ownerid);
-extern Acl *acldefault(void);
+extern Acl *aclownerdefault(char *relname, AclId ownerid);
+extern Acl *acldefault(char *relname);
 extern Acl *aclinsert3(Acl *old_acl, AclItem *mod_aip, unsigned modechg);
 
 extern char *aclmakepriv(char *old_privlist, char new_priv);
@@ -162,7 +162,6 @@ extern void ChangeAcl(char *relname, AclItem *mod_aip, unsigned modechg);
 extern AclId get_grosysid(char *groname);
 extern char *get_groname(AclId grosysid);
 
-/* XXX move these elsewhere -pma */
 extern int32 pg_aclcheck(char *relname, char *usename, AclMode mode);
 extern int32 pg_ownercheck(char *usename, char *value, int cacheid);
 extern int32
