@@ -15,7 +15,7 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: memutils.h,v 1.11 1997/09/08 20:59:22 momjian Exp $
+ * $Id: memutils.h,v 1.15 1998/04/12 02:58:22 scrappy Exp $
  *
  * NOTES
  *	  some of the information in this file will be moved to
@@ -67,7 +67,7 @@ s...)
  */
 #if defined(sun) && ! defined(sparc)
 #define LONGALIGN(LEN)	SHORTALIGN(LEN)
-#elif defined (alpha) || defined(linuxalpha)
+#elif defined (alpha)
 
  /*
   * even though "long alignment" should really be on 8-byte boundaries for
@@ -82,11 +82,16 @@ s...)
 		(((long)(LEN) + (sizeof (long) - 1)) & ~(sizeof (long) -1))
 #endif
 
+#if ! defined(sco)
 #define DOUBLEALIGN(LEN)\
 		(((long)(LEN) + (sizeof (double) - 1)) & ~(sizeof (double) -1))
 
 #define MAXALIGN(LEN)\
 		(((long)(LEN) + (sizeof (double) - 1)) & ~(sizeof (double) -1))
+#else
+#define DOUBLEALIGN(LEN) INTALIGN(LEN)
+#define MAXALIGN(LEN)	 INTALIGN(LEN)
+#endif
 
 /*****************************************************************************
  *	  oset.h --			Fixed format ordered set definitions.				 *
