@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-auth.c,v 1.11 1997/09/08 21:55:32 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-auth.c,v 1.16 1998/03/22 04:18:17 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -27,11 +27,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/param.h>			/* for MAXHOSTNAMELEN on most */
+#include <sys/types.h>
 #ifndef  MAXHOSTNAMELEN
 #include <netdb.h>				/* for MAXHOSTNAMELEN on some */
 #endif
 #include <unistd.h>
-#include <sys/types.h>
 #include <pwd.h>
 
 #include "postgres.h"
@@ -42,6 +42,11 @@
 #include "fe-auth.h"
 #include "fe-connect.h"
 
+#ifdef HAVE_CRYPT_H
+#include <crypt.h>
+#endif
+
+
 /*----------------------------------------------------------------
  * common definitions for generic fe/be routines
  *----------------------------------------------------------------
@@ -457,49 +462,49 @@ pg_krb5_sendauth(const char *PQerrormsg, int sock,
 #endif							/* KRB5 */
 
 static int
-pg_password_sendauth(Port *port, const char *user, const char *password)
+pg_password_sendauth(PGconn *conn, const char *password, AuthRequest areq)
 {
-	PacketBuf	buf;
-	char	   *tmp;
+	/* Encrypt the password if needed. */
 
-	buf.len = htonl(sizeof(PacketBuf));
-	buf.msgtype = STARTUP_PASSWORD_MSG;
-	buf.data[0] = '\0';
+	if (areq == AUTH_REQ_CRYPT)
+		password = crypt(password, conn->salt);
 
-	tmp = buf.data;
-	strncpy(tmp, user, strlen(user) + 1);
-	tmp += strlen(user) + 1;
-	strncpy(tmp, password, strlen(password) + 1);
-
-	return packetSend(port, &buf, sizeof(PacketBuf), BLOCKING);
+	return packetSend(conn, password, strlen(password) + 1);
 }
 
 /*
  * fe_sendauth -- client demux routine for outgoing authentication information
  */
 int
-fe_sendauth(MsgType msgtype, Port *port, const char *hostname,
-		  const char *user, const char *password, const char *PQerrormsg)
+fe_sendauth(AuthRequest areq, PGconn *conn, const char *hostname,
+			const char *password, char *PQerrormsg)
 {
-	switch (msgtype)
+	switch (areq)
 	{
+			case AUTH_REQ_OK:
+			break;
+
+		case AUTH_REQ_KRB4:
 #ifdef KRB4
-			case STARTUP_KRB4_MSG:
-			if (pg_krb4_sendauth(PQerrormsg, port->sock, &port->laddr,
-								 &port->raddr,
+			if (pg_krb4_sendauth(PQerrormsg, conn->sock, &conn->laddr.in,
+								 &conn->raddr.in,
 								 hostname) != STATUS_OK)
 			{
 				(void) sprintf(PQerrormsg,
 							"fe_sendauth: krb4 authentication failed\n");
-/*			fputs(PQerrormsg, stderr); */
 				return (STATUS_ERROR);
 			}
 			break;
+#else
+			(void) sprintf(PQerrormsg,
+					 "fe_sendauth: krb4 authentication not supported\n");
+			return (STATUS_ERROR);
 #endif
+
+		case AUTH_REQ_KRB5:
 #ifdef KRB5
-		case STARTUP_KRB5_MSG:
-			if (pg_krb5_sendauth(PQerrormsg, port->sock, &port->laddr,
-								 &port->raddr,
+			if (pg_krb5_sendauth(PQerrormsg, conn->sock, &conn->laddr.in,
+								 &conn->raddr.in,
 								 hostname) != STATUS_OK)
 			{
 				(void) sprintf(PQerrormsg,
@@ -507,14 +512,29 @@ fe_sendauth(MsgType msgtype, Port *port, const char *hostname,
 				return (STATUS_ERROR);
 			}
 			break;
+#else
+			(void) sprintf(PQerrormsg,
+					 "fe_sendauth: krb5 authentication not supported\n");
+			return (STATUS_ERROR);
 #endif
-		case STARTUP_MSG:
+
+		case AUTH_REQ_PASSWORD:
+		case AUTH_REQ_CRYPT:
+			if (pg_password_sendauth(conn, password, areq) != STATUS_OK)
+			{
+				(void) sprintf(PQerrormsg,
+				 "fe_sendauth: error sending password authentication\n");
+				return (STATUS_ERROR);
+			}
+
 			break;
-		case STARTUP_PASSWORD_MSG:
-			pg_password_sendauth(port, user, password);
+
 		default:
-			break;
+			(void) sprintf(PQerrormsg,
+			"fe_sendauth: authentication type %u not supported\n", areq);
+			return (STATUS_ERROR);
 	}
+
 	return (STATUS_OK);
 }
 
@@ -560,7 +580,7 @@ fe_getauthsvc(char *PQerrormsg)
  *					 name the user has authenticated to the system
  * if there is an error, return the error message in PQerrormsg
  */
-char	   *
+char *
 fe_getauthname(char *PQerrormsg)
 {
 	char	   *name = (char *) NULL;
