@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-connect.c,v 1.42 1997/09/18 20:22:46 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-connect.c,v 1.64 1998/03/26 23:46:14 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -21,14 +21,14 @@
 #include <ctype.h>
 #include <string.h>
 #include <netdb.h>
+#include <sys/un.h>
+#include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <errno.h>
 #include <signal.h>
 #include <ctype.h>				/* for isspace() */
 
 #include "postgres.h"
-#include "libpq/pqcomm.h"		/* for decls of MsgType, PacketBuf,
-								 * StartupInfo */
 #include "fe-auth.h"
 #include "fe-connect.h"
 #include "libpq-fe.h"
@@ -36,17 +36,20 @@
 #ifndef HAVE_STRDUP
 #include "strdup.h"
 #endif
+#ifdef HAVE_CRYPT_H
+#include <crypt.h>
+#endif
 
 
 /* use a local version instead of the one found in pqpacket.c */
 static ConnStatusType connectDB(PGconn *conn);
 
-static void startup2PacketBuf(StartupInfo *s, PacketBuf *res);
 static void freePGconn(PGconn *conn);
 static void closePGconn(PGconn *conn);
 static int	conninfo_parse(const char *conninfo, char *errorMessage);
 static char *conninfo_getval(char *keyword);
 static void conninfo_free(void);
+void		PQsetenv(PGconn *conn);
 
 #define NOTIFYLIST_INITIAL_SIZE 10
 #define NOTIFYLIST_GROWBY 10
@@ -74,6 +77,7 @@ static PQconninfoOption PQconninfoOptions[] = {
 /*	  Option-name		Environment-Var Compiled-in		Current value	*/
 /*						Label							Disp-Char		*/
 /* ----------------- --------------- --------------- --------------- */
+	/* "authtype" is ignored as it is no longer used. */
 	{"authtype", "PGAUTHTYPE", DefaultAuthtype, NULL,
 	"Database-Authtype", "", 20},
 
@@ -86,7 +90,7 @@ static PQconninfoOption PQconninfoOptions[] = {
 	{"dbname", "PGDATABASE", NULL, NULL,
 	"Database-Name", "", 20},
 
-	{"host", "PGHOST", DefaultHost, NULL,
+	{"host", "PGHOST", NULL, NULL,
 	"Database-Host", "", 40},
 
 	{"port", "PGPORT", DEF_PGPORT, NULL,
@@ -109,8 +113,26 @@ struct EnvironmentOptions
 }			EnvironmentOptions[] =
 
 {
+	/* common user-interface settings */
 	{
-		"PG_DATESTYLE", "datestyle"
+		"PGDATESTYLE", "datestyle"
+	},
+	{
+		"PGTZ", "timezone"
+	},
+
+	/* internal performance-related settings */
+	{
+		"PGCOSTHEAP", "cost_heap"
+	},
+	{
+		"PGCOSTINDEX", "cost_index"
+	},
+	{
+		"PGRPLANS", "r_plans"
+	},
+	{
+		"PGGEQO", "geqo"
 	},
 	{
 		NULL
@@ -120,7 +142,7 @@ struct EnvironmentOptions
 /* ----------------
  *		PQconnectdb
  *
- * establishes a connectin to a postgres backend through the postmaster
+ * establishes a connection to a postgres backend through the postmaster
  * using connection information in a string.
  *
  * The conninfo string is a list of
@@ -136,12 +158,12 @@ struct EnvironmentOptions
  * then some fields may be null'ed out instead of having valid values
  * ----------------
  */
-PGconn	   *
+PGconn *
 PQconnectdb(const char *conninfo)
 {
 	PGconn	   *conn;
-	PQconninfoOption *option;
 	char		errorMessage[ERROR_MSG_LENGTH];
+	char	   *tmp;
 
 	/* ----------
 	 * Allocate memory for the conn structure
@@ -168,37 +190,6 @@ PQconnectdb(const char *conninfo)
 		return conn;
 	}
 
-	/* ----------
-	 * Check that we have all connection parameters
-	 * ----------
-	 */
-	for (option = PQconninfoOptions; option->keyword != NULL; option++)
-	{
-		if (option->val != NULL)
-			continue;			/* Value was in conninfo */
-
-		/* ----------
-		 * No value was found for this option. Return an error.
-		 * ----------
-		 */
-		conn->status = CONNECTION_BAD;
-		sprintf(conn->errorMessage,
-				"ERROR: PQconnectdb(): Cannot determine a value for option '%s'.\n",
-				option->keyword);
-		strcat(conn->errorMessage,
-			   "Option not specified in conninfo string");
-		if (option->environ)
-		{
-			strcat(conn->errorMessage,
-				   ", environment variable ");
-			strcat(conn->errorMessage, option->environ);
-			strcat(conn->errorMessage, "\nnot set");
-		}
-		strcat(conn->errorMessage, " and no compiled in default value.\n");
-		conninfo_free();
-		return conn;
-	}
-
 	/* ----------
 	 * Setup the conn structure
 	 * ----------
@@ -207,17 +198,22 @@ PQconnectdb(const char *conninfo)
 	conn->Pfout = NULL;
 	conn->Pfin = NULL;
 	conn->Pfdebug = NULL;
-	conn->port = NULL;
 	conn->notifyList = DLNewList();
 
-	conn->pghost = strdup(conninfo_getval("host"));
-	conn->pgport = strdup(conninfo_getval("port"));
-	conn->pgtty = strdup(conninfo_getval("tty"));
-	conn->pgoptions = strdup(conninfo_getval("options"));
-	conn->pguser = strdup(conninfo_getval("user"));
-	conn->pgpass = strdup(conninfo_getval("password"));
-	conn->pgauth = strdup(conninfo_getval("authtype"));
-	conn->dbName = strdup(conninfo_getval("dbname"));
+	tmp = conninfo_getval("host");
+	conn->pghost = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("port");
+	conn->pgport = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("tty");
+	conn->pgtty = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("options");
+	conn->pgoptions = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("user");
+	conn->pguser = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("password");
+	conn->pgpass = tmp ? strdup(tmp) : NULL;
+	tmp = conninfo_getval("dbname");
+	conn->dbName = tmp ? strdup(tmp) : NULL;
 
 	/* ----------
 	 * Free the connection info - all is in conn now
@@ -225,14 +221,6 @@ PQconnectdb(const char *conninfo)
 	 */
 	conninfo_free();
 
-	/*
-	 * try to set the auth service if one was specified
-	 */
-	if (conn->pgauth)
-	{
-		fe_setauthsvc(conn->pgauth, conn->errorMessage);
-	}
-
 	/* ----------
 	 * Connect to the database
 	 * ----------
@@ -255,6 +243,8 @@ PQconnectdb(const char *conninfo)
 		PQclear(res);
 	}
 
+	PQsetenv(conn);
+
 	return conn;
 }
 
@@ -277,7 +267,7 @@ PQconndefaults(void)
 }
 
 /* ----------------
- *		PQsetdb
+ *		PQsetdbLogin
  *
  * establishes a connection to a postgres backend through the postmaster
  * at the specified host and port.
@@ -309,10 +299,15 @@ PQconndefaults(void)
  *
  *	  None of the above need be defined.  There are defaults for all of them.
  *
+ * To support "delimited identifiers" for database names, only convert
+ * the database name to lower case if it is not surrounded by double quotes.
+ * Otherwise, strip the double quotes but leave the reset of the string intact.
+ * - thomas 1997-11-08
+ *
  * ----------------
  */
-PGconn	   *
-PQsetdb(const char *pghost, const char *pgport, const char *pgoptions, const char *pgtty, const char *dbName)
+PGconn *
+PQsetdbLogin(const char *pghost, const char *pgport, const char *pgoptions, const char *pgtty, const char *dbName, const char *login, const char *pwd)
 {
 	PGconn	   *conn;
 	char	   *tmp;
@@ -335,54 +330,50 @@ PQsetdb(const char *pghost, const char *pgport, const char *pgoptions, const cha
 		conn->Pfout = NULL;
 		conn->Pfin = NULL;
 		conn->Pfdebug = NULL;
-		conn->port = NULL;
 		conn->notifyList = DLNewList();
 
-		if (!pghost || pghost[0] == '\0')
+		if ((pghost == NULL) || pghost[0] == '\0')
 		{
-			if (!(tmp = getenv("PGHOST")))
-			{
-				tmp = DefaultHost;
-			}
-			conn->pghost = strdup(tmp);
+			conn->pghost = NULL;
+			if ((tmp = getenv("PGHOST")) != NULL)
+				conn->pghost = strdup(tmp);
 		}
 		else
 			conn->pghost = strdup(pghost);
 
-		if (!pgport || pgport[0] == '\0')
+		if ((pgport == NULL) || pgport[0] == '\0')
 		{
-			if (!(tmp = getenv("PGPORT")))
-			{
+			if ((tmp = getenv("PGPORT")) == NULL)
 				tmp = DEF_PGPORT;
-			}
 			conn->pgport = strdup(tmp);
 		}
 		else
 			conn->pgport = strdup(pgport);
 
-		if (!pgtty || pgtty[0] == '\0')
+		if ((pgtty == NULL) || pgtty[0] == '\0')
 		{
-			if (!(tmp = getenv("PGTTY")))
-			{
+			if ((tmp = getenv("PGTTY")) == NULL)
 				tmp = DefaultTty;
-			}
 			conn->pgtty = strdup(tmp);
 		}
 		else
 			conn->pgtty = strdup(pgtty);
 
-		if (!pgoptions || pgoptions[0] == '\0')
+		if ((pgoptions == NULL) || pgoptions[0] == '\0')
 		{
-			if (!(tmp = getenv("PGOPTIONS")))
-			{
+			if ((tmp = getenv("PGOPTIONS")) == NULL)
 				tmp = DefaultOption;
-			}
 			conn->pgoptions = strdup(tmp);
 		}
 		else
 			conn->pgoptions = strdup(pgoptions);
 
-		if ((tmp = getenv("PGUSER")))
+		if (login)
+		{
+			error = FALSE;
+			conn->pguser = strdup(login);
+		}
+		else if ((tmp = getenv("PGUSER")) != NULL)
 		{
 			error = FALSE;
 			conn->pguser = strdup(tmp);
@@ -403,27 +394,38 @@ PQsetdb(const char *pghost, const char *pgport, const char *pgoptions, const cha
 			}
 		}
 
-		if ((tmp = getenv("PGPASSWORD")))
+		if (pwd)
+		{
+			conn->pgpass = strdup(pwd);
+		}
+		else if ((tmp = getenv("PGPASSWORD")) != NULL)
 		{
 			conn->pgpass = strdup(tmp);
 		}
 		else
-		{
-			conn->pgpass = 0;
-		}
+			conn->pgpass = strdup(DefaultPassword);
 
 		if (!error)
 		{
-			if (((tmp = (char *) dbName) && (dbName[0] != '\0')) ||
-				((tmp = getenv("PGDATABASE"))))
-			{
+			if ((((tmp = (char *) dbName) != NULL) && (dbName[0] != '\0'))
+				|| ((tmp = getenv("PGDATABASE"))))
 				conn->dbName = strdup(tmp);
-			}
 			else
 				conn->dbName = strdup(conn->pguser);
-			for (i = 0; conn->dbName[i]; i++)
-				if (isupper(conn->dbName[i]))
-					conn->dbName[i] = tolower(conn->dbName[i]);
+
+			/*
+			 * if the database name is surrounded by double-quotes, then
+			 * don't convert case
+			 */
+			if (*conn->dbName == '"')
+			{
+				strcpy(conn->dbName, conn->dbName + 1);
+				*(conn->dbName + strlen(conn->dbName) - 1) = '\0';
+			}
+			else
+				for (i = 0; conn->dbName[i]; i++)
+					if (isupper(conn->dbName[i]))
+						conn->dbName[i] = tolower(conn->dbName[i]);
 		}
 		else
 			conn->dbName = NULL;
@@ -450,6 +452,7 @@ PQsetdb(const char *pghost, const char *pgport, const char *pgoptions, const cha
 				}
 				PQclear(res);
 			}
+			PQsetenv(conn);
 		}
 	}
 	return conn;
@@ -467,71 +470,82 @@ connectDB(PGconn *conn)
 {
 	struct hostent *hp;
 
-	StartupInfo startup;
-	PacketBuf	pacBuf;
-	int			status;
-	MsgType		msgtype;
-	int			laddrlen = sizeof(struct sockaddr);
-	Port	   *port = conn->port;
-	int			portno;
+	StartupPacket sp;
+	AuthRequest areq;
+	int			laddrlen = sizeof(SockAddr);
+	int			portno,
+				family,
+				len;
 
 	/*
 	 * Initialize the startup packet.
-	 *
-	 * This data structure is used for the seq-packet protocol.  It describes
-	 * the frontend-backend connection.
-	 *
-	 *
 	 */
-	strncpy(startup.user, conn->pguser, sizeof(startup.user));
-	strncpy(startup.database, conn->dbName, sizeof(startup.database));
-	strncpy(startup.tty, conn->pgtty, sizeof(startup.tty));
+
+	MemSet((char *) &sp, 0, sizeof(StartupPacket));
+
+	sp.protoVersion = (ProtocolVersion) htonl(PG_PROTOCOL_LATEST);
+
+	strncpy(sp.user, conn->pguser, SM_USER);
+	strncpy(sp.database, conn->dbName, SM_DATABASE);
+	strncpy(sp.tty, conn->pgtty, SM_TTY);
+
 	if (conn->pgoptions)
-	{
-		strncpy(startup.options, conn->pgoptions, sizeof(startup.options));
-	}
-	else
-		startup.options[0] = '\0';
-	startup.execFile[0] = '\0'; /* not used */
+		strncpy(sp.options, conn->pgoptions, SM_OPTIONS);
 
 	/*
 	 * Open a connection to postmaster/backend.
-	 *
 	 */
-	port = (Port *) malloc(sizeof(Port));
-	MemSet((char *) port, 0, sizeof(Port));
 
-	if (!(hp = gethostbyname(conn->pghost)) || hp->h_addrtype != AF_INET)
+	if (conn->pghost != NULL)
 	{
-		(void) sprintf(conn->errorMessage,
-					   "connectDB() --  unknown hostname: %s\n",
-					   conn->pghost);
-		goto connect_errReturn;
+		hp = gethostbyname(conn->pghost);
+		if ((hp == NULL) || (hp->h_addrtype != AF_INET))
+		{
+			(void) sprintf(conn->errorMessage,
+						   "connectDB() --  unknown hostname: %s\n",
+						   conn->pghost);
+			goto connect_errReturn;
+		}
 	}
-	MemSet((char *) &port->raddr, 0, sizeof(port->raddr));
-	memmove((char *) &(port->raddr.sin_addr),
-			(char *) hp->h_addr,
-			hp->h_length);
-	port->raddr.sin_family = AF_INET;
-	portno = atoi(conn->pgport);
-	port->raddr.sin_port = htons((unsigned short) (portno));
+	else
+		hp = NULL;
 
+#if FALSE
+	MemSet((char *) &port->raddr, 0, sizeof(port->raddr));
+#endif
+	portno = atoi(conn->pgport);
+	family = (conn->pghost != NULL) ? AF_INET : AF_UNIX;
+	conn->raddr.sa.sa_family = family;
+	if (family == AF_INET)
+	{
+		memmove((char *) &(conn->raddr.in.sin_addr),
+				(char *) hp->h_addr,
+				hp->h_length);
+		conn->raddr.in.sin_port = htons((unsigned short) (portno));
+		len = sizeof(struct sockaddr_in);
+	}
+	else
+	{
+		len = UNIXSOCK_PATH(conn->raddr.un, portno);
+	}
 	/* connect to the server  */
-	if ((port->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+	if ((conn->sock = socket(family, SOCK_STREAM, 0)) < 0)
 	{
 		(void) sprintf(conn->errorMessage,
 					   "connectDB() -- socket() failed: errno=%d\n%s\n",
 					   errno, strerror(errno));
 		goto connect_errReturn;
 	}
-	if (connect(port->sock, (struct sockaddr *) & port->raddr,
-				sizeof(port->raddr)) < 0)
+	if (connect(conn->sock, &conn->raddr.sa, len) < 0)
 	{
 		(void) sprintf(conn->errorMessage,
-					   "connectDB() failed: Is the postmaster running at '%s' on port '%s'?\n",
-					   conn->pghost, conn->pgport);
+					   "connectDB() failed: Is the postmaster running and accepting%s connections at '%s' on port '%s'?\n",
+					   conn->pghost ? " TCP/IP(with -i)" : "",
+					   conn->pghost ? conn->pghost : "UNIX Socket",
+					   conn->pgport);
 		goto connect_errReturn;
 	}
+	if (family == AF_INET)
 	{
 		struct protoent *pe;
 		int			on = 1;
@@ -543,7 +557,7 @@ connectDB(PGconn *conn)
 						   "connectDB(): getprotobyname failed\n");
 			goto connect_errReturn;
 		}
-		if (setsockopt(port->sock, pe->p_proto, TCP_NODELAY,
+		if (setsockopt(conn->sock, pe->p_proto, TCP_NODELAY,
 					   &on, sizeof(on)) < 0)
 		{
 			(void) sprintf(conn->errorMessage,
@@ -553,8 +567,7 @@ connectDB(PGconn *conn)
 	}
 
 	/* fill in the client address */
-	if (getsockname(port->sock, (struct sockaddr *) & port->laddr,
-					&laddrlen) < 0)
+	if (getsockname(conn->sock, &conn->laddr.sa, &laddrlen) < 0)
 	{
 		(void) sprintf(conn->errorMessage,
 				   "connectDB() -- getsockname() failed: errno=%d\n%s\n",
@@ -562,42 +575,10 @@ connectDB(PGconn *conn)
 		goto connect_errReturn;
 	}
 
-	/* by this point, connection has been opened */
-	msgtype = fe_getauthsvc(conn->errorMessage);
-
-/*	  pacBuf = startup2PacketBuf(&startup);*/
-	startup2PacketBuf(&startup, &pacBuf);
-	pacBuf.msgtype = (MsgType) htonl(msgtype);
-	status = packetSend(port, &pacBuf, sizeof(PacketBuf), BLOCKING);
-
-	if (status == STATUS_ERROR)
-	{
-		sprintf(conn->errorMessage,
-				"connectDB() --  couldn't send complete packet: errno=%d\n%s\n", errno, strerror(errno));
-		goto connect_errReturn;
-	}
-
-	/* authenticate as required */
-	if (fe_sendauth(msgtype, port, conn->pghost,
-					conn->pguser, conn->pgpass,
-					conn->errorMessage) != STATUS_OK)
-	{
-		(void) sprintf(conn->errorMessage,
-					   "connectDB() --  authentication failed with %s\n",
-					   conn->pghost);
-		goto connect_errReturn;
-	}
-
-	/* free the password so it's not hanging out in memory forever */
-	if (conn->pgpass)
-	{
-		free(conn->pgpass);
-	}
-
 	/* set up the socket file descriptors */
-	conn->Pfout = fdopen(port->sock, "w");
-	conn->Pfin = fdopen(dup(port->sock), "r");
-	if (!conn->Pfout || !conn->Pfin)
+	conn->Pfout = fdopen(conn->sock, "w");
+	conn->Pfin = fdopen(dup(conn->sock), "r");
+	if ((conn->Pfout == NULL) || (conn->Pfin == NULL))
 	{
 		(void) sprintf(conn->errorMessage,
 					   "connectDB() -- fdopen() failed: errno=%d\n%s\n",
@@ -605,41 +586,124 @@ connectDB(PGconn *conn)
 		goto connect_errReturn;
 	}
 
-	conn->port = port;
+	/* Send the startup packet. */
 
+	if (packetSend(conn, (char *) &sp, sizeof(StartupPacket)) != STATUS_OK)
 	{
-		struct EnvironmentOptions *eo;
-		char		setQuery[80];		/* mjl: size okay? XXX */
-
-		for (eo = EnvironmentOptions; eo->envName; eo++)
-		{
-			const char *val;
-
-			if ((val = getenv(eo->envName)))
-			{
-				PGresult   *res;
-
-				sprintf(setQuery, "SET %s TO '%.60s'", eo->pgName, val);
-				res = PQexec(conn, setQuery);
-				PQclear(res);	/* Don't care? */
-			}
-		}
+		sprintf(conn->errorMessage,
+				"connectDB() --  couldn't send complete packet: errno=%d\n%s\n", errno, strerror(errno));
+		goto connect_errReturn;
 	}
+
+	/*
+	 * Get the response from the backend, either an error message or an
+	 * authentication request.
+	 */
+
+	do
+	{
+		int			beresp;
+
+		if ((beresp = pqGetc(conn->Pfin, conn->Pfdebug)) == EOF)
+		{
+			(void) sprintf(conn->errorMessage,
+				"connectDB() -- error getting authentication request\n");
+
+			goto connect_errReturn;
+		}
+
+		/* Handle errors. */
+
+		if (beresp == 'E')
+		{
+			pqGets(conn->errorMessage, sizeof(conn->errorMessage),
+				   conn->Pfin, conn->Pfdebug);
+
+			goto connect_errReturn;
+		}
+
+		/* Check it was an authentication request. */
+
+		if (beresp != 'R')
+		{
+			(void) sprintf(conn->errorMessage,
+					 "connectDB() -- expected authentication request\n");
+
+			goto connect_errReturn;
+		}
+
+		/* Get the type of request. */
+
+		if (pqGetInt((int *) &areq, 4, conn->Pfin, conn->Pfdebug))
+		{
+			(void) sprintf(conn->errorMessage,
+			"connectDB() -- error getting authentication request type\n");
+
+			goto connect_errReturn;
+		}
+
+		/* Get the password salt if there is one. */
+
+		if (areq == AUTH_REQ_CRYPT &&
+			pqGetnchar(conn->salt, sizeof(conn->salt),
+					   conn->Pfin, conn->Pfdebug))
+		{
+			(void) sprintf(conn->errorMessage,
+						 "connectDB() -- error getting password salt\n");
+
+			goto connect_errReturn;
+		}
+
+
+		/* Respond to the request. */
+
+		if (fe_sendauth(areq, conn, conn->pghost, conn->pgpass,
+						conn->errorMessage) != STATUS_OK)
+			goto connect_errReturn;
+	}
+	while (areq != AUTH_REQ_OK);
+
+	/* free the password so it's not hanging out in memory forever */
+	if (conn->pgpass != NULL)
+	{
+		free(conn->pgpass);
+		conn->pgpass = NULL;
+	}
+
 	return CONNECTION_OK;
 
 connect_errReturn:
-
-	/*
-	 * Igor/6/3/97 - We need to free it here...otherwise the function
-	 * returns without setting conn->port to port. Because of that any way
-	 * of referencing this variable will be lost and it's allocated memory
-	 * will not be freed.
-	 */
-	free(port);					/* PURIFY */
 	return CONNECTION_BAD;
 
 }
 
+void
+PQsetenv(PGconn *conn)
+{
+	struct EnvironmentOptions *eo;
+	char		setQuery[80];	/* mjl: size okay? XXX */
+
+	for (eo = EnvironmentOptions; eo->envName; eo++)
+	{
+		const char *val;
+
+		if ((val = getenv(eo->envName)))
+		{
+			PGresult   *res;
+
+			if (strcasecmp(val, "default") == 0)
+				sprintf(setQuery, "SET %s = %.60s", eo->pgName, val);
+			else
+				sprintf(setQuery, "SET %s = '%.60s'", eo->pgName, val);
+#ifdef CONNECTDEBUG
+			printf("Use environment variable %s to send %s\n", eo->envName, setQuery);
+#endif
+			res = PQexec(conn, setQuery);
+			PQclear(res);		/* Don't care? */
+		}
+	}
+}	/* PQsetenv() */
+
 /*
  * freePGconn
  *	 - free the PGconn data structure
@@ -664,8 +728,6 @@ freePGconn(PGconn *conn)
 		free(conn->pguser);
 	if (conn->notifyList)
 		DLFreeList(conn->notifyList);
-	if (conn->port)
-		free(conn->port);
 	free(conn);
 }
 
@@ -763,62 +825,25 @@ PQreset(PGconn *conn)
  *
  * RETURNS: STATUS_ERROR if the write fails, STATUS_OK otherwise.
  * SIDE_EFFECTS: may block.
- * NOTES: Non-blocking writes would significantly complicate
- *		buffer management.	For now, we're not going to do it.
- *
 */
 int
-packetSend(Port *port,
-		   PacketBuf *buf,
-		   PacketLen len,
-		   bool nonBlocking)
+packetSend(PGconn *conn, const char *buf, size_t len)
 {
-	PacketLen	totalLen;
-	int			addrLen = sizeof(struct sockaddr_in);
+	/* Send the total packet size. */
 
-	totalLen = len;
+	if (pqPutInt(4 + len, 4, conn->Pfout, conn->Pfdebug))
+		return STATUS_ERROR;
 
-	len = sendto(port->sock, (Addr) buf, totalLen, /* flags */ 0,
-				 (struct sockaddr *) & (port->raddr), addrLen);
+	/* Send the packet itself. */
 
-	if (len < totalLen)
-	{
-		return (STATUS_ERROR);
-	}
+	if (pqPutnchar(buf, len, conn->Pfout, conn->Pfdebug))
+		return STATUS_ERROR;
 
-	return (STATUS_OK);
+	pqFlush(conn->Pfout, conn->Pfdebug);
+
+	return STATUS_OK;
 }
 
-/*
- * startup2PacketBuf()
- *
- * this is just like StartupInfo2Packet(), defined in backend/libpq/pqpacket.c
- * but we repeat it here so we don't have to link in libpq.a
- *
- * converts a StartupInfo structure to a PacketBuf
- */
-static void
-startup2PacketBuf(StartupInfo *s, PacketBuf *res)
-{
-	char	   *tmp;
-
-/*	res = (PacketBuf*)malloc(sizeof(PacketBuf)); */
-	res->len = htonl(sizeof(PacketBuf));
-	/* use \n to delimit the strings */
-	res->data[0] = '\0';
-
-	tmp = res->data;
-
-	strncpy(tmp, s->database, sizeof(s->database));
-	tmp += sizeof(s->database);
-	strncpy(tmp, s->user, sizeof(s->user));
-	tmp += sizeof(s->user);
-	strncpy(tmp, s->options, sizeof(s->options));
-	tmp += sizeof(s->options);
-	strncpy(tmp, s->execFile, sizeof(s->execFile));
-	tmp += sizeof(s->execFile);
-	strncpy(tmp, s->tty, sizeof(s->tty));
-}
 
 /* ----------------
  * Conninfo parser routine
@@ -1087,7 +1112,7 @@ conninfo_free()
 }
 
 /* =========== accessor functions for PGconn ========= */
-char	   *
+char *
 PQdb(PGconn *conn)
 {
 	if (!conn)
@@ -1098,7 +1123,7 @@ PQdb(PGconn *conn)
 	return conn->dbName;
 }
 
-char	   *
+char *
 PQuser(PGconn *conn)
 {
 	if (!conn)
@@ -1109,7 +1134,7 @@ PQuser(PGconn *conn)
 	return conn->pguser;
 }
 
-char	   *
+char *
 PQhost(PGconn *conn)
 {
 	if (!conn)
@@ -1121,7 +1146,7 @@ PQhost(PGconn *conn)
 	return conn->pghost;
 }
 
-char	   *
+char *
 PQoptions(PGconn *conn)
 {
 	if (!conn)
@@ -1132,7 +1157,7 @@ PQoptions(PGconn *conn)
 	return conn->pgoptions;
 }
 
-char	   *
+char *
 PQtty(PGconn *conn)
 {
 	if (!conn)
@@ -1143,7 +1168,7 @@ PQtty(PGconn *conn)
 	return conn->pgtty;
 }
 
-char	   *
+char *
 PQport(PGconn *conn)
 {
 	if (!conn)
@@ -1165,7 +1190,7 @@ PQstatus(PGconn *conn)
 	return conn->status;
 }
 
-char	   *
+char *
 PQerrorMessage(PGconn *conn)
 {
 	if (!conn)
@@ -1199,7 +1224,6 @@ PQuntrace(PGconn *conn)
 	if (conn->Pfdebug)
 	{
 		fflush(conn->Pfdebug);
-		fclose(conn->Pfdebug);
 		conn->Pfdebug = NULL;
 	}
 }
