@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-exec.c,v 1.38 1997/09/08 21:55:41 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-exec.c,v 1.48 1998/03/15 08:11:11 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,6 +17,7 @@
 #include <signal.h>
 #include <string.h>
 #include <errno.h>
+#include <ctype.h>
 #include "postgres.h"
 #include "libpq/pqcomm.h"
 #include "libpq/pqsignal.h"
@@ -50,6 +51,8 @@ const char *pgresStatus[] = {
 	"PGRES_EMPTY_QUERY",
 	"PGRES_COMMAND_OK",
 	"PGRES_TUPLES_OK",
+	"PGRES_COPY_OUT", 
+	"PGRES_COPY_IN",
 	"PGRES_BAD_RESPONSE",
 	"PGRES_NONFATAL_ERROR",
 	"PGRES_FATAL_ERROR"
@@ -167,7 +170,7 @@ getTuple(PGconn *conn, PGresult *result, int binary)
 	if ((nfields % BYTELEN) > 0)
 		nbytes++;
 
-	if (pqGetnchar(bitmap, nbytes, pfin, pfdebug) == 1)
+	if (nbytes >= MAX_FIELDS || pqGetnchar(bitmap, nbytes, pfin, pfdebug) == 1)
 	{
 		sprintf(conn->errorMessage,
 			  "Error reading null-values bitmap from row data stream\n");
@@ -188,10 +191,10 @@ getTuple(PGconn *conn, PGresult *result, int binary)
 		else
 		{
 			/* get the value length (the first four bytes are for length) */
-			pqGetInt(&vlen, VARHDRSZ, pfin, pfdebug);
+			pqGetInt(&vlen, 4, pfin, pfdebug);
 			if (binary == 0)
 			{
-				vlen = vlen - VARHDRSZ;
+				vlen = vlen - 4;
 			}
 			if (vlen < 0)
 				vlen = 0;
@@ -455,12 +458,10 @@ process_response_from_backend(FILE *pfin, FILE *pfout, FILE *pfdebug,
 		if (id == EOF)
 		{
 			/* hmm,  no response from the backend-end, that's bad */
-			(void) sprintf(reason,
-				  "PQexec() -- Request was sent to backend, but backend "
-						   "closed the channel before "
-						   "responding.  This probably means the backend "
-					  "terminated abnormally before or while processing "
-						   "the request.\n");
+			(void) sprintf(reason, "PQexec() -- Request was sent to backend"
+					", but backend closed the channel before responding."
+			  "\n\tThis probably means the backend terminated abnormally"
+						   " before or while processing the request.\n");
 			conn->status = CONNECTION_BAD;		/* No more connection to
 												 * backend */
 			*result_p = (PGresult *) NULL;
@@ -1645,7 +1646,7 @@ PQnfields(PGresult *res)
 /*
    returns NULL if the field_num is invalid
 */
-char	   *
+char *
 PQfname(PGresult *res, int field_num)
 {
 	if (!res)
@@ -1676,6 +1677,7 @@ int
 PQfnumber(PGresult *res, const char *field_name)
 {
 	int			i;
+	char	   *field_case;
 
 	if (!res)
 	{
@@ -1688,13 +1690,27 @@ PQfnumber(PGresult *res, const char *field_name)
 		res->attDescs == NULL)
 		return -1;
 
+	field_case = strdup(field_name);
+	if (*field_case == '"')
+	{
+		strcpy(field_case, field_case + 1);
+		*(field_case + strlen(field_case) - 1) = '\0';
+	}
+	else
+		for (i = 0; field_case[i]; i++)
+			if (isupper(field_case[i]))
+				field_case[i] = tolower(field_case[i]);
+
 	for (i = 0; i < res->numAttributes; i++)
 	{
-		if (strcasecmp(field_name, res->attDescs[i].name) == 0)
+		if (strcmp(field_name, res->attDescs[i].name) == 0)
+		{
+			free(field_case);
 			return i;
+		}
 	}
+	free(field_case);
 	return -1;
-
 }
 
 Oid
@@ -1743,7 +1759,7 @@ PQfsize(PGresult *res, int field_num)
 		return 0;
 }
 
-char	   *
+char *
 PQcmdStatus(PGresult *res)
 {
 	if (!res)
@@ -1840,7 +1856,7 @@ PQcmdTuples(PGresult *res)
 
 	if res is not binary, a null-terminated ASCII string is returned.
 */
-char	   *
+char *
 PQgetvalue(PGresult *res, int tup_num, int field_num)
 {
 	if (!res)
