@@ -1,5 +1,5 @@
 /*
- * $Header: /usr/local/cvsroot/pgsql/src/test/regress/regress.c,v 1.16 1997/09/24 08:35:10 vadim Exp $
+ * $Header: /usr/local/cvsroot/pgsql/src/test/regress/regress.c,v 1.25 1998/04/07 18:13:08 momjian Exp $
  */
 
 #include <float.h>				/* faked on sunos */
@@ -30,7 +30,7 @@ extern char *reverse_c16(char *string);
 /*
 ** Distance from a point to a path
 */
-double	   *
+double *
 regress_dist_ptpath(pt, path)
 Point	   *pt;
 PATH	   *path;
@@ -43,7 +43,7 @@ PATH	   *path;
 	switch (path->npts)
 	{
 		case 0:
-			result = PALLOCTYPE(double);
+			result = palloc(sizeof(double));
 			*result = Abs((double) DBL_MAX);	/* +infinity */
 			break;
 		case 1:
@@ -56,14 +56,14 @@ PATH	   *path;
 			 * distance from the point to any of its constituent segments.
 			 */
 			Assert(path->npts > 1);
-			result = PALLOCTYPE(double);
+			result = palloc(sizeof(double));
 			for (i = 0; i < path->npts - 1; ++i)
 			{
 				regress_lseg_construct(&lseg, &path->p[i], &path->p[i + 1]);
 				tmp = dist_ps(pt, &lseg);
 				if (i == 0 || *tmp < *result)
 					*result = *tmp;
-				PFREE(tmp);
+				pfree(tmp);
 
 			}
 			break;
@@ -73,7 +73,7 @@ PATH	   *path;
 
 /* this essentially does a cartesian product of the lsegs in the
    two paths, and finds the min distance between any two lsegs */
-double	   *
+double *
 regress_path_dist(p1, p2)
 PATH	   *p1;
 PATH	   *p2;
@@ -97,18 +97,18 @@ PATH	   *p2;
 
 			if (*min < *(tmp = lseg_distance(&seg1, &seg2)))
 				*min = *tmp;
-			PFREE(tmp);
+			pfree(tmp);
 		}
 
 	return (min);
 }
 
-PATH	   *
+PATH *
 poly2path(poly)
 POLYGON    *poly;
 {
 	int			i;
-	char	   *output = (char *) PALLOC(2 * (P_MAXDIG + 1) * poly->npts + 64);
+	char	   *output = (char *) palloc(2 * (P_MAXDIG + 1) * poly->npts + 64);
 	char		buf[2 * (P_MAXDIG) + 20];
 
 	sprintf(output, "(1, %*d", P_MAXDIG, poly->npts);
@@ -125,7 +125,7 @@ POLYGON    *poly;
 }
 
 /* return the point where two paths intersect.	Assumes that they do. */
-Point	   *
+Point *
 interpt_pp(p1, p2)
 PATH	   *p1;
 PATH	   *p2;
@@ -207,7 +207,7 @@ int			pt_in_widget(Point *point, WIDGET * widget);
 
 #define NARGS	3
 
-WIDGET	   *
+WIDGET *
 widget_in(str)
 char	   *str;
 {
@@ -234,7 +234,7 @@ char	   *str;
 	return (result);
 }
 
-char	   *
+char *
 widget_out(widget)
 WIDGET	   *widget;
 {
@@ -275,11 +275,11 @@ BOX		   *box;
 	return (width * height);
 }
 
-char	   *
+char *
 reverse_c16(string)
 char	   *string;
 {
-	register	i;
+	int			i;
 	int			len;
 	char	   *new_string;
 
@@ -372,7 +372,7 @@ funny_dup17()
 			SPI_gettype(tupdesc, 1));
 
 	if ((ret = SPI_exec(sql, 0)) < 0)
-		elog(WARN, "funny_dup17 (fired %s) on level %3d: SPI_exec (insert ...) returned %d",
+		elog(ERROR, "funny_dup17 (fired %s) on level %3d: SPI_exec (insert ...) returned %d",
 			 when, *level, ret);
 
 	inserted = SPI_processed;
@@ -384,7 +384,7 @@ funny_dup17()
 			SPI_gettype(tupdesc, 1));
 
 	if ((ret = SPI_exec(sql, 0)) < 0)
-		elog(WARN, "funny_dup17 (fired %s) on level %3d: SPI_exec (select ...) returned %d",
+		elog(ERROR, "funny_dup17 (fired %s) on level %3d: SPI_exec (select ...) returned %d",
 			 when, *level, ret);
 
 	if (SPI_processed > 0)
@@ -412,17 +412,15 @@ funny_dup17()
 	return (tuple);
 }
 
-#include <ctype.h>				/* tolower () */
+HeapTuple	ttdummy(void);
+int32		set_ttdummy(int32 on);
 
-HeapTuple		ttdummy(void);
-int32			set_ttdummy(int32 on);
-
-extern int4	nextval(struct varlena * seqin);
+extern int4 nextval(struct varlena * seqin);
 
 #define TTDUMMY_INFINITY	999999
 
-static void    *splan = NULL;
-static bool		ttoff = false;
+static void *splan = NULL;
+static bool ttoff = false;
 
 HeapTuple
 ttdummy()
@@ -430,8 +428,10 @@ ttdummy()
 	Trigger    *trigger;		/* to get trigger name */
 	char	  **args;			/* arguments */
 	int			attnum[2];		/* fnumbers of start/stop columns */
-	Datum		oldon, oldoff;
-	Datum		newon, newoff;
+	Datum		oldon,
+				oldoff;
+	Datum		newon,
+				newoff;
 	Datum	   *cvals;			/* column values */
 	char	   *cnulls;			/* column nulls */
 	char	   *relname;		/* triggered relation name */
@@ -446,172 +446,173 @@ ttdummy()
 	int			i;
 
 	if (!CurrentTriggerData)
-		elog(WARN, "ttdummy: triggers are not initialized");
+		elog(ERROR, "ttdummy: triggers are not initialized");
 	if (TRIGGER_FIRED_FOR_STATEMENT(CurrentTriggerData->tg_event))
-		elog(WARN, "ttdummy: can't process STATEMENT events");
+		elog(ERROR, "ttdummy: can't process STATEMENT events");
 	if (TRIGGER_FIRED_AFTER(CurrentTriggerData->tg_event))
-		elog(WARN, "ttdummy: must be fired before event");
+		elog(ERROR, "ttdummy: must be fired before event");
 	if (TRIGGER_FIRED_BY_INSERT(CurrentTriggerData->tg_event))
-		elog (WARN, "ttdummy: can't process INSERT event");
+		elog(ERROR, "ttdummy: can't process INSERT event");
 	if (TRIGGER_FIRED_BY_UPDATE(CurrentTriggerData->tg_event))
 		newtuple = CurrentTriggerData->tg_newtuple;
-	
+
 	trigtuple = CurrentTriggerData->tg_trigtuple;
-	
+
 	rel = CurrentTriggerData->tg_relation;
 	relname = SPI_getrelname(rel);
-	
+
 	/* check if TT is OFF for this relation */
-	if (ttoff)				/* OFF - nothing to do */
+	if (ttoff)					/* OFF - nothing to do */
 	{
-		pfree (relname);
+		pfree(relname);
 		return ((newtuple != NULL) ? newtuple : trigtuple);
 	}
-	
+
 	trigger = CurrentTriggerData->tg_trigger;
 
 	if (trigger->tgnargs != 2)
-		elog(WARN, "ttdummy (%s): invalid (!= 2) number of arguments %d", 
-				relname, trigger->tgnargs);
-	
+		elog(ERROR, "ttdummy (%s): invalid (!= 2) number of arguments %d",
+			 relname, trigger->tgnargs);
+
 	args = trigger->tgargs;
 	tupdesc = rel->rd_att;
 	natts = tupdesc->natts;
-	
+
 	CurrentTriggerData = NULL;
-	
-	for (i = 0; i < 2; i++ )
+
+	for (i = 0; i < 2; i++)
 	{
-		attnum[i] = SPI_fnumber (tupdesc, args[i]);
-		if ( attnum[i] < 0 )
-			elog(WARN, "ttdummy (%s): there is no attribute %s", relname, args[i]);
-		if (SPI_gettypeid (tupdesc, attnum[i]) != INT4OID)
-			elog(WARN, "ttdummy (%s): attributes %s and %s must be of abstime type", 
-					relname, args[0], args[1]);
+		attnum[i] = SPI_fnumber(tupdesc, args[i]);
+		if (attnum[i] < 0)
+			elog(ERROR, "ttdummy (%s): there is no attribute %s", relname, args[i]);
+		if (SPI_gettypeid(tupdesc, attnum[i]) != INT4OID)
+			elog(ERROR, "ttdummy (%s): attributes %s and %s must be of abstime type",
+				 relname, args[0], args[1]);
 	}
-	
-	oldon = SPI_getbinval (trigtuple, tupdesc, attnum[0], &isnull);
+
+	oldon = SPI_getbinval(trigtuple, tupdesc, attnum[0], &isnull);
 	if (isnull)
-		elog(WARN, "ttdummy (%s): %s must be NOT NULL", relname, args[0]);
-	
-	oldoff = SPI_getbinval (trigtuple, tupdesc, attnum[1], &isnull);
+		elog(ERROR, "ttdummy (%s): %s must be NOT NULL", relname, args[0]);
+
+	oldoff = SPI_getbinval(trigtuple, tupdesc, attnum[1], &isnull);
 	if (isnull)
-		elog(WARN, "ttdummy (%s): %s must be NOT NULL", relname, args[1]);
-	
-	if (newtuple != NULL)						/* UPDATE */
+		elog(ERROR, "ttdummy (%s): %s must be NOT NULL", relname, args[1]);
+
+	if (newtuple != NULL)		/* UPDATE */
 	{
-		newon = SPI_getbinval (newtuple, tupdesc, attnum[0], &isnull);
+		newon = SPI_getbinval(newtuple, tupdesc, attnum[0], &isnull);
 		if (isnull)
-			elog(WARN, "ttdummy (%s): %s must be NOT NULL", relname, args[0]);
-		newoff = SPI_getbinval (newtuple, tupdesc, attnum[1], &isnull);
+			elog(ERROR, "ttdummy (%s): %s must be NOT NULL", relname, args[0]);
+		newoff = SPI_getbinval(newtuple, tupdesc, attnum[1], &isnull);
 		if (isnull)
-			elog(WARN, "ttdummy (%s): %s must be NOT NULL", relname, args[1]);
-		
-		if ( oldon != newon || oldoff != newoff )
-			elog (WARN, "ttdummy (%s): you can't change %s and/or %s columns (use set_ttdummy)",
-					relname, args[0], args[1]);
-		
-		if ( newoff != TTDUMMY_INFINITY )
+			elog(ERROR, "ttdummy (%s): %s must be NOT NULL", relname, args[1]);
+
+		if (oldon != newon || oldoff != newoff)
+			elog(ERROR, "ttdummy (%s): you can't change %s and/or %s columns (use set_ttdummy)",
+				 relname, args[0], args[1]);
+
+		if (newoff != TTDUMMY_INFINITY)
 		{
-			pfree (relname);	/* allocated in upper executor context */
+			pfree(relname);		/* allocated in upper executor context */
 			return (NULL);
 		}
 	}
 	else if (oldoff != TTDUMMY_INFINITY)		/* DELETE */
 	{
-		pfree (relname);
+		pfree(relname);
 		return (NULL);
 	}
-	
+
 	{
-		struct varlena *seqname = textin ("ttdummy_seq");
-		
-		newoff = nextval (seqname);
-		pfree (seqname);
+		struct varlena *seqname = textin("ttdummy_seq");
+
+		newoff = nextval(seqname);
+		pfree(seqname);
 	}
-	
+
 	/* Connect to SPI manager */
 	if ((ret = SPI_connect()) < 0)
-		elog(WARN, "ttdummy (%s): SPI_connect returned %d", relname, ret);
-	
+		elog(ERROR, "ttdummy (%s): SPI_connect returned %d", relname, ret);
+
 	/* Fetch tuple values and nulls */
-	cvals = (Datum *) palloc (natts * sizeof (Datum));
-	cnulls = (char *) palloc (natts * sizeof (char));
+	cvals = (Datum *) palloc(natts * sizeof(Datum));
+	cnulls = (char *) palloc(natts * sizeof(char));
 	for (i = 0; i < natts; i++)
 	{
-		cvals[i] = SPI_getbinval ((newtuple != NULL) ? newtuple : trigtuple, 
-								  tupdesc, i + 1, &isnull);
+		cvals[i] = SPI_getbinval((newtuple != NULL) ? newtuple : trigtuple,
+								 tupdesc, i + 1, &isnull);
 		cnulls[i] = (isnull) ? 'n' : ' ';
 	}
-	
+
 	/* change date column(s) */
-	if (newtuple)					/* UPDATE */
+	if (newtuple)				/* UPDATE */
 	{
-		cvals[attnum[0] - 1] = newoff;			/* start_date eq current date */
+		cvals[attnum[0] - 1] = newoff;	/* start_date eq current date */
 		cnulls[attnum[0] - 1] = ' ';
-		cvals[attnum[1] - 1] = TTDUMMY_INFINITY;	/* stop_date eq INFINITY */
+		cvals[attnum[1] - 1] = TTDUMMY_INFINITY;		/* stop_date eq INFINITY */
 		cnulls[attnum[1] - 1] = ' ';
 	}
-	else							/* DELETE */
+	else
+/* DELETE */
 	{
-		cvals[attnum[1] - 1] = newoff;			/* stop_date eq current date */
+		cvals[attnum[1] - 1] = newoff;	/* stop_date eq current date */
 		cnulls[attnum[1] - 1] = ' ';
 	}
-	
+
 	/* if there is no plan ... */
 	if (splan == NULL)
 	{
 		void	   *pplan;
 		Oid		   *ctypes;
 		char		sql[8192];
-		
+
 		/* allocate ctypes for preparation */
 		ctypes = (Oid *) palloc(natts * sizeof(Oid));
-		
+
 		/*
-		 * Construct query: 
-		 * 	INSERT INTO _relation_ VALUES ($1, ...)
+		 * Construct query: INSERT INTO _relation_ VALUES ($1, ...)
 		 */
 		sprintf(sql, "INSERT INTO %s VALUES (", relname);
 		for (i = 1; i <= natts; i++)
 		{
 			sprintf(sql + strlen(sql), "$%d%s",
-				i, (i < natts) ? ", " : ")");
+					i, (i < natts) ? ", " : ")");
 			ctypes[i - 1] = SPI_gettypeid(tupdesc, i);
 		}
-		
+
 		/* Prepare plan for query */
 		pplan = SPI_prepare(sql, natts, ctypes);
 		if (pplan == NULL)
-			elog(WARN, "ttdummy (%s): SPI_prepare returned %d", relname, SPI_result);
-		
+			elog(ERROR, "ttdummy (%s): SPI_prepare returned %d", relname, SPI_result);
+
 		pplan = SPI_saveplan(pplan);
 		if (pplan == NULL)
-			elog(WARN, "ttdummy (%s): SPI_saveplan returned %d", relname, SPI_result);
-		
+			elog(ERROR, "ttdummy (%s): SPI_saveplan returned %d", relname, SPI_result);
+
 		splan = pplan;
 	}
-	
+
 	ret = SPI_execp(splan, cvals, cnulls, 0);
-	
+
 	if (ret < 0)
-		elog(WARN, "ttdummy (%s): SPI_execp returned %d", relname, ret);
-	
+		elog(ERROR, "ttdummy (%s): SPI_execp returned %d", relname, ret);
+
 	/* Tuple to return to upper Executor ... */
-	if (newtuple)									/* UPDATE */
+	if (newtuple)				/* UPDATE */
 	{
 		HeapTuple	tmptuple;
-		
-		tmptuple = SPI_copytuple (trigtuple);
-		rettuple = SPI_modifytuple (rel, tmptuple, 1, &(attnum[1]), &newoff, NULL);
-		SPI_pfree (tmptuple);
+
+		tmptuple = SPI_copytuple(trigtuple);
+		rettuple = SPI_modifytuple(rel, tmptuple, 1, &(attnum[1]), &newoff, NULL);
+		SPI_pfree(tmptuple);
 	}
-	else											/* DELETE */
+	else
+/* DELETE */
 		rettuple = trigtuple;
-	
-	SPI_finish();		/* don't forget say Bye to SPI mgr */
-	
-	pfree (relname);
+
+	SPI_finish();				/* don't forget say Bye to SPI mgr */
+
+	pfree(relname);
 
 	return (rettuple);
 }
@@ -619,24 +620,24 @@ ttdummy()
 int32
 set_ttdummy(int32 on)
 {
-	
-	if (ttoff)				/* OFF currently */
+
+	if (ttoff)					/* OFF currently */
 	{
 		if (on == 0)
 			return (0);
-		
+
 		/* turn ON */
 		ttoff = false;
 		return (0);
 	}
-	
+
 	/* ON currently */
 	if (on != 0)
 		return (1);
-	
+
 	/* turn OFF */
 	ttoff = true;
-	
+
 	return (1);
 
 }
