@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Header: /usr/local/cvsroot/pgsql/src/test/regress/regress.sh,v 1.11 1997/06/03 14:19:28 thomas Exp $
+# $Header: /usr/local/cvsroot/pgsql/src/test/regress/regress.sh,v 1.18 1998/03/15 07:39:04 scrappy Exp $
 #
 if echo '\c' | grep -s c >/dev/null 2>&1
 then
@@ -14,17 +14,20 @@ if [ -d ./obj ]; then
 	cd ./obj
 fi
 
-TZ="PST8PDT"; export TZ
+PGTZ="PST8PDT"; export PGTZ
+PGDATESTYLE="Postgres,US"; export PGDATESTYLE
 
 #FRONTEND=monitor
 FRONTEND="psql -n -e -q"
 
+SYSTEM=`uname -s`
+
 echo "=============== Notes...                              ================="
 echo "postmaster must already be running for the regression tests to succeed."
-echo "The time zone must be set to PST/PDT for the date and time data types"
-echo " to pass the regression tests; to do this type"
-echo "   setenv TZ $TZ"
-echo " before starting postmaster. regress/README has more information."
+echo "The time zone is now set to PST8PDT explicitly by this regression test"
+echo " client frontend. Please report any apparent problems to"
+echo "   ports@postgresql.org"
+echo "See regress/README for more information."
 echo ""
 
 echo "=============== destroying old regression database... ================="
@@ -38,12 +41,29 @@ if [ $? -ne 0 ]; then
 fi
 
 echo "=============== running regression queries...         ================="
-for i in `cat sql/tests`
+echo "" > regression.diffs
+if [ a$MB != a ];then
+	mbtests=`echo $MB|tr A-Z a-z`
+else
+	mbtests=""
+fi
+for i in `cat sql/tests` $mbtests
 do
 	$ECHO_N "${i} .. " $ECHO_C
 	$FRONTEND regression < sql/${i}.sql > results/${i}.out 2>&1
-	if [ `diff expected/${i}.out results/${i}.out | wc -l` -ne 0 ]
+	if [ -f expected/${i}-${SYSTEM}.out ]
 	then
+		EXPECTED="expected/${i}-${SYSTEM}.out"
+	else
+		EXPECTED="expected/${i}.out"
+	fi
+  
+	if [ `diff ${EXPECTED} results/${i}.out | wc -l` -ne 0 ]
+	then
+		( diff -wC3 ${EXPECTED} results/${i}.out; \
+		echo "";  \
+		echo "----------------------"; \
+		echo "" ) >> regression.diffs
 		echo failed
 	else
 		echo ok
